#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
package CPIC;
use strict;

sub completion {
    my $cpic=shift;
    my ($edr,$cfg,$prod,$rel,$web);
    $edr=Obj::edr();
    $rel=$cpic->rel;

    $rel->remove_multilock();

    unless (Cfg::opt('precheck')) {
        $rel->completion();
        $prod=$cpic->prod;
        $cpic->set_reboot_info();
        $prod->completion_messages();
        $rel->config_without_reboot_message();

        # to set right values in responsefile for (precheck +) install+config
        $cfg=Obj::cfg();
        Cfg::set_opt($cpic->{task}) if (($cpic->{task} eq 'install') && Cfg::opt('configure'));

        unless($cpic->{noresponsefile}) {
            # Create files/response files
            $cpic->responsefile_comments();
            $edr->{all_tunables} = $cpic->all_tunables;
            $cfg->create_responsefile();
        }

        # Dump performance metrics data
        $edr->dumpmetrics();

        # Ask feedback
        $rel->cli_ask_feedback();

        if (Obj::webui()) {
            unless (Cfg::opt('vom')){
                $rel->check_update();
                $web=Obj::web();
                $web->web_script_form('completion');
            }
        }

        $edr->update_vom_task_status() if (Cfg::opt('vomplugin'));

        # Handle the log files
        $rel->upload_logs();
        $rel->check_update() unless (Obj::webui());
        $rel->reboot_messages();
    }

    if (Cfg::opt('vom')) {
        $web = Obj::web();
        my $trans;
        if ($web->{trans}) {
            $trans = $web->{trans};
        } else {
            $trans='Transport::VOM'->new();
        }
        $trans->complete_update_status();
    }
    $edr->exit_noexitfile();

    return '';
}

sub update_summary {
    my ($cpic,$msg,@items) = @_;
    my $summary="$msg->{msg}\n\n";
    for my $item(@items) { $summary.="\t$item\n"; }
    Msg::add_summary($summary,1);
    return;
}

sub set_reboot_info {
    my $cpic=shift;
    my (@start_sys,$prod);
    $cpic->{reboot_systems}=$cpic->systems_need_reboot("check_tunables");
    return unless (@{$cpic->{reboot_systems}});
    return unless ($cpic->{prod});
    $prod=$cpic->prod;
    for my $sys (@{$cpic->{systems}}) {
        $prod->require_start_after_reboot_sys($sys);
        push (@start_sys, $sys->{sys}) if ($sys->{require_start_after_reboot});
    }
    $cpic->{require_start_after_reboot_systems}=\@start_sys;
    return;
}

sub responsefile_comments {
    my $cpic=shift;
    my ($prod,$cmt,$pdfrs,$edr);

    # Each response file comment is a 4 item list
    # item 1 is the comment, previously translated in the prior line
    # item 2 a 0=optional, 1=required
    # item 3 is 0=scalar, 1=list
    # item 4 is 0=1d, 1=2d is SYSTEM, other=other second dimension

    # for makeresponsefile upgrade where sometimes fake systems are specified and product cannot be detected
    if ($cpic->{prod}) {
        $prod=$cpic->prod;
        $prod->responsefile_comments();
    }
    $pdfrs=Msg::get('pdfrs');
    # As create_responsefile() is in CFG using $edr->{rfc} hash instead of $cpic->{rfc}
    # EDR option comments are moved to EDR::responsefile_comments()
    $edr=Obj::edr();
    #$cmt=Msg::new("This variable defines a list of optional $pdfrs not to be installed on the systems");
    #$edr->{rfc}{donotinstall}=[$cmt->{msg},0,1,0];
    #$cmt=Msg::new("This variable defines a list of $pdfrs not to be removed from the systems during uninstall");
    #$edr->{rfc}{donotremove}=[$cmt->{msg},0,1,0];
    $cmt=Msg::new("This variable defines a list of license keys to be registered on the systems during install");
    $edr->{rfc}{keys}=[$cmt->{msg},0,1,1];
    $cmt=Msg::new("This variable designates a standard configure task");
    $edr->{rfc}{opt__configure}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs $pdfrs and starts the products that do not require configuration");
    $edr->{rfc}{opt__install}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable licenses the product only");
    $edr->{rfc}{opt__license}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs the product without requiring entry of a license key");
    $edr->{rfc}{opt__nolic}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable installs the product with keyless license");
    $edr->{rfc}{opt__vxkeyless}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines if you need to re-configure VCS");
    $edr->{rfc}{donotreconfigurevcs}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines if you need to re-configure fencing");
    $edr->{rfc}{donotreconfigurefencing}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable stores the keyless key you want to register");
    $edr->{rfc}{key}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable indicates that read access for VCS cluster information is granted for everyone");
    $edr->{rfc}{defaultaccess}=[$cmt->{msg},0,0,0];
#    $cmt=Msg::new("This variable updates the keyless license to the current version");
#    $edr->{rfc}{opt__updatekeys}=[$cmt->{msg},0,0,0];
#    $cmt=Msg::new("This variable defines a location, typically an NFS mount, from which all remote systems can install product patches. The location must be accessible from all target systems");
#    $edr->{rfc}{opt__patchpath}=[$cmt->{msg},0,0,0];
#    $cmt=Msg::new("This variable defines a location, typically an NFS mount, from which all remote systems can install product $pdfrs. The location must be accessible from all target systems");
#    $edr->{rfc}{opt__pkgpath}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the location where a working directory is created to store temporary files and $pdfrs needed during the install.  The default location is /var/tmp");
    $edr->{rfc}{opt__tmppath}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable uninstalls a product");
    $edr->{rfc}{opt__uninstall}=[$cmt->{msg},0,0,0];
    #$cmt=Msg::new("This variable installs only minimum product $pdfrs");
    #$edr->{rfc}{opt__installminpkgs}=[$cmt->{msg},0,0,0];
    #$cmt=Msg::new("This variable installs only recommended product $pdfrs");
    #$edr->{rfc}{opt__installrecpkgs}=[$cmt->{msg},0,0,0];
    #$cmt=Msg::new("This variable installs all product $pdfrs");
    #$edr->{rfc}{opt__installallpkgs}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the list of systems on which the product is to be installed, uninstalled, or configured");
    $edr->{rfc}{systems}=[$cmt->{msg},1,1,0];
    $cmt=Msg::new("This variable defines the list of systems to be recognized for configuration when a secure environment prevents all systems from being installed at once");
    $edr->{rfc}{systemscfg}=[$cmt->{msg},0,1,0];
    $cmt=Msg::new("This variable is an abbreviation that defines the product to be installed, uninstalled, or configured");
    $edr->{rfc}{prod}=[$cmt->{msg},1,0,0];
    $cmt=Msg::new("This variable defines the component that is being used");
    $edr->{rfc}{activecomponent}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines whether user agrees with EULA.pdf on the media");
    $edr->{rfc}{accepteula}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines whether user agrees to upload the log files onto our website");
    $edr->{rfc}{uploadlogs}=[$cmt->{msg},0,0,0];

    if ($cpic->maintenance_release) {
        $cmt=Msg::new("This variable defines the path of a base level release to be integrated with a maintenance level release for the two releases to be simultaneously installed");
        $edr->{rfc}{opt__base_path}=[$cmt->{msg},0,0,0];
    }

    $cmt=Msg::new("The variable defines the path of a patch-level release that is to be integrated with a base or maintenance-level release, when multiple releases are installed simultaneously.");
    $edr->{rfc}{opt__patch_path}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("The variable defines the path of a second patch-level release that is to be integrated with a base or maintenance-level release, when multiple releases are installed simultaneously.");
    $edr->{rfc}{opt__patch2_path}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("The variable defines the path of a third patch-level release that is to be integrated with a base or maintenance-level release, when multiple releases are installed simultaneously.");
    $edr->{rfc}{opt__patch3_path}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the path of a fourth patch level release to be integrated with a base or a maintenance level release for multiple releases to be simultaneously installed.");
    $edr->{rfc}{opt__patch4_path}=[$cmt->{msg},0,0,0];
    $cmt=Msg::new("This variable defines the path of a fifth patch level release to be integrated with a base or a maintenance level release for multiple releases to be simultaneously installed");
    $edr->{rfc}{opt__patch5_path}=[$cmt->{msg},0,0,0];

    return;
}


# EDR completion is coded in implementing module versus the
# EDR module to allow whatever additional customization is necessary
sub edr_completion {
    my $cpic=shift;
    my $web=Obj::web();
    my $cfg=Obj::cfg();
    my $edr=Obj::edr();

    # to set right values in responsefile for (precheck +) install+config
    Cfg::set_opt($cpic->{task}) if (($cpic->{task} eq 'install') && Cfg::opt('configure'));

    unless($cpic->{noresponsefile}) {
        $cpic->responsefile_comments();
        $edr->{all_tunables} = $cpic->all_tunables;
        $cfg->create_responsefile();
    }

    $web->web_script_form('completion') if (Obj::webui());
    $edr->update_vom_task_status() if (Cfg::opt('vomplugin'));

    $edr->exit_noexitfile();
    return '';
}

sub precheck_report {
    my $cpic=shift;
    my ($edr,$msg,$errors,$warnings,$task,$tmpsub);
    $edr=Obj::edr();

    Msg::n();
    $msg=Msg::new("Precheck report completed");
    $msg->print;

    # count errors and warnings
    for my $sys (@{$cpic->{systems}}) {
        $errors+=scalar(@{$sys->{errors}}) if ($sys->{errors});
        $warnings+=scalar(@{$sys->{warnings}}) if ($sys->{warnings});
    }
    $edr->display_errors_warnings();

    if (!$errors) {
        if (!$warnings) {
            $msg=Msg::new("No issues found in prechecks\n");
            $msg->print;
        }
        if ($cpic->ask_install()) {
            $task = Cfg::opt('upgrade') ? 'upgrade' :
                    Cfg::opt('patchupgrade') ? 'patchupgrade' :
                    Cfg::opt('hotfixupgrade') ? 'hotfixupgrade' : 'install';
            $cpic->set_task($task);
            if ($task =~ /^(install|upgrade)$/mx) {
                $tmpsub = $task.'_useragreement';
                $cpic->$tmpsub();
            }
            $cpic->cli_set_title();
            $cpic->options();
            #fix : incident 3879652 (precheck+)install+config  the install not set in response file
            $cpic->{task}=$task if (Cfg::opt('install'));
            return;
        }
    }
    $cpic->rel->completion();
    $cpic->edr_completion();
    return;
}

sub ask_install {
    my $cpic=shift;
    my ($rel,$ayn,$cfg,$msg,$prod,$systems,$continue);
    $prod=$cpic->prod;
    $cfg=Obj::cfg();
    $rel=$cpic->rel();
    $systems = join(' ',@{$cfg->{systems}});
    if (Cfg::opt('upgrade')) {
        $msg=Msg::new("Would you like to upgrade $prod->{abbr} on $systems?");
    } else {
        $msg=Msg::new("Would you like to install $prod->{abbr} on $systems?");
    }
    $ayn=$msg->aynn();

    if ($ayn eq 'Y') {
        $continue = 1;
        # down required path if users choose to go on
        if ($rel->{required_patch_id}) {
            $continue = $cpic->install_must_have_patch($rel->{required_patch_id});
        }
    }

    Msg::n();
    return $continue;
}

1;
