#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
package CPIC;
use strict;

sub rollback {
    my $cpic=shift;
    my ($phase,$msg,$failmsg,$prod,$web,$errors);
    return 1 if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));

    $web = Obj::web();
    $web->web_script_form('uninstalling') if (Obj::webui());

    $phase=$cpic->setup_rollback_phase();
    return 1 if (!$phase->tasks_steps());

    $phase->execute_tasks();

    $prod=$cpic->prod;
    $msg=Msg::new("$prod->{name} Rollback completed successfully");
    $failmsg=Msg::new("$prod->{name} Rollback did not complete successfully");
    $errors=$cpic->failures('uninstallfailmsg');
    $msg->display_completion($errors, $msg, $failmsg);
    # bail out uninstallrp if there is any patches failed to be rollback
    if ($#$errors>=0) {
        $msg=Msg::new("Manually uninstall the failed patches and reboot the systems to start the processes.\n");
        $msg->print();
        $cpic->completion();
    }
    return 1;
}

sub setup_rollback_phase {
    my $cpic=shift;
    my ($phase,$desc,$prod,$task,$task_name,%task_params,$patch,$rel_vers,$id,$upgraded_sub);

    $cpic->{falsefailures}=0;
    $cpic->{uninstallpatches}=$cpic->verify_list('uninstallpatches');

    $prod=$cpic->prod;
    $rel_vers=$cpic->rel->{vers};

    $phase= Phase->new('CPIC_rollback');
    $desc = Msg::new("Uninstalling $prod->{abbr}");
    $phase->set_description($desc);
    $phase->initialize_tasks();
    $upgraded_sub = upgrade_sub('preremove_sys');
    if ($prod->can($upgraded_sub)) {
        $id=10;
        $task_name = "product_preremove";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Performing $prod->{abbr} preremove tasks"),
            'description_sys' => Msg::new("Performing $prod->{abbr} preremove tasks on #{SYS}"),
            'if_per_system' => 1,
            'pre_handler' => \&CPIC::prod_preremove,
            'pre_handler_object_arg' => $cpic,
            'handler' => \&CPIC::prod_preremove_sys,
            'handler_object_arg' => $cpic,
            'post_handler' => \&CPIC::prod_preremove_errors,
            'post_handler_object_arg' => $cpic,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    $id=100;
    for my $patchi (@{$cpic->{uninstallpatches}}) {
        $patch=$cpic->patch($patchi);
        $task_name = "uninstall_patch_$patchi";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Uninstalling $patch->{showform} patch"),
            'description_sys' => Msg::new("Uninstalling $patch->{showform} patch on #{SYS}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::patch_uninstall_sys,
            'handler_object_arg' => $cpic,
            'handler_args' => [ $patchi, $rel_vers ],
            'rc_handler' => \&CPIC::patch_uninstall_rc_check_sys,
            'rc_handler_object_arg' => $cpic,
            'rc_handler_args' => [ $patchi ],
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);

        # Skip the task on some systems for serial mode
        for my $sys (@{$cpic->{systems}}) {
            if (!EDRu::inarr($patchi, @{$sys->{uninstallpatches}})) {
                $task->set_skip_sys($sys,1);
            }
        }
        $id++;
    }

    $id=400;
    $upgraded_sub = upgrade_sub('postremove_sys');
    if ($prod->can($upgraded_sub)) {
        $task_name = "product_postremove";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Performing $prod->{abbr} postremove tasks"),
            'description_sys' => Msg::new("Performing $prod->{abbr} postremove tasks on #{SYS}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::prod_postremove_sys,
            'handler_object_arg' => $cpic,
            'post_handler' => \&CPIC::prod_postremove_errors,
            'post_handler_object_arg' => $cpic,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    return $phase;
}

sub patch_uninstall_sys {
    my ($cpic,$sys,$patch,$rel_vers)=@_;
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $patch=$sys->patch($patch) unless (ref($patch));
    return 1 if (Cfg::opt('makeresponsefile'));
    Msg::log("BEGIN-patch_uninstall_sys($patch->{patch_vers},$sys->{sys})");
    # $patch->tsub("preremove_sys", $sys);
    $patch->uninstall_sys($sys);
    my $uof=EDRu::outputfile('uninstall', $sys->{sys}, "patch.$patch->{patch_vers}");
    $sys->copy_to_sys($cpic->localsys,$uof) unless ($sys->{islocal});
    $patch->postremove_sys($sys);
    Msg::log("END-patch_uninstall_sys($patch->{patch_vers},$sys->{sys})");
    # Adding temp failures to test for web script

    return 0 if ($cpic->false_failure);
    return 1 if ($patch->uninstall_success_sys($sys, $rel_vers));
    return 0;
}

sub patch_uninstall_failed_sys {
    my($sys,$patch)=@_;
    $sys->set_value('patchuninstallfail', 'push', $patch->{patch_vers});
    my $pdfr=Msg::get('pdfr');
    my $io=EDRu::readfile(EDRu::outputfile('uninstall', $sys->{sys}, "patch.$patch->{patch_vers}"));
    my $msg=Msg::new("$patch->{patch_vers} $pdfr failed to uninstall on $sys->{sys}");
    $msg->log;
    Msg::log("$io");
    push(@{$sys->{uninstallfailmsg}}, $msg);
    EDRu::appendfile($patch->{patch_vers},
        EDR::tmpdir()."/patch_uninstall_fail.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub patch_uninstall_passed_sys {
    my($sys,$patch)=@_;
    $sys->set_value('patchuninstallpass', 'push', $patch->{patch_vers});
    my $pdfr=Msg::get('pdfr');
    Msg::log("$patch->{patch_vers} $pdfr uninstalled successfully on $sys->{sys}");
    EDRu::appendfile($patch->{patch_vers},
        EDR::tmpdir()."/patch_uninstall_pass.$sys->{sys}") if (Cfg::opt('bait'));
    return '';
}

sub patch_uninstall_rc_check_sys {
    my ($cpic,$sys,$patch,$rc)= @_;
    $patch=$sys->patch($patch) unless (ref($patch));
    if ($rc) {
        patch_uninstall_passed_sys($sys, $patch);
    } elsif (defined $rc) {
        patch_uninstall_failed_sys($sys, $patch);
    }
    return $rc;
}

1;
