#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
package Prod;
use strict;
@Prod::ISA = qw(CPIP);

sub pre_init {
    my $prod=shift;
    (undef,$prod->{prodi},$prod->{padv})=split(/::/m,$prod->{class},3);
    return;
}

sub minpkgs { return $_[0]->{minpkgs} || []; }
sub recpkgs { return $_[0]->{recpkgs} || []; }
sub allpkgs { return $_[0]->{allpkgs} || []; }

# To get obsoleted packages of GA release
sub obsoleted_ga_release_pkgs {
    return ($_[0]->{obsoleted_ga_release_pkgs} || []);
}

# To get obsoleted packages of all previous releases
sub obsoleted_previous_releases_pkgs {
    return ($_[0]->{obsoleted_previous_releases_pkgs} || []);
}

# To get obsoleted packages which need refresh to new version when upgrade if it is installed
sub obsoleted_but_need_refresh_when_upgrade_pkgs {
    return ($_[0]->{obsoleted_but_need_refresh_when_upgrade_pkgs} || []);
}

# To get obsoleted packages which are still supported in current version
sub obsoleted_but_still_support_pkgs {
    return ($_[0]->{obsoleted_but_still_support_pkgs} || []);
}

sub allpkgs_on_media {
    my $prod=shift;
    return EDRu::arruniq(@{$prod->{allpkgs}},
                         @{$prod->{obsoleted_but_need_refresh_when_upgrade_pkgs}},
    );
}

sub set_pkgs {
    my $prod=shift;
    my ($rel,@categories,$category);

    $rel=$prod->rel;
    @categories=qw(minpkgs recpkgs allpkgs
                   obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$prod->{$category}});
    }

    return $prod->{allpkgs};
}

sub get_pkgs {
    my ($prod,$allpkgs_flag)= @_;
    return ($allpkgs_flag) ? $prod->allpkgs :
            Cfg::opt('installminpkgs') ? $prod->minpkgs :
            Cfg::opt('installrecpkgs') ? $prod->recpkgs : $prod->allpkgs;
}

sub install_thirdparty_pkgs_sys { return; }

sub obsoleted {
    my $prod = shift;
    my $obsoleted = $prod->{obsoleted} ||
                    $prod->{obsoleted_by_removing} ||
                    $prod->{obsoleted_by_merging};
    return $obsoleted;
}

sub pkgs_remove {
    my ($prod,@pkgs) = @_;
    $prod->{minpkgs}=EDRu::arrdel($prod->{minpkgs},@pkgs);
    $prod->{recpkgs}=EDRu::arrdel($prod->{recpkgs},@pkgs);
    $prod->{allpkgs}=EDRu::arrdel($prod->{allpkgs},@pkgs);
    return '';
}

sub geteula {
    my ($prod,$lang)=@_;
    my $cpic = Obj::cpic();
    my $rel = $prod->rel;
    #workaround for incident:3379647, change the name of EULA file if base_path is not same as basetitlevers
    if (Cfg::opt('base_path') && $cpic->{base_requires} ne $rel->{basetitlevers}) {
        if($cpic->{base_requires}) {
            $prod->{eula} =~ s/$rel->{basetitlevers}/$cpic->{base_requires}/;
        }
    }
    return "EULA/$lang/$prod->{eula}";
}

sub ask_upgrade_configure {
    my ($prod,$sys)=@_;
    return $prod->perform_task('ask_upgrade_configure');
}

sub ask_patchupgrade_configure {
    my ($prod,$sys)=@_;
    return $prod->perform_task('ask_patchupgrade_configure');
}

sub upgrade_prestop_config {
    my $prod = shift;
    return $prod->perform_task('upgrade_prestop_config');
}

# common Prod subs and trace-only stubs for optional Prod subs
sub perform_task_sys { return 1 }
sub perform_task { return 1 }
sub check_config { }

sub configure_alternate_prod { }

sub precheck_task { my ($prod,$sys)=@_; return $prod->perform_task('precheck_task'); }
sub post_precheck_task { my($prod,$sys)=@_; return $prod->perform_task('post_precheck_task'); }
sub preinstall_tasks { my ($prod,$sys)=@_; return $prod->perform_task('preinstall_tasks'); }
sub precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'precheck_sys'); }
sub patchupgrade_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'patchupgrade_precheck_sys'); }
sub hotfixupgrade_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'hotfixupgrade_precheck_sys'); }
sub uninstall_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'uninstall_precheck_sys'); }
sub upgrade_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_precheck_sys'); }
sub rollback_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'rollback_precheck_sys'); }
sub configure_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'configure_precheck_sys'); }
sub start_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'start_precheck_sys'); }
sub stop_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'stop_precheck_sys'); }
sub postcheck_precheck_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postcheck_precheck_sys'); }
sub require_start_after_reboot_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'require_start_after_reboot_sys'); }
sub licensed_sys { return 1; }
sub pre_shutdown_question {my ($prod,$sys)=@_; return $prod->perform_task('pre_shutdown_question'); }
sub handle_precheck_issues { my ($prod)=@_; return $prod->perform_task('handle_precheck_issues'); }
sub handle_precheck_fatal_errors { my ($prod)=@_; return $prod->perform_task('handle_precheck_fatal_errors'); }
sub any2any_precheck_sys { return 1 };
# Need add the following commented subs for SF/SFHA/SFCFS/SFCFSRAC/SFRAC
sub prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'prestop_sys'); }
sub poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'poststop_sys'); }
sub preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'preremove_sys'); }
sub postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postremove_sys'); }
sub preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'preinstall_sys'); }
sub postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'postinstall_sys'); }
sub configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'configure_sys'); }
#new add one
sub prestart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'prestart_sys'); }
sub poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'poststart_sys'); }

sub upgrade_prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_prestop_sys'); }
sub upgrade_poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststop_sys'); }
sub upgrade_preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preremove_sys'); }
sub upgrade_postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postremove_sys'); }
sub upgrade_preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preinstall_sys'); }
sub upgrade_postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postinstall_sys'); }
sub upgrade_configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_configure_sys'); }
sub upgrade_poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststart_sys'); }

sub patchupgrade_prestop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_prestop_sys'); }
sub patchupgrade_poststop_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststop_sys'); }
sub patchupgrade_preremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preremove_sys'); }
sub patchupgrade_postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postremove_sys'); }
sub patchupgrade_preinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_preinstall_sys'); }
sub patchupgrade_postinstall_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postinstall_sys'); }
sub patchupgrade_configure_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_configure_sys'); }
sub patchupgrade_poststart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_poststart_sys'); }

sub register_postchecks { my ($prod,$sys)=@_; return $prod->perform_task('register_postchecks'); }

# Keep the following 2 subs to be compatible with CPIP version before UXRT61.
sub register_postchecks_per_system { my ($prod,$sys)=@_; return $prod->perform_task('register_postchecks_per_system'); }
sub register_postchecks_per_cluster { my ($prod,$sys)=@_; return $prod->perform_task('register_postchecks_per_cluster'); }

sub verify_responsefile { my ($prod,$sys)=@_; return $prod->perform_task('verify_responsefile'); }
sub responsefile_prestart_config { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_prestart_config'); }
sub responsefile_poststart_config { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_poststart_config'); }
sub responsefile_comments { my ($prod,$sys)=@_; return $prod->perform_task('responsefile_comments'); }

sub cli_initiation_questions { }
sub web_initiation_questions { }
sub cli_prod_option { }
sub web_prod_option { }

sub cli_preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('cli_preinstall_messages'); }
sub web_preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('web_preinstall_messages'); }
sub preinstall_messages { my ($prod,$sys)=@_; return $prod->perform_task('preinstall_messages'); }
sub cli_prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('cli_prestart_config_questions'); }
sub web_prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('web_prestart_config_questions'); }
sub prestart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('prestart_config_questions'); }
sub cli_poststart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('cli_poststart_config_questions'); }
sub web_poststart_config_questions { my ($prod,$sys)=@_; return $prod->perform_task('web_poststart_config_questions'); }
sub completion_messages { my ($prod,$sys)=@_; return $prod->perform_task('completion_messages'); }
sub preremove_tasks {  my ($prod)=@_; return $prod->perform_task('preremove_tasks'); }


sub preinstall_errors {return handle_errors(@_);}
sub postinstall_errors {return handle_errors(@_);}
sub preremove_errors {return handle_errors(@_);}
sub postremove_errors {return handle_errors(@_);}
sub config_errors {return handle_errors(@_);}
sub prestop_errors {return handle_errors(@_);}
sub poststop_errors {return handle_errors(@_);}
sub check_dmp_native_support_reboot_sys { }
sub update_prod { }
sub copy_cpi_sys { }

sub other_requirements {
    my $prod=shift;
    my $msg=$prod->{other_requirements};
    return $msg->{msg} if (defined $msg);
    return;
}

sub description {
    my $prod=shift;
    my $msg=Msg::new("No description for $prod->{name} defined");
    $msg->print;
    return;
}

sub obsoleted_bundled_pkgs_sys {
    my ($prod,$sys) = @_;
    my ($msg,@bundled_pkgs,$pkgs);
    return '' unless ($prod->{padv} =~ /HPUX/m);
    # HPUX OEM bundle pkgs handling
    @bundled_pkgs=();
    if (defined $sys->{obsoleted_bundled_pkgs}) {
        push (@bundled_pkgs, @{$sys->{obsoleted_bundled_pkgs}});
    }
    for my $pkg (@{$prod->{obsoleted_bundled_pkgs}}) {
        push (@bundled_pkgs, $pkg) if ($sys->padv->bundle_pkg_sys($sys,$pkg));
    }
    if (@bundled_pkgs) {
        $sys->set_value('obsoleted_bundled_pkgs','list',@bundled_pkgs);
        $pkgs=join(' ',@bundled_pkgs);
        $msg=Msg::new("The following obsolete depot(s) bundled with HP-UX will be uninstalled on $sys->{sys}:\n\t$pkgs");
        if (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade))) {
            $sys->push_warning($msg);
        } else {
            $sys->push_note($msg);
        }
    }
    return 1;
}

sub obsoleted_bundles_sys {
    my ($prod,$sys) = @_;
    my ($msg,@bundles,$bundles);
    return '' unless ($prod->{padv} =~ /HPUX/m);
    # HPUX OEM bundle handling
    for my $bundle (@{$prod->{obsoleted_bundles}}) {
        push (@bundles, $bundle) if ($sys->padv->bundle_sys($sys, $bundle));
    }
    if (@bundles) {
        $sys->set_value('uninstallbundles','list',@bundles);
        ### push note that some OEM bunldles founded
        $bundles=join(' ',@bundles);
        $msg=Msg::new("The following obsolete bundle(s) with HP-UX will be uninstalled on $sys->{sys}:\n\t$bundles");
        if (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade))) {
            $sys->push_warning($msg);
        } else {
            $sys->push_note($msg);
        }
    }
    return 1;
}

sub disableconfig {
    my $prod=shift;
    return ($prod->{disable_config}) ? 1 : 0;
}

sub hasconfig {
    my $prod=shift;
    return ($prod->{has_config}) ? 1 : 0;
}

sub donotreconfig {
    my $prod=shift;
    my ($cfg,$rel,$previouscomponent,$activecomponent,$rtn);

    # Donnot stop VCS during reconfigure from VCS to SF
    $cfg=Obj::cfg();
    $rel=Obj::rel();
    $rtn=$rel->get_reconfigure_attr();
    return 1 if (($rtn eq 'Configure') || $cfg->{donotreconfigurevcs});
    return 0;
}

sub donotupgrade_pkg {
    my ($prod,$pkg)= @_;
    return EDRu::inarr($pkg->{pkgi}, @{$prod->{donotupgradepkgs}});
}

sub installtime_sys {
    my ($prod,$sys) = @_;
    return if(!$prod->{mainpkg});
    my $pkg=$sys->pkg($prod->{mainpkg});
    return $pkg->installtime_sys($sys);
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;
    my ($pkgvers,$rel,$mpvers,$pkg);
    return '' unless ($prod->{mainpkg});
    $pkg=$sys->pkg($prod->{mainpkg});
    $pkgvers=$pkg->version_sys($sys,$force_flag);
    $mpvers=$prod->{vers} if ($pkgvers && $prod->check_installed_patches_sys($sys,$pkgvers));
    $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag);
    return ($mpvers || $pkgvers);
}

sub version_with_extra_mainpkgs_sys {
    my ($prod,$sys) = @_;
    my ($extra_mainpkg_version, $extra_mainpkgs, $pkg, $pkgvers, $mpvers, $maxver, $cv);
    $maxver=$prod->version_sys($sys);
    if (defined $prod->{extra_mainpkgs}) {
        $extra_mainpkgs = $prod->{extra_mainpkgs};
        foreach my $extra_mainpkg (@$extra_mainpkgs) {
            $pkg = $sys->pkg($extra_mainpkg);
            next unless (defined $sys->{pkgvers}{$pkg->{pkg}});
            $pkgvers = $pkg->version_sys($sys);
            # Incident 3737712, product version depend on base version and patch version, not include
            # hotfix version. If base patches and mr patches have been installed properly, CPI reagrad it
            # as a completed product
            $mpvers = $prod->{vers} if ($pkgvers && $prod->check_installed_patches_sys($sys,$pkgvers,3));

            $pkgvers = $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers);
            if ($mpvers){
                $extra_mainpkg_version = $mpvers;
            } else {
                $extra_mainpkg_version = $pkgvers;
            }
            # For Rolling Upgrade phase 2, some extra mainpkgs has already been upgraded to media version.
            # We need to choose the highest version of extra mainpkgs that has not been upgraded to media version comparing prod main pkg.
            # If the extra mainpkg has already been upgraded, we skip this extra mainpkg.
            # E.g. 5.1RP2 SFCFSHA --> 6.0SP1 SFCFSHA, RU phase 2
            # 3 version of package: 5.1RP1(VRTScavf), 5.1RP2(VRTSvcs), 6.0SP1(VRTSvxfs, VRTSvxvm...)
            next if (Cfg::opt('upgrade_nonkernelpkgs') && !EDRu::compvers($extra_mainpkg_version,$pkg->{vers},4));
            $cv = EDRu::compvers($extra_mainpkg_version,$maxver,4);
            $maxver = $extra_mainpkg_version if ($cv == 1);
        }
    }

    if ($sys->hpux() && $prod->rp_installed_sys($sys) && $prod->rel->show_withrp($maxver)){
        $maxver .= " (with rp patches)";
    }
    return $maxver;
}

sub rp_installed_sys {
    my ($prod, $sys, $pkgi,$pkg);
    ($prod,$sys)=@_;
    $pkgi=$prod->{mainpkg};
    return 0 unless($pkgi);
    
    $pkg = $sys->pkg($pkgi);
    return $sys->padv->pkg_has_rp_sys($sys,$pkg);
}

sub revert_base_version_sys {
    my ($prod,$sys,$pkg,$pkgvers,$mpvers,$force_flag) = @_;
    my ($cpic,$pkg_mpvers,$rel,$basevers);
    $cpic = Obj::cpic();
    $rel=$sys->rel;
    # get the pstamp value of pkg to determine pkg version on Solaris
    $pkg_mpvers = $pkg->mpversion_sys($sys,$force_flag) if ($prod->{padv} =~ /Sol/m);
    $pkgvers = ($pkg_mpvers || $pkgvers);
    # cannot get patch version from disk, hence only check mainpkg version.
    # return mainpkg version if pkg only release
    #fix for e3403280, from disk behavior should be the same as from media
    return $pkgvers if (!$rel->{patches});
    # fix for e1989658, there is no need to revert on HPUX since HP is pkg release
    return $pkgvers if ($mpvers || !$pkgvers || $prod->{padv} =~ /HPUX/m);
    $basevers = @{$rel->{basevers}}[-1];
    return !EDRu::compvers($pkgvers,$prod->{vers},4) ? ($prod->{basevers} || $basevers) : $pkgvers;
}

sub register_postcheck_item {
    my ($prod,$ifpersystem,$sequence_id,$name,$desc,$handler) = @_;
    my ($key,$cpic);

    return unless($Obj::pool{CPIC});
    $cpic=Obj::cpic();

    $key="$sequence_id".'_'."$name";
    $desc = $desc->{msg} if (ref($desc) =~ m/^Msg/);
    $cpic->{postchecks}{$key} = {
         'product' => $prod,
         'description' => "$desc",
         'handler' => $handler,
         'if_per_system' => $ifpersystem,
    };

    return 1;
}

sub register_postcheck_item_per_system {
    my $prod=shift;
    $prod->register_postcheck_item(1, @_);
    return;
}

sub register_postcheck_item_per_cluster {
    my $prod=shift;
    $prod->register_postcheck_item(0, @_);
    return;
}

sub handle_errors {
    my ($prod,$cpic) = @_;
    my ($msg,$web,$edr,$errmsg,$exit,$warnings,$errors);
    $edr=Obj::edr();
    $web = Obj::web();

    return 1 if (Cfg::opt(qw(makeresponsefile)) && !Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)));

    # count errors and warnings
    for my $sys (@{$cpic->{systems}}) {
        $errors+=scalar(@{$sys->{errors}}) if ($sys->{errors});
        $warnings+=scalar(@{$sys->{warnings}}) if ($sys->{warnings});
    }

    # print errors and warnings
    if ($errors) {
        $msg=Msg::new("The following errors were discovered on the systems:");
        $msg->print;
        $msg->add_summary(1);
        $msg->addError($msg->{msg});

        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{errors}})) {
                Msg::print($errmsg);
                Msg::add_summary($errmsg);
                $msg->addError($errmsg);
            }
        }
        Msg::n();
    }

    if ($warnings) {
        $msg=Msg::new("The following warnings were discovered on the systems:");
        $msg->print;
        $msg->add_summary(1);
        $msg->addWarning($msg->{msg});

        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{warnings}})) {
                Msg::print($errmsg);
                Msg::add_summary($errmsg);
                $msg->addWarning($errmsg);
                $exit = 1 if ($sys->{warning_exit});
            }
        }
        Msg::n();
    }
    # Error happened, exit with $edr->{exitcode} or non-zero value.
    if ($errors || $exit) {
        if(Obj::webui()){
            my $webmsg;
            $webmsg .= $web->{errors} if ($web->{errors});
            $webmsg .= $web->{warnings} if ($web->{warnings});
            $web->before_cli_exit($webmsg);
        }
        EDR::exit_exitfile();
    }
    $edr->cli_handle_warnings() if ($warnings || $errors);
    return '';
}

sub poststart_errors {
    my ($prod,$cpic) = @_;

    return 1 if (Cfg::opt(qw(makeresponsefile)));
    if (Obj::webui()){
        my $msg = Msg::new('');
        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{warnings}})) {
                $msg->addWarning($errmsg);
            }
        }
    }
    for my $sys (@{$cpic->{systems}}) {
        for my $errmsg ((@{$sys->{warnings}}, @{$sys->{errors}})) {
            push(@{$sys->{startfailmsg}}, $errmsg);
        }
    }

    return;
}

# handles errors in CLI
sub cli_handle_errors {
    my ($msg, $ayn) = @_;

    $msg = Msg::new("Do you want to continue?");
    $ayn = $msg->aynn;
    # Error happened and user don't want to continue.
    # exit with $edr->{exitcode} or non-zero value.
    EDR::exit_exitfile() if ($ayn eq 'N');
    return;
}

# print the os dependency (pkgs/patches/libs)
sub cli_print_os_dependency {
    my ($prod) = @_;
    my (@platvers,$msg,$rel,%os_dependency,$max_length,@os_rpms,@os_libs,@os_build,$pkg,$suffix,$os_with_suffix,%minorverlib);
    $rel=$prod->rel;

    %os_dependency = (
        'ospatches' => Msg::new("OS patches")->{msg},
        'ospkgs'    => Msg::new("OS packages")->{msg},
    );
    if (defined $rel->{platvers}) {
        if ($rel->{padv} =~ /HPUX/m) {
            for my $patchindex (@{$rel->{platvers}}) {
                push (@platvers,"${patchindex}IA");
                push (@platvers,"${patchindex}PA");
            }
        } else {
            @platvers = @{$rel->{platvers}};
        }
    }
    push (@platvers,'all');

    if (Padv::plat($rel->{padv}) eq 'Linux') {
        for my $checked (qw(oslibs)) {
            @os_libs = ();
            @os_rpms = ();
            $max_length = 0;
            for my $pkgi(@{$prod->get_pkgs}) {
                $pkg=$prod->pkg($pkgi);
                for my $patchindex (sort keys %{$pkg->{$checked}}) {
                    @os_rpms = ();
                    while (my ($os_lib,$os_rpm) = each %{$pkg->{$checked}{$patchindex}}) {
                        push(@os_libs,$os_lib);
                        my @rpm_split = split(/\s+/,$os_rpm);
                        push(@os_rpms,@rpm_split);
                    }
                    push @{$minorverlib{$patchindex}},@os_rpms;
                }
            }
            for my $minorver(sort keys %minorverlib) {
                $msg = Msg::new("\nRequired OS rpms for $minorver:");
                $msg->print;
                $minorverlib{$minorver} = EDRu::arruniq(@{$minorverlib{$minorver}});
                if (scalar @{$minorverlib{$minorver}}) {
                    print_ordered_array(\@{$minorverlib{$minorver}});
                } else {
                    $msg = Msg::new("None");
                    $msg->print;
                }
            }
        }
    } else {
        for my $checked (qw(ospatches ospkgs)) {
            $msg=Msg::new("\nRequired $os_dependency{$checked}:");
            $msg->print;
            @os_build = ();
            $max_length = 0;
            for my $pkgi(@{$prod->get_pkgs}) {
                $pkg=$prod->pkg($pkgi);
                for my $patchindex (@platvers) {
                    $suffix = $patchindex eq 'all' ? '' : "($patchindex)";
                    for my $os_dep (@{$pkg->{$checked}{$patchindex}}) {
                        $os_with_suffix = $os_dep . $suffix;
                        push(@os_build,$os_with_suffix);
                        $max_length = length($os_with_suffix) if ($max_length < length($os_with_suffix));
                    }
                }
            }
            @os_build = @{EDRu::arruniq(@os_build)};
            if ($max_length){
                print_ordered_array(\@os_build,$max_length);
            } else {
                $msg = Msg::new("None");
                $msg->print;
            }
        }
    }
    return;
}

sub print_ordered_array {
    my ($arr_ref,$max_length) = @_;
    my (@target,$length,$num,$format,$count,@div);

    @target = sort @$arr_ref;
    unless ($max_length) {
        $max_length = 0;
        for my $sample (@target) {
            $max_length = $max_length > length($sample) ? $max_length : length($sample);
        }
    }
    $length = $max_length + 1;
    $num = int((EDR::get2('tput','termx') - 3) / $length);
    $num = 1 if $num == 0;
    $format = '%-' . "$length" . 's';
    $format = $format x $num;
    $count = 0;
    while ($count < scalar @target) {
        @div = @target[$count..($count + $num - 1)];
        $count += $num;
        Msg::print(Msg::string_sprintf($format,@div));
    }
    return;
}

sub cli_print_requirements {
    my ($prod) = @_;
    my ($otherreqs,$prodspace,$rel,$msg,$platreqs,$platreqs_str,$ps,$ms,@space,$cs,$prodname,$pkg,$pkgspace);
    $rel=$prod->rel;

    $rel->pre_platvers();

    $prodname = $prod->{name};
    $msg=Msg::new("The requirements for product '$prodname':\n");
    $msg->print;

    $platreqs = $rel->get_platreqs($prod);
    if (defined $platreqs) {
        for my $platreq (@{$platreqs}) {
            if ($platreq !~ /RHEL6 U6/) {
                $platreqs_str = $platreqs_str.$platreq."\n";
            }
        }

        $msg=Msg::new("Required OS/Kernel Versions:\n$platreqs_str");
        $msg->print;
    }

    $prod->cli_print_os_dependency();

    if ($prod->{minimal_memory_requirment}) {
        $msg=Msg::new("\nMinimal Memory Required: $prod->{minimal_memory_requirment}");
        $msg->print;
    }

    # Determine the space requirement for the product
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$prod->pkg($pkgi);
        $pkg->{space} = $pkg->space();
        $pkg->{space}||=[100,100,100,100];
        @space=@{$pkg->{space}};
        $pkgspace=$space[0]+$space[1]+$space[2]+$space[3];
        $prodspace+=$pkgspace;
    }
    $ps=int($prodspace/1024);
    $msg=Msg::new("\nSpace Requirements:\n$prod->{abbr} Package Size - $ps MB");
    $msg->print;

    $otherreqs=$prod->other_requirements||'';
    unless ($otherreqs){
        $otherreqs = Msg::new("None")->{msg};
    }

    $msg=Msg::new("\nOther Requirements:\n$otherreqs");
    $msg->print;

    return '';
}

sub cpu_number_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_cpu_number,$cpu_number);

    $cpu_number=$sys->padv->cpu_number_sys($sys);
    $sys->set_value('cpu_number', $cpu_number);

    $minimal_cpu_number=$prod->{minimal_cpu_number_requirment};
    return 1 unless ($minimal_cpu_number && $cpu_number);

    if ($cpu_number < $minimal_cpu_number) {
        if ($cpu_number < 2) {
            $msg=Msg::new("You have only 1 CPU on $sys->{sys}. It is required that you run $prod->{abbr} on servers with more than 1 CPU.");
        } else {
            $msg=Msg::new("You have only $cpu_number CPUs on $sys->{sys}. It is required that you run $prod->{abbr} on servers with minimal $minimal_cpu_number CPUs.");
        }
        $sys->push_warning($msg);
    }

    return 1;
}

sub cpu_speed_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_cpu_speed,$cpu_speed);

    $cpu_speed=$sys->padv->cpu_speed_sys($sys);
    $sys->set_value('cpu_speed', $cpu_speed);

    $minimal_cpu_speed=$prod->{minimal_cpu_speed_requirment};
    return 1 unless ($minimal_cpu_speed && $cpu_speed);

    # Convert the size based on MHz.
    if ($cpu_speed=~/\s*(\d+)\s*[kK]/mx) {
        $cpu_speed=int($1/1024);
    } elsif ($cpu_speed=~/\s*(\d+)\s*[mM]/mx) {
        $cpu_speed=$1;
    } elsif ($cpu_speed=~/\s*(\d+)\s*[gG]/mx) {
        $cpu_speed=$1*1024;
    }

    # Convert the size based on MHz.
    if ($minimal_cpu_speed=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_cpu_speed=int($1/1024);
    } elsif ($minimal_cpu_speed=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_cpu_speed=$1;
    } elsif ($minimal_cpu_speed=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_cpu_speed=$1*1024;
    }

    # In case the product require 2048MHZ CPU, but actually the machine has 2000M.
    if ($cpu_speed < int($minimal_cpu_speed * 0.95)) {
        $msg=Msg::new("The CPU speed on $sys->{sys} is $cpu_speed MHz, the minimal CPU speed required by product $prod->{abbr} is $minimal_cpu_speed MHz");
        $sys->push_warning($msg);
    }

    return 1;
}

sub memory_size_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_memory_size,$memory_size);

    $memory_size=$sys->padv->memory_size_sys($sys);
    return 1 unless ($memory_size);

    # Convert the size based on MegaByte.
    if ($memory_size=~/\s*(\d+)\s*[kK]/mx) {
        $memory_size=int($1/1024);
    } elsif ($memory_size=~/\s*(\d+)\s*[mM]/mx) {
        $memory_size=$1;
    } elsif ($memory_size=~/\s*(\d+)\s*[gG]/mx) {
        $memory_size=$1*1024;
    }
    $sys->set_value('memory_size', $memory_size);

    $minimal_memory_size=$prod->{minimal_memory_requirment}||'512M';
    # Convert the size based on MegaByte.
    if ($minimal_memory_size=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_memory_size=int($1/1024);
    } elsif ($minimal_memory_size=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_memory_size=$1;
    } elsif ($minimal_memory_size=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_memory_size=$1*1024;
    }

    # In case the product require 2048M memory, but actually the machine has 2046M.
    if ($memory_size < int($minimal_memory_size * 0.95)) {
        $msg=Msg::new("The RAM size on $sys->{sys} is $memory_size megabytes, the recommended RAM size required by product $prod->{abbr} is $minimal_memory_size megabytes");
        $sys->push_warning($msg);
    }

    return 1;
}

sub swap_size_sys {
    my ($prod,$sys) = @_;
    my ($msg,$minimal_swap_size,$swap_size);

    $swap_size=$sys->padv->swap_size_sys($sys);
    return 1 unless ($swap_size);

    if ($swap_size=~/\s*(\d+)\s*[kK]/mx) {
        $swap_size=int($1/1024);
    } elsif ($swap_size=~/\s*(\d+)\s*[mM]/mx) {
        $swap_size=$1;
    } elsif ($swap_size=~/\s*(\d+)\s*[gG]/mx) {
        $swap_size=$1*1024;
    }
    $sys->set_value('swap_size', $swap_size);

    $minimal_swap_size=$prod->{minimal_swap_requirment}||'200M';
    # Convert the size based on MegaByte.
    if ($minimal_swap_size=~/\s*(\d+)\s*[kK]/mx) {
        $minimal_swap_size=int($1/1024);
    } elsif ($minimal_swap_size=~/\s*(\d+)\s*[mM]/mx) {
        $minimal_swap_size=$1;
    } elsif ($minimal_swap_size=~/\s*(\d+)\s*[gG]/mx) {
        $minimal_swap_size=$1*1024;
    }

    # In case the product require 2048M swap, but actually the machine has 2046M.
    if ($swap_size < int($minimal_swap_size * 0.95)) {
        $msg=Msg::new("The size of free swap space on $sys->{sys} is $swap_size megabytes, the minimal free swap size required by product $prod->{abbr} is $minimal_swap_size megabytes");
        $sys->push_warning($msg);
        return 0;
    }

    return 1;
}



sub basic_precheck_sys {
    my ($prod,$sys)=@_;

    if (Cfg::opt(qw(install upgrade patchupgrade hotfixupgrade precheck))) {
        $prod->cpu_number_sys($sys);
        $prod->cpu_speed_sys($sys);
        $prod->memory_size_sys($sys);
        $prod->swap_size_sys($sys);
        $prod->system_info_sys($sys);
    }
    if (Cfg::opt(qw(install upgrade patchupgrade hotfixupgrade configure uninstall precheck))) {
        $prod->rel->verify_commands_sys($sys);
        $sys->padv->installcommand_precheck_sys($sys);
        $prod->check_dir_sys($sys);
    }

    return 1;
}

# get the system info with precheck
sub system_info_sys {
    my ($prod, $sys)=@_;
    my ($msg);

    return 1 unless (Cfg::opt('precheck'));

    if ($sys->{osversion}) {
        $msg = Msg::new("The system information on $sys->{hostname}: \n\tOperating system: $sys->{osversion} \n\tCPU number: $sys->{cpu_number} \n\tCPU speed: $sys->{cpu_speed} \n\tMemory size: $sys->{memory_size} MB \n\tSwap size: $sys->{swap_size} MB");
    } else {
        $msg = Msg::new("The system information on $sys->{hostname}: \n\tCPU number: $sys->{cpu_number} \n\tCPU speed: $sys->{cpu_speed} \n\tMemory size: $sys->{memory_size} MB \n\tSwap size: $sys->{swap_size} MB");
    }

    $sys->push_note($msg);
    return 1;
}

sub additional_dir_space_sys {
    my ($prod,$sys)=@_;
    return $prod->perform_task_sys($sys, 'additional_dir_space_sys');
}

sub check_dir_sys {
    my ($prod,$sys)=@_;
    my $msg;
    # check /tmp directory
    if (!$sys->is_dir('/tmp')) {
        $msg=Msg::new("There is no /tmp directory on $sys->{sys}");
        $sys->push_warning($msg);
    }

    return $prod->additional_dir_space_sys($sys)
}

sub check_installed_patches_sys {
    my ($prod,$sys,$pkg_iv,$dim) = @_;
    my ($rel,$vers,$padv,$patch,$cpic,$cv,$pkg,$hasPatch);
    $cpic = Obj::cpic();
    return '' if (EDRu::compvers($pkg_iv,$prod->{vers},2));
    $rel=$prod->rel;
    $padv=$sys->padv;
    return '' if (!$rel->{patches});
    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        next unless ($pkg->version_sys($sys));
        for my $patchi (@{$pkg->patches_sys($sys, $dim)})  {
            $hasPatch = 1;
            $patch=$sys->patch($patchi) unless (ref($patchi));
            if ($patch->{vers}) {
                $cv=EDRu::compvers($patch->{vers},$padv->patch_version_sys($sys, $patch),4);
                return '' if ($cv==1);
            } else {
                return '' unless ($prod->{padv} =~ /Sol/m);
                return '' unless ($patch->installed_sys($sys));
            }
        }
    }
    if($hasPatch){
        return 1;
    } else {
        return '';
    }
}

# requires some padv consistency for threaded any to many
# startprocs() and stopprocs() removed only add it if desperately needed for a2a

# again defined if system based control is needed, or cleaner -serial display
sub startprocs {
    my $prod=shift;
    my (@procs,$pkg,$rel);
    $rel = $prod->rel();
    # what about the hypothetical any to many that differ?
    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        if ($pkg->{startprocs}) {
            next if (($rel->{type} eq "H") && ($rel->{customized_stoplist}) &&
                     (!EDRu::inarr($pkgi,@{$rel->{hotfix_stopprocs}})));
            push(@procs, @{$pkg->{startprocs}}) if ($pkg->{startprocs});
        }
    }
    return EDRu::arruniq(@procs);
}

# again defined if system based control is needed, or cleaner -serial display
sub startprocs_sys {
    my ($prod,$sys) = @_;
    my ($pkg,@procs);
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);
        push(@procs, @{$pkg->{startprocs}})
            if (($pkg->{startprocs}) && ($pkg->version_sys($sys)));
    }
    return EDRu::arruniq(@procs);
}

sub stopprocs {
    my $prod=shift;
    my ($cpic,@procs,$pkg,$rel);
    # what about the hypothetical any to many that differ?
    $cpic=Obj::cpic() if Obj::created('CPIC');
    $rel = $prod->rel();
    @procs=();

    # if fresh install + configure, then no need to shutdown.
    # for upsell install, then definitely need to shutdown all processes
    if($cpic && Cfg::opt('configure') && Cfg::opt('install')) {
        if ($prod->check_if_upsell()){
            for my $pkgi (reverse(@{$prod->allpkgs})) {
                $pkg=$prod->pkg($pkgi);
                push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
            }
        }
        return EDRu::arruniq(@procs);
    }

    if($cpic && Cfg::opt(qw(patchupgrade hotfixupgrade upgrade install uninstall))) {
        for my $pkgi (@{$cpic->{uninstallpkgs_seq}}) {
            $pkg=Pkg->new_pkg($pkgi, $prod->{padv});
            if ($pkg->{stopprocs}) {
                next if (($rel->{type} eq "H") && ($rel->{customized_stoplist}) &&
                    (!EDRu::inarr($pkgi,@{$rel->{hotfix_stopprocs}})));
                push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
            }
        }
        for my $proci (@procs) {
            CPIC::init_cpip_object("Proc::$proci",$prod->{padv})
                unless (Obj::created('Proc', $proci, $prod->{padv}));
        }

        return EDRu::arruniq(@procs);
    }
    for my $pkgi (reverse(@{$prod->allpkgs})) {
        $pkg=$prod->pkg($pkgi);
        push(@procs, @{$pkg->{stopprocs}}) if ($pkg->{stopprocs});
    }
    return EDRu::arruniq(@procs);
}

sub stopprocs_sys {
    my ($prod,$sys) = @_;
    my ($cpic,@procs,$pkg);
    $cpic=Obj::cpic() if Obj::created('CPIC');
    @procs=();
    if($cpic && Cfg::opt('configure') && Cfg::opt('install')) {
        if ($prod->check_if_upsell($sys)){
            for my $pkgi (reverse(@{$prod->get_pkgs})) {
                $pkg=$sys->pkg($pkgi);
                push(@procs, @{$pkg->{stopprocs}})
                    if ($pkg->{stopprocs} && ($pkg->version_sys($sys)));
            }
        }
        return EDRu::arruniq(@procs);
    }

    if($cpic && Cfg::opt(qw(patchupgrade hotfixupgrade upgrade install uninstall))) {
        for my $pkgi (@{$cpic->{uninstallpkgs_seq}}) {
            $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
            # stop procs for those pkgs exist on the system.
            if ($pkg->{stopprocs} && ($pkg->version_sys($sys))) {
                if (Cfg::opt("install")) {
                    # stop procs of the pkgs which will be uninstalled (cross stack upgrade)
                    push(@procs, @{$pkg->{stopprocs}})
                        if (EDRu::inarr($pkgi,@{$sys->{uninstallpkgs}}));
                } else {
                    push(@procs, @{$pkg->{stopprocs}});
                }
            }
        }
        for my $proci (@procs) {
            CPIC::init_cpip_object("Proc::$proci", $sys->{padv})
                unless (Obj::created('Proc', $proci, $sys->{padv}));
        }

        return EDRu::arruniq(@procs);
    }
    for my $pkgi (reverse(@{$prod->get_pkgs})) {
        $pkg=$sys->pkg($pkgi);
        push(@procs, @{$pkg->{stopprocs}})
            if ($pkg->{stopprocs} && ($pkg->version_sys($sys)));
    }
    return EDRu::arruniq(@procs);
}

sub check_if_upsell {
    my ($prod,$sys) = @_;
    my $systems;

    return 0 unless Cfg::opt(qw(install));
    if ($sys) {
        $systems = [$sys];
    } else {
        $systems = CPIC::get('systems');
    }
    for my $sys (@$systems) {
        return 1 if defined(${$sys->{upgradeprod}}[0]);
    }
    return 0;
}

sub remove_procs_for_prod {
    my ($prod,$procs) = @_;
    for my $pkgi (reverse(@{$prod->get_pkgs})) {
        my $pkg=$prod->pkg($pkgi);
        $procs=EDRu::arrdel($procs, @{$pkg->{stopprocs}});
    }
    return $procs;
}

sub zru_version_sys {
    my ($prod,$sys) = @_;
    my ($vers,$cv);
    return 1 if(Cfg::opt("upgrade_nonkernelpkgs"));
    $vers=$prod->version_sys($sys);
    for my $zru_release (@{$prod->{zru_releases}}) {
        $cv=EDRu::compvers($zru_release,$vers,2);
        next if ($cv != 0);
        $cv=EDRu::compvers($zru_release,$vers);
        return 1 if ($cv != 1);
    }
    return 0;
}

sub upgradeable_version_sys {
    my ($prod,$sys) = @_;
    my ($vers,$cv);
    $vers=$prod->version_sys($sys);
    for my $ugvers (@{$prod->{upgradevers}}) {
        $cv=EDRu::compvers($ugvers,$vers,2);
        next if ($cv != 0);
        $cv=EDRu::compvers($ugvers,$vers);
        return 1 if ($cv != 1);
    }
    return 0;
}

sub pkgsetlist {
    my $cpic = Obj::cpic();
    my %pkgs = ();
    for my $padvi (@{$cpic->{padvs}}) {
        my $rel=$cpic->rel($padvi);
        my $prod=$cpic->prod($cpic->{prod},$padvi);
        my @pkgis = (@{$rel->allpkgs},@{$prod->allpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{allpkgs}})) {
                push @{$pkgs{$padvi}{allpkgs}},$pkg;
                $pkgs{$padvi}{allspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
        @pkgis = (@{$rel->recpkgs},@{$prod->recpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{recpkgs}})) {
                push @{$pkgs{$padvi}{recpkgs}},$pkg;
                $pkgs{$padvi}{recspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
        @pkgis = (@{$rel->minpkgs},@{$prod->minpkgs});
        for my $pkgi (@pkgis) {
            my $pkg=$cpic->pkg($pkgi,$padvi);
            unless (EDRu::inarr($pkg,@{$pkgs{$padvi}{minpkgs}})) {
                push @{$pkgs{$padvi}{minpkgs}},$pkg;
                $pkgs{$padvi}{minspace} +=$pkg->{space}[0]+$pkg->{space}[1]+$pkg->{space}[2]+$pkg->{space}[3];
            }
        }
    }
    return \%pkgs;
}

sub get_tunables{
    my ($prod) = @_;
    return $prod->{tunables} || [];
}

sub get_supported_tunables {
    my ($prod) = @_;
    return $prod->get_tunables;
}

sub validate_tunable_sys{
    my ($prod,$sys,$tunable,$value) =@_;
    my ($cpic, $prodname, $sysi, $tunable_info, @values,$msg, $rel, $basevers);
    # 0 valid
    # 1 Padv is not supported
    # 2 Prod(Ver) is not installed
    # 3 value is invalid

    $cpic = Obj::cpic();
    $rel = $prod->rel;
    $prodname = $prod->{name};
    $sysi = $sys->{sys};

    $basevers = $1 if (@{$rel->{basevers}}[-1]=~ /^(\d+\.\d+)/);
    if ($prod->version_sys($sys) !~ /^$basevers/) {
        $msg = Msg::new("$tunable is a tunable parameter of $prodname $rel->{titlevers}, which is neither installed nor being installed on $sysi");
        return (2, $msg);
    }

    $tunable_info = $cpic->get_tunable_info($tunable);
    @values = @{$tunable_info->{values}};
    if ( $tunable_info->{type} eq 'enum') {
        if (!EDRu::inarr($value, @values)) {
            $msg = Msg::new("Invalid value $value of $tunable specified for $sysi or all systems");
            return (3, $msg);
        }
    } elsif ($tunable_info->{type} eq 'range') {
        if ($value !~ /^-?(0x|0X)?\d+$/) {
            $msg = Msg::new("$tunable requires a numeric value but $value is specified for $sysi or all systems");
            return (3, $msg);
        }
        if (($values[0] ne undef && $value < $values[0]) || ($values[1] ne undef && $value > $values[1])) {
            if (!($value == 0 && $tunable_info->{allow_zero})) {
                if ($values[0] ne undef && $values[1] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value between $values[0] and $values[1] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                } elsif ($values[0] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value equal to or greater than $values[0] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                } else {
                    $msg = Msg::new("$tunable requires a numeric value equal to or less than $values[1] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                }
            }
        }

        if ($values[2] ne undef && $value % $values[2]) {
            $msg = Msg::new("Value of $tunable must be a multiple of $values[2] but $value is specified for $sysi or all systems");
            return (3, $msg);
        }
    }

    return 0;
}

sub set_tunable_sys{
    my ($prod,$sys,$tunable,$value) =@_;
    my ($origval,$ret,$out);

    $origval = $prod->get_tunable_value_sys($sys,$tunable);

    if ($origval eq $value) {
        Msg::log("$tunable has already had value $value on $sys->{sys}");
        return 2; # 2 - Not changed
    }

    ($ret, $out) = $prod->set_tunable_value_sys($sys,$tunable,$value);
    if ($ret) {
        return (0, $out); # 0 - Failed to set
    }
    return 1; # 1 - Done
}

sub adjust_pkglist{}

sub serial_not_supported{}

sub check_feature_required_os_pkgs_sys {
    my ($prod,$sys,$feature) =@_;
    my (@missing_ospkgs,@wrong_vers_ospkgs,$padv,$pdfrs,$pkg,$pkgs,$minvers,%checked_pkgs,$vers,$msg);

    @missing_ospkgs=();
    @wrong_vers_ospkgs=();
    $padv=$sys->padv();
    $pdfrs=Msg::get('pdfrs');
    $pkgs=$prod->{feature_required_os_pkgs}{$feature};
    return 1 unless ($pkgs);

    for my $item (@{$pkgs}) {
        next if (!$item);
        ($pkg,$minvers)=split(/\s+/,$item);
        next if (defined $checked_pkgs{$pkg});
        $checked_pkgs{$pkg}=1;

        $vers=$padv->pkg_version_sys($sys,$pkg,1);
        if ($vers) {
            if ($minvers && EDRu::compvers($vers,$minvers)==2) {
                push @wrong_vers_ospkgs,$pkg;
            }
        } else {
           push @missing_ospkgs,$pkg;
        }
    }

    if (@missing_ospkgs) {
        $sys->set_value("missing_ospkgs_for_feature,$feature",'list',@missing_ospkgs);
    }

    if (@wrong_vers_ospkgs) {
        $sys->set_value("wrong_vers_ospkgs_for_feature,$feature",'list',@wrong_vers_ospkgs);
    }

    return 0 if (@missing_ospkgs || @wrong_vers_ospkgs);
    return 1;
}

sub get_performed_component {
    my ($prod, $task) = @_;
    my ($rel);

    $rel = $prod->rel();

    return $prod->{allcomponent} if (EDRu::inarr($task, @{$rel->{allcomps_tasks}}));
    return $prod->{previouscomponent} if (EDRu::inarr($task, @{$rel->{precomps_tasks}}));
    return $prod->{activecomponent};
}

sub perform_component_task_sys {
    my ($prod,$sys,$task) = @_;
    my ($subprod,$components,$rtn,$rel,$vers);
    $rtn=1;
    #For upgrade, existing components should execute upgrade task, new components should execute fresh install task.
    if (Cfg::opt('upgrade')) {
        $rel = $prod->rel();
        $vers = $sys->{prodvers}[0];
        if (2 == EDRu::compvers($vers,'7.0') && EDRu::inarr($task, @{$rel->{upgrade_or_install_tasks}})) {
            return $rel->perform_upgrade_or_install_task_sys($sys,$prod,$task);
        }
    }
    $components ||= $prod->get_performed_component($task);
    push (@{$components}, $prod->{defaultcomponent}) unless ($components);
    Msg::log("Perform task: $task on component(s) are " . join(',',@{$components}));
    if ($components) {
        for my $component (@{$components}) {
            $subprod=$prod->prod($component);
            $rtn=$subprod->$task($sys) && $rtn if ($subprod->can($task));
        }
    } 
    return $rtn;
}

sub perform_component_task {
    my ($prod,$task) = @_;
    my ($subprod,$rtn,$components);
    $rtn=1;
    $components = $prod->get_performed_component($task);
    push (@{$components}, $prod->{defaultcomponent}) unless ($components);
    if ($components) {
        for my $component (@{$components}) {
            $subprod=$prod->prod($component);
            $rtn =$subprod->$task() && $rtn if ($subprod->can($task));
        }
    }
    return $rtn;
}

1;
