#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
package Rel;
use strict;
use LWP::UserAgent;
use JSON;

@Rel::ISA = qw(CPIP);

sub pre_init {
    my $rel=shift;
    (undef,$rel->{reli},$rel->{padv})=split(/::/m,$rel->{class},3);
    return;
}

sub allpkgs { return $_[0]->{allpkgs} || []; }
sub minpkgs { return $_[0]->{minpkgs} || []; }
sub recpkgs { return $_[0]->{recpkgs} || []; }

# common Rel subs and trace-only stubs for optional Rel subs

sub platvers_sys { return 1 }
sub osvers_sys { }
sub get_platreqs { }
sub post_platvers { return 1 }
sub upgradevers_sys { return 1 }
sub basic_precheck_sys { }
sub feature_usage_sys{ }
sub copy_cpi_sys { }
sub completion {}
sub detect_prior_rp_sys { }
sub determine_superset { }
sub determine_prodmix_upgrade { }
sub determine_cross_partial_upgrade { }
sub determine_upsell_supported {}
sub determine_rolling_upgrade {}
sub cli_prod_option { }
sub web_prod_option { }
sub license_option { }
sub cli_language_menu { }
sub preinstall_sys { }
sub preinstall_patch_sys { }
sub postinstall_sys { }
sub preremove_sys { }
sub postremove_sys { }
sub preset_uninstall_pkgs { }
sub upgrade_prod_mix_check { }
sub usage {}
sub process_args {}
sub config_without_reboot_message {}
sub reboot_messages {}
sub ru_switch_groups {}
sub minpkgs_nag {return 'Y';}
# should be defined in Pkg and Patch objs but easier to manage batched in Rel
sub pkg_space { }
sub pkg_versions { }
sub pkg_pass_error {return 0;}
sub patch_space { }
sub patch_versions { }
sub adjust_pkg_sequence { return $_[1] || []; }
sub adjust_options { }
sub rollback_check_task { return 0;}
sub rollback_fsspace_sys {}
sub adjust_pkgs { }
sub read_licenses_sys { }
sub check_tunable_info_sys { }
sub new_pkg_init_common { }
sub new_pkg_init_plat { }
sub new_pkg_init_padv { }
sub new_pkg_init {
    my ($rel,$pkg)= @_;
    $rel->new_pkg_init_common($pkg);
    $rel->new_pkg_init_plat($pkg);
    $rel->new_pkg_init_padv($pkg);
    return;
}

sub init_args { }
sub init_args2 { }
sub filter_args { }
sub define_patches { }
sub define_hotfixes { }
sub get_configured_component {}

# renaming and redefining to new release terminology
# new sub, not quite sure what to do with $release since $rel->{currentrelease}
# is defined for patch releases only.  copied define_private_hf_patches
# should eventually strip $release from define_mr_patches and $hotfix->define_hf_patches
sub define_base_patches {
    my ($rel,$pkg,$platvers,@patches)=(@_);
    my ($arg,$release);
    $arg='base_path' if (Cfg::opt('base_path'));
    $release=$rel->{currentrelease} || 'base';
    $rel->define_level_patches('basepatches', $arg, $pkg, $platvers,
                               $release, @patches);
    return 1;
}

sub define_mr_patches {
    #my ($rel,$pkgname,$platvers,$release,@patches)=(@_);
    my $rel=shift;
    $rel->define_level_patches('mrpatches', '', @_) if ($rel->{type}=~/M/);
    return 1;
}

# be aware of the inference to PRIVATE hotfixes here
sub define_private_hf_patches {
    my ($rel,$pkg,$platvers,@patches)=(@_);
    $rel->define_level_patches('privatehfpatches', '', $pkg, $platvers,
                               $rel->{currentrelease}, @patches);
    return 1;
}

sub define_level_patches {
    my ($rel,$level,$arg,$pkgi,$platvers,$release,@patches)=(@_);
    my $pkg=Obj::pkg($pkgi,$rel->{padv});
    foreach my $patchvers (@patches){
       my ($patchi,$vers)=split(/_/,$patchvers,2);
       if ($rel->{padv} =~ /HPUX/m) {
           $pkg->{patchvers}{$release}{$patchvers}='1.0';
           $pkg->{patchname}{$release}{$patchvers}=$patchvers;
       } else {
           $pkg->{patchvers}{$release}{$patchvers}=$vers if($vers);
           $pkg->{patchname}{$release}{$patchvers}=$patchi;
       }
       $rel->{patcharg}{$patchvers}=$arg if ($arg);
    }
    $pkg->{$level}{$release}{"$platvers"} = [ @patches ];
    return 1;
}

sub define_os_patches {
    my ($rel,$pkgname,$platvers,@patches)= @_;
    my $pkg=$rel->pkg($pkgname,1);
    $pkg->{ospatches}{"$platvers"} = [ @patches ] if($pkg);
    return 1;
}

# key is pkgpath or patchpath
# defined per release, no longer per product
# mediapath is now a required argument
sub pkgs_patches_dir {
    my ($rel,$key,$mediapath) = @_;
    my ($msg,$padv,$dir,$cpic,$path,$prod);
    # to only have to search once
    return $rel->{"ppd-$key-$mediapath"} if (defined($rel->{"ppd-$key-$mediapath"}));
    return "" if (($key eq "patchpath") && Cfg::opt("missingpkgs") && !$rel->{patches});
    $cpic=Obj::cpic();
    # for now to solve UXRT61.pm calls
    $mediapath||=$cpic->{basemediapath} if ($key eq 'pkgpath');
    $mediapath||=$cpic->{mediapath} if ($key eq 'patchpath');


    # can be defined at release level or fall to padv defaults
    $rel->{$key}||=$cpic->{pkgpath}{$rel->{padv}};
    # a2m pkgpath settings are stored in $cpic->{pkgpath}{$padv} by discover_prod_padv
    $rel->{$key}||=$rel->padv->{$key};
    EDR::die("Cannot find $key for $rel") unless ($rel->{$key});
    $dir=$rel->{$key};

    # probably rewrite this section
    # assumes path is raw /$padv and not /$rel->{key} of padv $rel
    $path=Cfg::opt($key);
    if ($path) {
        if ($#{$cpic->{padvs}}>0) {
            # must munge $path and $relpath for multipadv
            # assumes $relpath is path/padv for multi-padv
            $padv=EDRu::basename($dir);
            if ($^O =~ /Win32/i) {
                $path.="\\$padv";
            } else {
                $path.="/$padv";
            }
        }
        if (-d $path) {
            $rel->{"ppd-$key-$mediapath"}=$path;
            return $path
        }
        if ($#{$cpic->{padvs}}>0) {
            $msg=Msg::new("-$key subdirectory not found for $rel->{padv} at $path");
        } else {
            $msg=Msg::new("-$key directory: $path not found");
        }
    }

    # check in location stored in $dir
    my @mediapathdir;
    if ($^O =~ /Win32/i) {
        @mediapathdir = ("$mediapath\\$dir", "$mediapath\\..\\$dir");
    } else {
        @mediapathdir = ("$mediapath/$dir", "$mediapath/../$dir");
    }

    for my $path (@mediapathdir) {
        if (-d $path) {
            $rel->{"ppd-$key-$mediapath"}=$path;
            return $path;
        }
    }
    # can't hard-error here, with bundling, assume there are no pkgs/patches for that level
    Msg::log("Did not find $key $rel->{$key} under $mediapath");
    return '';
}

sub pkgdesc {
    my ($rel, $pkg) = @_;

    $pkg = $pkg->{pkg} if (ref($pkg) =~ m/^Pkg/);
    return $rel->{package_descriptions}{$pkg};
}

sub multilockfile {
    my $rel=shift;
    my $script=EDR::get('script');
    return if ($script && $script=~/^update/);
    return if (Cfg::opt(qw(precheck)));

    my $tmppath=EDR::get('tmppath') || '/var/tmp';
    return "$tmppath/.cpilock";
}

# Return 1 if no lock on the target system.
# Return 0 if lock exists on the target system.
sub check_multilock_sys {
    my ($rel,$sys)=@_;
    my ($lockfile,$localsys,$content,$driversys,$pid,$anotherinstaller,$rtn,$msg);

    $lockfile=$rel->multilockfile();
    return 1 unless ($lockfile);

    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $content=$sys->catfile($lockfile);
    return 1 unless ($content);

    ($driversys,$pid)=split(/\n/,$content);
    if ($driversys && $pid) {
        return 1 unless ($pid=~/^\d+$/mx);

        # Check if another installer instance running on same machine
        $anotherinstaller=1;
        $localsys=$rel->localsys();
        if ($driversys eq $localsys->{sys}) {
            # if the 2 installer instances run on same driver machine
            # if the pid is same with current installer, or the pid not exist,
            # regard another installer not running.
            if ($pid eq $$) {
                $anotherinstaller=0;
            } else {
                $rtn=$localsys->cmd("_cmd_ps -p $pid 2>/dev/null | _cmd_grep '$pid'");
                $anotherinstaller=0 if (!$rtn);
            }
        }

        if ($anotherinstaller) {
            $msg = Msg::new("Another installer running on $driversys is operating on $sys->{sys}, its process id is $pid. Make sure the installer completes execution before proceeding");
            $sys->push_warning($msg);
            return 0;
        }
    }

    return 1;
}

sub add_multilock_sys {
    my ($rel,$sys)=@_;
    my ($lockfile,$localsys,$content,$timestamp);

    $lockfile=$rel->multilockfile();
    return 1 unless ($lockfile);

    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 if ($sys->{locked});

    $localsys=$rel->localsys();
    $timestamp=$sys->cmd("_cmd_date 2>/dev/null");
    $content="$localsys->{sys}\n$$\n$timestamp\n";
    $sys->writefile($content,$lockfile);
    $sys->set_value('locked', 1);

    return 1;
}

sub remove_multilock_sys {
    my ($rel,$sys)=@_;

    my $lockfile=$rel->multilockfile();
    return 1 unless ($lockfile);

    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    return 1 unless ($sys->{locked});

    $sys->{locked}=0;
    $sys->rm($lockfile);
    return 1;
}

sub add_multilock {
    my $rel=shift;
    my $systems=CPIC::get('systems');

    for my $sys (@{$systems}) {
        $rel->add_multilock_sys($sys);
    }

    # register extra cleanup task when type Ctrl+C
    my $edr=Obj::edr();
    $edr->register_cleanup_task(\&Rel::remove_multilock, $rel);

    return 1;
}

sub remove_multilock {
    my $rel=shift;
    my $systems=CPIC::get('systems');

    for my $sys (@{$systems}) {
        $rel->remove_multilock_sys($sys);
    }
    return 1;
}

sub cli_installer_menu {
    my ($cpic,$prod,$rel);
    $rel = shift;
    $cpic=Obj::cpic();
    set_menu_msgs();
    $rel->cli_task_menu();
    $rel->cli_prod_menu();
    if (Cfg::opt(qw(desc requirements))) {
        $prod=$cpic->prod;
        Msg::n();
        (Cfg::opt(qw(desc))) ? $prod->description :
                               $prod->cli_print_requirements;
        Msg::prtc();
        $cpic->{prod}='';
        Cfg::unset_opt('desc');
        Cfg::unset_opt('requirements');
        $rel->cli_installer_menu();
    } elsif (Cfg::opt('requirements')) {
        Cfg::unset_opt('requirements');
        $rel->cli_installer_menu();
    }
    return;
}

sub cli_task_menu {
    my(@mk,@mo,@tasks,$buf,$cpic,$edr,$m,$mkey,$mm1,$mm2,$msg,$rel,$task,$tsl);
    $rel = shift;
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    if($cpic->{task}){
        $cpic->set_task($cpic->{task});
    }
    return if Cfg::opt(@{$rel->{args_task}});

    $msg=Msg::new("Task Menu:\n");
    $msg->bold;

    @tasks=qw(precheck install configure upgrade license uninstall desc start requirements stop quit help);
    @mo=@tasks;
    while (@mo) {
        $m=shift(@mo);
        next if(!EDRu::inarr($m,@{$rel->{args_task}},'quit','help'));
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-50s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl");
        $task=$msg->ask;
        $task=uc($task);
    }

    if ($task eq Msg::get('key_help')) {
        $rel->task_menu_help();
        $rel->cli_task_menu();
    } else {
        for my $m ('requirements', 'desc') {
            Cfg::set_opt($m) if ($task eq Msg::get("key_$m"));
        }
        for my $m (@{$rel->{args_task}}) {
            if ($task eq Msg::get("key_$m")) {
                $cpic->set_task($m);
                $cpic->{task}=$m;
            }
        }
    }
    return '';
}

sub cli_prod_menu {
    my(@prods,$cpic,$menu,$mprodi,$msg,$prod,$rel);
    $rel = shift;
    $cpic=Obj::cpic();
    return if ($cpic->{prod});

    if($#{$rel->{prods}}==0){
        $cpic->set_prod($rel->{prods}->[0]);
        return;
    }

    for my $mprodi (@{$rel->{prods}}) {
        $prod=$cpic->prod($mprodi);
        push(@prods,$prod->{name});
    }

    if (Cfg::opt('install')) {
        $msg=Msg::new("Select a product to install:");
    } elsif (Cfg::opt('uninstall')) {
        $msg=Msg::new("Select a product to uninstall:");
    } elsif (Cfg::opt('configure')) {
        $msg=Msg::new("Select a product to configure:");
    } elsif (Cfg::opt('start')) {
        $msg=Msg::new("Select a product to start:");
    } elsif (Cfg::opt('stop')) {
        $msg=Msg::new("Select a product to stop:");
    } elsif (Cfg::opt('upgrade')) {
        $msg=Msg::new("Select a product to upgrade:");
    } elsif (Cfg::opt('license')) {
        $msg=Msg::new("Select a product to license:");
    } elsif (Cfg::opt('precheck')) {
        $msg=Msg::new("Select a product to perform a pre-installation check for:");
    } elsif (Cfg::opt('desc')) {
        $msg=Msg::new("Select a product to view its product description:");
    } elsif (Cfg::opt('requirements')) {
        $msg=Msg::new("Select a product to view its product requirements:");
    } else {
        $msg=Msg::new("Select a product:");
    }

    Msg::n();
    $menu=$msg->menu(\@prods);
    $cpic->set_prod(${$rel->{prods}}[$menu-1]);
    return;
}

sub task_menu_help {
    menu_help(qw(install uninstall upgrade configure));
    menu_help(qw(start stop precheck license desc requirements));
    return '';
}

sub menu_help {
    my (@mo) = @_;
    my ($edr,$m,$mkey,$mm);
    Msg::title();

    $edr=Obj::edr();

    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        $mm="$edr->{tput}{bs}$mkey$edr->{tput}{be}) ".Msg::get("menu_$m");
        print "$mm\n\n";
        $mm=Msg::linebreak(Msg::get("help_$m"));
        print "$mm\n";
        print "\n" if (@mo);
    }
    Msg::prtc();
    return;
}

sub set_menu_msgs {
    my ($edr,$msg,$pdfrs);

    # menu license screen msgs
    $msg=Msg::new("Product");
    $msg->msg('menu_vendor_prod');
    $msg=Msg::new("Version Installed");
    $msg->msg('menu_version_installed');
    $msg=Msg::new("Licensed");
    $msg->msg('menu_version_licensed');

    # menu task selection
    # e1534297:  should not translate shortcut key.
    $edr=Obj::edr();
    #$msg=Msg::new("I");
    #$msg->msg("key_install");
    $edr->{msg}{key_install}='I';
    #$msg=Msg::new("G");
    #$msg->msg("key_upgrade");
    $edr->{msg}{key_upgrade}='G';
    #$msg=Msg::new("C");
    #$msg->msg("key_configure");
    $edr->{msg}{key_configure}='C';
    #$msg=Msg::new("L");
    #$msg->msg("key_license");
    $edr->{msg}{key_license}='L';
    #$msg=Msg::new("P");
    #$msg->msg("key_precheck");
    $edr->{msg}{key_precheck}='P';
    #$msg=Msg::new("U");
    #$msg->msg("key_uninstall");
    $edr->{msg}{key_uninstall}='U';
    #$msg=Msg::new("S");
    #$msg->msg("key_start");
    $edr->{msg}{key_start}='S';
    #$msg=Msg::new("X");
    #$msg->msg("key_stop");
    $edr->{msg}{key_stop}='X';
    #$msg=Msg::new("D");
    #$msg->msg("key_desc");
    $edr->{msg}{key_desc}='D';
    #$msg=Msg::new("?");
    #$msg->msg("key_help");
    $edr->{msg}{key_help}='?';

    $edr->{msg}{key_requirements}='R';
    $edr->{msg}{key_quit}='Q';

    # Installation Menu
    $msg=Msg::new("Install a Product");
    $msg->msg('menu_install');
    $msg=Msg::new("Upgrade a Product");
    $msg->msg('menu_upgrade');
    $msg=Msg::new("Configure a Product Component");
    $msg->msg('menu_configure');
    $msg=Msg::new("License a Product");
    $msg->msg('menu_license');
    $msg=Msg::new("Perform a Pre-Installation Check");
    $msg->msg('menu_precheck');
    $msg=Msg::new("Uninstall a Product");
    $msg->msg('menu_uninstall');
    $msg=Msg::new("Start a Product");
    $msg->msg('menu_start');
    $msg=Msg::new("Stop a Product");
    $msg->msg('menu_stop');
    $msg=Msg::new("View a Product Description");
    $msg->msg('menu_desc');
    $msg=Msg::new("View Product Requirements");
    $msg->msg('menu_requirements');
    $msg=Msg::new("Help");
    $msg->msg('menu_help');
    $msg=Msg::new("Quit");
    $msg->msg('menu_quit');

    # menu license msgs
    $msg=Msg::new("none");
    $msg->msg('installed_none');
    $msg=Msg::new("yes");
    $msg->msg('license_yes');
    $msg=Msg::new("no");
    $msg->msg('license_no');

    # menu Help msg
    $pdfrs=Msg::get('pdfrs');
    $msg=Msg::new("Installs $pdfrs and patches for an initial installation.");
    $msg->msg('help_install');
    $msg=Msg::new("Starts processes associated with product");
    $msg->msg('help_start');
    $msg=Msg::new("Stops processes associated with product");
    $msg->msg('help_stop');
    $msg=Msg::new("Upgrades all $pdfrs and patches installed on the systems .");
    $msg->msg('help_upgrade');
    $msg=Msg::new("Configures the product after installing a product without configuration.");
    $msg->msg('help_configure');
    $msg=Msg::new("Installs or updates a product license key. This option can be used to update an evaluation key to a permanent key, or to license a feature in a previously installed product.");
    $msg->msg('help_license');
    $msg=Msg::new("Performs a preinstallation check to determine if systems meet all installation requirements. No products are installed.");
    $msg->msg('help_precheck');
    $msg=Msg::new("Removes the selected product's $pdfrs, patches, files, and directories from the system if they are not used by another product.");
    $msg->msg('help_uninstall');
    $msg=Msg::new("Displays a brief overview of the selected product.");
    $msg->msg('help_desc');
    $msg=Msg::new("Displays a brief requirements of the selected product.");
    $msg->msg('help_requirements');

    return '';
}

sub define_inst_lic_prods {
    my ($cpic,$rel,$cfg);
    $rel=Obj::rel();
    $cfg=Obj::cfg();
    my (@ilprods,$localsys,$prod,$prodi,$prods);
    $cpic = Obj::cpic();
    $prods = $rel->{prods};

    @ilprods=();
    for my $prodi (@$prods) {
        $prod=$rel->prod($prodi);
        # do not need to check mediapath for update. Etrack 3084711
        push(@ilprods,$prodi) if ($prod->{obsoleted} || EDRu::inarr($prodi, @{$rel->{menuprods}}));
    }

    $localsys=Obj::localsys();
    # skipping installed products table if systems are declared because users 
    # were confused that the table should be relative to the systems, not localsys 
    $rel->inst_lic_sys($localsys, @ilprods) if ((!defined($cfg->{systems})) || ($#{$cfg->{systems}}<0));
    return;
}

sub prodlist {
    my $cpic=Obj::cpic();
    my @prods=();
    for my $padvi(@{$cpic->{padvs}}){
        my $rel=$cpic->rel($padvi);
        for my $prodi(@{$rel->{prods}}){
            if(EDRu::inarr($prodi,@{$rel->{menuprods}})){
                my $prod=$cpic->prod($prodi,$padvi);
                push(@prods,$prod);
            }
        }
    }
    return \@prods;
}

# To verify whether the commands exist on the system or not.
sub verify_commands_sys {
    my ($rel,$sys) = @_;
    my ($msg,$cmds,@keys,$key,$padv,$basekey,@cmds,$rtn,@cmds_bin,$cmd_bin,$padv_vers);

    $padv_vers=$sys->{local_platvers}||$sys->{platvers};
    $basekey='commands_need_be_checked';
    push @keys, $basekey;
    $key=$basekey . '_for_' . $padv_vers;
    push @keys, $key;
    for my $opt (@{$rel->{args_task}}) {
        if (Cfg::opt($opt)) {
            $key=$basekey . '_for_' . $opt;
            push @keys, $key;
            $key=$basekey . '_for_' . $opt . '_' . $padv_vers;
            push @keys, $key;
        }
    }

    @cmds=();
    for my $key (@keys) {
        push (@cmds, @{$rel->{$key}}) if (defined $rel->{$key});
    }
    return 1 unless (@cmds);

    $padv=$sys->padv;
    for my $cmd (@cmds) {
        $cmd_bin=$padv->{cmd}{$cmd};
        next unless ($cmd_bin);

        $cmd_bin=~s/\s+.*$//m;
        push (@cmds_bin, $cmd_bin);
    }

    if (@cmds_bin) {
        $cmds=join(' ', @cmds_bin);
        $rtn=$sys->cmd("_cmd_ls -L $cmds 2>&1 1>/dev/null");
        if (EDR::cmdexit()) {
            $msg=Msg::new("The following required OS commands are missing on $sys->{sys}:\n$rtn");
            $sys->push_error($msg);
        }
    }
    return '';
}

# return url config from VOS product server
sub get_upload_url {
    my ($rel) = @_;
    my ($upload_conf_file,@url_keys,$key,$url,$rtn,$conf,$line,$msg,$download_errors);

    return if(!$rel->{sort_upload_config_url});

    $upload_conf_file=EDR::tmpdir().'/upload_url.conf';
    $rtn=EDRu::download_file($rel->{sort_upload_config_url}, $upload_conf_file);
    # return "" if failed to get the URL config file.
    if (!$rtn || ! -f "$upload_conf_file") {
        $download_errors=EDR::get('download_errors');
        $msg=Msg::new("Could not download the upload configuration file '$rel->{sort_upload_config_url}'!\n\n$download_errors\n");
        Cfg::opt('testuploadurl') ? $msg->print : $msg->log;
        return;
    }

    $url='';
    @url_keys=();

    # check release specific upload url key, for example:  uxrt60_upload_url=...
    $key=$rel->{release_upload_url_key};
    push @url_keys, $key if (defined $key);

    # check base upload url key, for example:  uxrt_upload_url=...
    $key=$rel->{base_upload_url_key};
    push @url_keys, $key if (defined $key);

    # check the very basic upload url key, for example:  upload_url=...
    push @url_keys, 'upload_url';

    $conf=EDRu::readfile($upload_conf_file);
    Msg::log("The upload configure file's contents:\n$conf");
    for my $line (split(/\n/,$conf)) {
        for my $key (@url_keys) {
            if ($line=~/^\s*$key\s*=\s*(.*)\s*$/mx) {
                $url=$1;
                last;
            }
        }
        last if ($url);
    }

    unless ($url) {
        $msg=Msg::new("Invalid upload configuration url");
        $msg->print;
    }

    # return $url if the URL get correctly defined.
    if ($url && $url=~/^http/m) {
        $msg=Msg::new("The URL to upload log files: $url");
        Cfg::opt('testuploadurl') ? $msg->print : $msg->log;
        return $url;
    }
    return;
}

sub cli_ask_feedback {
    my ($rel) = @_;
    my ($cprod,$url,$cfg,$msg,$help,$ayn,$edr,$tmpdir);

    $cprod=CPIC::get('prod');
    $edr=Obj::edr();
    Cfg::unset_opt('silent') if ($cprod =~ /SVS/);
    return if(!$rel->need_ask_feedback());

    $tmpdir=$edr->{tmpdir};
    $msg=Msg::new("Installation procedures and diagnostic information are saved in the log files under directory $tmpdir. This information helps us identify and resolve failed operations performed by the installer. Would you like to send the information to us to help improve installation in the future?");
    $help=Msg::new("The installation log files will be uploaded to us for future analysis. The warnings, errors and installation operations are collected in the log.");
    $ayn=$msg->ayny($help);

    Msg::n();
    return if ($ayn eq 'N');

    $cfg=Obj::cfg();
    $cfg->{uploadlogs}=1;
    return 1;
}

sub upload_logs {
    my ($rel) = @_;
    my ($edr,$cfg,$url,$msg,$rtn,$files,$file,$scriptid,$error,$gzip,$tmppath,$upload_logfiles,$upload_errors);

    $cfg=Obj::cfg();
    return unless ($cfg->{uploadlogs});
    #return "" if (Cfg::opt("makeresponsefile"));
    $url=$rel->{upload_url};
    return unless ($url);

    $scriptid=EDR::get('scriptid');
    $tmppath=EDR::get('tmppath');
    $upload_logfiles=EDR::get('save_logfiles');
    $edr = Obj::edr();
    $edr->dumpmemory();
    $files='';
    for my $file (@{$upload_logfiles}) {
        if ($file=~m{^\/}) {
            $files.="$file ";
        } else {
            $files.="$scriptid/$file ";
        }
    }
    EDR::cmd_local("cd $tmppath; _cmd_tar -cf $scriptid.tar $files 2>/dev/null");

    $file="$tmppath/$scriptid.tar";
    # Compress the file
    $gzip='/usr/bin/gzip';
    if (! -f "$gzip") {
        $gzip=EDR::cmd_local('_cmd_which gzip');
        if (($gzip=~/no gzip/m) || (! -x $gzip)) {
            $gzip='';
        }
    }
    if ($gzip) {
        EDR::cmd_local("$gzip $file");
        $file="$tmppath/$scriptid.tar.gz";
    }

    Msg::log("Uploading $file to our website: $url");
    $rtn=EDRu::upload_file($url, $file, 'application/x-gzip');
    if ($rtn==1) {
        $msg=Msg::new("Upload completed successfully");
        $msg->bold;
    } else {
        $msg=Msg::new("Upload did not complete successfully due to internal issues");
        $msg->bold;

        $upload_errors=EDR::get('upload_errors');
        $error=EDRu::extract_upload_errors($upload_errors);
        Msg::log($error);
    }

    Msg::n();
    if (! Cfg::opt(qw(nocleanup debug))) {
        unlink("$file");
        unlink($edr->{memorydumpfile}) if(-f $edr->{memorydumpfile});
    }
    return;
}

# check if the feedback question needs to be asked
sub need_ask_feedback {
    my ($rel) = @_;
    my $url;

    return '' if (Cfg::opt('noipc'));
    # skip to ask feeback when run uninstaller from '/opt/VRTS/install' since VRTSperl uninstalled at that time.
    return '' if (Cfg::opt('uninstall') && EDR::get('fromdisk'));
    return '' unless (Cfg::opt(qw(install patchupgrade hotfixupgrade upgrade uninstall configure license makeresponsefile)));

    $url=$rel->get_upload_url();
    return '' unless ($url);
    $rel->{upload_url}=$url;
    return '' if (Cfg::opt('responsefile'));
    return 1;
}

sub get_sort_product_name {
    my ($rel,$prod) = @_;
    return $prod;
}

sub get_sort_product_name_for_latest_version {
    my ($rel,$prod) = @_;
    return $prod;
}

sub check_installed_pkgs_version_sys { return 1; }

#if $sort_padv is not null, get the matrix data for the $sort_padv
#if $sort_padv is null, get the matrix data for all the $sort_padv which exists under the $matrix_dir_path
sub init_releasematrix {
    my ($rel,$matrix_dir_path,$sort_padv) = @_;
    my ($filename,@supported_padvs,$release_matrix_for_padv,$patch_matrix_for_padv,$hotfix_matrix_for_padv);

    EDR::die("$matrix_dir_path is not found.") unless(-d $matrix_dir_path);

    if ($sort_padv) {
        #read release/ga json based on padv
        $filename = "$matrix_dir_path"."$sort_padv-ga.json";
        $release_matrix_for_padv = $rel->get_matrix_data($filename);
        #pre-process the psudo_matrix item, delete it from the hash
        delete $release_matrix_for_padv->{$sort_padv}{"99999"} if $release_matrix_for_padv->{$sort_padv}{"99999"};
        $rel->{vc_release_matrix_data}{$sort_padv} = $release_matrix_for_padv->{$sort_padv};

        #read patch json based on padv
        $filename = "$matrix_dir_path"."$sort_padv-patch.json";
        $patch_matrix_for_padv = $rel->get_matrix_data($filename);
        #pre-process the psudo_matrix item, delete it from the hash
        delete $patch_matrix_for_padv->{$sort_padv}{"99999"} if $patch_matrix_for_padv->{$sort_padv}{"99999"};
        $rel->{vc_patch_matrix_data}{$sort_padv} = $patch_matrix_for_padv->{$sort_padv};

        #read hotfix json based on padv
        $filename = "$matrix_dir_path"."$sort_padv-hotfix.json";
        $hotfix_matrix_for_padv = $rel->get_matrix_data($filename);
        #pre-process the psudo_matrix item, delete it from the hash
        delete $hotfix_matrix_for_padv->{$sort_padv}{"99999"} if $hotfix_matrix_for_padv->{$sort_padv}{"99999"};
        $rel->{vc_hotfix_matrix_data}{$sort_padv} = $hotfix_matrix_for_padv->{$sort_padv};
    } else {
        @supported_padvs = qw(aix53 aix61 aix71 hpux1123 hpux1131 rhel4_x86_64 rhel5_x86_64 rhel6_x86_64 rhel7_x86_64 sles9_x86_64 sles10_x86_64 sles11_x86_64 
                                sol8_sparc sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64 sol12_sparc sol12_x64 centos6_x86_64 centos7_x86_64);
        foreach my $padv (@supported_padvs) {
            $filename = "$matrix_dir_path"."$padv-ga.json";
            if (-f $filename) {
                $release_matrix_for_padv = $rel->get_matrix_data($filename);
                delete $release_matrix_for_padv->{$padv}{"99999"} if $release_matrix_for_padv->{$padv}{"99999"};
                $rel->{vc_release_matrix_data}{$padv} = $release_matrix_for_padv->{$padv};
            }
        }

        foreach my $padv (@supported_padvs) {
            $filename = "$matrix_dir_path"."$padv-patch.json";
            if (-f $filename) {
                $patch_matrix_for_padv = $rel->get_matrix_data($filename);
                delete $patch_matrix_for_padv->{$padv}{"99999"} if $patch_matrix_for_padv->{$padv}{"99999"};
                $rel->{vc_patch_matrix_data}{$padv} = $patch_matrix_for_padv->{$padv};
            }
        }

        foreach my $padv (@supported_padvs) {
            $filename = "$matrix_dir_path"."$padv-hotfix.json";
            if (-f $filename) {
                $hotfix_matrix_for_padv = $rel->get_matrix_data($filename);
                delete $hotfix_matrix_for_padv->{$padv}{"99999"} if $hotfix_matrix_for_padv->{$padv}{"99999"};
                $rel->{vc_hotfix_matrix_data}{$padv} = $hotfix_matrix_for_padv->{$padv};
            }
        }
    }
}

sub get_release_matrix {
    my ($rel,$padv,$level) = @_;

    return $rel->{vc_release_matrix_data}{$padv} if ($level eq 'ga');
    return $rel->{vc_patch_matrix_data}{$padv}   if ($level eq 'patch');
    return $rel->{vc_hotfix_matrix_data}{$padv};
}

sub get_matrix_data {
    my ($rel, $jsonfile, $allowerr) = @_;
    my $rf = EDRu::readfile($jsonfile);
    return '' if (!$rf);

    my $jsonhash = EDRu::eval_json($rf);
    if ($jsonhash == 0) {
        if ($allowerr) {
            Msg::log("Cannot decode json file: $jsonfile");
        } else {
            Msg::die("Cannot decode json file: $jsonfile");
        }
    }

    return $jsonhash;
}

sub set_matrix_path {
    my ($rel, $matrix_path) = @_;
    my ($cpic,$deploy,$valid_path,$perl_matrix,@matrix_paths);
    $cpic=Obj::cpic();
    $deploy=Obj::deploy();
    $perl_matrix=Cfg::opt('rootpath')."$cpic->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix/";
    # first validate the given $matrix_path if it's a directory
    if ($matrix_path && (-d $matrix_path)) {
        return $rel->set_matrix_path()
            if (!$rel->valid_matrix_path($matrix_path));
        $valid_path=$matrix_path;
    } else {
        # validate $rel->{matrix_path} if already set
        if (defined $rel->{matrix_path}) {
            $matrix_path=$rel->{matrix_path};
            return if ($rel->valid_matrix_path($matrix_path));
            undef($rel->{matrix_path});
        }
        # changes for E3088885 to make sure at least one matrix directory is found
        push (@matrix_paths, "$deploy->{preference}{Repository}{value}/.matrix") if (defined $deploy->{preference}{Repository}{value});
        push (@matrix_paths, "/opt/VRTS/install/.matrix") if (-d "/opt/VRTS/install/.matrix");
        push (@matrix_paths, $perl_matrix) if (-d "$perl_matrix");
        for my $path (@matrix_paths) {
            next unless ($rel->valid_matrix_path($path));
            $valid_path=$path;
            last;
        }
        if (!$valid_path) {
            # find and validate all the Matrix directory found under $cpic->{mediapath}
            $matrix_path=EDR::cmd_local("_cmd_find $cpic->{mediapath} -type d -name Matrix");
            for my $path (split /\n/, $matrix_path) {
                next if (!$rel->valid_matrix_path($path));
                $valid_path=$path; last;
            }
        }
    }

    if ($valid_path) {
        $valid_path=~s/\/$//g;
        $valid_path.="/";
        $rel->{matrix_path}=$valid_path;
        Msg::log("Pick up $rel->{matrix_path} as the release information matrix directory");
    } else {
        EDR::die("Cannot properly set matrix_path");
    }
    return;
}

sub valid_matrix_path {
    my ($rel,$matrix_path) = @_;
    my ($ret,$msg,$list);

    $matrix_path||=$rel->{matrix_path} if (defined $rel->{matrix_path});
    $ret=0;
    if (!$matrix_path) {
        $msg=Msg::new("No matrix_path defined. This is an invalid Rel Matrix directory\n");
        #$msg->print();
    } elsif (!-d $matrix_path) {
        $msg=Msg::new("$matrix_path is not a directory. We need a valid Rel Matrix directory\n");
        #$msg->print();
    } else {
        $list=EDR::cmd_local("_cmd_ls $matrix_path");
        for my $file (split /\n/, $list) {
            next unless ($file =~ /\.json$/);
            if (-f "$matrix_path/$file") {
                $ret=1;
                last;
            }
        }
    }
    return $ret;
}

# Copy json files from /opt/VRTSperl/../../Matrix to
# 1. /opt/VRTS/install/.matrix
# 2. $cpic->{preference}{Repository}{value}/.matrix (if defined)
sub copy_releasematrix {
    my ($rel,$matrix_arr)=@_;
    my ($cpic,$localsys,$rootpath);
    my ($perl_matrix,$default_matrix,$repository_matrix);
    my ($perl_matrix_date,$default_matrix_date,$repository_matrix_date);
    my $deploy=Obj::deploy();
    $cpic=Obj::cpic();
    

    # use $rel->{initial_padv_matrix_check} to determine
    # the json files in matrix/ need to be updated or not
    $rel->{initial_padv_matrix_check}='rhel6_x86_64-ga.json';
    $deploy->read_deploy_config();

    $rootpath=Cfg::opt('rootpath');
    $perl_matrix="$rootpath$cpic->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix";
    $default_matrix=$rootpath . "/opt/VRTS/install/.matrix";
    $repository_matrix="$deploy->{preference}{Repository}{value}/.matrix" if (defined($deploy->{preference}{Repository}{value}));
    $localsys=Obj::localsys();
    $localsys->mkdir($default_matrix) unless ($localsys->is_dir($default_matrix));

    if ($localsys->exists("$perl_matrix/$rel->{initial_padv_matrix_check}")) {
        $perl_matrix_date=$rel->matrix_maxid("$perl_matrix/$rel->{initial_padv_matrix_check}");
    }
    if ($localsys->exists("$default_matrix/$rel->{initial_padv_matrix_check}")) {
        $default_matrix_date=$rel->matrix_maxid("$default_matrix/$rel->{initial_padv_matrix_check}");
    }

    if (!$repository_matrix) {
        # if $repository_matrix not defined, only compare $perl_matrix and $default_matrix
        if (!$default_matrix_date || $default_matrix_date lt $perl_matrix_date) {
            $rel->merge_json($localsys,$perl_matrix,$default_matrix);
        }
        return '';
    }

    $localsys->mkdir($repository_matrix) unless ($localsys->is_dir($repository_matrix));
    if ($localsys->exists("$repository_matrix/$rel->{initial_padv_matrix_check}")) {
        $repository_matrix_date=$rel->matrix_maxid("$repository_matrix/$rel->{initial_padv_matrix_check}");
    }

    if ($default_matrix_date && $repository_matrix_date) {
        if ($perl_matrix_date ge $default_matrix_date && $perl_matrix_date ge $repository_matrix_date) {
            $rel->merge_json($localsys,$perl_matrix,$default_matrix) if($perl_matrix_date ne $default_matrix_date);
            $rel->merge_json($localsys,$perl_matrix,$repository_matrix) if($perl_matrix_date ne $repository_matrix_date);
        }
        if ($default_matrix_date gt $perl_matrix_date && $default_matrix_date gt $repository_matrix_date) {
            $rel->merge_json($localsys,$default_matrix,$repository_matrix);
        }
        if ($repository_matrix_date gt $perl_matrix_date && $repository_matrix_date gt $default_matrix_date) {
            $rel->merge_json($localsys,$repository_matrix,$default_matrix);
        }
    } elsif (!$default_matrix_date && $repository_matrix_date) {
        # Only need to compare $perl_matrix_date and $repository_matrix_date
        if ($repository_matrix_date ge $perl_matrix_date) {
            $rel->merge_json($localsys,$repository_matrix,$default_matrix);
        } else {
            $rel->merge_json($localsys,$perl_matrix,$default_matrix);
            $rel->merge_json($localsys,$perl_matrix,$repository_matrix);
        }
    } elsif ($default_matrix_date && !$repository_matrix_date) {
        # Only need to compare $perl_matrix_date and $default_matrix_date
        if ($default_matrix_date ge $perl_matrix_date) {
            $rel->merge_json($localsys,$default_matrix,$repository_matrix);
        } else {
            $rel->merge_json($localsys,$perl_matrix,$default_matrix);
            $rel->merge_json($localsys,$perl_matrix,$repository_matrix);
        }
    } elsif (!$default_matrix_date && !$repository_matrix_date) {
        $rel->merge_json($localsys,$perl_matrix,$default_matrix);
        $rel->merge_json($localsys,$perl_matrix,$repository_matrix);
    }
}

# Copy json files from $src_matrix to $dest_matrix
sub merge_json {
    my ($pkg,$sys,$src_matrix,$dest_matrix) = @_;
    my ($src_json,$dest_json);
    my $rel=Obj::rel();
    # Compare json files 1 by 1 and copy the newer ones to $src_matrix and $dest_matrix
    for my $json (split(/\n/,$sys->lsfile($src_matrix))) {
        $src_json="$src_matrix/$json";
        $dest_json="$dest_matrix/$json";
        # if json file exists in $dest_matrix dir
        if ($sys->exists($dest_json)) {
            # compare checksum first, if checksum equal, not copy the $src_json
            if (EDRu::cksum($src_json) ne EDRu::cksum($dest_json)) {
                # 1. $dest_json is sabotaged, cover the file with $src_json
                # 2. $dest_json is older than $src_json, cover the file with $src_json
                if($rel->matrix_maxid($src_json) gt $rel->matrix_maxid($dest_json)) {
                    $sys->copyfile($src_json,$dest_json);
                }
            }
        } else {
            $sys->copyfile($src_json,$dest_json);
        }
    }
    return;
}

# input: the file path of a matrix file
# output: 1. '' if the file does not exist or no valid items
#         2. return the max {$tag}
sub matrix_maxid {
    my ($rel,$file) = @_;
    my ($matrix,$contents,$tag);

    return '' unless -f $file;
    $matrix = $rel->get_matrix_data($file, 1);
    return '' unless ($matrix);
    $contents = [];
    $tag = 'release_date';
    @$contents = sort {$a->{$tag} cmp $b->{$tag}} values %{(values %$matrix)[0]};
    return '' unless scalar @$contents;
    return $contents->[-1]{$tag};
}

sub check_update {
    my ($rel) = @_;
    my ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj);
    my ($cpic,$prod,$prodi,$prodname,$relvers,$prodvers,$title,$msg,$patchname,$basevers,$web);
    return if (! Cfg::opt(qw(install upgrade patchupgrade hotfixupgrade)));
    return if (Cfg::opt(qw(noipc makeresponsefile)));
    $cpic=Obj::cpic();
    return if (! $cpic->{prod});
    return if ($cpic->{eebi});
    $prod = $cpic->prod();
    $prodname = $prod->{name};
    $prodi = lc($prod->{prod});
    $relvers = $rel->{titlevers};
    # for current naming of #.#.# P# may change in the future
    $relvers =~ s/\s(P|HF)\d+$// if ($rel->{type} eq "H");
    $prodvers = $relvers;
    $prodvers =~ s/\s//g;
    $basevers = $prodvers;
    $basevers =~ s/^([0-9\.]*).*$/$1/;
    $msg = Msg::new("Checking online updates for $prodname $relvers");
    $msg->bold;
    $title = Msg::new("Check online updates");
    $web = Obj::web();
#    if (Obj::webui()) {
#        $web->web_script_form('showstatus', $title);
#        my $left_info = Msg::new("Getting release patches");
#        $title->display_left($left_info);
#    }

    ($sort_status,$prod_upd_cv, $cpi_upd_cv, $lv_cb, $prod_upd_lv_cb, $cpi_upd_lv_cb, $lv_mj, $prod_upd_lv_mj, $cpi_upd_lv_mj) = $cpic->get_releases_patches_via_sort_webservice($prodi, $cpic->{padv}, $prodvers);
    $msg = "\n";
    if ($sort_status) {
        $title->display_right() if (Obj::webui());
        # current installed version
        if ($prod_upd_cv && @$prod_upd_cv || $cpi_upd_cv && @$cpi_upd_cv) {
            $msg .= Msg::new("\tAvailable updates for $prodname $relvers:\n")->{msg};
            for my $patch (@$prod_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    $patchname\n")->{msg};
            }
            for my $patch (@$cpi_upd_cv) {
                $patchname = $patch->{release_name};
                $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
            }
        } else {
            $msg .= Msg::new("\tNo updates available for $prodname $relvers\n")->{msg};
        }
        # latest version in current release branch.
        if ($lv_cb) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname $basevers release is: $lv_cb\n")->{msg};
            if ($prod_upd_lv_cb && @$prod_upd_lv_cb || $cpi_upd_lv_cb && @$cpi_upd_lv_cb) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_cb:\n")->{msg};
                for my $patch (@$prod_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n")->{msg};
                }
                for my $patch (@$cpi_upd_lv_cb) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_cb\n")->{msg};
            }
        }
        # latest major release version
        if ($lv_mj) {
            $msg .= "\n";
            $msg .= Msg::new("\tThe latest version of $prodname is: $lv_mj\n")->{msg};
            if ($prod_upd_lv_mj && @$prod_upd_lv_mj || $cpi_upd_lv_mj && @$cpi_upd_lv_mj) {
                $msg .= Msg::new("\t  Available updates for $prodname $lv_mj:\n")->{msg};
                for my $patch (@$prod_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    $patchname\n")->{msg};
                }
                for my $patch (@$cpi_upd_lv_mj) {
                    $patchname = $patch->{release_name};
                    $msg .= Msg::new("\t    (Installer) $patchname\n")->{msg};
                }
            } else {
                $msg .= Msg::new("\t  No updates available for $prodname $lv_mj\n")->{msg};
            }
        }

        $msg .= "\n";
        $msg .= Msg::new("\tVisit https://sort.veritas.com for more information.\n")->{msg};
    } else {
        $title->display_right('failed') if (Obj::webui());
        $msg .= Msg::new("\tA connection attempt to https://sort.veritas.com to check for product updates failed.\n")->{msg};
        $msg .= Msg::new("\tVisit https://sort.veritas.com to check for available product updates and information.\n")->{msg};
    }
    Msg::print($msg);
    if (Obj::webui()) {
#        $msg =~ s/(https\:\/\/\S+)\s+/<a href="$1">$1<\/a>/g;
        $msg =~ s/^\t\s{4}/&nbsp;&nbsp;&nbsp;&nbsp;/mg;
        $msg =~ s/^\t\s{2}/&nbsp;&nbsp;/mg;
        $msg =~ s/\t//g;
        $msg =~ s/\n/\\n/g;
        $web->web_script_form('check_update',$msg);
    }
    return;
}

sub get_tunables{
    my ($rel) = @_;
    return $rel->{tunables};
}

sub validate_tunable_sys{
    my ($rel,$sys,$tunable,$value) =@_;
    my ($cpic, $sysi, $tunable_info, @values, $msg);

    # 0 valid
    # 3 value is invalid
    $cpic=Obj::cpic();
    $sysi = $sys->{sys};
    $tunable_info = $cpic->get_tunable_info($tunable);
    @values = @{$tunable_info->{values}};
    if ( $tunable_info->{type} eq 'enum') {
        if (!EDRu::inarr($value, @values)) {
            $msg = Msg::new("Invalid value $value of $tunable specified for $sysi or all systems");
            return (3, $msg);
        }
    } elsif ($tunable_info->{type} eq 'range') {
        if ($value !~ /^-?(0x|0X)?\d+$/) {
            $msg = Msg::new("$tunable requires a numeric value but $value is specified for $sysi or all systems");
            return (3, $msg);
        }
        if (($values[0] ne undef && $value < $values[0]) || ($values[1] ne undef && $value > $values[1])) {
            if (!($value == 0 && $tunable_info->{allow_zero})) {
                if ($values[0] ne undef && $values[1] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value between $values[0] and $values[1] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                } elsif ($values[0] ne undef) {
                    $msg = Msg::new("$tunable requires a numeric value equal to or greater than $values[0] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                } else {
                    $msg = Msg::new("$tunable requires a numeric value equal to or less than $values[1] but $value is specified for $sysi or all systems");
                    return (3, $msg);
                }
            }
        }

        if ($values[2] ne undef && $value % $values[2]) {
            $msg = Msg::new("Value of $tunable must be a multiple of $values[2] but $value is specified for $sysi or all systems");
            return (3, $msg);
        }
    }
    return 0;
}

sub set_tunable_sys{
    my ($rel,$sys,$tunable,$value) =@_;
    my ($origval,$ret,$out);

    $origval = $rel->get_tunable_value_sys($sys,$tunable);

    if ($origval eq $value) {
        Msg::log("$tunable has already had value $value on $sys->{sys}");
        return 2; # 2 - Not changed
    }

    ($ret, $out) = $rel->set_tunable_value_sys($sys,$tunable,$value);
    if ($ret) {
        return (0, $out); # 0 - Failed to set
    }
    return 1; # 1 - Done
}

sub get_product_for_settunables{
    my ($rel) = @_;
    return '';
}

sub get_local_script_version_suffix{
    my ($rel) = @_;
    my $suffix = (defined $rel->{basetitlevers}) ? $rel->{basetitlevers} : $rel->{titlevers};
    $suffix =~ s/\s//mxg;
    $suffix =~ s/\.//mxg;
    return $suffix;
}

sub adjust_pkglist{
    my $rel = shift;
    for my $prodv(@{$rel->{prods}}){
        my $prod = Obj::prod($prodv,$rel->{padv});
        if(UNIVERSAL::can($prod,'adjust_pkglist')){
            $prod->adjust_pkglist();
        }
    }
}

sub self_healing {
    my ($release,$padv) = @_;
    my ($edr,$sort_hotfix,$sort_status,$local_status,$local_hotfix,$web);
    my $cpic=Obj::cpic();
    $web=Obj::web();
    return 1 if ($web->{run_type} && ($web->{run_type} eq "vom_addon"));

    # In case old CPIP releases prior to 6.1 do not support self healing
    return 1 unless UNIVERSAL::can('CPIP', 'get_release_version');

    $edr=Obj::edr();
    $release ||= $edr->{release};
    $padv ||= $edr->{padv};
    $release = CPIP::get_release_version($release);

    ($sort_status,$sort_hotfix) = get_hotfix_via_sort_webservice($padv,$release);
    ($local_status,$local_hotfix) = get_hotfix_via_local_storage($padv,$release);

    if ($sort_status) {
        $cpic->{sortready}=1;
    } else {
        $cpic->{sortready}=0;
    }

    if ($sort_status &&
        (!$local_status || !$sort_hotfix->{release_name} || ($sort_hotfix->{release_name} ne $local_hotfix->{release_name}))) {
        utilize_sort_hotfix($sort_hotfix,$padv,$release);
    } elsif ($local_status) {
        utilize_local_hotfix($local_hotfix);
    }
    return 1;
}


sub get_hotfix_content_from_gzfile {
    my ($sort_hotfix) = @_;
    my ($dir,$url,$filename,@array,$rtn,$download_errors,$file,$content);
    my ($gunzip,$tar,$cat,$exitcode);

    $url = $sort_hotfix->{upload_location};
    return '' unless $url;

    @array = split(/\//,$url);
    $filename = $array[-1];
    my $cpic=Obj::cpic();
    $dir = $cpic->{hotfixdir} . '/tmp_' . EDRu::randomstr(6);
    EDRu::mkdir_local_nosys($dir) || return '';
    $file = "$dir/$filename";

    my $time1 = time;
    $rtn=EDRu::download_file($url, $file);
    my $time2 = time;
    my $totaltime = $time2 - $time1;
    my $clocktime=EDRu::clocktime($totaltime);
    Msg::log("Download Patch from URL '$url' took $totaltime seconds($clocktime)");

    # return "" if failed to get the URL config file.
    if (!$rtn || ! -f $file) {
        $download_errors=EDR::get('download_errors');
        Msg::log("Could not download file $file!\n\n$download_errors\n");
        EDRu::rmdir_local_nosys($dir);
        return '';
    }

    $gunzip = get_cmd(qw(/usr/bin/gunzip /usr/contrib/bin/gunzip /bin/gunzip));
    $tar = get_cmd(qw(/usr/bin/tar /bin/tar));
    $cat = get_cmd(qw(/usr/bin/cat /bin/cat));
    unless ($gunzip && $tar && $cat) {
        Msg::log("Skip downloading Patch because command not found: gunzip tar cat");
        EDRu::rmdir_local_nosys($dir);
        return '';
    }
    (undef,$exitcode) = cmd_local("cd $dir; $gunzip -c $filename | $tar xvf - 2>/dev/null");
    if ($exitcode) {
        Msg::log("Failed to un-zip and un-tar the file $file");
        EDRu::rmdir_local_nosys($dir);
        return '';
    }
    ($content,undef) = cmd_local("$cat $dir/*/*.pl 2>/dev/null");
    EDRu::rmdir_local_nosys($dir);
    return $content;
}

sub cmd_local {
    my ($cmd) = @_;
    #Msg::log("Run command: $cmd");
    my ($rtn,$stderr,$exitcode) = Sys::run_local_command($cmd);
    #Msg::log("Exitcode is $exitcode, Return:\n$rtn");
    return ($rtn,$exitcode);
}

sub get_cmd {
    my (@cmds) = @_;

    for my $cmd (@cmds) {
        return $cmd if -x $cmd;
    }
    return '';
}

sub get_hotfix_json_via_sort {
    my $padv = shift;
    my ($response,$url,$param,$jsonhash);

    my $time1 = time;
    $padv = Padv::cpi_to_sort_padv($padv);

    # do not try download hotfix from sort if noipc is used
    return 0 if EDRu::inarr('-noipc', @ARGV);

    if (EDRu::inarr('-pseudo_hotfix', @ARGV)) {
        $param = '&pseudo=1';
    } else {
        $param = '';
    }
    
    $url = CPIP::get_hotfix_sort_url();
    $url .= "?padv=$padv&level=cpi$param";
    Msg::log( "Downloading Patch definition from URL '$url'");
    $response = EDRu::download_page($url);
    $response =~ s/\n+|\r+|\t+|\f+//mg;
    $response =~ s/^.*<pre>//mg;
    $response =~ s/<\/pre>.*$//mg;
    $response =~ s/(.*)}.*$/$1}/mg;

    if ($response) {
        Msg::debug("RESULT is :$response");
        $jsonhash = EDRu::eval_json($response);
        if ($jsonhash == 0) {
            Msg::log("Failed to parse the Patch definition.\n");
            return 0;
        }
    } else {
        Msg::log("Failed to contact our SORT website");
        return 0;
    }

    my $time2 = time;
    my $totaltime = $time2 - $time1;
    my $clocktime=EDRu::clocktime($totaltime);
    Msg::log( "Download Patch definition took $totaltime seconds($clocktime)");

    return ($response,$jsonhash);
}

sub get_hotfix_via_sort_webservice {
    my ($padv,$release) = @_;
    my ($response,$json,$sort_hotfix,$ahotfix);

    $padv = Padv::cpi_to_sort_padv($padv);

    # do not try download hotfix from sort if noipc is used
    return 0 if EDRu::inarr('-noipc', @ARGV);

    if (EDRu::inarr('-pseudo_hotfix', @ARGV)) {
        $release = '9.9pseudo';
    }
    ($response,$json) = get_hotfix_json_via_sort($padv);
    $sort_hotfix = {};
    if (($response) && (ref($json->{$padv}) eq 'HASH')) {
        for my $id (keys %{$json->{$padv}}) {
            $ahotfix = $json->{$padv}{$id};
            if ($ahotfix->{release_version} eq $release){
                $sort_hotfix = $ahotfix;
                last;
            }
        }
    }
    return ($response,$sort_hotfix);
}

sub get_hotfix_via_local_storage {
    my ($padv,$release) = @_;
    my ($file,$content,$local_hotfix,$hotfix_version);

    $hotfix_version = '';
    $file = get_local_hotfix_file($padv,$release);
    if (-f $file) {
        $content = EDRu::readfile($file);
        Msg::log("Getting Patch from local file $file");
        $local_hotfix->{content} = $content;
        if ($content =~ /^\s*#HOTFIX_VERSION:(\S+)/) {
            $hotfix_version = $1;
        }
        $local_hotfix->{release_name} = $hotfix_version;
        return (1,$local_hotfix);
    }
    return 0;

}

sub utilize_sort_hotfix {
    my ($sort_hotfix,$padv,$release) = @_;
    my ($file,$dirname,$content);

    return 1 unless ($sort_hotfix);

    $content = get_hotfix_content_from_gzfile($sort_hotfix);
    if (!$content) {
        Msg::log("There is no available installer Patch on SORT website for this release $release $padv");
        return 0;
    }
    return 0 unless eval_hotfix($content);

    # save the hotfix to local
    $file = get_local_hotfix_file($padv,$release);
    $dirname = File::Basename::dirname($file);
    EDRu::mkdir_local_nosys($dirname) || return '';
    Msg::log("Save the Patch to local file $file");
    $content = "#HOTFIX_VERSION:$sort_hotfix->{release_name}\n$content";
    EDRu::writefile($content,$file);
    return 1;
}

sub utilize_local_hotfix {
    my ($local_hotfix) = @_;
    my ($content);

    $content = $local_hotfix->{content};
    return 0 unless eval_hotfix($content);
    return 1;
}

sub eval_hotfix {
    my ($hotfix) = @_;
    my ($eval_err,$msg);

    return 0 if (!$hotfix || $hotfix =~ /^\s*$/);

    Msg::log("The Patch content:\n$hotfix");
    Msg::log("Eval the Patch ...");
    eval $hotfix;
    $eval_err=$@;
    if ($eval_err) {
        $eval_err =~ s/\\/\\\\/g;
        Msg::log("The content of Patch is not in correct Perl format\n\n$eval_err");
        return 0;
    }
    Msg::log("Eval done");
    return 1;
}

sub get_local_hotfix_file {
    my ($padv,$release) = @_;

    my $cpic=Obj::cpic();
    return "$cpic->{hotfixdir}/${release}_${padv}.pl";
}

sub initiate_kernel_padv_hash {
    my ($rel) = @_;
    my ($response,$url,$kernel_padv_file,$jsonhash,$jsontext,$jsonfile);
    my ($rootpath,$cpic,$edr,$perl_matrix,$local_matrix,$image_matrix,$level);

    return 1 if ($rel->{kernel_padv} && %{$rel->{kernel_padv}});
    $rootpath=Cfg::opt("rootpath");
    $cpic=Obj::cpic();
    $edr=Obj::edr();

    $jsonfile="kernel_padv.json";
    $perl_matrix="$rootpath/$edr->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix/$jsonfile";
    $local_matrix="$rootpath/opt/VRTS/install/.matrix/$jsonfile";
    $image_matrix="$edr->{scriptdir}/scripts/CPIP/Rel/Matrix/$jsonfile";

    if ($cpic->{sortready}) {
        # with SORT connection, download the latest kernel-padv matrix
        $url=CPIP::get_kernel_padv_sort_url();
        Msg::log("Downloading kernel padv matrix from URL '$url'");
        $response = EDRu::download_page($url);
        if ($response) {
            $jsonhash = EDRu::eval_json($response);
            if ($jsonhash) {
                # Save the matrix to local
                $jsontext=EDRu::to_json($jsonhash,{pretty=>1});
                EDRu::mkdir_local_nosys(EDRu::dirname($local_matrix));
                EDRu::writefile($jsontext,$local_matrix);
            }
        }
    }
    if (!$jsonhash) {
        # without SORT connection/matrix download failed, use the local kernel-padv matrix
        for my $file($local_matrix,$image_matrix,$perl_matrix) {
            if (-f $file) {
                $jsonhash=$rel->get_matrix_data($file);
            }
            if ($jsonhash) {
                Msg::log("Using local kernel padv matrix json: $file");
                last;
            }
        }
    }

    if (!$jsonhash) {
        Msg::die("Cannot find kernel-padv json hash");
        return 0;
    }
    
    # initiate the $jsonhash content into $rel->{kernel_padv}{$padv}{$kernel}
    for my $key (keys %{$jsonhash}) {
        $level=$jsonhash->{$key}{level};
        $level=~s/\D+(\d+)\D*/$1/mx;
        $rel->{kernel_padv}{$jsonhash->{$key}{padv}}{$key}=$level;
    }
    return 1;
}

# initiate the release matrix for target padv
sub init_sort_platvers {
    my ($rel)=@_;
    my ($response,$padv,$url,$jsontext,$jsonfile);
    my ($rootpath,$cpic,$edr,$perl_matrix_dir,$local_matrix_dir,$image_matrix_dir,$level);
    return 1 if (Padv::plat($rel->{padv}) ne 'Linux');
    return 1 if ($rel->{release_matrix} && %{$rel->{release_matrix}});
    $rootpath=Cfg::opt("rootpath");
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    $padv=Padv::cpi_to_sort_padv($rel->{padv});
    $rel->set_value("sort_padv",$padv);

    $perl_matrix_dir="$rootpath/$edr->{perlpath}/lib/site_perl/$cpic->{release}/CPIP/Rel/Matrix/";
    $local_matrix_dir="$rootpath/opt/VRTS/install/.matrix/";
    $image_matrix_dir="$edr->{scriptdir}/scripts/CPIP/Rel/Matrix/";
    
    # with SORT connection, download the latest $padv ga/patch/hotfix matrix
    for my $level (qw/ga patch hotfix/) {
        my $jsonhash;
        if ($cpic->{sortready}) {
            $url = CPIP::get_hotfix_sort_url();
            $url.= "?padv=$padv&level=$level&pretty=1&product=SFHA";
            Msg::log("Downloading release matrix data for $padv $level");
            $response = EDRu::download_page($url);
            if ($response) {
                $jsonhash = EDRu::eval_json($response);
                if ($jsonhash) {
                    # Save the matrix to local
                    $jsontext=EDRu::to_json($jsonhash,{pretty=>1});
                    EDRu::mkdir_local_nosys($local_matrix_dir);
                    EDRu::writefile($jsontext,"$local_matrix_dir/$padv-$level.json");
                }
            }
        }
        if (!$jsonhash) {
            for my $dir ($local_matrix_dir,$image_matrix_dir,$perl_matrix_dir) {
                my $file="$dir/$padv-$level.json";
                if (-f $file) {
                    $jsonhash=$rel->get_matrix_data($file);
                }
                if ($jsonhash) {
                    Msg::log("Using local release matrix $padv $level json: $file");
                    last;
                }
            }
            if (!$jsonhash) {
                Msg::die("Cannot find release matrix json hash for $padv $level");
                return 0;
            }
        }
        $rel->set_hash_value("release_matrix,$padv,$level",$jsonhash->{$padv});
    }
    return 1;
}

sub pre_platvers {
    my ($rel)=@_;
    my ($sort_padv,$matrix,$level);
    my ($msg,$common_sys,$cpic);
    $cpic=Obj::cpic();

    return 1 if (Padv::plat($rel->{padv}) ne 'Linux');
    return 1 if (Cfg::opt('ignorechecks'));
    $sort_padv=$rel->{sort_padv};
    if ($rel->{type}=~/M/) {
        $level='patch';
    } elsif ($rel->{type} eq 'H') {
        if ($rel->{currentrelease} =~ /^UXRT\d{4}/) {
            $level='patch';
        } else {
            $level='ga';
        }
    } else {
        $level='ga';
    }
    $matrix=$rel->{release_matrix}{$sort_padv}{$level};
    while ( my ($id, $val) = each %{$matrix} ) {
        if (EDRu::compvers($val->{release_version},$rel->{vers},3)==0) {
            $rel->set_value('os_update_min', $val->{os_update_min});
            $rel->set_value('os_update_max', $val->{os_update_max});
            $rel->set_value('os_update_err', $val->{os_update_err});
            while ( my ($required_patch_update, $required_patch_id) = each %{$val->{required_patch}} ) {
                $rel->set_value("os_update_patchid_mapping,$required_patch_update", $required_patch_id);
            }
            last;
        }
    }
    Msg::log("Minimum OS update level supported: $rel->{os_update_min}");
    Msg::log("Maximum OS update level supported: $rel->{os_update_max}");
    Msg::log("Unsupported OS update level: $rel->{os_update_err}") if ($rel->{os_update_err});
    
    unless (defined($rel->{os_update_min}) && defined($rel->{os_update_max})) {
        $common_sys=@{$cpic->{systems}}[0];
        $msg=Msg::new("Failed to retrieve supported $rel->{padv} update level from the release matrix, contact our technical support for further troubleshooting.");
        $common_sys->push_error($msg);
        for my $sys (@{$cpic->{systems}}) {
            $sys->set_value('stop_checks',1);
        }
    }

    return 1;
}
sub rel_handle_publisher_errors { 
    # get handler and args then run the handles
    my $rel = shift;
    my ($edr, $errmsg);
	my ($rtn, $rootpath, @lines);
    my ($position, $msg, $ayn, $publisher, $url, %sys_and_problem_publisher) ;
    $edr=Obj::edr();
    
    for my $sys (@{$edr->{systems}}) {        
        my (@problem_publisher);

        if ($sys->{publisher_err_args}){
            $errmsg = $sys->{publisher_err_args};                   
            $rootpath=Cfg::opt('rootpath')||'';
            $rootpath="-R $rootpath" if ($rootpath);
            $rtn=$sys->cmd("_cmd_pkg $rootpath publisher 2>/dev/null");
            @lines = split("\n", $rtn);
            
            for my $line (@lines){
                #the last colomn is url,where data of it may ends with '/', remove the last '/'
                #in order to be searched in $errmsg.
                if ($line =~ /^(\S+).*\s+(\S+)\/$/){
                    $publisher = $1;
                    $url = $2;
                    $position = index($errmsg, $url);
                    if (-1 != $position){
                        push (@problem_publisher, $publisher)
                    }
                }
            }
			if (@problem_publisher){
			    $sys_and_problem_publisher{"$sys->{sys}"} = \@problem_publisher;
			}
		}
	}
    return if (!%sys_and_problem_publisher);
    $msg = Msg::new("Unable to contact the following configured publishers:");
    $msg->bold;
    $msg->n();
    for my $sys (@{$edr->{systems}}) {
        if ($sys_and_problem_publisher{$sys->{sys}}) {
            Msg::print("On $sys->{sys}:\n");
            for $publisher (@{$sys_and_problem_publisher{$sys->{sys}}}){
                Msg::print("  $publisher");
            }
            Msg::n();
        }
    }
    $msg=Msg::new("Do you want the installer to unset them and continue?");
    $ayn = $msg->aynn();
    if ($ayn eq 'Y'){
        for my $sys (@{$edr->{systems}}) {
            if ($sys_and_problem_publisher{$sys->{sys}}){
                for $publisher (@{$sys_and_problem_publisher{$sys->{sys}}}){
                    $sys->cmd("_cmd_pkg $rootpath unset-publisher $publisher");
                }
            }
        }
        return;
    }
	my $cpic = Obj::cpic();
    $cpic->rel->completion();
    $cpic->edr_completion();
	return;
}
sub rel_handle_precheck_errors{
    my $rel = shift;
    $rel->rel_handle_publisher_errors();
    return;
}
1;
