#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSfsadv731::Common;
@Pkg::VRTSfsadv731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSfsadv';
    $pkg->{name}=Msg::new("File System Advanced Solutions")->{msg};
    return;
}

sub donotinstall_sys {
    my ($pkg,$sys) = @_;
    my ($pkgsys);

    $pkgsys = Obj::pkg($pkg->{pkgi},$sys->{padv},1);
    if (!$pkgsys) {
        $pkgsys = $pkg;
    }
    return $pkgsys->{donotinstall} || EDRu::inarr($pkgsys->{pkgi},@{$sys->{donotinstallpkgs}});
}

sub donotuninstall_sys {
    my ($pkg,$sys) = @_;
    my ($pkgsys);

    $pkgsys = Obj::pkg($pkg->{pkgi},$sys->{padv},1);
    if (!$pkgsys) {
        $pkgsys = $pkg;
    }
    return $pkgsys->{donotuninstall} || EDRu::inarr($pkgsys->{pkgi},@{$sys->{donotuninstallpkgs}});
}

sub donotupgrade_sys {
    my ($pkg,$sys) = @_;
    my ($pkgsys);

    $pkgsys = Obj::pkg($pkg->{pkgi},$sys->{padv},1);
    if (!$pkgsys) {
        $pkgsys = $pkg;
    }
    return $pkgsys->{donotupgrade} || EDRu::inarr($pkgsys->{pkgi},@{$sys->{donotupgradepkgs}});
}

package Pkg::VRTSfsadv731::AIX;
@Pkg::VRTSfsadv731::AIX::ISA = qw(Pkg::VRTSfsadv731::Common);

package Pkg::VRTSfsadv731::HPUX;
@Pkg::VRTSfsadv731::HPUX::ISA = qw(Pkg::VRTSfsadv731::Common);

package Pkg::VRTSfsadv731::HPUX1131par;
@Pkg::VRTSfsadv731::HPUX1131par::ISA = qw(Pkg::VRTSfsadv731::HPUX);

sub init_padv {
    my $pkg=shift;
    $pkg->{donotinstall} = 1;
    $pkg->{donotuninstall} = 1;
    $pkg->{donotupgrade} = 1;
    return;
}

package Pkg::VRTSfsadv731::Linux;
@Pkg::VRTSfsadv731::Linux::ISA = qw(Pkg::VRTSfsadv731::Common);

package Pkg::VRTSfsadv731::SunOS;
@Pkg::VRTSfsadv731::SunOS::ISA = qw(Pkg::VRTSfsadv731::Common);

package Pkg::VRTSfsadv731::RHEL6x8664;
@Pkg::VRTSfsadv731::RHEL6x8664::ISA = qw(Pkg::VRTSfsadv731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libacl.so.1()(64bit)" => "libacl.x86_64",
		"libacl.so.1(ACL_1.0)(64bit)" => "libacl.x86_64",
		"libcrypto.so.10()(64bit)" => "openssl.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libgcc_s.so.1()(64bit)" => "libgcc.x86_64",
		"libgcc_s.so.1(GCC_3.0)(64bit)" => "libgcc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libssl.so.10()(64bit)" => "openssl.x86_64",
		"libstdc++.so.6()(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(CXXABI_1.3)(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4.5)(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4)(64bit)" => "libstdc++.x86_64",
	};
    return;
}

package Pkg::VRTSfsadv731::RHEL7x8664;
@Pkg::VRTSfsadv731::RHEL7x8664::ISA = qw(Pkg::VRTSfsadv731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libacl.so.1()(64bit)" => "libacl.x86_64",
		"libacl.so.1(ACL_1.0)(64bit)" => "libacl.x86_64",
		"libcrypto.so.10()(64bit)" => "openssl-libs.x86_64",
		"libcrypto.so.10(libcrypto.so.10)(64bit)" => "openssl-libs.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libgcc_s.so.1()(64bit)" => "libgcc.x86_64",
		"libgcc_s.so.1(GCC_3.0)(64bit)" => "libgcc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libssl.so.10()(64bit)" => "openssl-libs.x86_64",
		"libssl.so.10(libssl.so.10)(64bit)" => "openssl-libs.x86_64",
		"libstdc++.so.6()(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(CXXABI_1.3)(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4.5)(64bit)" => "libstdc++.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4)(64bit)" => "libstdc++.x86_64",
	};
    return;
}

package Pkg::VRTSfsadv731::OL6x8664;
@Pkg::VRTSfsadv731::OL6x8664::ISA = qw(Pkg::VRTSfsadv731::RHEL6x8664);

package Pkg::VRTSfsadv731::OL7x8664;
@Pkg::VRTSfsadv731::OL7x8664::ISA = qw(Pkg::VRTSfsadv731::RHEL7x8664);

package Pkg::VRTSfsadv731::CentOS6x8664;
@Pkg::VRTSfsadv731::CentOS6x8664::ISA = qw(Pkg::VRTSfsadv731::RHEL6x8664);

package Pkg::VRTSfsadv731::CentOS7x8664;
@Pkg::VRTSfsadv731::CentOS7x8664::ISA = qw(Pkg::VRTSfsadv731::RHEL7x8664);

package Pkg::VRTSfsadv731::SLES11x8664;
@Pkg::VRTSfsadv731::SLES11x8664::ISA = qw(Pkg::VRTSfsadv731::Linux);

sub init_padv {
	my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libacl.so.1()(64bit)" => "libacl.x86_64",
		"libacl.so.1(ACL_1.0)(64bit)" => "libacl.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libgcc_s.so.1()(64bit)" => "libgcc_s1.x86_64",
		"libgcc_s.so.1(GCC_3.0)(64bit)" => "libgcc_s1.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libstdc++.so.6()(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(CXXABI_1.3)(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4.5)(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4)(64bit)" => "libstdc++6.x86_64",
	};
	return;
}

package Pkg::VRTSfsadv731::SLES12x8664;
@Pkg::VRTSfsadv731::SLES12x8664::ISA = qw(Pkg::VRTSfsadv731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libacl.so.1()(64bit)" => "libacl1.x86_64",
		"libacl.so.1(ACL_1.0)(64bit)" => "libacl1.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libgcc_s.so.1()(64bit)" => "libgcc_s1.x86_64",
		"libgcc_s.so.1(GCC_3.0)(64bit)" => "libgcc_s1.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libstdc++.so.6()(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(CXXABI_1.3)(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4.5)(64bit)" => "libstdc++6.x86_64",
		"libstdc++.so.6(GLIBCXX_3.4)(64bit)" => "libstdc++6.x86_64",
	};
    return;
}

1;
