#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSgab731::Common;
@Pkg::VRTSgab731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSgab';
    $pkg->{name}=Msg::new("Group Membership and Atomic Broadcast")->{msg};
    # some HPUX deviance
    $pkg->{startprocs}=[ qw(gab731) ];
    $pkg->{stopprocs}=[ qw(gab731) ];
    return;
}

package Pkg::VRTSgab731::AIX;
@Pkg::VRTSgab731::AIX::ISA = qw(Pkg::VRTSgab731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{previouspkgnames}=[ qw(VRTSgab.rte) ];
    return;
}
sub donotuninstall_sys {
    my ($pkg,$sys)=@_;
    $pkg->{donotrmonupgrade}=1 if(Cfg::opt("upgrade_kernelpkgs"));
    return;
}


package Pkg::VRTSgab731::HPUX;
@Pkg::VRTSgab731::HPUX::ISA = qw(Pkg::VRTSgab731::Common);

sub preremove_sys {
    my ($pkg,$sys) = @_;
    my ($vers);
    if (Cfg::opt('upgrade')) {
        $vers = $pkg->version_sys($sys);
        if (EDRu::compvers($vers,'4.1',2) == 0) {
            $pkg->{force_uninstall} = 1;
        }
    }
    return;
}

package Pkg::VRTSgab731::Linux;
@Pkg::VRTSgab731::Linux::ISA = qw(Pkg::VRTSgab731::Common);

package Pkg::VRTSgab731::RHEL6x8664;
@Pkg::VRTSgab731::RHEL6x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::RHEL7x8664;
@Pkg::VRTSgab731::RHEL7x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::OL6x8664;
@Pkg::VRTSgab731::OL6x8664::ISA = qw(Pkg::VRTSgab731::RHEL6x8664);

package Pkg::VRTSgab731::OL7x8664;
@Pkg::VRTSgab731::OL7x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::CentOS6x8664;
@Pkg::VRTSgab731::CentOS6x8664::ISA = qw(Pkg::VRTSgab731::RHEL6x8664);

package Pkg::VRTSgab731::CentOS7x8664;
@Pkg::VRTSgab731::CentOS7x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::SLES11x8664;
@Pkg::VRTSgab731::SLES11x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::SLES12x8664;
@Pkg::VRTSgab731::SLES12x8664::ISA = qw(Pkg::VRTSgab731::Linux);

package Pkg::VRTSgab731::SunOS;
@Pkg::VRTSgab731::SunOS::ISA = qw(Pkg::VRTSgab731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{smf}=['/var/svc/manifest/system/gab.xml', '/lib/svc/method/gab'];
    return;
}

sub preremove_sys {
    my ($pkg, $sys) = (@_);
    my $vcs = $pkg->prod('VCS731');
    $vcs->backup_smf_scripts_sys($sys,$pkg);
    return;
}

sub postremove_sys {
    my ($pkg, $sys) = (@_);
    my $vcs = $pkg->prod('VCS731');
    $vcs->restore_smf_scripts_sys($sys,$pkg);
    return;
}

1;
