#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSgms731::Common;
@Pkg::VRTSgms731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSgms';
    $pkg->{name}=Msg::new("Group Messaging Services")->{msg};

    $pkg->{startprocs}=[ qw(vxgms731) ];
    $pkg->{stopprocs}=[ qw(vxgms731) ];
    $pkg->{initconf}='/etc/vx/gms.config';
    return;
}

sub preinstall_sys {
    my ($pkg,$sys)=@_;
    my ($conf,$dir,$file,$rootpath);

    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $pkg->{initconf};
    if ($sys->exists("$file")) {
        $conf=$sys->catfile($file);
        if ($conf =~ /GMS_START/mx) {
            $conf =~ s/GMS_START\s*=\s*1/GMS_START=0/mx;
            $sys->movefile($file,"$file.prev");
            $sys->writefile($conf,$file);
            if ($sys->exists("$file")) {
                $sys->rm("$file.prev");
            }
        } else {
            $sys->appendfile('GMS_START=0', $file);
        }
    } else {
        $dir=EDRu::dirname($file);
        $sys->cmd("_cmd_mkdir -p $dir");
        $sys->cmd("echo 'GMS_START=0' > $file 2>/dev/null");
    }
    
    unless($rootpath) {
        if ($sys->{padv} =~ /Sol11/) {
            $sys->cmd('pkg change-facet --no-backup-be facet.vxgms.start=False');
        }
    }
    return;
}

sub postremove_sys {
    my ($pkg, $sys) = (@_);
    $sys->cmd("_cmd_rm $pkg->{initconf}");
    return;
}

package Pkg::VRTSgms731::AIX;
@Pkg::VRTSgms731::AIX::ISA = qw(Pkg::VRTSgms731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{donotcheckinstalllog}=1;
    return;
}

package Pkg::VRTSgms731::HPUX;
@Pkg::VRTSgms731::HPUX::ISA = qw(Pkg::VRTSgms731::Common);

package Pkg::VRTSgms731::Linux;
@Pkg::VRTSgms731::Linux::ISA = qw(Pkg::VRTSgms731::Common);

package Pkg::VRTSgms731::RHEL6x8664;
@Pkg::VRTSgms731::RHEL6x8664::ISA = qw(Pkg::VRTSgms731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"rtld(GNU_HASH)" => "glibc.x86_64", 
	};
	return;
}

package Pkg::VRTSgms731::RHEL7x8664;
@Pkg::VRTSgms731::RHEL7x8664::ISA = qw(Pkg::VRTSgms731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"rtld(GNU_HASH)" => "glibc.x86_64",
	};
	return;
}

package Pkg::VRTSgms731::OL6x8664;
@Pkg::VRTSgms731::OL6x8664::ISA = qw(Pkg::VRTSgms731::RHEL6x8664);

package Pkg::VRTSgms731::OL7x8664;
@Pkg::VRTSgms731::OL7x8664::ISA = qw(Pkg::VRTSgms731::RHEL7x8664);

package Pkg::VRTSgms731::CentOS6x8664;
@Pkg::VRTSgms731::CentOS6x8664::ISA = qw(Pkg::VRTSgms731::RHEL6x8664);

package Pkg::VRTSgms731::CentOS7x8664;
@Pkg::VRTSgms731::CentOS7x8664::ISA = qw(Pkg::VRTSgms731::RHEL7x8664);

package Pkg::VRTSgms731::SLES11x8664;
@Pkg::VRTSgms731::SLES11x8664::ISA = qw(Pkg::VRTSgms731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
	};
    return;
}

package Pkg::VRTSgms731::SLES12x8664;
@Pkg::VRTSgms731::SLES12x8664::ISA = qw(Pkg::VRTSgms731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
	};
    return;
}

package Pkg::VRTSgms731::SunOS;
@Pkg::VRTSgms731::SunOS::ISA = qw(Pkg::VRTSgms731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{donotrmonupgrade}=0;
    return;
}

1;
