#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSob34::Common;
@Pkg::VRTSob34::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSob';
    $pkg->{name}=Msg::new("Enterprise Administrator Service")->{msg};
    $pkg->{stopprocs}=[ qw(vxsvc34) ];
    $pkg->{startprocs}=[ qw(vxsvc34) ];
    $pkg->{gavers}='3.4.700.0';
    return;
}

package Pkg::VRTSob34::AIX;
@Pkg::VRTSob34::AIX::ISA = qw(Pkg::VRTSob34::Common);

package Pkg::VRTSob34::HPUX;
@Pkg::VRTSob34::HPUX::ISA = qw(Pkg::VRTSob34::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{donotrmonupgrade}=1;
    $pkg->{ospatches}{'11.31IA'}=['PHSS_39898'];
    return;
}

package Pkg::VRTSob34::Linux;
@Pkg::VRTSob34::Linux::ISA = qw(Pkg::VRTSob34::Common);

package Pkg::VRTSob34::RHEL6x8664;
@Pkg::VRTSob34::RHEL6x8664::ISA = qw(Pkg::VRTSob34::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{'RHEL6.4'}={
        "libcrypt.so.1"  =>  "glibc-2.12-1.107.el6.i686",
        "libcrypt.so.1(GLIBC_2.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GCC_3.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.1)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.1.2)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.1.3)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.2)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.3)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.3.2)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.3.3)"  =>  "glibc-2.12-1.107.el6.i686",
        "libc.so.6(GLIBC_2.3.4)"  =>  "glibc-2.12-1.107.el6.i686",
        "libdl.so.2"  =>  "glibc-2.12-1.107.el6.i686",
        "libdl.so.2(GLIBC_2.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libdl.so.2(GLIBC_2.1)"  =>  "glibc-2.12-1.107.el6.i686",
        "libgcc_s.so.1"  =>  "libgcc-4.4.7-3.el6.i686",
        "libgcc_s.so.1(GCC_3.0)"  =>  "libgcc-4.4.7-3.el6.i686",
        "libgcc_s.so.1(GCC_3.3)"  =>  "libgcc-4.4.7-3.el6.i686",
        "libm.so.6"  =>  "glibc-2.12-1.107.el6.i686",
        "libm.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libnsl.so.1"  =>  "glibc-2.12-1.107.el6.i686",
        "libpam.so.0"  =>  "pam-1.1.1-13.el6.i686",
        "libpthread.so.0"  =>  "glibc-2.12-1.107.el6.i686",
        "libpthread.so.0(GLIBC_2.0)"  =>  "glibc-2.12-1.107.el6.i686",
        "libpthread.so.0(GLIBC_2.1)"  =>  "glibc-2.12-1.107.el6.i686",
        "libpthread.so.0(GLIBC_2.2)"  =>  "glibc-2.12-1.107.el6.i686",
        "libpthread.so.0(GLIBC_2.3.2)"  =>  "glibc-2.12-1.107.el6.i686",
        "libresolv.so.2"  =>  "glibc-2.12-1.107.el6.i686",
        "librt.so.1"  =>  "glibc-2.12-1.107.el6.i686",
        "libstdc++.so.6"  =>  "libstdc++-4.4.7-3.el6.i686",
        "libstdc++.so.6(CXXABI_1.3)"  =>  "libstdc++-4.4.7-3.el6.i686",
        "libstdc++.so.6(GLIBCXX_3.4)"  =>  "libstdc++-4.4.7-3.el6.i686",
    };
    $pkg->{oslibs}{'RHEL6.5'}={
        "libcrypt.so.1"  =>  "glibc-2.12-1.132.el6.i686",
        "libcrypt.so.1(GLIBC_2.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GCC_3.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.1)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.1.2)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.1.3)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.2)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.3)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.3.2)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.3.3)"  =>  "glibc-2.12-1.132.el6.i686",
        "libc.so.6(GLIBC_2.3.4)"  =>  "glibc-2.12-1.132.el6.i686",
        "libdl.so.2"  =>  "glibc-2.12-1.132.el6.i686",
        "libdl.so.2(GLIBC_2.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libdl.so.2(GLIBC_2.1)"  =>  "glibc-2.12-1.132.el6.i686",
        "libgcc_s.so.1"  =>  "libgcc-4.4.7-4.el6.i686",
        "libgcc_s.so.1(GCC_3.0)"  =>  "libgcc-4.4.7-4.el6.i686",
        "libgcc_s.so.1(GCC_3.3)"  =>  "libgcc-4.4.7-4.el6.i686",
        "libm.so.6"  =>  "glibc-2.12-1.132.el6.i686",
        "libm.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libnsl.so.1"  =>  "glibc-2.12-1.132.el6.i686",
        "libpam.so.0"  =>  "pam-1.1.1-17.el6.i686",
        "libpthread.so.0"  =>  "glibc-2.12-1.132.el6.i686",
        "libpthread.so.0(GLIBC_2.0)"  =>  "glibc-2.12-1.132.el6.i686",
        "libpthread.so.0(GLIBC_2.1)"  =>  "glibc-2.12-1.132.el6.i686",
        "libpthread.so.0(GLIBC_2.2)"  =>  "glibc-2.12-1.132.el6.i686",
        "libpthread.so.0(GLIBC_2.3.2)"  =>  "glibc-2.12-1.132.el6.i686",
        "libresolv.so.2"  =>  "glibc-2.12-1.132.el6.i686",
        "librt.so.1"  =>  "glibc-2.12-1.132.el6.i686",
        "libstdc++.so.6"  =>  "libstdc++-4.4.7-4.el6.i686",
        "libstdc++.so.6(CXXABI_1.3)"  =>  "libstdc++-4.4.7-4.el6.i686",
        "libstdc++.so.6(GLIBCXX_3.4)"  =>  "libstdc++-4.4.7-4.el6.i686",
    };
    $pkg->{oslibs}{'RHEL6.6'}={
        "libcrypt.so.1"  =>  "glibc-2.12-1.149.el6.i686",
        "libcrypt.so.1(GLIBC_2.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GCC_3.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.1)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.1.2)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.1.3)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.2)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.3)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.3.2)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.3.3)"  =>  "glibc-2.12-1.149.el6.i686",
        "libc.so.6(GLIBC_2.3.4)"  =>  "glibc-2.12-1.149.el6.i686",
        "libdl.so.2"  =>  "glibc-2.12-1.149.el6.i686",
        "libdl.so.2(GLIBC_2.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libdl.so.2(GLIBC_2.1)"  =>  "glibc-2.12-1.149.el6.i686",
        "libgcc_s.so.1"  =>  "libgcc-4.4.7-11.el6.i686",
        "libgcc_s.so.1(GCC_3.0)"  =>  "libgcc-4.4.7-11.el6.i686",
        "libgcc_s.so.1(GCC_3.3)"  =>  "libgcc-4.4.7-11.el6.i686",
        "libm.so.6"  =>  "glibc-2.12-1.149.el6.i686",
        "libm.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libnsl.so.1"  =>  "glibc-2.12-1.149.el6.i686",
        "libpam.so.0"  =>  "pam-1.1.1-20.el6.i686",
        "libpthread.so.0"  =>  "glibc-2.12-1.149.el6.i686",
        "libpthread.so.0(GLIBC_2.0)"  =>  "glibc-2.12-1.149.el6.i686",
        "libpthread.so.0(GLIBC_2.1)"  =>  "glibc-2.12-1.149.el6.i686",
        "libpthread.so.0(GLIBC_2.2)"  =>  "glibc-2.12-1.149.el6.i686",
        "libpthread.so.0(GLIBC_2.3.2)"  =>  "glibc-2.12-1.149.el6.i686",
        "libresolv.so.2"  =>  "glibc-2.12-1.149.el6.i686",
        "librt.so.1"  =>  "glibc-2.12-1.149.el6.i686",
        "libstdc++.so.6"  =>  "libstdc++-4.4.7-11.el6.i686",
        "libstdc++.so.6(CXXABI_1.3)"  =>  "libstdc++-4.4.7-11.el6.i686",
        "libstdc++.so.6(GLIBCXX_3.4)"  =>  "libstdc++-4.4.7-11.el6.i686",
    };
    $pkg->{oslibs}{'RHEL6.7'}={
        "libcrypt.so.1"  =>  "glibc-2.12-1.166.el6.i686",
        "libcrypt.so.1(GLIBC_2.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GCC_3.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.1)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.1.2)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.1.3)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.2)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.3)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.3.2)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.3.3)"  =>  "glibc-2.12-1.166.el6.i686",
        "libc.so.6(GLIBC_2.3.4)"  =>  "glibc-2.12-1.166.el6.i686",
        "libdl.so.2"  =>  "glibc-2.12-1.166.el6.i686",
        "libdl.so.2(GLIBC_2.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libdl.so.2(GLIBC_2.1)"  =>  "glibc-2.12-1.166.el6.i686",
        "libgcc_s.so.1"  =>  "libgcc-4.4.7-16.el6.i686",
        "libgcc_s.so.1(GCC_3.0)"  =>  "libgcc-4.4.7-16.el6.i686",
        "libgcc_s.so.1(GCC_3.3)"  =>  "libgcc-4.4.7-16.el6.i686",
        "libm.so.6"  =>  "glibc-2.12-1.166.el6.i686",
        "libm.so.6(GLIBC_2.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libnsl.so.1"  =>  "glibc-2.12-1.166.el6.i686",
        "libpam.so.0"  =>  "pam-1.1.1-20.el6.i686",
        "libpthread.so.0"  =>  "glibc-2.12-1.166.el6.i686",
        "libpthread.so.0(GLIBC_2.0)"  =>  "glibc-2.12-1.166.el6.i686",
        "libpthread.so.0(GLIBC_2.1)"  =>  "glibc-2.12-1.166.el6.i686",
        "libpthread.so.0(GLIBC_2.2)"  =>  "glibc-2.12-1.166.el6.i686",
        "libpthread.so.0(GLIBC_2.3.2)"  =>  "glibc-2.12-1.166.el6.i686",
        "libresolv.so.2"  =>  "glibc-2.12-1.166.el6.i686",
        "librt.so.1"  =>  "glibc-2.12-1.166.el6.i686",
        "libstdc++.so.6"  =>  "libstdc++-4.4.7-16.el6.i686",
        "libstdc++.so.6(CXXABI_1.3)"  =>  "libstdc++-4.4.7-16.el6.i686",
        "libstdc++.so.6(GLIBCXX_3.4)"  =>  "libstdc++-4.4.7-16.el6.i686",
    };
    for my $minorvers (keys %{$pkg->{oslibs}}) {
        if ($minorvers=~/RHEL(.*)/) {
            $pkg->{oslibs}{"OL$1"} = { %{$pkg->{oslibs}{$minorvers}} };
        }
    }
    return;
}

package Pkg::VRTSob34::RHEL7x8664;
@Pkg::VRTSob34::RHEL7x8664::ISA = qw(Pkg::VRTSob34::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{'RHEL7.0'}={
        "libcrypt.so.1"  =>  "glibc-2.17-55.el7.i686",
        "libcrypt.so.1(GLIBC_2.0)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GCC_3.0)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.0)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.1)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.1.2)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.1.3)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.2)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.3)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.3.2)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.3.3)"  =>  "glibc-2.17-55.el7.i686",
        "libc.so.6(GLIBC_2.3.4)"  =>  "glibc-2.17-55.el7.i686",
        "libdl.so.2"  =>  "glibc-2.17-55.el7.i686",
        "libdl.so.2(GLIBC_2.0)"  =>  "glibc-2.17-55.el7.i686",
        "libdl.so.2(GLIBC_2.1)"  =>  "glibc-2.17-55.el7.i686",
        "libgcc_s.so.1"  =>  "libgcc-4.8.2-16.el7.i686",
        "libgcc_s.so.1(GCC_3.0)"  =>  "libgcc-4.8.2-16.el7.i686",
        "libgcc_s.so.1(GCC_3.3)"  =>  "libgcc-4.8.2-16.el7.i686",
        "libm.so.6"  =>  "glibc-2.17-55.el7.i686",
        "libm.so.6(GLIBC_2.0)"  =>  "glibc-2.17-55.el7.i686",
        "libnsl.so.1"  =>  "glibc-2.17-55.el7.i686",
        "libpam.so.0"  =>  "pam-1.1.8-9.el7.i686",
        "libpthread.so.0"  =>  "glibc-2.17-55.el7.i686",
        "libpthread.so.0(GLIBC_2.0)"  =>  "glibc-2.17-55.el7.i686",
        "libpthread.so.0(GLIBC_2.1)"  =>  "glibc-2.17-55.el7.i686",
        "libpthread.so.0(GLIBC_2.2)"  =>  "glibc-2.17-55.el7.i686",
        "libpthread.so.0(GLIBC_2.3.2)"  =>  "glibc-2.17-55.el7.i686",
        "libresolv.so.2"  =>  "glibc-2.17-55.el7.i686",
        "librt.so.1"  =>  "glibc-2.17-55.el7.i686",
        "libstdc++.so.6"  =>  "libstdc++-4.8.2-16.el7.i686",
        "libstdc++.so.6(CXXABI_1.3)"  =>  "libstdc++-4.8.2-16.el7.i686",
        "libstdc++.so.6(GLIBCXX_3.4)"  =>  "libstdc++-4.8.2-16.el7.i686",
    };

    $pkg->{oslibs}{'RHEL7.1'}={
    };

    for my $minorvers (keys %{$pkg->{oslibs}}) {
        if ($minorvers=~/RHEL(.*)/) {
            $pkg->{oslibs}{"OL$1"} = { %{$pkg->{oslibs}{$minorvers}} };
        }
    }
    return;
}

package Pkg::VRTSob34::OL6x8664;
@Pkg::VRTSob34::OL6x8664::ISA = qw(Pkg::VRTSob34::RHEL6x8664);

package Pkg::VRTSob34::OL7x8664;
@Pkg::VRTSob34::OL7x8664::ISA = qw(Pkg::VRTSob34::RHEL7x8664);

package Pkg::VRTSob34::CentOS6x8664;
@Pkg::VRTSob34::CentOS6x8664::ISA = qw(Pkg::VRTSob34::RHEL6x8664);

package Pkg::VRTSob34::CentOS7x8664;
@Pkg::VRTSob34::CentOS7x8664::ISA = qw(Pkg::VRTSob34::RHEL7x8664);

package Pkg::VRTSob34::SunOS;
@Pkg::VRTSob34::SunOS::ISA = qw(Pkg::VRTSob34::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{responsefile}="SECURITYADMINPASSWORD=\nROOTAGENTPASSWORD=\nCONFIGURESECURITY=n\n";
    $pkg->{languagepkgs}{mu}=[qw(VRTSmuob)];
    return;
}

1;
