#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSodm731::Common;
@Pkg::VRTSodm731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSodm';
    $pkg->{name}=Msg::new("Oracle Disk Manager")->{msg};

    $pkg->{startprocs}=[ qw(odm731) ];
    $pkg->{stopprocs}=[ qw(odm731) ];
    $pkg->{initconf}='/etc/vx/odm.config';
    return;
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;
    my ($conf,$dir,$file,$rootpath,$proc);

    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $pkg->{initconf};
    if ($sys->exists("$file")) {
        $conf=$sys->catfile($file);
        if ($conf =~ /ODM_START/mx) {
            $conf =~ s/ODM_START\s*=\s*1/ODM_START=0/mx;
            $sys->movefile($file,"$file.prev");
            $sys->writefile($conf,$file);
            if ($sys->exists("$file")) {
                $sys->rm("$file.prev");
            }
        } else {
            $sys->appendfile('ODM_START=0', $file);
        }
    } else {
        $dir=EDRu::dirname($file);
        $sys->cmd("_cmd_mkdir -p $dir");
        $sys->cmd("echo 'ODM_START=0' > $file 2>/dev/null");
    }

    unless($rootpath){
        #Package will start odm after installation.
        $proc = $pkg->proc('odm731');
        $proc->stop_sys($sys);
    }
    return;
}

sub postremove_sys {
    my ($pkg, $sys) = (@_);
    
    #Packages should be removed for uninstall.
    if (Cfg::opt('uninstall')) {
        $sys->cmd("_cmd_rm $pkg->{initconf}");
    }
    return;
}

package Pkg::VRTSodm731::AIX;
@Pkg::VRTSodm731::AIX::ISA = qw(Pkg::VRTSodm731::Common);

package Pkg::VRTSodm731::HPUX;
@Pkg::VRTSodm731::HPUX::ISA = qw(Pkg::VRTSodm731::Common);

package Pkg::VRTSodm731::Linux;
@Pkg::VRTSodm731::Linux::ISA = qw(Pkg::VRTSodm731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{previouspkgnames}=[ qw(VRTSodm-platform VRTSodm-common) ];

    return;
}

package Pkg::VRTSodm731::RHEL6x8664;
@Pkg::VRTSodm731::RHEL6x8664::ISA = qw(Pkg::VRTSodm731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "module-init-tools.x86_64",
		"rtld(GNU_HASH)" => "glibc.x86_64", 
	};
	return;
}

package Pkg::VRTSodm731::RHEL7x8664;
@Pkg::VRTSodm731::RHEL7x8664::ISA = qw(Pkg::VRTSodm731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"kmod" => "kmod.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"rtld(GNU_HASH)" => "glibc.x86_64", 
	};
	return;
}

package Pkg::VRTSodm731::OL6x8664;
@Pkg::VRTSodm731::OL6x8664::ISA = qw(Pkg::VRTSodm731::RHEL6x8664);

package Pkg::VRTSodm731::OL7x8664;
@Pkg::VRTSodm731::OL7x8664::ISA = qw(Pkg::VRTSodm731::RHEL7x8664);

package Pkg::VRTSodm731::CentOS6x8664;
@Pkg::VRTSodm731::CentOS6x8664::ISA = qw(Pkg::VRTSodm731::RHEL6x8664);

package Pkg::VRTSodm731::CentOS7x8664;
@Pkg::VRTSodm731::CentOS7x8664::ISA = qw(Pkg::VRTSodm731::RHEL7x8664);

package Pkg::VRTSodm731::SLES11x8664;
@Pkg::VRTSodm731::SLES11x8664::ISA = qw(Pkg::VRTSodm731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "module-init-tools.x86_64",
	};
    return;
}

package Pkg::VRTSodm731::SLES12x8664;
@Pkg::VRTSodm731::SLES12x8664::ISA = qw(Pkg::VRTSodm731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "kmod-compat.x86_64",
	};
	return;
}

package Pkg::VRTSodm731::SunOS;
@Pkg::VRTSodm731::SunOS::ISA = qw(Pkg::VRTSodm731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{donotrmonupgrade}=0;
    return;
}

1;
