#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSsfcpi731::Common;
@Pkg::VRTSsfcpi731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSsfcpi';
    $pkg->{name}=Msg::new("Storage Foundation Installer")->{msg};
    $pkg->{installpkgslast}=1;
    $pkg->{installpatcheslast}=1;
	$pkg->{donotrmonupgrade}=1;
    return;
}

# Add dependency for other products only when uninstall VRTSsfcpi
sub query_pkgdep_sys {
    my ($pkg,$sys)=@_;
    my ($vxvm,$vxfs,$vcs);
    my (@softdeps,$rel,$pkgver);
    $rel=Obj::rel();
    $pkgver=$pkg->version_sys($sys);

    if (Cfg::opt('uninstall')) {
        $vxvm = $sys->pkg('VRTSvxvm731') if (EDRu::inarr("VM731", @{$rel->{prods}}));
        $vxfs = $sys->pkg('VRTSvxfs731') if (EDRu::inarr("FS731", @{$rel->{prods}}));
        $vcs = $sys->pkg('VRTSvcs731') if (EDRu::inarr("VCS731", @{$rel->{prods}}));
        # Only need to compare base version.
        push (@softdeps, $vxvm->{pkg}) if ($vxvm && EDRu::compvers($vxvm->version_sys($sys),$pkgver,2)==0);
        push (@softdeps, $vxfs->{pkg}) if ($vxfs && EDRu::compvers($vxfs->version_sys($sys),$pkgver,2)==0);
        push (@softdeps, $vcs->{pkg})  if ($vcs && EDRu::compvers($vcs->version_sys($sys),$pkgver,2)==0);
        return \@softdeps;
    }
    return [];
}

# Deal with situations that won't uninstall old VRTSsfcpi package
sub donotuninstall_sys {
	return 0 unless (Cfg::opt('upgrade'));
    return 1;
}

# Deal with VRTSsfcpi packages that should be uninstalled during upgrade
sub set_sfcpi_upgrade_sys {
    my ($pkg,$sys) = @_;
    my ($cpic, $activecomponent, $prod, $prodver, $rel);
    my ($pkgver,$oldcpipkg);
    $cpic = Obj::cpic();
    $rel = Obj::rel();
    return 1 unless (Cfg::opt(qw/upgrade/));
    # Partial upgrade will not uninstall old sfcpi package
    return 1 if ($cpic->{'partial_upgrade'});
    $activecomponent = $rel->get_activecomponent();
    $prod = $cpic->rel->prod($activecomponent);
    $prodver = $prod->{vers};
    for my $oldpkg (keys%{$sys->{pkgvers}}) {
        if ($oldpkg =~ /VRTSsfcpi/ && ($oldpkg ne 'VRTSsfcpi')) {
            $oldcpipkg=Pkg->new_pkg($oldpkg, $cpic->{padv});
            $oldcpipkg=Pkg->new_pkg($oldpkg, $sys->{padv});
            $oldcpipkg->add_uninstall_sys($sys);
            push (@{$cpic->{uninstallpkgs}},$oldpkg) unless (EDRu::inarr($oldpkg,@{$cpic->{uninstallpkgs}}));
        }
    }
    return 1;
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;
    my $rootpath = Cfg::opt('rootpath');
    my $cpic = Obj::cpic();
    my @files;
    my $scriptfile="$rootpath/opt/VRTS/install/bin/UXRT731/add_install_scripts";
    my $prod=CPIC::get('prod');
    if(Cfg::opt("upgrade_kernelpkgs")){
        if($sys->exists($scriptfile,$sys)){
            Msg::log("Creating install scripts");
            $sys->cmd("CALLED_BY=CPI $scriptfile force $prod 2>/dev/null");
        }
    }

    if ($cpic->{'partial_upgrade'}) {
        my $binpath = "$rootpath/opt/VRTS/install/bin";
        my $files = $sys->cmd("_cmd_ls $binpath | grep -v UXRT731");
        @files = split(/\s+/m, $files);
        for my $oldscriptpath (@files) {
            $sys->cmd("CALLED_BY=CPI $binpath/$oldscriptpath/add_install_scripts delete 2>/dev/null");
            $sys->cmd("CALLED_BY=CPI $binpath/$oldscriptpath/add_install_scripts 2>/dev/null");
        }   
    }   
        
    return;
}

package Pkg::VRTSsfcpi731::AIX;
@Pkg::VRTSsfcpi731::AIX::ISA = qw(Pkg::VRTSsfcpi731::Common);

package Pkg::VRTSsfcpi731::HPUX;
@Pkg::VRTSsfcpi731::HPUX::ISA = qw(Pkg::VRTSsfcpi731::Common);

package Pkg::VRTSsfcpi731::Linux;
@Pkg::VRTSsfcpi731::Linux::ISA = qw(Pkg::VRTSsfcpi731::Common);

package Pkg::VRTSsfcpi731::SunOS;
@Pkg::VRTSsfcpi731::SunOS::ISA = qw(Pkg::VRTSsfcpi731::Common);

sub preremove_sys {
    my ($pkg,$sys)=@_;
    my $rootpath = Cfg::opt('rootpath');
    my $scriptfile="$rootpath/opt/VRTS/install/bin/UXRT731/add_install_scripts";
    $sys->cmd("CALLED_BY=CPI $scriptfile delete 2>/dev/null");
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;
    my $rootpath = Cfg::opt('rootpath');
    my $scriptfile="$rootpath/opt/VRTSsfcpi/bin/run-once";
    if ($sys->exists($scriptfile)) {
        Msg::log("Running postinstall script of $pkg->{pkg}");
        $sys->cmd("BASEDIR=$rootpath VRTSSFCPISILENT=1 $scriptfile 2>/dev/null");
    }
    return $pkg->SUPER::postinstall_sys($sys);
}

sub postremove_sys {
	my ($pkg,$sys)=@_;
	my $cpic = Obj::cpic();
	my $rootpath = Cfg::opt('rootpath');
	my @files;
	my $uninstalled_scripts="$rootpath/opt/VRTS/install/bin/UXRT731";

#for Sol11, CPI need remove add_install_scripts for current VRTSsfpi manully.
	$sys->cmd("_cmd_rmr $uninstalled_scripts");
#
#        #need refresh local scripts if there is any other product installed
	my $binpath = "$rootpath/opt/VRTS/install/bin";
	my $files = $sys->cmd("_cmd_ls $binpath");
	@files = split(/\s+/m, $files);
	for my $scriptpath (@files) {
		$sys->cmd("CALLED_BY=CPI $binpath/$scriptpath/add_install_scripts delete 2>/dev/null");
		$sys->cmd("CALLED_BY=CPI $binpath/$scriptpath/add_install_scripts 2>/dev/null");
	}
	return $pkg->SUPER::postremove_sys($sys);
}

1;
