#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSsfmh731::Common;
@Pkg::VRTSsfmh731::Common::ISA = qw(Pkg);

use JSON;

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSsfmh';
    $pkg->{name}=Msg::new("Storage Foundation Managed Host")->{msg};
    # sfmh-discovery.pl needs to be stopped
    # since it restarts vxnotify
    $pkg->{stopprocs}=[ qw(sfmhdiscovery731 vxdclid731) ];
    $pkg->{startprocs}=[ qw(sfmhdiscovery731 xprtld731) ];
    $pkg->{installpkgslast}=1;
    $pkg->{installpatcheslast}=1;
    $pkg->{donotrmonupgrade}=1;
    # TODO: need update with last build.
    $pkg->{gavers}='7.1.0.0';

    return;
}

sub check_if_managedhost_connected_to_central_server_sys {
    my ($pkg,$sys)=@_;
    my ($rtn,$jsonobj,@mgmt_servers,$mgmt_server);

    @mgmt_servers=();
    $rtn=$sys->cmd("/opt/VRTSsfmh/bin/xprtlc -l https://localhost:5634/world/getvitals 2>/dev/null");
    if (!EDR::cmdexit()) {
        # xprtld is running on the system
        $jsonobj=JSON::from_json($rtn);
        if (defined $jsonobj->{DOMAINS}) {
            for my $mgmt_server (keys %{$jsonobj->{DOMAINS}}) {
                $mgmt_server=~s/.*:\/\/(.*):\d+.*$/$1/m;
                push (@mgmt_servers, $mgmt_server);
            }
        }
    } else {
        # xprtld is not running on the system, checking '/etc/default/sfm_resolv.conf'
        $rtn=$sys->catfile("/etc/default/sfm_resolv.conf");
        for my $line (split(/\n/, $rtn)) {
            if ($line=~m{ \[(.*)\] }mxs) {
                $mgmt_server=$1;
                $mgmt_server=~s/.*:\/\/(.*):\d+.*$/$1/m;
                push (@mgmt_servers, $mgmt_server) if ($mgmt_server ne 'domains');
            }
        }
    }
    return join (' ', @mgmt_servers);
}

#Add dependency for other products only when uninstall VRTSperl
sub query_pkgdep_sys {
    my ($pkg,$sys)=@_;
    my ($vxvm,$vxfs,$vcs,@softdeps,$rel);
    $rel=Obj::rel();
    if (Cfg::opt('uninstall')) {
	$vxvm = $sys->pkg('VRTSvxvm731') if (EDRu::inarr("VM731",@{$rel->{prods}}));
	$vxfs = $sys->pkg('VRTSvxfs731') if (EDRu::inarr("FS731",@{$rel->{prods}}));
	$vcs = $sys->pkg('VRTSvcs731') if (EDRu::inarr("VCS731",@{$rel->{prods}}));
        push (@softdeps, $vxvm->{pkg}) if ($vxvm && $vxvm->version_sys($sys));
        push (@softdeps, $vxfs->{pkg}) if ($vxfs && $vxfs->version_sys($sys));
        push (@softdeps, $vcs->{pkg}) if ($vcs && $vcs->version_sys($sys));
        return \@softdeps;
    }
    return [];
}

# Check the remote CMS version based on the xprtld version
sub remote_vomcs_version_sys {
    my ($pkg,$sys,$remoteSys) = @_;
    my $rtn=$sys->cmd("/opt/VRTSsfmh/bin/xprtlc -l https://$remoteSys:5634/world/getvitals 2>/dev/null");
    if (!EDR::cmdexit()) {
        # xprtld is running on the system
        my $jsonobj=JSON::from_json($rtn);
        if (defined $jsonobj->{XPRTLD_VERSION}) {
            return $jsonobj->{XPRTLD_VERSION};
        }
    }
    return '';
}

#e3712926:Stop vxdclid process after fresh installation
sub postinstall_sys {
    my ($pkg,$sys)=@_;
    my ($vxdclid);
    $vxdclid = $sys->proc('vxdclid731');
    if ($vxdclid->check_sys($sys)) {
        $sys->cmd("/opt/VRTSsfmh/etc/vxdcli.sh stop"); 
    }
    return;
}

package Pkg::VRTSsfmh731::AIX;
@Pkg::VRTSsfmh731::AIX::ISA = qw(Pkg::VRTSsfmh731::Common);

package Pkg::VRTSsfmh731::HPUX;
@Pkg::VRTSsfmh731::HPUX::ISA = qw(Pkg::VRTSsfmh731::Common);

package Pkg::VRTSsfmh731::Linux;
@Pkg::VRTSsfmh731::Linux::ISA = qw(Pkg::VRTSsfmh731::Common);

package Pkg::VRTSsfmh731::SunOS;
@Pkg::VRTSsfmh731::SunOS::ISA = qw(Pkg::VRTSsfmh731::Common);

1;
