#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSvcs731::Common;
@Pkg::VRTSvcs731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSvcs';
    $pkg->{name}=Msg::new("Cluster Server")->{msg};
    $pkg->{startprocs}=[ qw(had731 CmdServer731) ];
    $pkg->{stopprocs}=[ qw(had731 CmdServer731) ];
    $pkg->{autoremovedependentpkgs}=1;
    $pkg->{unkernelpkg}=1;
    return;
}

package Pkg::VRTSvcs731::AIX;
@Pkg::VRTSvcs731::AIX::ISA = qw(Pkg::VRTSvcs731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{previouspkgnames}=[ qw(VRTSvcs.rte) ];
    return;
}

package Pkg::VRTSvcs731::HPUX;
@Pkg::VRTSvcs731::HPUX::ISA = qw(Pkg::VRTSvcs731::Common);

sub define_ospatches {
    my ($pkg,$padv,$sys) = @_;

    # we need to check PHCO_43449 only if HPUX OS version is 11.31.1303
    if (EDRu::compvers($sys->{fusionversion}, '11.31.1303') < 2) {
        push(@{$pkg->{ospatches}{"11.31IA"}},"PHCO_43449");
        push(@{$pkg->{ospatches}{"11.31PA"}},"PHCO_43449");
    }
}

package Pkg::VRTSvcs731::Linux;
@Pkg::VRTSvcs731::Linux::ISA = qw(Pkg::VRTSvcs731::Common);

package Pkg::VRTSvcs731::RHEL6x8664;
@Pkg::VRTSvcs731::RHEL6x8664::ISA = qw(Pkg::VRTSvcs731::Linux);

sub init_padv {               
    my $pkg=shift;            
    $pkg->{oslibs}{all}={              
        "/bin/ksh"  =>  "ksh.x86_64",   
    }; 
    return;                   
}

package Pkg::VRTSvcs731::RHEL7x8664;
@Pkg::VRTSvcs731::RHEL7x8664::ISA = qw(Pkg::VRTSvcs731::Linux);

sub init_padv {
    my $pkg=shift;            
    $pkg->{oslibs}{all}={              
       "/bin/ksh"  =>  "ksh.x86_64",
    }; 
    return;     
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;
    $pkg->SUPER::postinstall_sys($sys);
    $sys->padv->systemd_reload($sys);
    return;
}

package Pkg::VRTSvcs731::OL6x8664;
@Pkg::VRTSvcs731::OL6x8664::ISA = qw(Pkg::VRTSvcs731::RHEL6x8664);

package Pkg::VRTSvcs731::OL7x8664;
@Pkg::VRTSvcs731::OL7x8664::ISA = qw(Pkg::VRTSvcs731::RHEL7x8664);

package Pkg::VRTSvcs731::CentOS6x8664;
@Pkg::VRTSvcs731::CentOS6x8664::ISA = qw(Pkg::VRTSvcs731::RHEL6x8664);

package Pkg::VRTSvcs731::CentOS7x8664;
@Pkg::VRTSvcs731::CentOS7x8664::ISA = qw(Pkg::VRTSvcs731::RHEL7x8664);

package Pkg::VRTSvcs731::SLES11x8664;
@Pkg::VRTSvcs731::SLES11x8664::ISA = qw(Pkg::VRTSvcs731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
        "/bin/ksh"  =>  "ksh.x86_64",
    };
    return;
}

package Pkg::VRTSvcs731::SLES12x8664;
@Pkg::VRTSvcs731::SLES12x8664::ISA = qw(Pkg::VRTSvcs731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
        "/bin/ksh"  =>  "mksh.x86_64",
    };
    return;
}

package Pkg::VRTSvcs731::SunOS;
@Pkg::VRTSvcs731::SunOS::ISA = qw(Pkg::VRTSvcs731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{ospkgs}{all}=[ 'resource-pools' ];
    return;
}

# workaround of 1638382, remove the preremove script of obsolete pkg
sub preremove_sys {
    my ($pkg,$sys) = @_;
    my ($rootpath,$vers,$preremove_script);

    $vers=$sys->padv->pkg_version_sys($sys, $pkg);
    if (Cfg::opt('rootpath') && ($vers eq '5.0')) {
        $rootpath = Cfg::opt('rootpath');
        $preremove_script="$rootpath/var/sadm/pkg/VRTSvcs/install/preremove";
        $sys->cmd("_cmd_rm $preremove_script");
    }
    my $vcs = $pkg->prod('VCS731');
    $vcs->backup_smf_scripts_sys($sys,$pkg);
    return 1;
}

sub postremove_sys {
    my ($pkg, $sys) = (@_);
    my $vcs = $pkg->prod('VCS731');
    $vcs->restore_smf_scripts_sys($sys,$pkg);
    return;
}

#e3654406:stop had to make sure the had has been stopped.
#e3738609:kill remaining had/hashadow/cmdserver procs after running hastop
sub postinstall_sys {
    my ($pkg,$sys)=@_;
    my ($vcs);
    if (Cfg::opt('upgrade') && !Cfg::opt('rootpath')) {
        $vcs = $pkg->prod('VCS731');
	    $vcs->stop_vcs_upgrade($sys);
    }
    return;
}

1;
