#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSvxfs731::Common;
@Pkg::VRTSvxfs731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSvxfs';
    $pkg->{name}=Msg::new("File System")->{msg};
    $pkg->{softdeps} = [ qw(VRTSdbed VRTSodm) ];
    return;
}

package Pkg::VRTSvxfs731::AIX;
@Pkg::VRTSvxfs731::AIX::ISA = qw(Pkg::VRTSvxfs731::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{startprocs}=[ qw(vxfs731 vxportal731 qio731 vxcafs731) ];
    $pkg->{stopprocs}=[ qw(vxcafs731 qio731 vxportal731 vxfs731) ];
    return;
}
sub donotuninstall_sys {
    my ($pkg,$sys)=@_;
    $pkg->{donotrmonupgrade}=1 if(Cfg::opt("upgrade_kernelpkgs") && $sys->{pkgvers}{VRTScavf});
    return;
}

package Pkg::VRTSvxfs731::HPUX;
@Pkg::VRTSvxfs731::HPUX::ISA = qw(Pkg::VRTSvxfs731::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{donotrmonupgrade}=1;
    $pkg->{ospatches}{'11.31IA'}=['PHKL_38651','PHKL_38952','PHKL_40944','PHKL_41086'];
    $pkg->{ospatches}{'11.31PA'}=['PHKL_38651','PHKL_38952','PHKL_40944','PHKL_41086'];
    return;
}

package Pkg::VRTSvxfs731::Linux;
@Pkg::VRTSvxfs731::Linux::ISA = qw(Pkg::VRTSvxfs731::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{startprocs}=[ qw(vxportal731 fdd731 vxcafs731) ];
    $pkg->{stopprocs}=[ qw(vxcafs731 fdd731 vxportal731 vxfs731) ];
    $pkg->{previouspkgnames} = [ qw(VRTSvxfs-platform VRTSvxfs-common)];
    return;
}

package Pkg::VRTSvxfs731::RHEL6x8664;
@Pkg::VRTSvxfs731::RHEL6x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);

sub init_padv {
    my $pkg=shift;
	$pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"findutils" => "findutils.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.10)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libreadline.so.6()(64bit)" => "readline.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "module-init-tools.x86_64",
		"policycoreutils" => "policycoreutils.x86_64",
	};
    return;
}

package Pkg::VRTSvxfs731::RHEL7x8664;
@Pkg::VRTSvxfs731::RHEL7x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{startprocs}=[ qw(vxfs731 vxportal731 fdd731 vxcafs731) ];
	$pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"findutils" => "findutils.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.10)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libgcc_s.so.1()(64bit)" => "libgcc.x86_64",
		"libgcc_s.so.1(GCC_3.0)(64bit)" => "libgcc.x86_64",
		"libgcc_s.so.1(GCC_3.3.1)(64bit)" => "libgcc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libncurses.so.5()(64bit)" => "ncurses-libs.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libtinfo.so.5()(64bit)" => "ncurses-libs.x86_64",
		"policycoreutils" => "policycoreutils.x86_64",
	};
    return;
}

package Pkg::VRTSvxfs731::OL6x8664;
@Pkg::VRTSvxfs731::OL6x8664::ISA = qw(Pkg::VRTSvxfs731::RHEL6x8664);

package Pkg::VRTSvxfs731::OL7x8664;
@Pkg::VRTSvxfs731::OL7x8664::ISA = qw(Pkg::VRTSvxfs731::RHEL7x8664);

package Pkg::VRTSvxfs731::CentOS6x8664;
@Pkg::VRTSvxfs731::CentOS6x8664::ISA = qw(Pkg::VRTSvxfs731::RHEL6x8664);

package Pkg::VRTSvxfs731::CentOS7x8664;
@Pkg::VRTSvxfs731::CentOS7x8664::ISA = qw(Pkg::VRTSvxfs731::RHEL7x8664);

package Pkg::VRTSvxfs731::SLES11x8664;
@Pkg::VRTSvxfs731::SLES11x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);

sub init_padv {
	my $pkg=shift;
	$pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"findutils" => "findutils.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.10)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libncurses.so.5()(64bit)" => "libncurses5.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "module-init-tools.x86_64",
	};
	return;
}

package Pkg::VRTSvxfs731::SLES12x8664;
@Pkg::VRTSvxfs731::SLES12x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);

sub init_padv {
	my $pkg=shift;
    $pkg->{startprocs}=[ qw(vxfs731 vxportal731 fdd731 vxcafs731) ];
	$pkg->{oslibs}{all}={
		"coreutils" => "coreutils.x86_64",
		"ed" => "ed.x86_64",
		"findutils" => "findutils.x86_64",
		"libc.so.6()(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.10)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.14)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3.4)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.3)(64bit)" => "glibc.x86_64",
		"libc.so.6(GLIBC_2.4)(64bit)" => "glibc.x86_64",
		"libdl.so.2()(64bit)" => "glibc.x86_64",
		"libdl.so.2(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libm.so.6()(64bit)" => "glibc.x86_64",
		"libm.so.6(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libnsl.so.1()(64bit)" => "glibc.x86_64",
		"libpthread.so.0()(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.2)(64bit)" => "glibc.x86_64",
		"libpthread.so.0(GLIBC_2.3.3)(64bit)" => "glibc.x86_64",
		"librt.so.1()(64bit)" => "glibc.x86_64",
		"librt.so.1(GLIBC_2.2.5)(64bit)" => "glibc.x86_64",
		"modutils" => "kmod-compat.x86_64",
	};
	return;
}

package Pkg::VRTSvxfs731::SunOS;
@Pkg::VRTSvxfs731::SunOS::ISA = qw(Pkg::VRTSvxfs731::Common);

sub init_plat {
    my $pkg=shift;

    #$pkg->{startprocs}=[ qw(vxportal731 fdd731 vxcafs731) ];
    $pkg->{startprocs}=[ qw(fdd731 vxcafs731 vxportal731) ];
    $pkg->{stopprocs}=[ qw(vxportal731 vxcafs731 fdd731 qlog731 vxfs731) ];
    $pkg->{startprocs}=[ qw(fdd731 vxcafs731 vxportal731 odm731) ] if (Obj::rel()->{simple});
    $pkg->{stopprocs}=[ qw(vxportal731 odm731 vxcafs731 fdd731 qlog731 vxfs731) ] if (Obj::rel()->{simple});
    $pkg->{donotrmonupgrade}=0;
    
    return;
}

1;
