#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg::VRTSvxvm731::Common;
@Pkg::VRTSvxvm731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSvxvm';
    $pkg->{name}=Msg::new("Volume Manager Binaries")->{msg};
    $pkg->{vminstall}='/tmp/.cpivminstall';
    $pkg->{softdeps} = [ qw(VRTSdbed VRTSodm) ];
    return;
}

# touch this file to inform VM native scripts that it is a CPI installation
# VM native scripts will skip the checks for encapsulated and mirrored bootdisk
# if it is a CPI installation
sub preinstall_sys {
    my ($pkg,$sys)=@_;
    $sys->cmd("_cmd_rmr $pkg->{vminstall} 2>/dev/null");
    $sys->cmd("_cmd_touch $pkg->{vminstall} 2>/dev/null") if ($sys->{encap});

    return;
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;
    $sys->cmd("_cmd_rmr $pkg->{vminstall} 2>/dev/null");

    return;
}

package Pkg::VRTSvxvm731::AIX;
@Pkg::VRTSvxvm731::AIX::ISA = qw(Pkg::VRTSvxvm731::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{startprocs}=[ qw(vxconfigd731 vxloggerd731 vxvollogd731 vxesd731 vxrelocd731 vxcached731 vxconfigbackupd731 vxattachd731 vvr731 vxvvrstatd731) ];
    $pkg->{stopprocs}=[ qw(vxcloud731 vxvvrstatd731 vvr731 vxconfigbackupd731 vxsited731 vxcached731
                           vxrelocd731 vxnotify731 vxattachd731 vxesd731 vxvollogd731 vxloggerd731 vxconfigd731) ];
    $pkg->{ospkgs}{'5.3'}=['xlC.aix50.rte 6.0.0.7', 'devices.fcp.disk.rte 5.2.0.41', 'devices.scsi.disk.rte 5.2.0.41', 'devices.common.IBM.fc.hba-api'];
    $pkg->{ospkgs}{'6.1'}=['xlC.aix61.rte 9.0.0.0', 'devices.common.IBM.fc.hba-api' ];
    $pkg->{ospkgs}{'7.1'}=['xlC.aix61.rte 11.1.0.1', 'devices.common.IBM.fc.hba-api' ];
    $pkg->{has_install_export_cmd} = 1;
    return;
}
sub donotuninstall_sys {
    my ($pkg,$sys)=@_;
    $pkg->{donotrmonupgrade}=1 if(Cfg::opt("upgrade_kernelpkgs") && $sys->{pkgvers}{VRTScavf});
    return;
}

package Pkg::VRTSvxvm731::HPUX;
@Pkg::VRTSvxvm731::HPUX::ISA = qw(Pkg::VRTSvxvm731::Common);

sub init_plat {
    my $pkg=shift;

    $pkg->{startprocs}=[ qw(vxconfigd731 vxloggerd731 vxvollogd731 vxesd731 vxrelocd731 vxcached731 vxconfigbackupd731 vxattachd731 vvr731 vxvvrstatd731) ];
    $pkg->{stopprocs}=[ qw(vxvvrstatd731 vvr731 vxconfigbackupd731 vxsited731 vxcached731
                           vxrelocd731 vxnotify731 vxattachd731 vxesd731 vxvollogd731 vxloggerd731 vxconfigd731) ];
    $pkg->{reinstall}=1;
    $pkg->{donotrmonupgrade}=1;
    return;
}

package Pkg::VRTSvxvm731::Linux;
@Pkg::VRTSvxvm731::Linux::ISA = qw(Pkg::VRTSvxvm731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{startprocs}=[ qw(vxdmp731 vxio731 vxspec731 vxconfigd731 vxloggerd731 vxvollogd731 vxesd731 vxrelocd731 vxcached731 vxconfigbackupd731 vxattachd731 vvr731 vxencryptd731 vxvvrstatd731 vxcloud731) ];
    $pkg->{stopprocs}=[ qw(vxcloud731 vxvvrstatd731 vxencryptd731 vvr731 vxconfigbackupd731 vxsited731 vxattachd731 vxcached731 vxrelocd731 vxnotify731 vxesd731 vxvollogd731 vxloggerd731 vxconfigd731 vxspec731 vxio731 vxdmp731) ];
    $pkg->{previouspkgnames} = [ qw(VRTSvxvm-platform VRTSvxvm-common) ];
    return;
}

package Pkg::VRTSvxvm731::RHEL6x8664;
@Pkg::VRTSvxvm731::RHEL6x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

sub init_padv {
	my $pkg=shift;
	$pkg->{oslibs}{all}={
		"policycoreutils" => "policycoreutils.x86_64",
	};
	return;
}

package Pkg::VRTSvxvm731::RHEL7x8664;
@Pkg::VRTSvxvm731::RHEL7x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{startprocs}=[ qw(vxdmp731 vxio731 vxspec731 vxconfigd731 vxrecover731 vxencryptd731 vvr731 vxcloud731) ];
    # From 7.3, VM using systemctl to start vxvm-recover service, still stop old procs in upgrade stop process
    # Add all process here, will divide the old/new processes in verify_procs_list
    $pkg->{stopprocs}=[ qw(vxcloud731 vxvvrstatd731 vvr731 vxgin731 vxcfsplugin731 vxencryptd731 vxconfigbackupd731 vxsited731 vxattachd731 vxcached731 vxrelocd731 vxnotify731 vxesd731 vxvollogd731 vxloggerd731 vxrecover731 vxconfigd731 vxspec731 vxio731 vxdmp731) ];

	$pkg->{oslibs}{all}={
		"bc" => "bc.x86_64",
		"policycoreutils" => "policycoreutils.x86_64",
	};

    return;
}

sub postinstall_sys {
    my ($pkg,$sys)=@_;

    # the above docker files are from 7.1 and needs to be removed in post-7.1
    my @dockerfiles=qw(
        /usr/sbin/vxvolapi
        /usr/sbin/cfs-plugin
        /etc/sysconfig/vxgin
        /usr/lib/systemd/system/vxgin.service
        /etc/vx/docker/default.conf
        /etc/sysconfig/vxcfsplugin
        /usr/lib/systemd/system/vxcfsplugin.service
    );
    $sys->rm(join(' ', @dockerfiles));

    return $pkg->SUPER::postinstall_sys($sys);
}

package Pkg::VRTSvxvm731::OL6x8664;
@Pkg::VRTSvxvm731::OL6x8664::ISA = qw(Pkg::VRTSvxvm731::RHEL6x8664);

package Pkg::VRTSvxvm731::OL7x8664;
@Pkg::VRTSvxvm731::OL7x8664::ISA = qw(Pkg::VRTSvxvm731::RHEL7x8664);

package Pkg::VRTSvxvm731::CentOS6x8664;
@Pkg::VRTSvxvm731::CentOS6x8664::ISA = qw(Pkg::VRTSvxvm731::RHEL6x8664);

package Pkg::VRTSvxvm731::CentOS7x8664;
@Pkg::VRTSvxvm731::CentOS7x8664::ISA = qw(Pkg::VRTSvxvm731::RHEL7x8664);

package Pkg::VRTSvxvm731::SLES11x8664;
@Pkg::VRTSvxvm731::SLES11x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

package Pkg::VRTSvxvm731::SLES12x8664;
@Pkg::VRTSvxvm731::SLES12x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

sub init_plat {
    my $pkg=shift;
    $pkg->{startprocs}=[ qw(vxdmp731 vxio731 vxspec731 vxconfigd731 vxrecover731 vxencryptd731 vvr731 vxcloud731) ];
    $pkg->{stopprocs}=[ qw(vxcloud731 vxvvrstatd731 vxencryptd731 vvr731 vxrecover731 vxconfigbackupd731 vxsited731 vxattachd731 vxcached731 vxrelocd731 vxnotify731 vxesd731 vxvollogd731 vxloggerd731 vxconfigd731 vxspec731 vxio731 vxdmp731) ];
    $pkg->{previouspkgnames} = [ qw(VRTSvxvm-platform VRTSvxvm-common) ];
    return;
}

package Pkg::VRTSvxvm731::SunOS;
@Pkg::VRTSvxvm731::SunOS::ISA = qw(Pkg::VRTSvxvm731::Common);

sub init_plat {
    my $pkg=shift;
    $pkg->{responsefile}=' ';
    $pkg->{startprocs}=[qw(vxdmp731 vxio731 vxspec731 vxconfigd731 vxloggerd731 vxvollogd731 vxesd731
                           vxrelocd731 vxcached731 vxconfigbackupd731 vxattachd731 vvr731 vxvvrstatd731) ];
    $pkg->{stopprocs}=[ qw(vxvvrstatd731 vvr731 vxconfigbackupd731 vxsited731 vxattachd731 vxcached731
                           vxrelocd731 vxnotify731 vxesd731 vxvollogd731 vxloggerd731 vxconfigd731
                           vxspec731 vxio731 vxdmp731) ];
    $pkg->{donotrmonupgrade}=0;
    $pkg->{softdeps} = [];
    return;
}

sub define_ospatches {
    my ($pkg,$padv,$sys) = @_;
    my ($sunwcfcl_pkg);

    # we need to check for 114477-04 only if SUNWcfcl present
    $sunwcfcl_pkg=Pkg::new_pkg('Pkg','SUNWcfcl',$sys->{padv});
    push(@{$pkg->{ospatches}{'5.9'}},'114477-04') if ($sunwcfcl_pkg->version_sys($sys));

    return;
}

1;
