#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::odm731::Common;
@Proc::odm731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='odm';
    $proc->{name}='odm name';
    $proc->{desc}='odm description';
    $proc->{fatal}=1;
    $proc->{start_period}=20;
    $proc->{initconf}='/etc/vx/odm.config';
    return;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my ($mod,$prod,$rtn,$vxodmd,$cpic,$rel,$activecomponent);
    $cpic = Obj::cpic();
    $rel = $cpic->rel;
    $activecomponent = $rel->get_activecomponent();
    $prod = $proc->prod($activecomponent);
    $mod=$sys->get_drivers($proc->{proc});
    if ($mod eq '') {
        Msg::log("odm not loaded on sys $sys->{sys}");
        return 0;
    } else {
        Msg::log("odm loaded on sys $sys->{sys}");
    }
    if (($state =~ /start/m) && ($prod->{prod} =~ /^(SFRAC|SVS|SFCFS|SFCFSHA)$/mx)) {
        $vxodmd=$sys->cmd('_cmd_gabconfig -a 2>/dev/null');
        return ($vxodmd !~ /Port d/m) ? 0 : 1;
    }
    $rtn=$sys->cmd("_cmd_mount 2> /dev/null | _cmd_grep -w '/dev/odm'");
    if ($rtn ne '') {
        Msg::log("odm configured on $sys->{sys}");
        return 1;
    } else {
        Msg::log("odm loaded not configured on $sys->{sys}");
        # stopping, need to unload
        return 1 if ($state =~ /stop/m);
    }
    # starting, need to configure
    return 0;
}


sub start_failed_sys {
    my ($proc,$sys) = @_;
    my ($lic,$prod,$activecomponent,$rel,$cpic);

    $cpic = Obj::cpic();
    $rel = $cpic->rel;

    $activecomponent = $rel->get_activecomponent();
    $prod = $proc->prod($activecomponent);

    if ($prod->{prod} =~ /^(SFCFS|SFCFSHA)$/mx) {
        # Check 'CFSODM' for SFCFS/SFCFSHA
        $lic='ODM#VERITAS File System';
        Msg::log("## Check license bit '$lic' needed for odm starting for $prod:");
        $sys->cmd("/opt/VRTSvlic/bin/vxlicrep -i | _cmd_grep '$lic' 2>&1");
        if (EDR::cmdexit()) {
            Msg::log("license bit '$lic' is not enabled, odm failed to start.");
        }
    } elsif ($prod->{prod} =~ /^(SFRAC|SVS)$/mx) {
        # Check 'VXCFS#VERITAS File System' for SFRAC/SVS
        $lic='VXCFS#VERITAS File System';
        Msg::log("## Check license bit '$lic' needed for odm starting for $prod:");
        $sys->cmd("/opt/VRTSvlic/bin/vxlicrep -i | _cmd_grep '$lic' 2>&1");
        if (EDR::cmdexit()) {
            Msg::log("license bit '$lic' is not enabled, odm failed to start.");
        }
    }

    return 1;
}

sub enable_sys {
    my ($proc,$sys) = @_;
    my ($file,$conf,$rootpath);
    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    $conf =~ s/ODM_START\s*=\s*0/ODM_START=1/mx;
    $conf .= "\n";
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->rm("$file.prev");
    }

    if ($sys->{plat} eq 'AIX') {
        $sys->cmd('/etc/methods/vxkextadm vxodm config');
    }

    return 1;

}

sub correct_config_file_sys {
    my ($proc,$sys,$file)=@_;
    my ($conf,$dir);
    
    #This function works for the scenario: using native install to instlal VRTSodm, ODM_START won't be added.
    if ($sys->exists("$file")) {
        $conf=$sys->catfile($file);
        if ($conf =~ /ODM_START/mx) {
            return 1;
        } else {
            $sys->appendfile('ODM_START=0', $file);
        }
    } else {
        $dir=EDRu::dirname($file);
        $sys->cmd("_cmd_mkdir -p $dir");
        $sys->cmd("echo 'ODM_START=0' > $file 2>/dev/null");
    }
    #Package will start odm after installation.
    $proc->stop_sys($sys);

    return;
}

sub is_enabled_sys {
    my ($proc,$sys)=@_;
    my ($conf,$rootpath,$initconf);
    $rootpath = Cfg::opt('rootpath') || '';
    $initconf = $rootpath . $proc->{initconf};
    $proc->correct_config_file_sys($sys,$initconf);
    $conf = $sys->cmd("unset ODM_START 2>/dev/null >/dev/null ; . $initconf 2>/dev/null >/dev/null ; echo \$ODM_START");
    chomp $conf;
    if ($conf eq '0'){
        return 0;
    } else {
        return 1;
    }
}

sub disable_sys {
    my ($proc,$sys) = @_;
    my ($file,$conf,$rootpath);
    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    $conf =~ s/ODM_START\s*=\s*1/ODM_START=0/mx;
    $conf .= "\n";
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->rm("$file.prev");
    }

    if ($sys->{plat} eq 'AIX') {
        $sys->cmd('/etc/methods/vxkextadm vxodm unconfig');
    }
    return 1;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## ps -ef output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_ps -ef 2>/dev/null');

    Msg::log("## cat /dev/odm/fid for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_cat /dev/odm/fid 2>/dev/null');

    Msg::log("## cat /dev/odm/cluster for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_cat /dev/odm/cluster 2>/dev/null');

    return;
}

package Proc::odm731::AIX;
@Proc::odm731::AIX::ISA = qw(Proc::odm731::Common);

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my ($prod,$odmconf1,$odmconf2,$rtn,$activecomponent,$rel,$cpic);

    $cpic = Obj::cpic();
    $rel = $cpic->rel;

    $activecomponent = $rel->get_activecomponent();
    $prod = $proc->prod($activecomponent);
    $rtn=$sys->cmd("/etc/methods/vxkextadm vxodm status 2>/dev/null | _cmd_grep 'is not loaded'");
    if ($rtn) {
        Msg::log("odm not loaded on sys $sys->{sys}");
        return 0;
    } else {
        Msg::log("odm loaded on sys $sys->{sys}");
    }
    if (($state =~ /start/m) && ($prod->{prod} =~ /^(SFRAC|SVS|SFCFS|SFCFSHA)$/mx)) {
        $odmconf1=$sys->cmd('_cmd_gabconfig -a 2> /dev/null');
        $odmconf2=$sys->cmd("_cmd_mount 2>/dev/null | _cmd_grep -w '/dev/odm'");
        return (($odmconf1 !~ /Port d/m) || ($odmconf2 eq '')) ? 0 : 1;
    }
    $rtn=$sys->cmd("_cmd_mount 2>/dev/null | _cmd_grep -w '/dev/odm'");
    if ($rtn ne '') {
        Msg::log("odm configured on $sys->{sys}");
        return 1;
    } else {
        Msg::log("odm loaded not configured on $sys->{sys}");
        # stopping, need to unload
        return 1 if ($state =~ /stop/m);
    }
    # starting, need to configure
    return 0;
}

sub start_sys {
    my ($proc,$sys)=@_;
    # if /dev/odm is already mounted, unmount it first.
    if ($sys->cmd("_cmd_mount 2>/dev/null | _cmd_grep -w '/dev/odm'")) {
        $sys->cmd('/etc/rc.d/rc2.d/S99odm stop 2>/dev/null');
        $sys->cmd('_cmd_umount /dev/odm') if (EDR::cmdexit());
        $sys->cmd('/etc/methods/vxkextadm vxodm unload 2> /dev/null');
    }
    $sys->cmd('/etc/methods/vxkextadm vxodm load 2> /dev/null');
    $sys->cmd('/etc/rc.d/rc2.d/S99odm start 2>/dev/null');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/rc.d/rc2.d/S99odm stop 2>/dev/null');
    $sys->cmd('_cmd_umount /dev/odm') if (EDR::cmdexit());
    $sys->cmd('/etc/methods/vxkextadm vxodm unload 2> /dev/null');
    return 1;
}

package Proc::odm731::HPUX;
@Proc::odm731::HPUX::ISA = qw(Proc::odm731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/sbin/init.d/odm start 2>/dev/null');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/sbin/init.d/odm stop 2>/dev/null');
    return 1;
}

package Proc::odm731::Linux;
@Proc::odm731::Linux::ISA = qw(Proc::odm731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{proc}='vxodm';
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/init.d/vxodm restart 2>/dev/null');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/init.d/vxodm stop 2>/dev/null');
    if (!EDR::cmdexit()) {
        if ($sys->get_drivers( $proc->{proc})) {
            Msg::log('Unloading vxodm');
            $sys->unload_driver( $proc->{proc});
        } else {
            Msg::log('vxodm already unloaded');
        }
    } else {
            Msg::log('Failed to stop ODM');
    }
    return 1;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my ($odmconf,$prod,$mod,$modid,$activecomponent,$rel,$cpic);

    $cpic = Obj::cpic();
    $rel = $cpic->rel;

    $activecomponent = $rel->get_activecomponent();
    $prod = $proc->prod($activecomponent);
    $mod=$sys->get_drivers($proc->{proc});
    if ($mod eq '') {
        Msg::log("odm not loaded on sys $sys->{sys}");
        return 0;
    } else {
        Msg::log("odm loaded on sys $sys->{sys}");
    }
    # For SFRAC, port 'd' is the indication
    # that ODM has started successfully.
    if ($prod->{prod} =~ /^(SFRAC|SVS|SFCFS|SFCFSHA)$/mx) {
    if ($state =~ /start/m) {
            $odmconf=$sys->cmd('_cmd_gabconfig -a 2> /dev/null');
            Msg::log("## gabconfig -a output: \n $odmconf \n");
            $odmconf = $sys->cmd("_cmd_gabconfig -a 2> /dev/null | _cmd_grep 'Port d'");
            return ($odmconf eq '') ? 0 : 1;
    } elsif ($state =~ /stop/m) {
            $modid = $sys->cmd("_cmd_lsmod 2>/dev/null | _cmd_grep 'vxodm'");
        return 1 if ($modid);
    }
    }
    # For DBED, there is no port 'd' (Standalone mode).
    # Check if /dev/odm is mounted or not.
    $odmconf = $sys->cmd("/etc/init.d/vxodm status 2>/dev/null | _cmd_grep 'is stopped'");
    if ($odmconf eq '') {
        Msg::log("odm started on $sys->{sys}");
        return 1;
    } else {
        Msg::log("odm loaded not started on $sys->{sys}");
        # stopping, need to unload
        return 1 if ($state =~ /stop/m);
    }
    return 0;
}

package Proc::odm731::RHEL7x8664;
@Proc::odm731::RHEL7x8664::ISA = qw(Proc::odm731::Linux);

sub start_sys {
    my ($proc,$sys)=@_;
    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if (EDRu::compvers(${$sys->{prodvers}}[0],'7.3')==2) {
        $proc->SUPER::stop_sys($sys);
        return 1;
    }
    $sys->padv->systemctl_stop_service($sys, $proc->{proc});
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::odm731::SLES12x8664;
@Proc::odm731::SLES12x8664::ISA = qw(Proc::odm731::RHEL7x8664);

package Proc::odm731::SunOS;
@Proc::odm731::SunOS::ISA = qw(Proc::odm731::Common);

sub init_plat {
    my ($proc,$sys)=@_;
    $proc->{controlfile}='/etc/rc2.d/S92odm';
    $proc->{smf_manifest}='/var/svc/manifest/system/vxodm/odm.xml';
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd('_cmd_svcadm disable -s system/vxodm 2>/dev/null');
        $sys->cmd('_cmd_svcadm enable system/vxodm 2>/dev/null');
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} restart 2>/dev/null");
    } else {
        return 0;
    }
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd('_cmd_svcadm disable -st system/vxodm 2>/dev/null');
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} stop 2>/dev/null");
    }

    $sys->unload_driver($proc->{proc});
    return 1;
}

1;
