#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::veki731::Common;
@Proc::veki731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='veki';
    $proc->{name}='veki name';
    $proc->{desc}='veki description';
    $proc->{fatal}=1;
    return;
}

package Proc::veki731::AIX;
@Proc::veki731::AIX::ISA = qw(Proc::veki731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{driverfile}='/usr/lib/drivers/veki.ext';
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/vekiextadm load');
    $sys->cmd('/etc/methods/vekiextadm config');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/vekiextadm unconfig');
    $sys->cmd('/etc/methods/vekiextadm unload');
    sleep 3;
    if($proc->check_sys($sys)){
        Msg::log("Unload Driver on $sys->{sys}");
        $sys->unload_driver($proc->{driverfile});
    }
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $mod;
    $mod=$sys->get_drivers( $proc->{driverfile});
    if ($mod eq '') {
        return 0;
    }
    return 1;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## ps -ef output for $proc->{proc} on $sys->{sys}:\n\n");
    $sys->cmd('_cmd_ps -aef');
    return;
}

package Proc::veki731::HPUX;
@Proc::veki731::HPUX::ISA = qw(Proc::veki731::Common);

package Proc::veki731::Linux;
@Proc::veki731::Linux::ISA = qw(Proc::veki731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{controlfile} = '/etc/init.d/veki';
    return;
}

package Proc::veki731::RHEL7x8664;
@Proc::veki731::RHEL7x8664::ISA = qw(Proc::veki731::Linux);

sub start_sys {
    my ($proc,$sys)=@_;

    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;
    my $status;
    if (EDRu::compvers(${$sys->{prodvers}}[0],'7.3')==2) {
        $sys->cmd("$proc->{controlfile} stop");
        ## if veki service state is active in old version, install new VRTSveki, postinstall script in rpm cannot load veki module
        ## and VRTSvxvm will failed to be installed
        $status = $sys->cmd("_cmd_systemctl show -p ActiveState $proc->{proc}");
        $sys->cmd("_cmd_systemctl stop $proc->{proc}") if $status =~ /=active/;
        return 1;
    }
    return $sys->padv->systemctl_stop_service($sys, $proc->{proc});
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $mod;
    $mod=$sys->get_drivers( 'veki');
    if ($mod eq '') {
        return 0;
    }
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::veki731::OL7x8664;
@Proc::veki731::OL7x8664::ISA = qw(Proc::veki731::RHEL7x8664);

package Proc::veki731::SLES12x8664;
@Proc::veki731::SLES12x8664::ISA = qw(Proc::veki731::RHEL7x8664);

package Proc::veki731::SunOS;
@Proc::veki731::SunOS::ISA = qw(Proc::veki731::Common);

1;
