#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxcafs731::Common;
@Proc::vxcafs731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxcafs';
    $proc->{name}='vxcafs name';
    $proc->{desc}='vxcafs description';
    return;
}


package Proc::vxcafs731::Linux;
@Proc::vxcafs731::Linux::ISA = qw(Proc::vxcafs731::Common);
# load to kernel
sub start_sys {
    my ($proc,$sys)=@_;
    $sys->load_driver($proc->{proc});
    return 1;
}

# unload from kernel
sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

# check whether mod is loaded
# 1 means loaded, 0 means not loaded
sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn=$sys->get_drivers($proc->{proc});
    return ($rtn?1:0);
}
package Proc::vxcafs731::AIX;
@Proc::vxcafs731::AIX::ISA = qw(Proc::vxcafs731::Common);
sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_vxkextadm cafs load");
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_vxkextadm cafs unload");
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $ret = $sys->cmd("_cmd_vxcfg cafs status 2>&1");
    return 0 if ($ret =~ /USAGE:/);
    return !EDR::cmdexit();
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## ps -ef output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_ps -ef');
    return;
}

package Proc::vxcafs731::HPUX;
@Proc::vxcafs731::HPUX::ISA = qw(Proc::vxcafs731::Common);

package Proc::vxcafs731::SunOS;
@Proc::vxcafs731::SunOS::ISA = qw(Proc::vxcafs731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{reboot_donotstart}=1;
    return;
}

# load to kernel
sub start_sys {
    my ($proc,$sys)=@_;
    my ($mn,$driver);
    $driver=$proc->{proc};
    $mn=$sys->get_drivers( $driver);
    if ($mn eq '') {
        $sys->cmd("_cmd_adddrv -f -m '* 0664 root sys' $driver 2>/dev/null; _cmd_modload -p drv/$driver 2>/dev/null");
    }
    return 1;
}

# unload from kernel
sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

# check whether mod is loaded
# 1 means loaded, 0 means not loaded
sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn=$sys->get_drivers($proc->{proc});
    return ($rtn?1:0);
}

1;