#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxfen731::Common;
@Proc::vxfen731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxfen';
    $proc->{name}='I/O Fencing';
    $proc->{desc}='I/O Fencing';
    # increase this time from 180 to 300, refer to e2806636
    $proc->{start_period}=300;
    $proc->{stop_period}=10;
    $proc->{fatal}=1;
    return;
}

sub prestop_sys {
    my ($proc,$sys) = @_;
    my ($vxfenversion,$output,$vxfencmd);
    return 1 unless (Cfg::opt('upgrade_kernelpkgs'));
    return '' unless ($proc->check_sys($sys,'start'));
    $output=$sys->cmd("_cmd_vxfenconfig -W 2>/dev/null | _cmd_grep 'Current Protocol Version'");
    $vxfenversion= $1 if ($output =~/Current Protocol Version\s+:\s+(\d+)/m);
    return '' unless (EDRu::isint($vxfenversion));
    #backup /etc/vxfenmode
    $output=$sys->cmd('_cmd_cat /etc/vxfenmode');
    if($output=~/vxfen_protocol_version/m){
        $output=~s/vxfen_protocol_version(.*)//mxg;
        $sys->writefile($output,'/etc/vxfenmode.rubak');
    } else {
        $sys->cmd('_cmd_cp -rf  /etc/vxfenmode /etc/vxfenmode.rubak');
        $vxfencmd="vxfen_protocol_version=$vxfenversion";
        $sys->appendfile($vxfencmd,'/etc/vxfenmode');
    }
    return 1;
}


sub parse_vxfenadm_output {
    my ($proc,$sys) = @_;
    my (%vxfen_configuration,$flag,@cmlist,$cm,@silist,$si,$nothing);

    my $vxfenadm_d=$sys->cmd('_cmd_vxfenadm -d 2>&1');
    if (EDR::cmdexit()) {
        $vxfen_configuration{cluster_member}{self_node_state}='error in using vxfenadm';
        return %vxfen_configuration;
    }
    $vxfenadm_d =~ m/Fencing Protocol Version: ([^\r\n]+)[\r\n]/;
    $vxfen_configuration{protocol_version}=$1;
    $vxfenadm_d =~ m/Fencing Mode: ([^\r\n]+)[\r\n]/;
    $vxfen_configuration{fencing_mode}=$1;
    if ($vxfenadm_d =~ m/Fencing .*? Disk Policy: ([^\r\n]+)[\r\n]/) {
        $vxfen_configuration{disk_policy}=$1;
    }
    if ($vxfenadm_d =~ m/Fencing Mechanism:\s*(\S+)\s*/) {
        $vxfen_configuration{mechanism}=$1;
    }
    ($nothing, $cm)=split(/Cluster Members:/m, $vxfenadm_d);
    ($cm, $nothing)=split(/RFSM/m, $cm);
    $cm=~s/^[\r\n]*//;  # remove newline chars from beginning
    $cm=~s/[\r\n]*$//;  # remove newline chars from end
    @cmlist=split(/[\r\n]+/, $cm);   # get entries
    $flag=0;    # track the '*' in Cluster Member list (* indicates the self node)
    for my $cm (@cmlist) {
        $flag=1 if $cm =~ m/\*/;
        if ($cm=~m/^[ \t\*]*(\d+) (\(.*\))[ \t]*/) {
            $vxfen_configuration{cluster_member}{node_name}{"$1"}=$2;
            if ($flag) {
                $vxfen_configuration{cluster_member}{self_node_id}=$1;
                $vxfen_configuration{cluster_member}{self_node_name}=$2;
                last;
            }
        }
    }
    ($nothing, $si)= split(/RFSM State Information:/m,$vxfenadm_d);
    $si=~s/^[\r\n]*//;  # remove newline chars from beginning
    $si=~s/[\r\n]*$//;  # remove newline chars from end
    @silist=split(/[\r\n]+/, $si);   # get entries
    for my $si (@silist) {
        if ($si=~m/^[ \t]*node\s+(\d+)\s+in\s+state\s+\d+\s+\((.*)\)[ \t]*/) {
            $vxfen_configuration{cluster_member}{node_state}{"$1"}=$2;
            $vxfen_configuration{cluster_member}{self_node_state}=$2 if ($vxfen_configuration{cluster_member}{self_node_id}==$1);
        }
    }
    return %vxfen_configuration;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my $mod;
    $mod=$sys->get_drivers($proc->{proc});
    if ($mod eq '') {
        Msg::log("vxfen not loaded on sys $sys->{sys}");
        return 0;
    }
    my %vxfen_configs=$proc->parse_vxfenadm_output($sys);
    if ($state =~ /start/m) {
        # start phase
        if ( $vxfen_configs{cluster_member}{self_node_state} =~ /running/m ) {
            return 1;
        } else {
            return 0;
        }
    } else {
        # stop phase, vxfen is loaded
        return 1;
    }
}

sub dump_vxfen_parameters_sys {
    my ($proc,$sys,$msg)=@_;
    my ($debuglines,$loglines,$total_log_line,$logmsg,$vxfen_debug_info,$vxfen_log_file,$vxfen_log); 

    $debuglines = 100; 
    $loglines = 50;
    $total_log_line = $debuglines + $loglines;
    $vxfen_log_file = "/var/VRTSvcs/log/vxfen/vxfen.log";

    $vxfen_debug_info=$sys->cmd("_cmd_vxfendebug -p 2>/dev/null | _cmd_tail -$debuglines");
    $vxfen_debug_info = "\n$vxfen_debug_info";
    $vxfen_debug_info =~ s/\n/\n\t(sys: $sys->{sys}) /g;
    $logmsg="VxFEN $msg on system $sys->{sys}, dumping debug information:\n";
    $vxfen_log = $sys->cmd("_cmd_tail -$loglines $vxfen_log_file");
    
    $logmsg.="(sys: $sys->{sys}) VxFEN Logs (last $total_log_line lines):\nBEGIN VxFEN LOG {\n$vxfen_debug_info\n\nLogs from vxfen.log\n$vxfen_log\nEND VxFEN LOG }\n";
    Msg::log($logmsg);
    return;
}

sub enable_sys {
    my ($proc,$sys) = @_;
    my ($cfg,$file,$conf,$rootpath,$stat);
    $cfg = Obj::cfg();
    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    return 1 if (($conf =~ /\nVXFEN_START\s*=\s*1/x) && ($conf =~ /\nVXFEN_STOP\s*=\s*1/x));
    $conf =~ s/VXFEN_START\s*=\s*0/VXFEN_START=1/mx;
    $conf =~ s/VXFEN_STOP\s*=\s*0/VXFEN_STOP=1/mx;
    $conf .= "\n" if ($conf !~ /\n$/);
    $stat=$sys->filestat($file);
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->change_filestat($file,$stat);
        $sys->rm("$file.prev");
    }
    if($proc->{driver_config}=~ /\S/){
        $sys->cmd_script($proc->{driver_config});
    }
    return 1;
}

sub disable_sys {
    my ($proc,$sys,$postinstall) = @_;
    my ($cfg,$file,$conf,$rootpath,$stat);

    $cfg = Obj::cfg();

    if(!$postinstall && $proc->{driver_unconfig}=~ /\S/){
        if(!$cfg->{vcs_allowcomms} && ($cfg->{systemscfg} && $#{$cfg->{systemscfg}} == 0 || !$cfg->{systemscfg} &&  $#{$cfg->{systems}} == 0)){
            $sys->cmd_script($proc->{driver_unconfig});
        }
    }

    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    return 1 if (($conf =~ /\nVXFEN_START\s*=\s*0/x) && ($conf =~ /\nVXFEN_STOP\s*=\s*0/x));
    $conf =~ s/VXFEN_START\s*=\s*1/VXFEN_START=0/mx;
    $conf =~ s/VXFEN_STOP\s*=\s*1/VXFEN_STOP=0/mx;
    $conf .= "\n" if ($conf !~ /\n$/);
    $stat=$sys->filestat($file);
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->change_filestat($file,$stat);
        $sys->rm("$file.prev");
    }
    return 1;
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("$proc->{controlfile} start");
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("$proc->{controlfile} stop");
    if ($sys->get_drivers($proc->{proc})) {
        $sys->unload_driver($proc->{proc});
    }
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;
    return $proc->dump_vxfen_parameters_sys($sys,'start failed');
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;
    return $proc->dump_vxfen_parameters_sys($sys,'stop failed');
}

sub check_service_sys {
    my ($proc,$sys)=@_;
    my ($conf);
    $conf = $sys->cmd("unset VXFEN_START VXFEN_STOP; . $proc->{initconf} 2>/dev/null ; set |_cmd_grep '^VXFEN_ST'|_cmd_grep -v BASH_EXECUTION_STRING");
    return 0 unless ($conf);
    if (($conf =~ /VXFEN_START=1/mx) && ($conf =~ /VXFEN_STOP=1/mx)){
        return 1;
    } else {
        return 0;
    }
}

package Proc::vxfen731::AIX;
@Proc::vxfen731::AIX::ISA = qw(Proc::vxfen731::Common);
sub init_plat {
    my $proc=shift;
    $proc->{controlfile}='/etc/rc.d/rc2.d/S97vxfen';
    $proc->{driverfile}='/usr/lib/drivers/vxfen';
    $proc->{initconf}='/etc/default/vxfen';
    $proc->{kernelfile}='/etc/methods/vxfenext';
    $proc->{driver_config} = <<'AIX_VXFEN_CONFIG';
odmadd /usr/lib/methods/vxfen.odm
odmadd /usr/lib/methods/vxfend.odm
/etc/methods/vxfenext -start -dvxfen > /dev/null
/etc/methods/vxfenext -start -dvxfend > /dev/null
exit 0
AIX_VXFEN_CONFIG
    $proc->{driver_unconfig} = <<'AIX_VXFEN_UNCONFIG';
/etc/methods/vxfenext -stop -dvxfend > /dev/null 2>&1
/etc/methods/vxfenext -stop -dvxfen > /dev/null 2>&1
odmdelete -q"name=vxfend" -o CuAt >/dev/null 2>&1
odmdelete -q"name=vxfend" -o CuDv >/dev/null 2>&1
odmdelete -q"uniquetype='vxfend/node/vxfend'" -o PdAt >/dev/null 2>&1
odmdelete -q"uniquetype='vxfend/node/vxfend'" -o PdDv >/dev/null 2>&1
odmdelete -q"rule='/etc/methods/vxfenext -define -dvxfend'" -o Config_Rules >/dev/null 2>&1
odmdelete -q"name=vxfen" -o CuAt >/dev/null 2>&1
odmdelete -q"name=vxfen" -o CuDv >/dev/null 2>&1
odmdelete -q"uniquetype='vxfen/node/vxfen'" -o PdAt >/dev/null 2>&1
odmdelete -q"uniquetype='vxfen/node/vxfen'" -o PdDv >/dev/null 2>&1
odmdelete -q"rule='/etc/methods/vxfenext -define -dvxfen'" -o Config_Rules >/dev/null 2>&1
exit 0
AIX_VXFEN_UNCONFIG

    return;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my $mod;
    $mod=$sys->cmd("$proc->{kernelfile} -status");
    if ($mod !~ /vxfen: loaded/m) {
        Msg::log("vxfen not loaded on sys $sys->{sys}");
        return 0;
    }
    my %vxfen_configs=$proc->parse_vxfenadm_output($sys);
    if ($state =~ /start/m) {
        # start phase
        if ( $vxfen_configs{cluster_member}{self_node_state} =~ /running/m ) {
            return 1;
        } else {
            return 0;
        }
    } else {
        # stop phase, vxfen is loaded
        return 1;
    }
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("$proc->{kernelfile} -start");
    $sys->cmd("$proc->{kernelfile} -start dvxfend");
    if ($sys->{rsh} eq 'rsh') {
        $sys->cmd("$proc->{controlfile} start 1>/dev/null");
    } else {
        $sys->cmd("$proc->{controlfile} start");
    }
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("$proc->{controlfile} stop");
    $sys->cmd("$proc->{kernelfile} -stop");
    return 1;
}

package Proc::vxfen731::HPUX;
@Proc::vxfen731::HPUX::ISA = qw(Proc::vxfen731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{controlfile}='/sbin/init.d/vxfen';
    $proc->{initconf}='/etc/rc.config.d/vxfenconf';
    return;
}

package Proc::vxfen731::Linux;
@Proc::vxfen731::Linux::ISA = qw(Proc::vxfen731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{controlfile}='/etc/init.d/vxfen';
    $proc->{initconf}='/etc/sysconfig/vxfen';
    return;
}

package Proc::vxfen731::RHEL7x8664;
@Proc::vxfen731::RHEL7x8664::ISA = qw(Proc::vxfen731::Linux);

sub init_plat {
    my $proc=shift;
    $proc->{controlfile}='/opt/VRTSvcs/vxfen/bin/vxfen';
    $proc->{initconf}='/etc/sysconfig/vxfen';
    return;
}

# In upgrade scenario, before 7.2 the controfile should be /etc/init.d/
# From 7.3, controfile is moved from /etc/init.d/ to /opt/VRTS
sub reset_controlfile {
    my ($proc, $sys)=@_;
    if ($sys->exists('/etc/init.d/vxfen')) {
        $proc->{controlfile}='/etc/init.d/vxfen';
    }
}

sub start_sys {
    my ($proc,$sys)=@_;
    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;

    # upgrade from 7.2 and earlier, we still using /etc/init.d/script to stop sys
    $proc->reset_controlfile($sys);
    if ($proc->{controlfile} =~ /etc/) {
        $sys->cmd("$proc->{controlfile} stop");
        return 1;
    }
    return $sys->padv->systemctl_stop_service($sys, $proc->{proc});
}

package Proc::vxfen731::SLES12x8664;
@Proc::vxfen731::SLES12x8664::ISA = qw(Proc::vxfen731::RHEL7x8664);

package Proc::vxfen731::SunOS;
@Proc::vxfen731::SunOS::ISA = qw(Proc::vxfen731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{controlfile}='/lib/svc/method/vxfen';
    $proc->{smf_manifest}='/var/svc/manifest/system/vxfen.xml';
    $proc->{initconf}='/etc/default/vxfen';
    
    $proc->{driver_config} = <<'SOL_VXFEN_CONFIG';
/usr/sbin/add_drv -v -f -m '* 0600 root sys' vxfen
svcs -Ho state vxfen | grep -i disabled
ENABLED=$?
svccfg -s system/vxfen delcust
if [ $ENABLED -ne 0 ]
then
    svcadm enable system/vxfen
fi
SOL_VXFEN_CONFIG
    $proc->{driver_unconfig} = <<'SOL_VXFEN_UNCONFIG';
svcadm disable system/vxfen
svccfg delete -f system/vxfen
rem_drv vxfen
modinfo | grep -w vxfen | awk '{print $1}' |xargs -i modunload -i {}
SOL_VXFEN_UNCONFIG
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd('_cmd_svcadm disable -st system/vxfen');
        $sys->cmd('_cmd_svcadm enable system/vxfen');
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} start");
    } else {
        return 0;
    }
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd('_cmd_svcadm disable -st system/vxfen');
    } elsif ($sys->exists($proc->{controlfile})) {
        $sys->cmd("$proc->{controlfile} stop");
    }
    return 0 if (EDR::cmdexit());
    return 1;
}

sub check_service_sys {
    my ($proc,$sys)=@_;
    my ($svcinfo_vxfen,$conf);
    if ($sys->exists($proc->{smf_manifest})) { # Solaris 10
        $svcinfo_vxfen = $sys->cmd("_cmd_svcs -l system/vxfen 2>/dev/null | _cmd_grep '^enabled' ");
        if ($svcinfo_vxfen =~ /true/m) {
            if ( $svcinfo_vxfen =~ /temporary/m) {
                return 0;
            } else {
                return 1;
            }
        } elsif ($svcinfo_vxfen =~ /false/m ) {
            if ( $svcinfo_vxfen =~ /temporary/m) {
                return 1;
            } else {
                return 0;
            }
        }
    } else { # Solaris 9
        $conf = $sys->cmd("unset VXFEN_START VXFEN_STOP; . $proc->{initconf} 2>/dev/null ; set |_cmd_grep '^VXFEN_ST'|_cmd_grep -v BASH_EXECUTION_STRING");
        return 0 unless ($conf);
        if (($conf =~ /VXFEN_START=1/mx) && ($conf =~ /VXFEN_STOP=1/mx)){
            return 1;
        } else {
            return 0;
        }
    }
}

sub disable_service_sys {
    my ($proc,$sys)=@_;
    if ($sys->exists($proc->{smf_manifest})) {
        $sys->cmd('_cmd_svcadm disable system/vxfen');
    }
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn = $sys->cmd("$proc->{controlfile} status 2>/dev/null | _cmd_grep 'running'");
    return 1 if ($rtn);
    return 0;
}

1;
