#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxfs731::Common;
@Proc::vxfs731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxfs';
    $proc->{name}='vxfs name';
    $proc->{desc}='vxfs description';
    $proc->{fatal}=1;
    return;
}

package Proc::vxfs731::AIX;
@Proc::vxfs731::AIX::ISA = qw(Proc::vxfs731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_vxkextadm $proc->{proc} load");
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_vxkextadm $proc->{proc} unload");
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_vxcfg $proc->{proc} status");
    return !EDR::cmdexit();
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## ps -ef output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_ps -ef');
    return;
}

package Proc::vxfs731::HPUX;
@Proc::vxfs731::HPUX::ISA = qw(Proc::vxfs731::Common);

package Proc::vxfs731::Linux;
@Proc::vxfs731::Linux::ISA = qw(Proc::vxfs731::Common);

# load to kernel
sub start_sys {
    my ($proc,$sys)=@_;
    $sys->load_driver($proc->{proc});
    return 1;
}

# unload from kernel
sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

# check whether mod is loaded
# 1 means loaded, 0 means not loaded
sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn=$sys->get_drivers($proc->{proc});
    return ($rtn?1:0);
}

package Proc::vxfs731::RHEL7x8664;
@Proc::vxfs731::RHEL7x8664::ISA = qw(Proc::vxfs731::Linux);

## e3916239, change vxfs to use systemd control
sub start_sys {
    my ($proc,$sys)=@_;
    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if (EDRu::compvers(${$sys->{prodvers}}[0],'7.3')==2) {
        $sys->unload_driver($proc->{proc});
        return 1;
    }
    $sys->padv->systemctl_stop_service($sys, $proc->{proc});
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::vxfs731::SLES12x8664;
@Proc::vxfs731::SLES12x8664::ISA = qw(Proc::vxfs731::RHEL7x8664);

package Proc::vxfs731::SunOS;
@Proc::vxfs731::SunOS::ISA = qw(Proc::vxfs731::Common);

# load to kernel
sub start_sys {
    my ($proc,$sys)=@_;
    $sys->load_driver($proc->{proc});
    return 1;
}

# unload from kernel
sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

# check whether mod is loaded
# 1 means loaded, 0 means not loaded
sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn=$sys->get_drivers($proc->{proc});
    return ($rtn?1:0);
}

1;
