#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxglm731::Common;
@Proc::vxglm731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxglm';
    $proc->{name}='vxglm name';
    $proc->{desc}='vxglm description';
    $proc->{fatal}=1;
    return;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## gabconfig -a output:\n\n");
    $sys->cmd('_cmd_gabconfig -a 2> /dev/null');
    Msg::log("## ps -ef output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_ps -ef');
    return;
}

sub enable_sys {
    my ($proc,$sys)=@_;
    return 1;
}

sub disable_sys {
    my ($proc,$sys)=@_;
    return 1;
}

package Proc::vxglm731::AIX;
@Proc::vxglm731::AIX::ISA = qw(Proc::vxglm731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/glmkextadm load 2> /dev/null');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/glmkextadm unload 2> /dev/null') if($sys->exists('/etc/methods/glmkextadm'));
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    return 0 unless ($sys->exists('/etc/methods/glmkextadm'));
    my $glmconf = $sys->cmd("/etc/methods/glmkextadm status | _cmd_grep 'not loaded'");
    return 1 if ($glmconf eq '');
    return 0;
}

package Proc::vxglm731::HPUX;
@Proc::vxglm731::HPUX::ISA = qw(Proc::vxglm731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('_cmd_kcmodule -B vxglm=loaded');
    $sys->cmd('/sbin/init.d/vxglm start');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/sbin/init.d/vxglm stop');
    $sys->cmd('_cmd_kcmodule vxglm=unused');
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $pids=$sys->proc_pids('vxglmd');
    return 1 if ($#$pids>=0);
    return 0;
}

package Proc::vxglm731::Linux;
@Proc::vxglm731::Linux::ISA = qw(Proc::vxglm731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{start_period}=10;
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/init.d/vxglm start');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("export PATH=/bin:/sbin:\$PATH; /etc/init.d/vxglm stop");
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn = $sys->get_drivers($proc->{proc});
    return 1 if($rtn ne '');
    return 0;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## ps -ef output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_ps -aef');
    Msg::log("## modinfo output for $proc->{proc}:\n\n");
    $sys->cmd('_cmd_lsmod');
    Msg::log("## gabconfig -a output:\n\n");
    $sys->cmd('_cmd_gabconfig -a 2> /dev/null');
    return;
}

package Proc::vxglm731::RHEL7x8664;
@Proc::vxglm731::RHEL7x8664::ISA = qw(Proc::vxglm731::Linux);

sub init_plat {
    my $proc=shift;
    $proc->{start_period}=20;
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if (EDRu::compvers(${$sys->{prodvers}}[0],'7.3')==2) {
        $sys->cmd("export PATH=/bin:/sbin:\$PATH; /etc/init.d/vxglm stop");
        return 1;
    }
    $sys->padv->systemctl_stop_service($sys, $proc->{proc});
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::vxglm731::SLES12x8664;
@Proc::vxglm731::SLES12x8664::ISA = qw(Proc::vxglm731::RHEL7x8664);

package Proc::vxglm731::SunOS;
@Proc::vxglm731::SunOS::ISA = qw(Proc::vxglm731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    my ($mn,$driver);
    $driver=$proc->{proc};
    $mn=$sys->get_drivers( $driver);
    if ($mn eq '') {
        $sys->cmd("_cmd_adddrv -f -m '* 0640 root sys' $driver 2>/dev/null; _cmd_modload -p drv/$driver 2>/dev/null");
    }
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn = $sys->get_drivers($proc->{proc});
    return 1 if ($rtn ne '');
    return 0;
}

1;
