#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxgms731::Common;
@Proc::vxgms731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxgms';
    $proc->{name}='vxgms name';
    $proc->{desc}='vxgms description';
    $proc->{fatal}=1;
    $proc->{initconf}='/etc/vx/gms.config'; 
    return;
}

sub enable_sys {
    my ($proc,$sys) = @_;
    my ($file,$conf,$rootpath);
    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    $conf =~ s/GMS_START\s*=\s*0/GMS_START=1/mx;
    $conf .= "\n";
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->rm("$file.prev");
    }

    if ($sys->{plat} eq 'AIX') {
        $sys->cmd('/etc/methods/gmskextadm config');
    } elsif ($sys->{plat} eq 'SunOS') {
        $sys->cmd('pkg change-facet --no-backup-be facet.vxgms.start=True'); 
    }

    return 1;

}

sub correct_config_file_sys {
    my ($proc,$sys,$file)=@_;
    my ($conf,$dir);
    
    #This function works for the scenario: using native installer to install VRTSgms, gms.config won't be created.
    if ($sys->exists("$file")) {
        $conf=$sys->catfile($file);
        if ($conf =~ /GMS_START/mx) {
            return 1; 
        } else {
            $sys->appendfile('GMS_START=0', $file);
        }
    } else {
        $dir=EDRu::dirname($file);
        $sys->cmd("_cmd_mkdir -p $dir");
        $sys->cmd("echo 'GMS_START=0' > $file 2>/dev/null");
    }

    if ($sys->{plat} eq 'SunOS') { 
        $sys->cmd('pkg change-facet --no-backup-be facet.vxgms.start=False'); 
    }

    $proc->stop_sys($sys);

    return 1;
}

sub is_enabled_sys {
    my ($proc,$sys)=@_;
    my ($conf,$rootpath,$initconf);
    $rootpath = Cfg::opt('rootpath') || '';
    $initconf = $rootpath . $proc->{initconf};
    $proc->correct_config_file_sys($sys,$initconf);
    $conf = $sys->cmd("unset GMS_START 2>/dev/null >/dev/null ; . $initconf 2>/dev/null >/dev/null ; echo \$GMS_START");
    chomp $conf;
    if ($conf eq '0'){
        return 0;
    } else {
        return 1;
    }
}

sub disable_sys {
    my ($proc,$sys) = @_;
    my ($file,$conf,$rootpath);
    $rootpath = Cfg::opt('rootpath') || '';
    $file= $rootpath . $proc->{initconf};
    $conf=$sys->catfile($file);
    return 0 unless ($conf);
    $conf =~ s/GMS_START\s*=\s*1/GMS_START=0/mx;
    $conf .= "\n";
    $sys->movefile($file,"$file.prev");
    $sys->writefile($conf,$file);
    if ($sys->exists("$file")) {
        $sys->rm("$file.prev");
    }

    if ($sys->{plat} eq 'AIX') {
        $sys->cmd('/etc/methods/gmskextadm unconfig');
    } elsif ($sys->{plat} eq 'SunOS') {
        $sys->cmd('pkg change-facet --no-backup-be facet.vxgms.start=False');
    }
    return 1;
}

package Proc::vxgms731::AIX;
@Proc::vxgms731::AIX::ISA = qw(Proc::vxgms731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/gmskextadm load');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/methods/gmskextadm unload') if($sys->exists('/etc/methods/gmskextadm'));
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    return 0 unless $sys->exists('/etc/methods/gmskextadm');
    my $gmsconf = $sys->cmd("/etc/methods/gmskextadm status | _cmd_grep 'not loaded'");
    return 1 if ($gmsconf eq '');
    return 0;
}

package Proc::vxgms731::HPUX;
@Proc::vxgms731::HPUX::ISA = qw(Proc::vxgms731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/sbin/init.d/vxgms start');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/sbin/init.d/vxgms stop');
    $sys->cmd('_cmd_kcmodule vxgms=unused');
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $pids=$sys->proc_pids('vxgmsd');
    return 1 if ($#$pids>=0);
    return 0;
}

package Proc::vxgms731::Linux;
@Proc::vxgms731::Linux::ISA = qw(Proc::vxgms731::Common);

sub init_plat {
    my $proc=shift;
    $proc->{start_period}=20;
    return;
}

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/init.d/vxgms start');
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->cmd('/etc/init.d/vxgms stop');
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn = $sys->get_drivers($proc->{proc});
    return 1 if ($rtn ne '');
    return 0;
}

package Proc::vxgms731::RHEL7x8664;
@Proc::vxgms731::RHEL7x8664::ISA = qw(Proc::vxgms731::Linux);

sub start_sys {
    my ($proc,$sys)=@_;
    return $sys->padv->systemctl_start_service($sys, $proc->{proc});
}

sub stop_sys {
    my ($proc,$sys)=@_;
    if (EDRu::compvers(${$sys->{prodvers}}[0],'7.3')==2) {
        $sys->cmd("/etc/init.d/vxgms stop");
        return 1;
    }
    $sys->padv->systemctl_stop_service($sys, $proc->{proc});
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::vxgms731::SLES12x8664;
@Proc::vxgms731::SLES12x8664::ISA = qw(Proc::vxgms731::RHEL7x8664);

package Proc::vxgms731::SunOS;
@Proc::vxgms731::SunOS::ISA = qw(Proc::vxgms731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    my ($mn,$driver);
    $driver=$proc->{proc};
    $mn=$sys->get_drivers( $driver);
    if ($mn eq '') {
        $sys->cmd("_cmd_adddrv -f -m '* 0640 root sys' $driver 2>/dev/null; _cmd_modload -p drv/$driver 2>/dev/null");
    }
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->unload_driver($proc->{proc});
    return 1;
}

sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn = $sys->get_drivers($proc->{proc});
    return 1 if ($rtn ne '');
    return 0;
    # TODO: start/stop/stat ?
}

1;
