#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxio731::Common;
@Proc::vxio731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxio';
    $proc->{name}='vxio name';
    $proc->{desc}='vxio description';
    $proc->{fatal}=1;
    return;
}

sub start_sys {
    return 1;
}

# unload from kernel
sub stop_sys {
    my ($proc,$sys)=@_;
    my ($vxspec);
    $sys->unload_driver($proc->{proc});
    if ($proc->check_sys($sys)) {
        # try to unload vxspec and then unload vxio again.
        $vxspec = $sys->proc('vxspec731');
        if ($vxspec && $vxspec->check_sys($sys)) {
            $vxspec->stop_sys($sys);
            sleep 5;
        }
        $sys->unload_driver($proc->{proc});
    }
    return 1;
}

# check whether mod is loaded
# 1 means loaded, 0 means not loaded
sub check_sys {
    my ($proc,$sys)=@_;
    my $rtn=$sys->get_drivers($proc->{proc});
    return ($rtn?1:0);
}

package Proc::vxio731::AIX;
@Proc::vxio731::AIX::ISA = qw(Proc::vxio731::Common);

package Proc::vxio731::HPUX;
@Proc::vxio731::HPUX::ISA = qw(Proc::vxio731::Common);

package Proc::vxio731::Linux;
@Proc::vxio731::Linux::ISA = qw(Proc::vxio731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->load_driver($proc->{proc});
    return 1;
}

package Proc::vxio731::SunOS;
@Proc::vxio731::SunOS::ISA = qw(Proc::vxio731::Common);

sub start_sys {
    my ($proc,$sys)=@_;
    $sys->cmd("_cmd_adddrv $proc->{proc} 2>/dev/null");
    $sys->padv->load_driver_sys($sys,$proc->{proc});
    $sys->padv->add_etc_system_entry_sys($sys,[$proc->{proc}]);
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $proc->SUPER::stop_sys($sys);
    $sys->cmd("_cmd_rmdrv $proc->{proc} 2>/dev/null");
    $sys->padv->rm_etc_system_entry_sys($sys,[$proc->{proc}]);
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures in /var/adm/messages:\n");
    $sys->cmd('_cmd_tail -10 /var/adm/messages 2>/dev/null');
    return 1;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check stop failures in /var/adm/messages:\n");
    $sys->cmd('_cmd_tail -10 /var/adm/messages 2>/dev/null');
    return 1;
}

1;
