#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Proc::vxrecover731::Common;
@Proc::vxrecover731::Common::ISA = qw(Proc);

sub init_common {
    my $proc=shift;
    $proc->{proc}='vxvm-recover';
    $proc->{name}='recover';
    $proc->{desc}='start recover relevant processes';
    $proc->{start_period}=20;
    $proc->{stop_period}=20;
    $proc->{multisystemserialstart}=1;
    $proc->{procs}=[qw(vxesd vxrelocd vxrecover vxcached vxvvrstatd vxvvrsecdgd vxconfigbackupd vxvollogd vxloggerd)];
    return;
}

sub check_sys {
    my ($proc,$sys,$state)=@_;
    my ($vxvmproc,$pids,$rc,$rtn);
    for $vxvmproc (@{$proc->{procs}}) {
        $pids=$sys->proc_pids($vxvmproc);
        $rc = ($#$pids == -1) ? 0 : 1;
        $rtn ||= $rc;
    }
    if ($rtn) {
        return 1;
    } else {
        return 0;
    }
}

package Proc::vxrecover731::AIX;
@Proc::vxrecover731::AIX::ISA = qw(Proc::vxrecover731::Common);

package Proc::vxrecover731::HPUX;
@Proc::vxrecover731::HPUX::ISA = qw(Proc::vxrecover731::Common);

package Proc::vxrecover731::Linux;
@Proc::vxrecover731::Linux::ISA = qw(Proc::vxrecover731::Common);

package Proc::vxrecover731::RHEL7x8664;
@Proc::vxrecover731::RHEL7x8664::ISA = qw(Proc::vxrecover731::Linux);

# From 7.3, we would call VM script to start/stop process/module on RHEL7&SLES12
sub start_sys {
    my ($proc,$sys)=@_;
    $sys->padv->systemctl_start_service($sys, $proc->{proc});

    sleep 5;
    return 1;
}

sub stop_sys {
    my ($proc,$sys)=@_;
    $sys->padv->systemctl_stop_service($sys, $proc->{proc});

    sleep 5;
    return 1;
}

sub start_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check start failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

sub stop_failed_sys {
    my ($proc,$sys)=@_;

    Msg::log("## Check stop failures on system $sys->{sys}:\n");
    $sys->cmd("_cmd_systemctl status $proc->{proc}");

    return 1;
}

package Proc::vxrecover731::SLES12x8664;
@Proc::vxrecover731::SLES12x8664::ISA = qw(Proc::vxrecover731::RHEL7x8664);

package Proc::vxrecover731::SunOS;
@Proc::vxrecover731::SunOS::ISA = qw(Proc::vxrecover731::Common);

1;

