#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Prod::AVAILABILITY731::Common;
@Prod::AVAILABILITY731::Common::ISA = qw(Prod);
sub init_common {
    my ($prod) = @_;
    $prod->{prod}="AVAILABILITY";
    $prod->{abbr}=Msg::new("InfoScale Availability")->{msg};
    $prod->{vers}='7.3.1.000';
    $prod->{eula}='EULA.pdf';
    $prod->{name}=Msg::new("Veritas InfoScale Availability")->{msg};
    $prod->{mainpkg}='VRTSvcs731';
    $prod->{lic_names}=['Veritas InfoScale Availability'];
    $prod->{lic_keyless_name} = 'AVAILABILITY';
    $prod->{licsuperprods} = [ qw(ENTERPRISE731) ];
    $prod->{upgradevers}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{responsefileupgradeok}=1;
    $prod->{subprods}=[qw(VCS731)];
    $prod->{allcomponent}=[qw(VCS731)];
    $prod->{defaultcomponent}='VCS731';
    $prod->{has_config} = 1;
    return;
}

sub description {
    my ($msg);

    $msg=Msg::new("Veritas InfoScale Availability helps keep organizations' information available and critical business services up and running with a robust software-defined approach. Organizations can innovate and gain cost benefits of physical and virtual across commodity server deployments. Maximum IT service continuity is ensured at all times, moving resiliency from the infrastructure layer to the application layer.");

    $msg->print;
    return;
}

sub perform_task_sys {
    my ($prod,$sys,$task) = @_;
    return $prod->perform_component_task_sys($sys,$task);
}

sub perform_task {
    my ($prod,$task) = @_;
    return $prod->perform_component_task($task);
}

sub set_pkgs {
    my($prod,$rel,$vcs,$category,@categories,@extrapkgs);
    $prod=shift;

    $rel=$prod->rel;
    $vcs=$prod->prod('VCS731');

    #todo: add VRTSvcsvmw
    @categories=qw(allpkgs);
    @extrapkgs=qw(VRTSfssdk731);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$vcs->{$category}});
        $prod->{$category}=EDRu::arrdel($prod->{$category},@extrapkgs);
    }

    @categories=qw(obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_previous_releases_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$vcs->{$category}});
    }
    $prod->{installonupgradepkgs} = $prod->{allpkgs};
    return $prod->{allpkgs};
}

sub responsefile_prestart_config {
    my ($prod) = @_;
    my ($cfg);
    $cfg=Obj::cfg();
    if ($cfg->{opt}{activecomponent}) {
        $prod->{activecomponent}=$cfg->{opt}{activecomponent};
    }
    $prod->perform_task('responsefile_prestart_config');
    return;
}

sub licensed_sys {
    my ($prod,$sys)=@_;
    my $rel = Obj::rel();
    return $rel->prod_licensed_sys($sys);
}

sub get_tunables {
    my ($prod)=@_;
    my ($subprodobj,$subprodtunables,@tunables);
    for my $subprod (@{$prod->{subprods}}) {
        $subprodobj=$prod->prod($subprod);
        $subprodtunables=$subprodobj->get_supported_tunables();
        push(@tunables,@{$subprodtunables});
    }
    return EDRu::arruniq(@tunables) || [];
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;  
    my ($pkg,$pkgvers,$mpvers,$vers,$prodvers);

    #install VRTSllt VRTSvcsea packages will be regarded as AVAILABILITY installed.
    my @pkgs=qw(VRTSllt731 VRTSvcsea731);
    @pkgs=qw(VRTSvcsk731) if (Obj::rel()->{simple} && !Cfg::opt('upgrade'));
    for my $pkgname (@pkgs) {
        $pkg=$sys->pkg($pkgname);
        $pkgvers=$pkg->version_sys($sys,$force_flag);
        $mpvers=$prod->{vers} if ($prod->check_installed_patches_sys($sys,$pkgvers)); 
        $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag); 
        return '' unless ($pkgvers);
        # pick up the lowest version as the product version
        $vers = ($mpvers || $pkgvers);

        #since aix 7.0.1 doesn't have VRTSvcsea patch installed, don't check VRTSvceea in this case
        next if ($sys->aix() && ($pkg->{pkg} eq 'VRTSvcsea') && (0 == EDRu::compvers($prodvers, '7.0.1', 3))); 
        if (!$prodvers || (EDRu::compvers($prodvers,$vers,4)==1)) {
            $prodvers = $vers;
        }
    }
    
    return '' if (2 == EDRu::compvers($prodvers,'7.0'));
    return $prodvers;
}

# When calling Prod::stopprocs, it will retrieve packages from the new product,
# causing more product processes been added into this list.
sub stopprocs {  
    my ($prod)=@_; 
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs($prod);
    return $procs;
}

sub stopprocs_sys { 
    my ($prod,$sys)=@_;
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs_sys($prod,$sys);
    return $procs;
}

sub cli_prod_option {
    my $prod=shift;
    $prod->perform_task('cli_prod_option');
    return;
}

sub precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_sys($prod,$sys);
}

sub configure_precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::configure_precheck_sys($prod,$sys);
}

sub precheck_task {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_task($prod);
}

sub addnode_poststart {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::addnode_poststart($prod);
}

sub delnode_precheck {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::delnode_precheck($prod);
}

sub default_systemnames {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::default_systemnames($prod);
}

package Prod::AVAILABILITY731::AIX;
@Prod::AVAILABILITY731::AIX::ISA = qw(Prod::AVAILABILITY731::Common);

package Prod::AVAILABILITY731::HPUX;
@Prod::AVAILABILITY731::HPUX::ISA = qw(Prod::AVAILABILITY731::Common);

package Prod::AVAILABILITY731::Linux;
@Prod::AVAILABILITY731::Linux::ISA = qw(Prod::AVAILABILITY731::Common);

package Prod::AVAILABILITY731::SunOS;
@Prod::AVAILABILITY731::SunOS::ISA = qw(Prod::AVAILABILITY731::Common);

package Prod::AVAILABILITY731::Sol11sparc;
@Prod::AVAILABILITY731::Sol11sparc::ISA = qw(Prod::AVAILABILITY731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(VCS731)];
    $prod->{upgradevers}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{allcomponent}=[qw(VCS731)];
    return;
}

package Prod::AVAILABILITY731::Sol12sparc;
@Prod::AVAILABILITY731::Sol12sparc::ISA = qw(Prod::AVAILABILITY731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(VCS731)];
    $prod->{upgradevers}=[qw(7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.2 7.3 7.3.1)];
    $prod->{allcomponent}=[qw(VCS731)];
    return;
}

package Prod::AVAILABILITY731::Sol11x64;
@Prod::AVAILABILITY731::Sol11x64::ISA = qw(Prod::AVAILABILITY731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(VCS731)];
    $prod->{upgradevers}=[qw(6.0.10 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(6.0.10 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{allcomponent}=[qw(VCS731)];
    $prod->{obsoleted_but_need_refresh_when_upgrade_pkgs} = [];
    return;
}

package Prod::AVAILABILITY731::Sol12x64;
@Prod::AVAILABILITY731::Sol12x64::ISA = qw(Prod::AVAILABILITY731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(VCS731)];
    $prod->{upgradevers}=[qw(7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.2 7.3 7.3.1)];
    $prod->{allcomponent}=[qw(VCS731)];
    $prod->{obsoleted_but_need_refresh_when_upgrade_pkgs} = [];
    return;
}

1;
