#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Prod::ENTERPRISE731::Common;
@Prod::ENTERPRISE731::Common::ISA = qw(Prod);
sub init_common {
    my ($prod) = @_;
    $prod->{prod}="ENTERPRISE";
    $prod->{abbr}=Msg::new("InfoScale Enterprise")->{msg};
    $prod->{vers}='7.3.1.000';
    $prod->{eula}='EULA.pdf';
    $prod->{name}=Msg::new("Veritas InfoScale Enterprise")->{msg};
    $prod->{mainpkg}='VRTSdbac731';
    $prod->{lic_names}=['Veritas InfoScale Enterprise'];
    $prod->{lic_keyless_name} = 'ENTERPRISE';
    $prod->{upgradevers}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{responsefileupgradeok}=1;
    $prod->{subprods}=[qw(VCS731 SF731 SFHA731 SFCFSHA731 SFRAC731 SFSYBASECE731)];
    $prod->{allcomponent}=[qw(SFRAC731)];
    $prod->{defaultcomponent}='SFRAC731';
    $prod->{has_config} = 1;
    
    if (Obj::rel()->{simple}) {
        $prod->{mainpkg}='VRTScfs731';
    }

    return;
}

sub description {
    my ($msg);

    $msg=Msg::new("Veritas InfoScale Enterprise addresses enterprise IT service continuity needs.  It provides resiliency and software defined storage for critical services across your datacenter infrastructure.  Realize better ROI and unlock high performance by integrating next-generation storage technologies. The solution provides high availability and disaster recovery for complex multi-tiered applications across any distance in physical and virtual environments.");

    $msg->print;
    return;
}

sub perform_task_sys {
    my ($prod,$sys,$task) = @_;
    return $prod->perform_component_task_sys($sys,$task);
}

sub perform_task {
    my ($prod,$task) = @_;
    return $prod->perform_component_task($task);
}

sub set_pkgs {
    my($prod,$rel,$sfrac,$category,@categories,@extrapkgs);
    $prod=shift;

    $rel=$prod->rel;
    $sfrac=$prod->prod('SFRAC731');

    #todo: add VRTSvcsvmw into the list somewhere
    @categories=qw(allpkgs);
    @extrapkgs=qw(VRTSfssdk731);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$sfrac->{$category}});
        $prod->{$category}=EDRu::arrdel($prod->{$category},@extrapkgs);
    }

    @categories=qw(obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_previous_releases_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$sfrac->{$category}});
    }
    #e3780989,SFRAC's obsoleted_previous_releases_pkgs doesn't include VRTSsvs.
    push(@{$prod->{obsoleted_previous_releases_pkgs}},'VRTSsvs731');
    $prod->{installonupgradepkgs} = $prod->{allpkgs};
    return $prod->{allpkgs};
}

sub responsefile_prestart_config {
    my ($prod) = @_;
    my ($cfg);
    $cfg=Obj::cfg();
    if ($cfg->{opt}{activecomponent}) {
        $prod->{activecomponent}=$cfg->{opt}{activecomponent};
    }
    $prod->perform_task('responsefile_prestart_config');
    return;
}

sub licensed_sys {
    my ($prod,$sys)=@_;
    my $rel = Obj::rel();
    return $rel->prod_licensed_sys($sys);
}

sub poststart_configure_sys {
    my ($prod,$sys)=@_; 
    return $prod->perform_task_sys($sys, 'poststart_configure_sys'); 
}

sub get_tunables {
    my ($prod)=@_;
    my ($subprodobj,$subprodtunables,@tunables);
    for my $subprod (@{$prod->{subprods}}) {
        $subprodobj=$prod->prod($subprod);
        $subprodtunables=$subprodobj->get_supported_tunables();
        push(@tunables,@{$subprodtunables});
    }
    return EDRu::arruniq(@tunables) || [];
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;  
    my ($pkg,$pkgvers,$mpvers,$vers,$prodvers);

    #VRTSllt VRTSvcsea VRTSvxvm
    #install the 3 packages will be regarded as ENTERPRISE installed.
    my @pkgs=qw(VRTSllt731 VRTSvcsea731 VRTSvxvm731);
    @pkgs=qw(VRTSvcsk731 VRTSvxvm731) if (Obj::rel()->{simple} && !Cfg::opt('upgrade'));
    for my $pkgname (@pkgs) {
        $pkg=$sys->pkg($pkgname);
        $pkgvers=$pkg->version_sys($sys,$force_flag);
        $mpvers=$prod->{vers} if ($prod->check_installed_patches_sys($sys,$pkgvers)); 
        $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag); 
        return '' unless ($pkgvers);
        # pick up the lowest version as the product version
        $vers = ($mpvers || $pkgvers);
        # Skip checking VRTSvcsea in RU phase because VRTSvcsea is non-kernel package
        next if (Cfg::opt('rolling_upgrade') && ($pkg->{pkg} eq 'VRTSvcsea') && $vers);

        #since aix 7.0.1 doesn't have VRTSvcsea patch installed, don't check VRTSvceea in this case  
        next if ($sys->aix() && ($pkg->{pkg} eq 'VRTSvcsea') && (0 == EDRu::compvers($prodvers, '7.0.1', 3))); 
        if (!$prodvers || (EDRu::compvers($prodvers,$vers,4)==1)) {
            $prodvers = $vers;
        }
    }
    
    return '' if (2 == EDRu::compvers($prodvers,'7.0'));  
    return $prodvers;
}

# When calling Prod::stopprocs, it will retrieve packages from the new product,
# causing more product processes been added into this list.
sub stopprocs {  
    my ($prod)=@_; 
    my ($subprod,$subprodprocs,$component,$procs);
    $procs=[];
    if ($prod->{previouscomponent} && @{$prod->{previouscomponent}}) {
        for my $component (@{$prod->{previouscomponent}}) {
            $subprod=$prod->prod($component);
            $subprodprocs=$subprod->stopprocs();
            push(@$procs,@$subprodprocs) if ($subprodprocs);
        }
        $procs=EDRu::arruniq(@$procs);
    } else {
        $subprod = $prod->prod($prod->{defaultcomponent});
        $procs=$subprod->stopprocs();
    }
    return $procs;
}

sub stopprocs_sys { 
    my ($prod,$sys)=@_;
    my ($subprod,$subprodprocs,$component,$procs);
    $procs=[];
    if ($prod->{previouscomponent} && @{$prod->{previouscomponent}}) {
        for my $component (@{$prod->{previouscomponent}}) {
            $subprod=$prod->prod($component);
            $subprodprocs=$subprod->stopprocs_sys($sys);
            push(@$procs,@$subprodprocs) if ($subprodprocs);
        }
        $procs=EDRu::arruniq(@$procs);
    } else {
        $subprod = $prod->prod($prod->{defaultcomponent});
        $procs=$subprod->stopprocs_sys($sys);
    }
    return $procs;
}

sub postinstall_sys {
    my ($prod,$sys)=@_;
    my ($sfsybasece,$vcs,$sf);
    
    if (EDRu::inarr('SFSYBASECE731', @{$prod->{subprods}})) {
        $sfsybasece = $prod->prod('SFSYBASECE731');
        $sfsybasece->copy_resstatechange_sys($sys);
    } 
    #For upsell, postinstall_sys should execute different component based on installed product.
    if (Cfg::opt(qw(install)) && $sys->{upgradeprod}) {
        if($sys->{upgradeprod}[0] =~ /FOUNDATION/mi) {
            $vcs = $prod->prod('VCS731');
            $vcs->postinstall_sys($sys) if ($vcs->can('postinstall_sys'));
            return;
        } elsif ($sys->{upgradeprod}[0] =~ /STORAGE/mi) {
            return;
        } elsif ($sys->{upgradeprod}[0] =~ /AVAILABILITY/mi) {
            $sf = $prod->prod('SF731');
            $sf->postinstall_sys($sys) if ($sf->can('postinstall_sys'));
            return;
        }
    }
    $prod->perform_component_task_sys($sys,'postinstall_sys',$prod->{allcomponent});
    return;
}

sub upgrade_postinstall_sys {
    my ($prod,$sys)=@_;
    my ($rel,$proc);

    $rel=$prod->rel;
    $rel->perform_upgrade_or_install_task_sys($sys,$prod,'upgrade_postinstall_sys');
    #e3780193:disable vcsmm if previous component is not SFRAC.
    my $activecomponent=$rel->get_activecomponent();
    if ($activecomponent !~ /SFRAC/mi) {
        $proc = $sys->proc('vcsmm731');
        $proc->disable_sys($sys);
    }
    
    return;
}

sub cli_prod_option {
    my $prod=shift;

    $prod->perform_task('cli_prod_option');
    return;
}

sub precheck_sys {
    my ($prod,$sys)=@_;
    if ($prod->{activecomponent}) {
        $prod=$prod->prod("${$prod->{activecomponent}}[0]");
        $prod->precheck_sys($sys);
    }
    return;
}

sub configure_precheck_sys {
    my ($prod,$sys)=@_;
    if ($prod->{activecomponent}) {
        $prod=$prod->prod("${$prod->{activecomponent}}[0]");
        $prod->configure_precheck_sys($sys);
    }
    return;
}

sub precheck_task {
    my ($prod)=@_;
    if ($prod->{activecomponent}) {
        $prod=$prod->prod("${$prod->{activecomponent}}[0]");
        $prod->precheck_task();
    }
    return 1;
}

sub addnode_poststart {
    my $prod = shift;
    if ($prod->{activecomponent}) {
        $prod=$prod->prod("${$prod->{activecomponent}}[0]");
        $prod->addnode_poststart();
    }
    return;
}

sub delnode_precheck {
    my $prod = shift;
    if ($prod->{activecomponent}) {
        $prod=$prod->prod("${$prod->{activecomponent}}[0]");
        $prod->delnode_precheck();
    }
    return;
}

sub default_systemnames {
    my $prod = shift;
    my ($component,$subprod);
    $component = $prod->{defaultcomponent};
    $subprod = $prod->prod($component);
    return $subprod->default_systemnames;
}

package Prod::ENTERPRISE731::AIX;
@Prod::ENTERPRISE731::AIX::ISA = qw(Prod::ENTERPRISE731::Common);

sub init_plat {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    return;
}

sub memory_size_sys {
    my ($prod,$sys) = @_;
    $prod->SUPER::memory_size_sys($sys);
    return Prod::FS731::AIX::memory_size_sys($prod,$sys);
}

package Prod::ENTERPRISE731::HPUX;
@Prod::ENTERPRISE731::HPUX::ISA = qw(Prod::ENTERPRISE731::Common);

package Prod::ENTERPRISE731::Linux;
@Prod::ENTERPRISE731::Linux::ISA = qw(Prod::ENTERPRISE731::Common);

package Prod::ENTERPRISE731::RHEL7x8664;
@Prod::ENTERPRISE731::RHEL7x8664::ISA = qw(Prod::ENTERPRISE731::Linux);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    return;
}

package Prod::ENTERPRISE731::SLES11x8664;
@Prod::ENTERPRISE731::SLES11x8664::ISA = qw(Prod::ENTERPRISE731::Linux);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    return;
}

package Prod::ENTERPRISE731::CentOS6x8664;
@Prod::ENTERPRISE731::CentOS6x8664::ISA = qw(Prod::ENTERPRISE731::Linux);

package Prod::ENTERPRISE731::CentOS7x8664;
@Prod::ENTERPRISE731::CentOS7x8664::ISA = qw(Prod::ENTERPRISE731::Linux);

package Prod::ENTERPRISE731::SLES12x8664;
@Prod::ENTERPRISE731::SLES12x8664::ISA = qw(Prod::ENTERPRISE731::Linux);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    return;
}

package Prod::ENTERPRISE731::SunOS;
@Prod::ENTERPRISE731::SunOS::ISA = qw(Prod::ENTERPRISE731::Common);

package Prod::ENTERPRISE731::Sol11sparc;
@Prod::ENTERPRISE731::Sol11sparc::ISA = qw(Prod::ENTERPRISE731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    $prod->{upgradevers}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{minimal_cpu_speed_requirment} = '1 GHz';
    $prod->{allcomponent}=[qw(SFRAC731)];
    return;
}

package Prod::ENTERPRISE731::Sol12sparc;
@Prod::ENTERPRISE731::Sol12sparc::ISA = qw(Prod::ENTERPRISE731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=EDRu::arrdel($prod->{subprods},qw/SFSYBASECE731/);
    $prod->{upgradevers}=[qw(7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.2 7.3 7.3.1)];
    $prod->{minimal_cpu_speed_requirment} = '1 GHz';
    $prod->{allcomponent}=[qw(SFRAC731)];
    return;
}

package Prod::ENTERPRISE731::Sol11x64;
@Prod::ENTERPRISE731::Sol11x64::ISA = qw(Prod::ENTERPRISE731::SunOS);

package Prod::ENTERPRISE731::Sol12x64;
@Prod::ENTERPRISE731::Sol12x64::ISA = qw(Prod::ENTERPRISE731::SunOS);

1;
