#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

# 1. No webinstaller.
# 2. Only support one package set --> allpkgs
# 3.
# 4.

package Prod::FOUNDATION731::Common;
@Prod::FOUNDATION731::Common::ISA = qw(Prod);
sub init_common {
    my ($prod) = @_;
    $prod->{prod}="FOUNDATION";
    $prod->{abbr}=Msg::new("InfoScale Foundation")->{msg};
    $prod->{vers}='7.3.1.000';
    $prod->{eula}='EULA.pdf';
    $prod->{name}=Msg::new("Veritas InfoScale Foundation")->{msg};
    $prod->{mainpkg}='VRTSvxvm731';
    $prod->{lic_names}=['Veritas InfoScale Foundation'];
    $prod->{lic_keyless_name} = 'FOUNDATION';
    $prod->{licsuperprods}=[ qw(STORAGE731) ];
    $prod->{upgradevers}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{responsefileupgradeok}=1;
    $prod->{subprods}=[qw(SF731)];
    $prod->{allcomponent}=[qw(SF731)];
    $prod->{defaultcomponent}='SF731';
    $prod->{has_config} = 1;
    return;
}

sub perform_task_sys {
    my ($prod,$sys,$task) = @_;
    return $prod->perform_component_task_sys($sys,$task);
}

sub perform_task {
    my ($prod,$task) = @_;
    return $prod->perform_component_task($task);
}

sub set_pkgs {
    my($prod,$rel,$sf,$category,@categories,@extrapkgs);
    $prod=shift;

    $rel=$prod->rel;
    $sf=$prod->prod('SF731');

    @categories=qw(allpkgs);
    @extrapkgs=qw(VRTSdbed731 VRTSfsadv731 VRTSodm731 VRTSfssdk731);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$sf->{$category}});
        $prod->{$category}=EDRu::arrdel($prod->{$category},@extrapkgs);
    }

    @categories=qw(obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_previous_releases_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$sf->{$category}});
    }
    $prod->{installonupgradepkgs} = $prod->{allpkgs};
    return $prod->{allpkgs};
}

sub description {
    my ($msg);

    $msg=Msg::new("Veritas InfoScale Foundation is specifically designed for an enterprise' edge-tier, departmental, and test/development systems. InfoScale Foundation combines the industry-leading File System and Volume Manager technology, and delivers a complete solution for heterogeneous online storage management while increasing storage utilization and enhancing storage I/O path availability.");

    $msg->print;
    return;
}

sub licensed_sys {
    my ($prod,$sys)=@_;
    my $rel=Obj::rel();
    return $rel->prod_licensed_sys($sys);
}

sub get_tunables {
    my ($prod)=@_;
    my ($subprodobj,$subprodtunables,@tunables);
    for my $subprod (@{$prod->{subprods}}) {
        $subprodobj=$prod->prod($subprod);
        $subprodtunables=$subprodobj->get_supported_tunables();
        push(@tunables,@{$subprodtunables});
    }
    return EDRu::arruniq(@tunables) || [];
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;  
    my ($pkg,$pkgvers,$mpvers,$vers,@chk_pkgs);

    @chk_pkgs = qw(VRTSvxvm731);
    for my $pkgname (@chk_pkgs) {
        $pkg=$sys->pkg($pkgname);
        $pkgvers=$pkg->version_sys($sys,$force_flag);
        $mpvers=$prod->{vers} if ($prod->check_installed_patches_sys($sys,$pkgvers)); 
        $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag); 
        return '' unless ($pkgvers);
    }
        
    $vers = ($mpvers || $pkgvers);
    return '' if (2 == EDRu::compvers($vers,'7.0'));  
    return $vers;
}

sub poststart_configure_sys {
    my ($prod,$sys)=@_;
    return $prod->perform_task_sys($sys, 'poststart_configure_sys');
}

sub stopprocs {
    my ($prod)=@_;
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs($prod);
    return $procs;
}

sub stopprocs_sys {
    my ($prod,$sys)=@_;
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs_sys($prod,$sys);
    return $procs;
}

sub cli_prod_option {
    my $prod=shift;
    $prod->perform_task('cli_prod_option');
    return;
}

sub precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_sys($prod,$sys);
}

sub configure_precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::configure_precheck_sys($prod,$sys);
}

sub precheck_task {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_task($prod);
}

sub default_systemnames {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::default_systemnames($prod);
}

package Prod::FOUNDATION731::AIX;
@Prod::FOUNDATION731::AIX::ISA = qw(Prod::FOUNDATION731::Common);

sub memory_size_sys {
    my ($prod,$sys) = @_;
    $prod->SUPER::memory_size_sys($sys);
    return Prod::FS731::AIX::memory_size_sys($prod,$sys);
}

package Prod::FOUNDATION731::HPUX;
@Prod::FOUNDATION731::HPUX::ISA = qw(Prod::FOUNDATION731::Common);

package Prod::FOUNDATION731::Linux;
@Prod::FOUNDATION731::Linux::ISA = qw(Prod::FOUNDATION731::Common);

package Prod::FOUNDATION731::SunOS;
@Prod::FOUNDATION731::SunOS::ISA = qw(Prod::FOUNDATION731::Common);

package Prod::FOUNDATION731::Sol11sparc;
@Prod::FOUNDATION731::Sol11sparc::ISA = qw(Prod::FOUNDATION731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(SF731)];
    $prod->{upgradevers}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{minimal_cpu_speed_requirment} = '1 GHz';
    $prod->{allcomponent}=[qw(SF731)];
    return;
}

package Prod::FOUNDATION731::Sol12sparc;
@Prod::FOUNDATION731::Sol12sparc::ISA = qw(Prod::FOUNDATION731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{subprods}=[qw(SF731)];
    $prod->{upgradevers}=[qw(7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.2 7.3 7.3.1)];
    $prod->{minimal_cpu_speed_requirment} = '1 GHz';
    $prod->{allcomponent}=[qw(SF731)];
    return;
}

package Prod::FOUNDATION731::Sol11x64;
@Prod::FOUNDATION731::Sol11x64::ISA = qw(Prod::FOUNDATION731::SunOS);

package Prod::FOUNDATION731::Sol12x64;
@Prod::FOUNDATION731::Sol12x64::ISA = qw(Prod::FOUNDATION731::SunOS);

1;
