#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Prod::SFCFSHA731::Common;
@Prod::SFCFSHA731::Common::ISA = qw(Prod);

sub init_common {
    my $prod = shift;
    $prod->{vers}='7.3.1.000';
    $prod->{default_cluster_protocol_version} = 190;

    if ($prod->sfcfsha) {
        $prod->{prod}='SFCFSHA';
        $prod->{abbr}='SFCFSHA';
        $prod->{name}=Msg::new("Storage Foundation Cluster File System HA")->{msg};
        $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];
        $prod->{proddir}='storage_foundation_cluster_file_system_ha';
        $prod->{eula}='EULA_SFHA_Ux_7.3.1.pdf';

        $prod->{installscript_prod}='SFCFSHA731';
        $prod->{installscript_name}='SFCFSHA';
        $prod->{mainpkg}='VRTScavf731';
        $prod->{licsuperprods}=[ qw(SFRAC731) ];

        $prod->{extra_mainpkgs}=[ qw(VRTSvxvm731 VRTSvxfs731 VRTSvcs731 VRTSllt731 VRTSgab731 VRTSvxfen731 VRTSvcsag731)];
        $prod->{superprods} = [qw(SFRAC731)];

    } elsif ($prod->svs) {
        $prod->{prod}='SVS';
        $prod->{abbr}='SVS';
        $prod->{name}=Msg::new("VirtualStore")->{msg};
        $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];
        $prod->{not_prompt_minpkgs_warning}=1;
        $prod->{proddir}='virtualstore';
        $prod->{eula}='EULA_VirtualStore_Ux_7.3.1.pdf';
        $prod->{mainpkg}='VRTSsvs731';
        $prod->{licsuperprods}=[ qw(ENTERPRISE731) ];

        $prod->{extra_mainpkgs}=[ qw(VRTSvxvm731 VRTSvxfs731 VRTSvcs731 VRTSllt731 VRTSgab731 VRTSvxfen731 VRTSvcsag731)];
        $prod->{obsoleted_by_merging}=1;

    } elsif ($prod->sfsybasece) {
        $prod->{prod}='SFSYBASECE';
        $prod->{abbr}='SFSYBASECE';
        $prod->{name}=Msg::new("Storage Foundation for Sybase ASE CE")->{msg};
        $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];
        $prod->{proddir}='storage_foundation_for_sybase_ce';
        $prod->{eula}='EULA_SF_Sybase_CE_Ux_7.3.1.pdf';

        $prod->{installscript_prod}='SFSYBASECE731';
        $prod->{installscript_name}='SFSYBASECE';
        $prod->{lic_names}=['Veritas Storage Foundation for Sybase ASE CE'];
        $prod->{licsuperprods}=[ qw(ENTERPRISE731) ];
        $prod->{mainpkg}='VRTScavf731';

        $prod->{extra_mainpkgs}=[ qw(VRTSvxvm731 VRTSvxfs731 VRTSvcs731 VRTSllt731 VRTSgab731 VRTSvxfen731 VRTSvcsag731)];

        $prod->{menu_sfsybasece} = ['config_cfs', 'config_fencing', 'config_sfsybasece', 'exit_cleanly'];

        my $msg = Msg::new("Configure Cluster File System");
        $msg->msg('sfsybasece_config_cfs');
        $msg = Msg::new("Configure I/O Fencing in Sybase Mode");
        $msg->msg('sfsybasece_config_fencing');
        $msg = Msg::new("Configure Sybase ASE CE Instance in VCS");
        $msg->msg('sfsybasece_config_sfsybasece');
        $msg = Msg::new("Exit $prod->{abbr} Configuration");
        $msg->msg('sfsybasece_exit_cleanly');

    } elsif ($prod->sfcfs) {
        $prod->{prod}='SFCFS';
        $prod->{abbr}='SFCFS';
        $prod->{name}=Msg::new("Storage Foundation Cluster File System")->{msg};
        $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];
        $prod->{proddir}='storage_foundation_cluster_file_system';
        $prod->{eula}='EULA_SFHA_Ux_7.3.1.pdf';

        $prod->{installscript_prod}='SFCFS731';
        $prod->{installscript_name}='SFCFS';
        $prod->{mainpkg}='VRTScavf731';
        $prod->{licsuperprods}=[ qw(STORAGE731 SFRAC731) ];

        $prod->{extra_mainpkgs}=[ qw(VRTSvxvm731 VRTSvxfs731 VRTSvcs731 VRTSllt731 VRTSgab731 VRTSvxfen731 VRTSvcsag731)];
        $prod->{superprods} = [qw(SFRAC731)];
    }
    if (Obj::rel()->{simple}) {
        $prod->{minpkgs}=$prod->{recpkgs}=$prod->{allpkgs}=[ qw(VRTScfs731) ];
        $prod->{mainpkg}='VRTScfs731';
    }
    $prod->{subprods}=[qw(VCS731 SF731 SFHA731)];

    $prod->{upgrade_flag_file}="/opt/VRTS/install/.$prod->{prod}.upgrade";

    if ($prod->svs) {
        $prod->{lic_names}=['VirtualStore'];
    } elsif ($prod->sfsybasece) {
        $prod->{lic_names}=['Veritas Storage Foundation for Sybase ASE CE'];
    } else {
        $prod->{lic_names}=['Storage Foundation for Cluster File System',
                            'SANPoint Foundation Suite',
                            'VirtualStore',
                            'Storage Foundation Cluster File System'];
    }

    $prod->{responsefileupgradeok}=1;
    $prod->{installonupgradepkgs} = [ qw(VRTSfsadv731 VRTSdbed731 VRTSamf731 VRTSsfmh731 VRTSvbs731 VRTSvcswiz731) ];

    $prod->{cfsbin}='/opt/VRTS/bin';

    $prod->{cfscluster_config_pending} = 1;

    $prod->{has_poststart_config} = 1;
    $prod->{has_config} = 1;
    $prod->{multisystemserialpoststart}=1;

    $prod->{minimal_cpu_number_requirment} = 2;
    $prod->{minimal_memory_requirment} = '2 GB';
    $prod->{minimal_swap_requirment} = '1 GB';

    my $padv=$prod->padv();
    $padv->{cmd}{cfscluster}="$prod->{cfsbin}/cfscluster";
    $padv->{cmd}{cfsdgadm}="$prod->{cfsbin}/cfsdgadm";
    $padv->{cmd}{cfsmntadm}="$prod->{cfsbin}/cfsmntadm";
    $padv->{cmd}{cfsmount}="$prod->{cfsbin}/cfsmount";
    $padv->{cmd}{cfsumount}="$prod->{cfsbin}/cfsumount";
    $padv->{cmd}{fsclustadm}="$prod->{cfsbin}/fsclustadm";
    $padv->{cmd}{vcsmmconfig}='/sbin/vcsmmconfig';
    $padv->{cmd}{odmclusteradm}='/opt/VRTSodm/bin/odmclustadm';
    $padv->{cmd}{cfsshare}="$prod->{cfsbin}/cfsshare";
    $padv->{cmd}{vxprint}='/usr/sbin/vxprint';

    return;
}

sub default_systemnames {
    my $prod=shift;
    my $vcs=$prod->prod('VCS731');
    return $vcs->default_systemnames;
}

sub adjust_ru_procs {
    my $prod=shift;
    if(Cfg::opt('upgrade_nonkernelpkgs')){
        my $procs=[qw(had731 CmdServer731)];
        return $procs;
    }
}

sub set_options {
    my ($prod,$options) = @_;
    my ($id,$option);

    $id=0;
    for my $option (@{$options}) {
        if($option) {
           if ($id == 0) {
               Cfg::set_opt(lc($option), 1);
               if (lc($option) eq 'vvr') {
                   Cfg::unset_opt('vvr');
                   Cfg::set_opt('vr', 1);
               }
           } elsif ($id == 1) {
               Cfg::set_opt('gco', 1);
           }
        }
        $id++;
    }
    return;
}

sub set_pkgs {
    my($prod,$rel,$vcs,$vm,$fs,$sf,$sf1,$category,@categories);
    $prod=shift;

    $rel=$prod->rel;
    $vm=$prod->prod('VM731');
    $fs=$prod->prod('FS731');
    $vcs=$prod->prod('VCS731');
    $sf=$prod->prod('SF731');

    #Ensure VRTSodm731 is after VRTSgsm51 in the pkg list.
    $sf1->{allpkgs} = EDRu::arrdel($sf->{allpkgs},'VRTSodm731');
    $sf1->{minpkgs} = $sf->{minpkgs};
    $sf1->{recpkgs} = EDRu::arrdel($sf->{recpkgs},'VRTSodm731');

    @categories=qw(minpkgs recpkgs allpkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$vm->{$category}},
                                        @{$fs->{$category}},
                                        @{$vcs->{$category}},
                                        @{$sf1->{$category}},
                                        @{$prod->{$category}});
       if($prod->sfsybasece()){
            $prod->{$category}=EDRu::arrdel($prod->{$category},'VRTSgms731');
            $prod->{$category}=EDRu::arrdel($prod->{$category},'VRTSodm731');
            $prod->{$category}=EDRu::arrdel($prod->{$category},'VRTScps731');
            $prod->{$category}=EDRu::arrdel($prod->{$category},'VRTSdbed731');
       }
    }

    @categories=qw(obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                        @{$prod->{$category}},
                                        @{$sf->{$category}},
                                        @{$vcs->{$category}},
                                        @{$vm->{$category}},
                                        @{$fs->{$category}});
    }
    return $prod->{allpkgs};
}

# return 1 if using installsfcfs/installsvs to configure SFCFSHA
# and SFCFSHA is licensed
sub configure_alternate_prod {
    my ($prod)=@_;
    my $rel=Obj::rel();
    my $activecomponent=$rel->get_activecomponent();
    return 1 if (($activecomponent =~/(SFCFS|SVS)(\d+)/m) && $prod->sfcfsha );
    return;
}

# check the license and update prod from SFCFS to SFCFSHA
sub update_prod {
    my ($prod) = @_;
    my ($sys,$cpic,$name_sfcfsha,$ivers,$prod_sfcfsha);
    $cpic=Obj::cpic();
    $sys=@{CPIC::get('systems')}[0];
    $name_sfcfsha='SFCFSHA731';

    $prod_sfcfsha=$prod->prod($name_sfcfsha);
    $ivers=$prod_sfcfsha->version_sys($sys,1);
    if ($ivers && $prod_sfcfsha->configure_alternate_prod()) {
        $cpic->set_prod($name_sfcfsha);
        $cpic->cli_set_title();
    }
    return;
}

sub clear_and_start_cvm {
    my ($cvm_state, $sys, $msg,$vcs,$sysname,$syslist,$prod);
    $prod = shift;

    $vcs = $prod->prod('VCS731');
    $syslist=CPIC::get('systems');
    for my $sys (@$syslist) {
        $sysname=$sys->{vcs_sysname};
        if (!$sysname) {
            $sysname = $vcs->get_vcs_sysname_sys($sys);
            $sys->{vcs_sysname} = $sysname;
        }
        $cvm_state = $sys->cmd("$vcs->{bindir}/hagrp -state cvm -sys $sysname");
        if ($cvm_state eq 'ONLINE') {
            Msg::log('cvm already Online');
        } else {
            $sys->cmd("$vcs->{bindir}/hagrp -clear cvm -sys $sysname");
            $sys->cmd("$vcs->{bindir}/hagrp -online cvm -sys $sysname");
            if (!EDR::cmdexit()) {
               $sys->cmd("$vcs->{bindir}/hagrp -wait cvm State ONLINE -sys $sysname -time 300 ");
            }

            if (EDR::cmdexit()) {
               $msg = Msg::new("cvm service group failed to online on $sys->{sys}");
               push(@{$sys->{startfailmsg}}, $msg);
            }
        }
    }
    return;
}

sub freeze_cvm {
    my ($prod,$sys)=@_;
    my $cvm_vxconfigd_state;
    my $vcs = $prod->prod('VCS731');
    my $sysname;

    $sysname=$sys->{vcs_sysname};
    $cvm_vxconfigd_state = $sys->cmd("$vcs->{bindir}/hares -state cvm_vxconfigd -sys $sysname");
    if ($cvm_vxconfigd_state eq 'ONLINE') {
        $sys->cmd("$vcs->{bindir}/hagrp -freeze cvm");
    }

    return;
}

sub cli_prod_option
{
    my $prod=shift;
    $prod->perform_task('cli_prod_option');
    return;
}

sub web_prod_option
{
    my $prod=shift;
    $prod->perform_task('web_prod_option');
    return;
}

# Add a HA menu to fix 1988793
sub cli_initiation_questions {
    my ($prod)=@_;
    my ($cpic,$cfg,$mode,$mprod);

    #if (($prod->svs()) && (Cfg::opt('install'))) {
    #    $mode=$prod->svs_config_mode('install');
    #    if ($mode eq 'typical') {
    #        Cfg::set_opt('silent');
    #        $prod->{silent_installconfig} = 1;
    #    }
    #}
    return unless (Cfg::opt('makeresponsefile'));
    $cpic=Obj::cpic();
    $cfg=Obj::cfg();
    $mprod=$prod->rel->prod_ha_menu('makeresponsefile',$cpic->{prod});
    $cpic->{prod}=$mprod;
    $cfg->{prod}=$mprod;
    return;
}

sub web_initiation_questions {
    my ($prod) = @_;
    return unless (Cfg::opt('install') || Cfg::opt('configure'));
    return if (Cfg::opt( qw(addnode fencing security configcps) ));

    my $web = Obj::web();
    if ($prod->svs()&& !$prod->{silent_installconfig}) {
        my $mode = $web->web_script_form('config_mode');
        if ($mode eq 'typical') {
            $prod->{silent_installconfig} = 1;
        }
    }
    return;
}

sub perform_task_sys {
    my ($prod,$sys,$task) = @_;
    my ($sf_rtn,$vcs_rtn,$vcs,$sf);

    $sf=$prod->prod('SF731');
    $sf_rtn=$sf->$task($sys) if ($sf->can($task));

    $vcs=$prod->prod('VCS731');
    $vcs_rtn=$vcs->$task($sys) if ($vcs->can($task));
    return;
}

sub perform_task {
    my ($prod,$task) = @_;
    my ($sfha);

    $sfha=$prod->prod('SFHA731');
    return $sfha->$task();
}

sub is_rdma_supported {
    my ($prod) = @_;
    my ($vcs,$vcs_rtn);

    $vcs=$prod->prod('VCS731');
    $vcs_rtn=$vcs->is_rdma_supported() if ($vcs->can('is_rdma_supported'));
    return $vcs_rtn;
}

sub verify_responsefile {
    my ($prod) = @_;
    $prod->perform_task('verify_responsefile');
    return;
}

sub require_start_after_reboot_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys, 'require_start_after_reboot_sys');
    return;
}

sub responsefile_comments {
    my ($prod) = @_;
    $prod->perform_task('responsefile_comments');
    return;
}

sub responsefile_prestart_config {
    my ($prod) = @_;
    $prod->perform_task('responsefile_prestart_config');
    return;
}

sub responsefile_poststart_config {
    my $prod =shift;
    $prod->perform_task('responsefile_poststart_config');
    return;
}

sub preinstall_messages {
    my ($prod) = @_;
    $prod->perform_task('preinstall_messages');
    return;
}

sub cli_prestart_config_questions {
    my ($prod) = @_;
    my ($mode,$ret);
    my $installconfig = CPIC::get('installconfig');

    if ($prod->svs()) {
        # if separate configure then provide menu to choose mode
        # if custom install then custom configure without asking mode
        while ((!$installconfig) || ($prod->{silent_installconfig})) {
            # skip SVS-specific install/config mode
            # due to the merging of SVS into SFCFSHA
            last;
            if ($prod->{silent_installconfig}) {
                $mode = 'typical';
            } else {
                $mode = $prod->svs_config_mode('configure');
            }
            if ($mode eq 'typical') {
                $ret = $prod->svs_auto_config();
                next if (EDR::getmsgkey($ret, 'back'));
                return 1;
            } else {
                Msg::title();
                last;
            }
        }
    }

    $prod->perform_task('cli_prestart_config_questions');
    return;
}

sub svs_config_mode {
    my ($prod,$task) = @_;
    my ($default,$help,$menu_opts,$mode,$msg);

    $default = 1;

    Msg::title();
    if ($task eq 'install') {
        $help = Msg::new("To install $prod->{abbr} in typical mode, installer will ask minimum questions during installation and automatically configure cluster heartbeat links.\nTo install $prod->{abbr} in custom mode, installer will ask all the questions during installation.\nA typical installation includes installing $prod->{abbr}, licensing the product and then configuring the product with little user interaction.");
    } else {
        $help = Msg::new("To configure $prod->{abbr} in typical mode, installer will ask minimum questions during configuration and automatically configure cluster heartbeat links.\nTo configure $prod->{abbr} in custom mode, installer will ask all the questions during configuration.");
    }
    $msg = Msg::new("Typical");
    $menu_opts = [$msg->{msg}];
    $msg = Msg::new("Custom");
    push(@{$menu_opts},$msg->{msg});
    if ($task eq 'install') {
        $msg = Msg::new("Choose the mode you would like to install $prod->{abbr}:");
    } else {
        $msg = Msg::new("Choose the mode you would like to configure $prod->{abbr}:");
    }
    $mode = $msg->menu($menu_opts,$default,$help);
    return 'typical' if ($mode == 1);
    return 'custom';
}

sub svs_auto_config {
    my ($ayn,$cfg,$clus_id,$clus_name,$msg,$prod,$rhbn,$vcs_prod);
    $prod = shift;
    $cfg = Obj::cfg();

    $cfg->{vcs_allowcomms} = 1;
    $vcs_prod = $prod->prod('VCS731');
    Cfg::set_opt('silent');

    $rhbn = $vcs_prod->auto_config_llt();
    return $rhbn if (EDR::getmsgkey($rhbn, 'back'));
    # in case llt auto detection failed and silent option was unsest.
    $cfg->{vcs_clusterid} = int(rand(65535)) if (!defined($cfg->{vcs_clusterid}));
    $clus_id = $cfg->{vcs_clusterid};
    if ($vcs_prod->check_clusterid($clus_id,$rhbn)) {
        Cfg::unset_opt('silent');
        $clus_id = $vcs_prod->config_clusterid($clus_id,$rhbn);
    }
    # in case cluster id auto detection failed and silent option was unset.
    Cfg::set_opt('silent');
    $clus_name = lc("$prod->{abbr}$clus_id");
    $msg = $vcs_prod->display_config_info($clus_name, $clus_id, $rhbn);
    $msg->add_summary(1);
    unless (Cfg::opt('responsefile')) {
        $cfg->{vcs_clusterid}=$clus_id;
        $vcs_prod->set_hb_nics($rhbn, CPIC::get('systems'));
        $cfg->{vcs_clustername}=$clus_name;
    }
    delete($cfg->{autocfgllt});
    return 'done';
}

sub web_prestart_config_questions {
    my ($prod) = @_;
    my $edr = Obj::edr();
    my $web = Obj::web();
    my $cfg = Obj::cfg();

    if ($prod->{silent_installconfig}) {
        $cfg->{vcs_allowcomms} = 1;
        my $vcs = $prod->prod('VCS731');
        if ($vcs->web_autocfg_hbnics()) {
            $vcs->set_hb_nics($web->{rhbn},$edr->{systems});
            delete $cfg->{autocfgllt};

            if (!$cfg->{vcs_clusterid}) {
                $cfg->{vcs_clusterid} = int(rand(65535));
                if ($vcs->check_clusterid($cfg->{vcs_clusterid}, $web->{rhbn})) {
                    $vcs->web_config_clusterid($web->{rhbn});
                }
            }
            $cfg->{vcs_clustername} = lc("$prod->{abbr}$cfg->{vcs_clusterid}");

            return;
        }
    }
    $prod->perform_task('web_prestart_config_questions');
    return;
}

sub cli_poststart_config_questions {
    my ($prod) = @_;
    $prod->perform_task('cli_poststart_config_questions');
    return;
}

sub web_poststart_config_questions {
    my ($prod) = @_;
    $prod->perform_task('web_poststart_config_questions');
    return;
}

sub stop_precheck_sys {
    my ($prod,$sys) = @_;
    $prod->check_cvmstatus_sys($sys);
    $prod->perform_task_sys($sys,'stop_precheck_sys');
    return;
}

sub uninstall_precheck_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'uninstall_precheck_sys');
    return;
}

sub precheck_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'precheck_sys');
    return;
}

sub hotfixupgrade_precheck_sys {
    my ($prod, $sys) = @_;

    $prod->perform_task_sys($sys,"hotfixupgrade_precheck_sys");
    return;
}

sub configure_precheck_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'configure_precheck_sys');
    return;
}

sub start_precheck_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'start_precheck_sys');
    return;
}

sub check_vmware_vsphere_sdk_sys {
    my ($prod,$sys) = @_;
    my ($msg,$exitcode,$rtn);

    $rtn=$sys->cmd("/usr/bin/perl -e 'eval {require VMware::VIRuntime}; die \$\@ if \$\@'");
    $exitcode=EDR::cmdexit();

    if ($exitcode) {
        $msg=Msg::new("VMware vSphere SDK for Perl is not installed or not working as expected on $sys->{sys}. Download and install the SDK from VMware website and re-run the installation program.");
        $sys->push_warning($msg);
    }
    return;
}

sub check_cvmstatus_sys {
    my ($prod,$sys) = @_;
    my ($sysi,$msg,@temp,$grp,$vcs,$obj_sysi,$syslist,$line,$rtn,$cvm_online,$cvm_frozen);

    # Check if cvm is frozen and online.
    # If so, quit upgrading and ask user to unfreeze cvm before upgrading.
    $vcs=$prod->prod('VCS731');
    unless (Cfg::opt('rootpath')) {
        $rtn = $sys->cmd("$vcs->{bindir}/hagrp -list Frozen=1 2>/dev/null");
        for my $line (split(/\n+/,$rtn)) {
            ($grp)=split(/\s+/m,$line, 2);
            if($grp && ($grp eq 'cvm')){
                $cvm_frozen=1;
                last;
            }
        }

        if ($cvm_frozen) {
            $rtn = $sys->cmd("$vcs->{bindir}/hagrp -display cvm -attribute State");
            $syslist=CPIC::get('systems');
            for my $line (split(/\n+/,$rtn)) {
                @temp = split(/\s+/m, $line);
                next if ($temp[0] eq '#Group');
                $sysi = $temp[2];
                for my $obj_sysi (@$syslist) {
                    my $sysname = $obj_sysi->{vcs_sysname};
                    if (($sysi eq $sysname) && ($temp[3] =~ /ONLINE|PARTIAL/mx)) {
                        $cvm_online=1;
                        last;
                    }
                }
            }
            if ($cvm_online) {
                if(Cfg::opt('upgrade')) {
                    $msg = Msg::new("Cannot upgrade if cvm service group is frozen and online. Unfreeze cvm service group before upgrading.");
                } elsif(Cfg::opt('stop')) {
                    $msg = Msg::new("Cannot stop if cvm service group is frozen and online. Unfreeze cvm service group before stop.");
                }
                $sys->push_error($msg);
                return 0;
            }
        }
    }
}

sub upgrade_precheck_sys {
    my ($prod,$sys) = @_;

    $prod->perform_task_sys($sys,'upgrade_precheck_sys');
    $prod->check_cvmstatus_sys($sys);
    return;
}

sub prestop_sys {
    my ($prod,$sys) = @_;

    if (!Cfg::opt('rootpath')) {
        $prod->stop_agents_sys($sys);
    }

    $prod->perform_task_sys($sys,'prestop_sys');
    return;
}

sub preremove_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'preremove_sys');
    return;
}

sub preremove_tasks {
    my $prod = shift;
    $prod->perform_task('preremove_tasks');
    return;
}

sub postremove_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'postremove_sys');
    return;
}

sub preinstall_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'preinstall_sys');
    return;
}

sub postinstall_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'postinstall_sys');
    return;
}

# return 0 : sfsybasece is not configured
# return 1 : sfsybasece is configured
sub is_sfsybasece_configured {
    my ($prod,$sys) = @_;
    my ($output,$rootpath,$vcs,$component_file);
    return 0 unless ($prod->sfsybasece());
    return 0 if ($sys->cmd('_cmd_gabconfig -a 2>/dev/null') =~ /Port d/m);

    $vcs = $prod->prod("VCS731");
    $rootpath = Cfg::opt('rootpath') || '';
    if ($sys->exists($rootpath.$vcs->{vxfenmode})) {
        $output=$sys->cmd("_cmd_grep 'vxfen_mode' $rootpath/etc/vxfenmode 2>/dev/null");
        return 1 if ($output =~/^\s*vxfen_mode\s*=\s*sybase\s*/mx); 
    }

    $component_file = "$rootpath/opt/VRTS/install/\.component";
    if ($sys->exists($component_file)) {
        $output=$sys->cmd("_cmd_grep 'configured_component' $component_file 2>/dev/null");
        return 1 if ($output =~/^\s*configured_component\s*=\s*SFSYBASECE731\s*/mx); 
    }

    return 0;
    
}

sub check_config {
    my ($prod, $no_double_check) = @_;
    my ($syslist,$cvm_state,$had,$maincf,$port_f,$port_v,$port_w,$prodi,$rootpath,$sf,$sf_config,$sys,$sysname,$vcs,$vcs_config);
    
    $syslist = CPIC::get('systems');
    $prodi = $prod->{prodi};
    for my $sysi (@$syslist) {
        return 0 if (!EDRu::inarr($prodi, @{$sysi->{upgradeprod}}));
    }
 
    return 1 if($no_double_check);
    $rootpath = Cfg::opt('rootpath') || '';
    $sf=$prod->prod('SF731');
    $sf_config=$sf->check_config();
    return 0 unless $sf_config;
    $vcs=$prod->prod('VCS731');
    $vcs_config=$vcs->check_config();
    return 0 unless $vcs_config;

    # Check SFCFS configuration here.
    # As long as group cvm keyword exist in main.cf, we should regard SFCFSHA is configured
    # If Port d is not exist, we should regard SFSYBASECE is configured
    $sys=$$syslist[0];
    $maincf=$sys->readfile("$rootpath$vcs->{configdir}/main.cf");
    return 0 if ($maincf!~/\n\s*group\s+cvm[\s+\(]/x);
    return 0 if ($prod->sfsybasece() && !$prod->is_sfsybasece_configured($sys));
    return 1;
}

sub prestart_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'prestart_sys'); }

sub configure_sys {
    my ($prod,$sys) = @_;
    # for SVS typical installation/configuration, backup existing conf files
    $prod->backup_conf_files_sys($sys) if (Cfg::opt('silent'));
    $prod->perform_task_sys($sys,'configure_sys');
    return;
}

sub backup_conf_files_sys {
    my ($prod,$sys) = @_;
    my (@files,$confdir,$out,$rootpath,$vcs);
    $vcs = $prod->prod('VCS731');
    $rootpath = Cfg::opt('rootpath') || '';
    $confdir = "$rootpath$vcs->{configdir}";
    if ($sys->exists($confdir)) {
        $out = $sys->cmd("_cmd_ls $confdir/*.cf 2> /dev/null");
        for my $file(split(/\n/,$out)) {
            $sys->cmd("_cmd_cp -p $file $file.cpisave");
        }
    }
    @files = ("$rootpath$vcs->{llthosts}","$rootpath$vcs->{llttab}","$rootpath$vcs->{gabtab}","$rootpath$vcs->{vxfenmode}","$rootpath$vcs->{vxfendg}");
    for my $file(@files) {
        if ($sys->exists($file)) {
            $sys->cmd("_cmd_cp -p $file $file.cpisave");
        }
    }
    return;
}

sub poststart_sys {
    my ($prod,$sys) = @_;
    my ($vcs,$vm);
    $vcs=$prod->prod('VCS731');
    $vm=$prod->prod('VM731');

    if ($sys->system1) {
        $prod->sfcfs_start_agents();
        EDRu::create_flag('sfcfs_start_agents_done');
    } else {
        EDRu::wait_for_flag('sfcfs_start_agents_done');
    }
    $vm->poststart_sys($sys) if ($vm->can('poststart_sys'));

    $vcs->poststart_sys($sys) if ($vcs->can('poststart_sys'));

    # Start CVM and CFS Agents ..
    if ($sys->system1) {
        $prod->clear_and_start_cvm();
        EDRu::create_flag('clear_and_start_cvm_done');
    } else {
        EDRu::wait_for_flag('clear_and_start_cvm_done');
    }
    return;
}

sub ru_prestop_sys {
    my ($prod,$sys) = @_;
    my ($ver1,$ver2,$ver3,$output,$msg,$had);
    my $vcs=$sys->prod('VCS731');

    if (Cfg::opt('upgrade_kernelpkgs')) {
        $output=$sys->cmd('_cmd_vxdctl protocolversion 2>&1');
        $ver1=$1 if($output=~/Cluster\s+running\s+at\s+protocol\s+(\d+)/mx);
        $ver2=$sys->cmd("_cmd_fsclustadm protoversion 2>&1 | _cmd_awk '/local/ {print \$3}'");
        $ver3=$sys->cmd("_cmd_odmclustadm protoversion 2>&1 | _cmd_awk '/local/ {print \$3}'");

        Msg::log("Starting Stopping CVM in rolling upgrade on $sys->{sys}");
        if ($prod->stop_agents_sys($sys)) {
            # Print Error Message Here
            # Directly exit CPI
            $msg=Msg::new("Failed to stop CVM/CFS agents, contact our technical support or refer to the Veritas InfoScale Configuration and Upgrade Guide for further troubleshooting.");
            $sys->push_error($msg);
            return 1;
        }
        Msg::log("Finished Stopping CVM in rolling upgrade on $sys->{sys}");

        # if ru_prestop_sys failed, return immediately.
        return 1 if ($vcs->ru_prestop_sys($sys));
        #set fs/vm/odm version
        $sys->cmd("_cmd_vxdctl setversion $ver1 2>&1") if($ver1);
        $sys->cmd("_cmd_fsclustadm protoset $ver2 2>&1") if($ver2);
        $sys->cmd("_cmd_odmclustadm protoset $ver3 2>&1") if($ver3);
    }
    return;
}

sub upgrade_prestop_sys {
    my ($prod,$sys) = @_;

    my $vcs=$prod->prod('VCS731');
    my $vm=$prod->prod('VM731');
    my $sfha=$prod->prod('SFHA731');

    $vm->upgrade_prestop_sys($sys) if ($vm->can('upgrade_prestop_sys'));
    $sfha->upgrade_rolling_post_sys($sys) if($sys->system1 &&  Cfg::opt('upgrade_nonkernelpkgs'));
    $prod->ru_prestop_sys($sys) if(Cfg::opt('upgrade_kernelpkgs'));


    if (!Cfg::opt('rootpath')) {
        $prod->stop_agents_sys($sys) unless(Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs)));
    }

    $vcs->upgrade_prestop_sys($sys) if ($vcs->can('upgrade_prestop_sys'));
    return;
}


sub upgrade_preremove_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_preremove_sys');
    return;
}

sub upgrade_postremove_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_postremove_sys');
    return;
}

sub upgrade_preinstall_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_preinstall_sys');
    return;
}

sub upgrade_postinstall_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_postinstall_sys');
    return;
}

sub upgrade_configure_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_configure_sys');
    return;
}

sub upgrade_poststart_sys {
    my ($prod,$sys) = @_;
    my ($vcs,$sfha,$csystems,$nsystems);
    $vcs=$prod->prod('VCS731');
    $sfha=$prod->prod('SFHA731');

    if ($sys->system1) {
        $prod->sfcfs_start_agents();
        EDRu::create_flag('sfcfs_start_agents_done');
    } else {
        EDRu::wait_for_flag('sfcfs_start_agents_done');
    }

    # Get number of systems from the main.cf
    $nsystems = $sys->cmd("_cmd_grep '^system' $vcs->{maincf} 2> /dev/null | _cmd_wc -l");
    $csystems = scalar(@{CPIC::get('systems')});
    Msg::log("Number of systems in main.cf = $nsystems");
    Msg::log("Number of systems provided = $csystems");

    # Start CVM and CFS Agents ..
    # It is Phased Upgrade if number of systems in main.cf != number of
    # systems in $cpic->{systems} (systems passed to CPI)
    # If Phased Upgrade, do not start the Agents.
    # Refer to the Installation Guide for the Correct procedure.
    if ($csystems == $nsystems) {
        Msg::log('Starting CVM as it is not phased upgrade');
        if ($sys->system1) {
            $prod->start_cvm_cfs_agents($sys);
            EDRu::create_flag('start_cvm_cfs_agents_done');
        } else {
            EDRu::wait_for_flag('start_cvm_cfs_agents_done');
        }
    }
    $sfha->upgrade_poststart_sys($sys) if ($sfha->can('upgrade_poststart_sys'));
    if ($sys->exists("$prod->{upgrade_flag_file}")) {
        # Remove .<UPI>.upgrade file
        $sys->cmd("_cmd_rmr $prod->{upgrade_flag_file}");
    }
    return;
}

sub completion_messages {
    my ($prod,$msg,$activecomponent,$rel,$reboot_systems);
    $prod=shift;
    $rel=Obj::rel();
    $activecomponent=$rel->get_activecomponent();
    $activecomponent=~s/\d+$//m;
    $reboot_systems=CPIC::get('require_start_after_reboot_systems');
    $prod->perform_task('completion_messages');

    return unless (Cfg::opt('configure'));
    return if (Cfg::opt(qw(fencing addnode security configcps rootpath makeresponsefile)));

    if ($prod->{cfscluster_config_pending}==1) {
        $msg = Msg::new("Unable to complete $activecomponent configuration at this time\n");
        unless ( defined($reboot_systems) && (@$reboot_systems) ) {
            $msg->{msg} .= Msg::new("</opt/VRTS/bin/cfscluster config -s> must be run after the system is rebooted\n")->{msg};
        }
        $msg->print();
        if (Obj::webui()) {
            $msg->{msg} =~ s/<(.*)>/&lt;$1&gt;/mg;
            my $web = Obj::web();
            $web->web_script_form('alert',$msg);
        }
    }
    if ($prod->svs()) {
        $msg = Msg::new("Refer to $prod->{name} installation and configuration guide for how to set up VirtualStore and register VMware vSphere Plug-in for VirtualStore.");
        $msg->bold;
        Msg::n();
    }

    return;
}

# start CVM and CFS Agents
sub start_cvm_cfs_agents {
    my ($prod,$sys) = @_;
    my ($cvmstate,$grp,$sysi,$state,$vcs,$syslist,@fields);

    $vcs = $prod->prod('VCS731');
    return if $prod->{cvmcfs_started};
    $syslist=CPIC::get('systems');
    $cvmstate = $sys->cmd("$vcs->{bindir}/hagrp -display cvm -attribute State");
    for my $line (split(/\n/,$cvmstate)) {
        next if ($line=~/^#Group/);
        @fields = split(/\s+/, $line);
        $grp=$fields[0];
        $sysi=$fields[2];
        $state=$fields[3];

        # Only Online on nodes passed to CPI.
        # This is done to handle phased upgrades.
        for my $obj_sysi (@$syslist) {
            my $sysname = $obj_sysi->{vcs_sysname};
            if (($sysi eq $sysname) && ($state =~ /OFFLINE/m)) {
                Msg::log("Online cvm Group on $sysi");
                $sys->cmd("$vcs->{bindir}/hagrp -online $grp -sys $sysi");
                if (!EDR::cmdexit()) {
                    $sys->cmd("$vcs->{bindir}/hagrp -wait $grp State ONLINE -sys $sysi -time 300");
                } else {
                    Msg::log("Unable to online cvm service group on $sysi");
                }
                Msg::log("cvm Group on $sysi is online");
            }
        }
    }
    $prod->{cvmcfs_started}=1;

    return;
}

sub sfcfs_start_agents {
    my ($msg,$sys,$syslist,$activecomponent,$rel,$vm,$vcs);
    my $prod = shift;

    $syslist=CPIC::get('systems');
    $rel=Obj::rel();
    $activecomponent=$rel->get_activecomponent();
    $activecomponent=~s/\d+$//m;
    $vm = $prod->prod('VM731');
    $vcs = $prod->prod('VCS731');

    Msg::log("\nConfiguring CFS agents:\n");
    Msg::log("Confirming $prod->{abbr} configuration daemon startup");

    for my $sys (@$syslist) {
        if ($vm->vold_status_sys($sys) ne 'enabled') {
            Msg::log('Disabled');
            if (Cfg::opt('start')) {
                $msg=Msg::new("Unable to start CFS agents at this time. vxconfigd is not enabled.");
            } else {
                $msg=Msg::new("Unable to complete $activecomponent configuration at this time.\n</opt/VRTS/bin/cfscluster config -s> must be run after the system is rebooted.");
            }
            $sys->push_warning($msg);
            return 0;
        }
    }
    $prod->set_value('cfscluster_config_pending',0);
    Msg::log('All systems Enabled');
    # TODO: check for update
    return 1 if (Cfg::opt('upgrade'));
    Msg::log('Starting CFS agents');
    $sys = $$syslist[0];
    $sys->cmd('_cmd_cfscluster config -t 200 -s');
    if (EDR::cmdexit() != 0) {
        $msg = Msg::new("Unable to start CFS agents.");
        $sys->push_error($msg);
        $prod->set_value('cfscluster_config_pending',1);
        return 0;
    }
    Msg::log('CFS agents are started');

    # Copy CVMTypes.cf and CFSTypes.cf to /etc/VRTSvcs/conf for Dynamic Upgrade.
    for my $sys (@$syslist) {
        if (($sys->exists("$vcs->{configdir}/CVMTypes.cf"))
            && (!$sys->exists("$vcs->{confdir}/CVMTypes.cf"))) {
            $sys->cmd("_cmd_cp -p $vcs->{configdir}/CVMTypes.cf $vcs->{confdir}/CVMTypes.cf");
        }
        if (($sys->exists("$vcs->{configdir}/CFSTypes.cf"))
            && (!$sys->exists("$vcs->{confdir}/CFSTypes.cf"))) {
            $sys->cmd("_cmd_cp -p $vcs->{configdir}/CFSTypes.cf $vcs->{confdir}/CFSTypes.cf");
        }
    }
    return;
}

sub stop_agents_sys {
    my ($prod,$sys) = @_;
    my ($cfscluster,$fsclustadm,$failure);

    $cfscluster = $sys->cmd_bin('cfscluster');
    $fsclustadm = $sys->cmd_bin('fsclustadm');
    unless ($sys->exists($cfscluster) && $sys->exists($fsclustadm)){
        if ($sys->system1) {
            if (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade rollback))) {
                EDRu::create_flag('stop_cvm_cfs_agents_done');
            }
            EDRu::create_flag('sfcfs_stop_agents_all_done');
        }
        return 0;
    }

    #Etrack 3046613 : merging SVS back to SFCFSHA
    $prod->stop_svsweb_sys($sys);
    $prod->stop_odm_sys($sys);

    # If upgrade or patchupgrade, do not stop VCS.
    # Just stop the CVM and CFS Agents. (Offline group cvm)
    # Otherwise the service group frozen operation will fail in later steps(ET3436664)
    if (Cfg::opt(qw(upgrade patchupgrade))) {
        if ($sys->system1) {
            if ($sys->{prod_running}) {
                $failure=$prod->stop_cvm_cfs_agents($sys); 
            } else {
                $prod->sfcfs_stop_agents_all($sys);
            }
            EDRu::create_flag('stop_cvm_cfs_agents_done');
        } else {
            EDRu::wait_for_flag('stop_cvm_cfs_agents_done');
        }
        return 1 if ($failure);
        return 0;
    }

    if ($sys->system1) {
        $prod->sfcfs_stop_agents_all($sys);
        EDRu::create_flag('sfcfs_stop_agents_all_done');
    } else {
        EDRu::wait_for_flag('sfcfs_stop_agents_all_done');
    }

    return 0;
}

# used to get which groups should be offline before offlining $group
# returns @groups
sub all_groups_depend_on_group {
    my ($prod,$sys,$group) = @_;
    my ($vcs, @groups, $deps,@fields, @depgroups);
    $vcs = $prod->prod('VCS731');
    @groups = ();
    if ($group) {
        $deps = $sys->cmd("$vcs->{bindir}/hagrp -dep $group 2>/dev/null");
        for my $dep (split(/\n/,$deps)) {
            @fields=split(/\s/, $dep);
            next if ($fields[0] eq $group);
            next if ($fields[0] =~ /^#Parent/);
            @depgroups = $prod->all_groups_depend_on_group($sys, $fields[0]);
            if (@depgroups) {
                push(@groups, @depgroups);
            }
            push(@groups, $fields[0]);
        }
    }
    return @groups;
}

# stop CVM and CFS Agents
sub stop_cvm_cfs_agents {
    my ($prod,$sys) = @_;
    my ($padv,$port_f,$vcs,@deps,@cvmdeps,$syslist,$port_q,$offlinefailuregroups,$offlinefailuregrouplist,$failure);
    $syslist=CPIC::get('systems');
    $vcs = $prod->prod('VCS731');
    $padv=$sys->padv;

    Msg::log('Stopping CVM and CFS Agents');
    @cvmdeps = $prod->all_groups_depend_on_group($sys, 'cvm');
    if (@cvmdeps) {
        push(@deps, @cvmdeps);
    }
    push(@deps, 'cvm');
    $offlinefailuregroups=$prod->offline_groups($sys,\@deps);
    if (@{$offlinefailuregroups}) {
        # If there're groups depends on CVM and failed to offline
        # No need to go further and attempt to stop CVM/CFS related agents.
        $offlinefailuregrouplist=join(',',@{$offlinefailuregroups});
        Msg::log("The following service groups failed to offline: $offlinefailuregrouplist");
        $failure=1;
    }
    for my $sys (@$syslist) {
        $port_f = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port f gen' ");
        my $sysname = $sys->{vcs_sysname};
        $sys->cmd("$vcs->{bindir}/hares -offline vxfsckd -sys $sysname")
            if (($port_f >=1) && ($sys->proc_pids('vxfsckd')));
        $sys->cmd('_cmd_fsclustadm cfsdeinit 2>/dev/null');
        # Stop Qlog for SunOS
        if ($sys->sunos()) {
            $port_q = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port q gen' ");
            $sys->unload_driver('qlog') if ($port_q >= 1);
            $port_q = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port q gen' ");
            if ($port_q >= 1) {
                Msg::log("Failed to unload driver Qlog");
            }
        }
    }
    Msg::log('CVM and CFS Agents are stopped');
    return 1 if ($failure);
    return 0;
}

sub offline_groups {
    my ($prod,$sys,$deps) = @_;
    my ($grpstate,$sysi,$sys_master,$vxdctl_mode,$state,$vcs,$count,$syslist,@fields,$temp_sys);
    my ($master_node,$master_cvm_state);
    my (@offlinefailuregroups);
    $syslist=CPIC::get('systems');
    $vcs = $prod->prod('VCS731');

    for my $grp (@{$deps}) {
        $grpstate = $sys->cmd("$vcs->{bindir}/hagrp -display $grp -attribute State");
        for my $line (split(/\n/,$grpstate)) {
            next if ($line=~/^#Group/);

            @fields = split(/\s+/, $line);
            $sysi=$fields[2];
            $state=$fields[3];

            # Only Offline on nodes passed to CPI.
            # This is done to handle phased upgrades.
            $count=0;
            for my $obj_sysi (@$syslist) {
                my $sysname = $obj_sysi->{vcs_sysname};
                if ($sysname && ($sysi eq $sysname)) {
                    $temp_sys=$obj_sysi;
                    last;
                }
                $count ++;
            }
            next if ($count == scalar(@$syslist));
            # Offline Slave nodes before Master node.
            if ($grp eq 'cvm') {
                $vxdctl_mode = $temp_sys->cmd("_cmd_vxdctl -c mode 2>/dev/null | _cmd_grep 'mode:'");
                if ($vxdctl_mode=~/MASTER/m) {
                    $master_node=$sysi;
                    $master_cvm_state=$state;
                } else {
                    push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$sysi,$grp,$state));
                }
            } else {
                push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$sysi,$grp,$state));
            }
        }
        if ($grp eq 'cvm') {
            push (@offlinefailuregroups,$grp) if ($prod->offline_group_sys($sys,$master_node,$grp,$master_cvm_state));
        }
    }
    return \@offlinefailuregroups;
}

sub offline_group_sys {
    my ($prod,$sys,$sysi,$grp,$state) = @_;
    my ($vcs);
    $vcs = $prod->prod('VCS731');
    if ($state =~ /FAULTED/m) {
        $sys->cmd("$vcs->{bindir}/hagrp -clear $grp -sys $sysi");
    }
    if ($state =~ /ONLINE|PARTIAL/mx) {
        $sys->cmd("$vcs->{bindir}/hagrp -offline $grp -sys $sysi");
        if (!EDR::cmdexit()) {
            $sys->cmd("$vcs->{bindir}/hagrp -wait $grp State OFFLINE -sys $sysi -time 300");
        } else {
            Msg::log("Unable to offline $grp on $sysi");
            return 1;
        }
    }
    return 0;
}

# Common define for AIX and Linux.
# Solaris and HPUX will define their own sfcfs_stop_agents_all
sub sfcfs_stop_agents_all {
    my ($prod,$sys) = @_;
    my ($sysi,$port_h,$sys_master,$port_v,$vxdctl_mode,$padv,$syslist,$port_q,@master_nodes);
    $syslist=CPIC::get('systems');
    $padv = $sys->padv;

    for my $sysi (@$syslist) {
        $port_v = $sysi->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port v gen' ");
        $port_h = $sysi->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port h gen' ");
        if ($port_v >= 1) {
            if ($port_h == 0) {
                $sysi->cmd('_cmd_cfscluster start');
            }
        }
        # Stop Qlog only for SunOS
        if ($sys->sunos()) {
            $port_q = $sysi->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port q gen' ");
            $sysi->unload_driver('qlog') if ($port_q >= 1);
        }
    }

    if ($prod->svs()) {
       my @cfsnfsdeps = $prod->all_groups_depend_on_group($sys, 'cfsnfssg');
       push (@cfsnfsdeps,'cfsnfssg');
       $prod->offline_groups($sys,\@cfsnfsdeps);
    }
    # Stop Slave nodes before Master node.
    for my $sysi (@$syslist) {
        $vxdctl_mode = $sysi->cmd("_cmd_vxdctl -c mode 2>/dev/null | _cmd_grep 'mode:'");
        if ($vxdctl_mode=~/MASTER/m) {
            push (@master_nodes, $sysi);
        } else {
            $prod->cfscluster_stop_sys($sysi);
        }
    }
    for my $sys_master (@master_nodes) {
        $prod->cfscluster_stop_sys($sys_master);
    }
    return;
}

sub cfscluster_stop_sys {
    my ($prod,$sys) = @_;
    Msg::log("Stopping Cluster Manager and Agents on $sys->{sys}");
    $sys->cmd('_cmd_cfscluster stop -f');
    $sys->cmd('_cmd_fsclustadm cfsdeinit');
    Msg::log("Cluster Manager and Agents on $sys->{sys} are stopped");
    return;
}

#installer should stop odm before stopping VCS because odm register on gab,
#if odm isn't stopped, gab can't be stopped also
sub stop_odm_sys {
    my ($prod,$sys) = @_;
    my ($odmver,$odmpkg,$odm,$gmspkg,$vxgms,$gmsver);
    $odmpkg=$sys->pkg('VRTSodm731');
    if (defined $odmpkg) {
        $odm=$sys->proc('odm731');
        $odmver=$odmpkg->version_sys($sys);
        if ($odmver && ($odm->check_sys($sys,'stop'))) {
            $odm->stop_sys($sys);
            return 1 if ($odm->check_sys($sys,'stop'));
        }
    }
    $gmspkg=$sys->pkg('VRTSgms731');
    if (defined $gmspkg) {
        $vxgms=$sys->proc('vxgms731');
        $gmsver=$gmspkg->version_sys($sys);
        if ($gmsver && ($vxgms->check_sys($sys,'stop'))) {
            $vxgms->stop_sys($sys);
            return 1 if ($vxgms->check_sys($sys,'stop'));
        }
    }
    return 0;
}

#Etrack 3046613
#For SVS merging into SFCFSHA, svsweb should be stopped before all the other
#processes of SFCFSHA
sub stop_svsweb_sys {
    my ($prod,$sys) = @_;
    my ($cpic,$svsver,$svspkg,$svs);

    return unless (Cfg::opt('upgrade'));

    $cpic=Obj::cpic();
    for my $oldpkg (keys%{$sys->{pkgvers}}) {
        next unless ($oldpkg =~ /VRTSsvs/);
        $svspkg=$oldpkg;
        last;
    }

    return unless ($svspkg);
    $svspkg=$sys->pkg($svspkg);
    if (defined $svspkg) {
        $svs=$sys->proc('svsweb731');
        $svsver=$svspkg->version_sys($sys);
        $svs->stop_sys($sys) if ($svsver);
    }
    return;
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;
    my ($pkgvers,$mpvers,$cpic,$rel,$cv,$pkg);
    my ($sfcfsha_licensed,$sfsybasece_licensed,$svs_licensed,$svspkg);
    $cpic=Obj::cpic();
    $rel=$cpic->rel;
    return '' unless ($prod->{mainpkg});

    $pkg=$sys->pkg($prod->{mainpkg});
    $pkgvers=$pkg->version_sys($sys,$force_flag);
    $mpvers=$prod->{vers} if ($pkgvers && $prod->check_installed_patches_sys($sys,$pkgvers));
    $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag);
    $pkgvers = $prod->version_mapping_sys($sys,$pkgvers) if $prod->can('version_mapping_sys');
    return '' if (!$mpvers && !$pkgvers);

    if ($prod->sfsybasece()) {
        # Skip super product license check(last parameter=1)
        # because SFCFSHA has SFSYBASECE, SFRAC as it's super product
        # If SFSYBASECE is licensed, $rel->prod_licensed_sys($sys, 'SFCFSHA731') will return licensed
        $sfcfsha_licensed=$rel->prod_licensed_sys($sys, 'SFCFSHA731', 1);
        $sfsybasece_licensed=$rel->prod_licensed_sys($sys,$prod->{prodi}, 1);
        if ($pkgvers) {
            return $pkgvers if ($sfsybasece_licensed);
            #e3872414:Check whether sfsybasece configured
            return $pkgvers if ($prod->is_sfsybasece_configured($sys));
        }
        # in other scenario(e.g. no license upgrade with installer script), installer auto detect product. we assume product is SFCFSHA
        return '';
    } elsif ($prod->svs()) {
        # this if block is fix for etrack 3115659
        # aiming at reporting SVS version properly
        $cv=$prod->prod("SFCFSHA731");
        $sfcfsha_licensed=$cv->version_sys($sys);
        $svs_licensed=$rel->prod_licensed_sys($sys,$prod->{prodi}, 1);
        if ($pkgvers) {
            if ($sfcfsha_licensed  && $svs_licensed) {
                $cv=EDRu::compvers($svs_licensed, $sfcfsha_licensed,2);
                if ($cv eq "2") {
                    return $sfcfsha_licensed;
                } else {
                    return ($mpvers || $pkgvers);
                }
            } elsif ($sfcfsha_licensed) {
                return $sfcfsha_licensed;
            } elsif ($svs_licensed) {
                return $pkgvers;
            }
        }
        # in other scenario(e.g. no license upgrade with installer script), installer auto detect product. we assume product is SFCFSHA
        return '';
    } elsif ($prod->sfcfsha()) {
        for my $oldpkg (keys %{$sys->{pkgvers}}) {
            next unless ($oldpkg =~ /VRTSsvs/ && defined $sys->{pkgvers}{'VRTSsvs'} && $sys->{pkgvers}{'VRTSsvs'} != '');
            $svspkg=$oldpkg;
            last;
        }
        $svs_licensed=$rel->prod_licensed_sys($sys, 'SVS731', 1) if ($svspkg);
        if ($pkgvers && $svs_licensed && Cfg::opt(qw(upgrade upgrade_kernelpkgs))) {
            $prod->{upgrade_from}="SVS";
        }
    }

    $cv=EDRu::compvers($pkgvers,$prod->{vers},2);
    if ($cv) {
        if ($rel->prod_licensed_sys($sys,$prod->{prodi})) {
            return ($mpvers || $pkgvers);
        }
    } else {
        return ($mpvers || $pkgvers);
    }
    return '';
}

sub description {
    my ($prod,$msg);
    $prod=shift;

    if ($prod->sfcfsha()) {
        $msg=Msg::new("Storage Foundation Clustered File System HA adds the full functionality of Cluster Server, and features the power and flexibility to protect everything from a single critical database instance to very large multi-application clusters in networked storage environments. In addition, increased automation and intelligent workload management allow cluster administrators to maximize individual resources by moving beyond reactive recovery to proactive management of availability and performance. The SFCFS HA bundle includes support for application monitoring and failover through the Cluster Server product.");
        $msg->print;
    } elsif ($prod->svs()) {
        $msg=Msg::new("VirtualStore powered by Cluster File System serves as a highly scalable, highly available NFS data storage for your VMware virtual machine images. VirtualStore is built on top of Cluster File System which provides high availability and linear scalability across the cluster. With integration with VMware Virtual Center, VirtualStore gives you a complete solution for managing your VMware virtual machine images.");
        $msg->print;
    } else {
        $msg=Msg::new("Storage Foundation Clustered File System extends Veritas File System and Veritas Volume Manager to support shared data in a SAN environment. Using Storage Foundation Cluster File System, multiple servers can access shared storage and files, transparently to the applications and concurrently with each other. SFCFCS by itself is not licensed for application monitoring and is targeted for parallel applications, applications with their own built in high availability, or only the need for shared data access.");
        $msg->print;
    }
    return;
}

sub licensed_sys {
    my ($prod,$sys) = @_;
    my ($rel);

    $rel = Obj::rel();
    my ($features,$fv);
    # HA mode: check
    $features = $rel->feature_values_sys($sys, 'Mode#VERITAS Cluster Server');
    for my $fv (@$features) {
        if ($fv eq 'VCS') {
            $rel->gco_licensed_sys($sys);
            last;
        }
    }

    # set vvr option per license check
    $rel->vr_licensed_sys($sys);
    $rel->vfr_licensed_sys($sys);
    return $rel->prod_licensed_sys($sys,$prod->{prodi});
}

sub reorder_vxglm {
    my ($prod,$ref_procs) = @_;
    my ($v,@a);
    if((EDRu::inarr('vxglm731', @{$ref_procs})) && (EDRu::inarr('had731', @{$ref_procs}))) {
        $ref_procs=EDRu::arrdel($ref_procs, 'vxglm731');
        for my $v (@{$ref_procs}) {
            if ($v eq 'had731') {
                push (@a,'vxglm731',$v);
            } else {
                push (@a,$v);
            }
        }
        return \@a;
    }
    return $ref_procs;
}

sub startprocs {
    my $prod=shift;
    return adjust_ru_procs() if(Cfg::opt('upgrade_nonkernelpkgs'));
    my $ref_procs = Prod::startprocs($prod);
    $ref_procs=$prod->reorder_vxglm($ref_procs);
    $ref_procs=$prod->remove_vcsmm_lmx($ref_procs);
    return $ref_procs;
}

sub startprocs_sys {
    my ($prod,$sys)=@_;
    my ($sf,$vm,$ref_procs);
    return adjust_ru_procs() if(Cfg::opt('upgrade_nonkernelpkgs'));
    $vm = $prod->prod('VM731');
    $ref_procs = Prod::startprocs_sys($prod, $sys);
    $ref_procs = $vm->verify_procs_list_sys($sys,$ref_procs,'start');
    $ref_procs=$prod->reorder_vxglm($ref_procs);
    $sf = $prod->prod('SF731');
    $ref_procs = $sf->adjust_vxdbd_nostart_sys($sys,$ref_procs);
    $ref_procs=$prod->remove_vcsmm_lmx($ref_procs);
    return $ref_procs;
}

sub stopprocs {
    my $prod=shift;
    my ($ref_procs,$sfha);
    return adjust_ru_procs() if(Cfg::opt('upgrade_nonkernelpkgs'));
    $sfha = $prod->prod('SFHA731');
    $ref_procs = Prod::stopprocs($prod);
    $ref_procs = $sfha->verify_procs_list($ref_procs,'stop');
    $ref_procs=$prod->remove_vcsmm_lmx($ref_procs);
    return $ref_procs;
}

sub stopprocs_sys {
    my ($prod,$sys)=@_;
    my ($ref_procs,$sfha);
    return adjust_ru_procs() if(Cfg::opt('upgrade_nonkernelpkgs'));
    $sfha = $prod->prod('SFHA731');
    $ref_procs = Prod::stopprocs_sys($prod, $sys);
    $ref_procs = $sfha->verify_procs_list_sys($sys,$ref_procs,'stop');
    $ref_procs=$prod->remove_vcsmm_lmx($ref_procs);
    return $ref_procs;
}

sub remove_vcsmm_lmx {
    my ($prod,$procs) = @_;
    $procs=EDRu::arrdel($procs, 'vcsmm731', 'lmx731');
    return $procs;
}

sub postcheck_gabconfig_sys {
    my ($prod,$sys) = @_;
    my ($msg,$ports,%gab_ports,%gab_jeopardy_ports,$port,@jeopardy_ports,$line,$rtn,@ports);

    $rtn=$sys->cmd("_cmd_gabconfig -l 2> /dev/null| _cmd_grep 'Driver state' | _cmd_awk '{ print \$4;}'");
    if (($rtn ne 'Configured') && ($rtn ne 'ConfiguredPartition')) {
        $msg=Msg::new("GAB is not in configured state on $sys->{sys}");
        $sys->push_error($msg);
        return 0;
    }

    $rtn=$sys->cmd("_cmd_gabconfig -a 2>&1 | _cmd_grep 'Port' | _cmd_grep 'gen'");
    for my $line (split(/\n/,$rtn)) {
        if ($line=~/GAB gabconfig ERROR/m) {
            $msg=Msg::new("GAB is not started on $sys->{sys}");
            $sys->push_error($msg);
            return 0;
        } elsif ($line=~/^Port (\S)\s+gen\s+\S+\s+membership/m) {
            $gab_ports{$1}=1;
        } elsif ($line=~/^Port (\S)\s+gen\s+\S+\s+jeopardy/m) {
            $gab_jeopardy_ports{$1}=1;
        }
    }

    for my $port (qw(a b d f h u v w y)) {
        next if($port=~/d/m && $prod->sfsybasece());
        push (@ports, $port) unless($gab_ports{$port});
        push (@jeopardy_ports, $port) if($gab_jeopardy_ports{$port});
    }

    if (@ports) {
        $ports=join(',',@ports);
        $msg=Msg::new("The following gab ports are not started on $sys->{sys}:\n\t$ports");
        $sys->push_warning($msg);
    }

    if (@jeopardy_ports) {
        $ports=join(',',@jeopardy_ports);
        $msg=Msg::new("The following gab ports are in jeopardy state on $sys->{sys}:\n\t$ports");
        $sys->push_warning($msg);
        return 0;
    }

    return 1 unless (@ports);
    return 0;
}

sub postcheck_cvmconfig_sys{
    my ($prod,$sys) = @_;
    my ($msg,$provers,$cpic,$rtn,$rtn1,$tmpdir,$nodeid,$nodeid1,$output,$rel,$activecomponent,@m);
    $cpic=Obj::cpic();
    $rtn=$sys->cmd("_cmd_hasys -state $sys->{vcs_sysname} 2>/dev/null");
    $tmpdir=EDR::tmpdir();
    $rel=Obj::rel();
    $activecomponent=$rel->get_activecomponent();
    $activecomponent=~s/\d+//g;
    if ($rtn !~/RUNNING/){
        $msg=Msg::new("$activecomponent is not in RUNNING status on $sys->{sys}. It is recommended to start VCS and then start or configure $activecomponent.");
        $sys->push_warning($msg);
        return 0;
    }
    if($sys->system1){
        $rtn=$sys->cmd("_cmd_hagrp -state cvm 2>/dev/null | _cmd_grep 'ONLINE' | _cmd_grep -v '#' | _cmd_sed -n '1p' |_cmd_awk '{print \$3}'");
        my $tsys=($Obj::pool{"Sys::$rtn"}) ? Obj::sys($rtn) : '';
        if($tsys){
            $provers=$tsys->cmd("_cmd_grep volboot /etc/vx/volboot 2>/dev/null | _cmd_awk '{print \$4}'");
            EDRu::writefile($provers, "$tmpdir/protocolversion", 1) if($provers);
        }
    }
    $rtn=$sys->cmd("_cmd_hagrp -state cvm -sys $sys->{vcs_sysname} 2>/dev/null");
    if($rtn !~/ONLINE/){
        $msg=Msg::new("CVM is not running on $sys->{sys}");
        push(@m,$msg);
        $rtn1=$sys->cmd("_cmd_grep '^group cvm ' /etc/VRTSvcs/conf/config/main.cf 2>/dev/null");
        unless($rtn1){
            $msg=Msg::new("\n\tThe group cvm is not configured on $sys->{sys}");
            push(@m,$msg);
        }

        $rtn1=$sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep 'Port b'");
        unless($rtn1){
            $msg=Msg::new("\n\tvxfen is not running on $sys->{sys}");
            push(@m,$msg);
        }
        $rtn1=$sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep 'Port a'");
        unless($rtn1){
            $msg=Msg::new("\n\tgab is not running on $sys->{sys}");
            push(@m,$msg);
        }

        $rtn1=$sys->cmd("/opt/VRTSvcs/bin/hagrp -value cvm AutoStartList 2>/dev/null | _cmd_grep $sys->{sys}");
        unless($rtn1){
            $msg=Msg::new("\n\tThe attribute AutoStartList is not configured on $sys->{sys}");
            push(@m,$msg);
        }
        $rtn1=$sys->cmd("_cmd_hares -value cvm_clus CVMNodeId 2>/dev/null");
        if($rtn1){
            $nodeid=$1 if($rtn1=~/$sys->{sys}\cI(\d+)/);
            $output=$sys->cmd("_cmd_grep $sys->{sys} /etc/llthosts 2>/dev/null");
            if($output){
                $nodeid1= $1 if($output=~/(\d+)\s+$sys->{vcs_sysname}/);
                if($nodeid ne $nodeid1){
                    $msg=Msg::new("\n\tThe attribute CVMNodeId is not consistent with the defenition in /etc/llthosts");
                    push(@m,$msg);
                }
            }
        }
        my ($pv,$localsys,$pv1,$out);
        $out=$sys->cmd("_cmd_grep volboot /etc/vx/volboot 2>/dev/null ");
        if($out){
            (undef,undef,undef,$pv)=split(/\s+/,$out);
        }

        $localsys = $prod->localsys;
        $pv1=$localsys->catfile("$tmpdir/protocolversion") if($localsys->exists("$tmpdir/protocolversion"));
        if($pv & $pv1 && ($pv ne $pv1)){
              $msg=Msg::new("\n\tCVM protocol version is $pv on $sys->{sys}. It is recommended to check the protocol version.");
              push(@m,$msg);
        }

        $sys->push_warning(@m);
        return 0;
    }
    return 1;
}

sub postcheck_fsproto_sys {
    my ($prod,$sys) = @_;
    my ($supported_protos,$current_proto,$max_proto,$msg,$currentmsg,$maxmsg,$supportedmsg,$versmsg);
    $currentmsg = Msg::new("Current version");
    $maxmsg = Msg::new("Latest version");
    $supportedmsg = Msg::new("Supported version(s)");
    if ($sys->exists($sys->padv->{cmd}{fsclustadm})) {
        $supported_protos = $sys->cmd("_cmd_fsclustadm protoversion 2>/dev/null | _cmd_awk '/^local:/ {print \$2}'");
        $current_proto = $sys->cmd("_cmd_fsclustadm protoversion 2>/dev/null | _cmd_awk '/^local:/ {print \$3}'");
        if ( $supported_protos && $current_proto && $current_proto ne '-') {
            $max_proto = (reverse sort {$a<=>$b} split(/,/,$supported_protos))[0];
            if ($current_proto != $max_proto) {
                $versmsg = sprintf("\n\t%-21s: %s\n\t%-21s: %s\n\t%-21s: %s", $currentmsg->{msg},$current_proto, $maxmsg->{msg},$max_proto, $supportedmsg->{msg},$supported_protos);
                $msg = Msg::new("VxFS cluster is not running at the latest protocol version on $sys->{sys}:$versmsg");
                $sys->push_warning($msg);
                return 0;
            }
        }
    }
    return 1;
}

sub postcheck_odmproto_sys {
    my ($prod,$sys) = @_;
    my ($supported_protos,$current_proto,$max_proto,$msg,$currentmsg,$maxmsg,$supportedmsg,$versmsg);
    $currentmsg = Msg::new("Current version");
    $maxmsg = Msg::new("Latest version");
    $supportedmsg = Msg::new("Supported version(s)");
    if ($sys->exists($sys->padv->{cmd}{odmclustadm})) {
        $supported_protos = $sys->cmd("_cmd_odmclustadm protoversion 2>/dev/null | _cmd_awk '/^local:/ {print \$2}'");
        $current_proto = $sys->cmd("_cmd_odmclustadm protoversion 2>/dev/null | _cmd_awk '/^local:/ {print \$3}'");
        if ( $supported_protos && $current_proto && $current_proto ne '-') {
            $max_proto = (reverse sort {$a<=>$b} split(/,/,$supported_protos))[0];
            if ($current_proto != $max_proto) {
                $versmsg = sprintf("\n\t%-21s: %s\n\t%-21s: %s\n\t%-21s: %s", $currentmsg->{msg},$current_proto, $maxmsg->{msg},$max_proto, $supportedmsg->{msg},$supported_protos);
                $msg = Msg::new("VxODM cluster is not running at the latest protocol version on $sys->{sys}:$versmsg");
                $sys->push_warning($msg);
                return 0;
            }
        }
    }
    return 1;
}

sub postcheck_mountedcfs_sys {
    my ($prod,$sys) = @_;
    my ($fs, $mounted_bds, $vcs, $vcs_bds, $dgname, $dgflag, $msg, $nonvcsbdsmsg);
    $fs= $prod->prod('FS731');
    $mounted_bds = $fs->mounted_vxfs_blockdevice_sys($sys);
    $vcs = $prod->prod('VCS731');
    $vcs_bds = $vcs->cfsmount_res_sys($sys);
    $nonvcsbdsmsg = '';
    for my $bd (@{$mounted_bds}) {
        if ($bd =~ /^\/dev\/vx\/dsk\/([^\/]*)\/.*$/mx) {
            $dgname = $1;
            $dgflag = $sys->cmd("_cmd_vxdg list $dgname 2>/dev/null | _cmd_grep '^flags:'");
            if ( $dgflag =~ /shared/) {
                if (!EDRu::inarr($bd, @{$vcs_bds})) {
                    $nonvcsbdsmsg .= "\n\t$bd";
                }
            }
        }
    }
    if ($nonvcsbdsmsg) {
        $msg = Msg::new("The following mounted CFS block devices are not configured in VCS on $sys->{sys}:$nonvcsbdsmsg");
        $sys->push_warning($msg);
        return 0;
    } else {
        return 1;
    }
}

sub register_postchecks {
    my $prod=shift;
    my ($sequence_id,$name,$desc,$handler);

    $prod->perform_task('register_postchecks');

    $sequence_id=510;
    $name='cvmconfig';
    $desc=Msg::new("cvm status");
    $handler=\&postcheck_cvmconfig_sys;
    $prod->register_postcheck_item_per_system($sequence_id,"$name",$desc,$handler);

    $sequence_id=511;
    $name='fsproto';
    $desc=Msg::new("CFS protocol version");
    $handler=\&postcheck_fsproto_sys;
    $prod->register_postcheck_item_per_system($sequence_id,"$name",$desc,$handler);

    $sequence_id=512;
    $name='odmproto';
    $desc=Msg::new("ODM protocol version");
    $handler=\&postcheck_odmproto_sys;
    $prod->register_postcheck_item_per_system($sequence_id,"$name",$desc,$handler);

    $sequence_id=513;
    $name='cfsmounts';
    $desc=Msg::new("Mounted CFS file systems");
    $handler=\&postcheck_mountedcfs_sys;
    $prod->register_postcheck_item_per_system($sequence_id,"$name",$desc,$handler);

    # need override VCS 330_gabconfig postcheck.
    $sequence_id=330;
    $name='gabconfig';
    $desc=Msg::new("gabconfig ports status");
    $handler=\&postcheck_gabconfig_sys;
    $prod->register_postcheck_item_per_system($sequence_id,"$name",$desc,$handler);

    return 1;
}

sub addnode_poststart {
    my $prod = shift;
    my ($cfsnfssg,$firstnode,$msg,$errmsg,$out,$proc,$proci,$rtn,$startprocs,$sys,$sysi);
    my ($rel,$activecomponent,@not_startprocs);
    my $cpic = Obj::cpic();
    my $cfg = Obj::cfg();

    $rel = Obj::rel();
    $activecomponent = $rel->get_activecomponent();
    $firstnode = Obj::sys(${$cfg->{clustersystems}}[0]);
    # start other SFCFS processes
    for my $sysi (@{$cfg->{newnodes}}) {
        $sys = Obj::sys($sysi);
        $startprocs = $prod->startprocs_sys($sys);
        # if the active component is sfsybase, don't start vxgms and vxodm
        if (EDRu::inarr($activecomponent,qw/SFSYBASECE731/)) {
            @not_startprocs = qw(vxgms731 odm731);
            $startprocs = EDRu::arrdel($startprocs,@not_startprocs);
        }    
        for my $proci (@{$startprocs}) {
            $proc = $sys->proc($proci);
            next if ($proc->{donotstart});
            next if ($proc->check_sys($sys, 'prestart'));
            $msg = Msg::new("Starting $proc->{proc} on $sysi");
            if ($cpic->proc_start_sys($sys, $proc)) {
                CPIC::proc_start_passed_sys($sys, $proc);
                $msg->display_status();
            } else {
                $errmsg = Msg::new("Failed to start $proc->{proc} on $sysi");
                $msg->addError($errmsg->{msg});
                CPIC::proc_start_failed_sys($sys, $proc);
                $msg->display_status('failed');
            }
        }
    }

    $msg = Msg::new("Configure CVM and CFS");
    $msg->left();
    $rtn = $prod->addnode_config_cvm_cfs();
    if ($rtn) {
        Msg::right_done();
        # configure cnfs on new nodes if configured
        $out = $firstnode->cmd("_cmd_grep '^CFSSG_SVC_GRP' /opt/VRTSvxfs/cfs/lib/cfsadm.env 2> /dev/null");
        if ($out =~ /CFSSG_SVC_GRP\s*=\s*(.*)/m) {
            $cfsnfssg = $1;
            $cfsnfssg = EDRu::despace($cfsnfssg);
            $firstnode->cmd("_cmd_hagrp -display $cfsnfssg");
            if (!EDR::cmdexit()) {
                $rtn = $prod->addnode_config_cnfs();
                sleep 5;
            }
        }
        $rtn = $prod->addnode_mount_share_dg();
    } else {
        Msg::right_failed();
    }

    if (!$rtn) {
        $msg = Msg::new("Addnode poststart did not complete successfully");
        $sys->push_error($msg);
    }
    return $rtn;
}

sub addnode_config_cnfs {
    my ($msg,$status,$sys,$sysname);
    my $prod = shift;
    my $cfg = Obj::cfg();

    $status = 1;
    for my $sysi (@{$cfg->{newnodes}}) {
        $sys = Obj::sys($sysi);
        $sysname = $sys->{vcs_sysname};
        $msg = Msg::new("Configure clustered NFS on $sysname");
        $msg->left;
        $sys->cmd("_cmd_cfsshare addnode $sysname");
        if (EDR::cmdexit()) {
            $msg->right_failed();
            $status = 0;
        } else {
            $msg->right_done();
        }
    }
    return $status;
}

sub addnode_config_cvm_cfs {
    my ($firstnode,$n,$out,$rtn,$rel,$status,$sys,$sysi,$system,$vxfsckd,$vxfsckd_activation,$asymmetry_key,$asymmetry_value,$cvmres,$cvmsg_name,$max_pri);
    my $prod = shift;
    $rel = Obj::rel();
    my $activecomponent=$rel->get_activecomponent();
    my $cfg = Obj::cfg();
    my $vcs = $prod->prod('VCS731');

    $firstnode = Obj::sys(${$cfg->{clustersystems}}[0]);
    $cvmsg_name = $firstnode->cmd("_cmd_hares -display -attribute Group -type CVMCluster -localclus 2>/dev/null| _cmd_tail -1 | _cmd_awk '{print \$4}'");
    if (!EDRu::inarr($cvmsg_name, split (/\n/, $firstnode->cmd("$vcs->{bindir}/hagrp -list 2>/dev/null | _cmd_awk '{print \$1}' | _cmd_sort | _cmd_uniq")))) {
        Msg::log('Group cvm does not exist in cluster');
        return 1;
    }

    $status = 1;
    $vcs->haconf_makerw();
    # get ActivationMode attribute
    $out = $firstnode->cmd("$vcs->{bindir}/hares -list Group=$cvmsg_name Type=CFSfsckd 2>/dev/null| _cmd_awk '{print \$1}'");
    $vxfsckd = (split(/\n/,$out))[0] if ($out);
    if ($vxfsckd) {
        $system = $firstnode->{vcs_sysname};
        $out = $firstnode->cmd("$vcs->{bindir}/hares -value $vxfsckd ActivationMode $system 2>/dev/null");
        chomp($out);
        $vxfsckd_activation = $out if ($out);
    }
    $max_pri = $vcs->get_sg_max_priority($cvmsg_name);
    for my $sysi (@{$cfg->{newnodes}}) {
        $sys = Obj::sys($sysi);
        $system = $sys->{vcs_sysname};
        $n = $firstnode->cmd("$vcs->{bindir}/hasys -value $system LLTNodeId");
        $max_pri+=1;
        $rtn = $firstnode->cmd("$vcs->{bindir}/hagrp -modify $cvmsg_name SystemList -add $system $max_pri");
        if(EDRu::isverror($rtn)) {
            Msg::log("Modify cvm SystemList to add $sysi failed.");
            $status = 0;
        }

        $rtn = $firstnode->cmd("$vcs->{bindir}/hagrp -modify $cvmsg_name AutoStartList -add $system");
        if (EDRu::isverror($rtn)) {
            Msg::log("Modify cvm AutoStartList to add $sysi failed");
            $status = 0;
        }

        $cvmres = $firstnode->cmd("_cmd_hares -list Type=CVMCluster -localclus 2>/dev/null| _cmd_awk '{print \$1}' | _cmd_uniq");
        $rtn = $firstnode->cmd("$vcs->{bindir}/hares -modify $cvmres CVMNodeId -add $system $n");
        if(EDRu::isverror($rtn)) {
            Msg::log("Modify $cvmres CVMNodeId to add $sysi $n failed.");
            $status = 0;
        }

        # update ActivationMode attribute with new nodes
        if ($vxfsckd_activation) {
            $rtn=$firstnode->cmd("$vcs->{bindir}/hares -modify $vxfsckd ActivationMode $vxfsckd_activation -sys $system");
            if(EDRu::isverror($rtn)) {
                Msg::log("Modify $vxfsckd ActivationMode to add $sysi failed.");
                $status = 0;
            }
        }
    }
    $vcs->haconf_dumpmakero();

    # Run vxclustadm on all nodes of the cluster except new nodes
    for my $sysi (@{$cfg->{clustersystems}}) {
        $sys = Obj::sys($sysi);
        $rtn = $sys->cmd("$prod->{cfsbin}/vxclustadm -m vcs -t gab reinit");
        if (EDRu::isverror($rtn)) {
            Msg::log('Make the nodes of the cluster re-read the cluster configuration file failed.');
            $status = 0;
        }
    }

    # set the asymmetry value on all new nodes, according to the value of existing node
    # changed to storage_connectivity using the command vxtune, refer to e2744683
    $asymmetry_key = 'storage_connectivity';
    $rtn = $firstnode->cmd("_cmd_vxtune $asymmetry_key");
    if ((!EDRu::isverror($rtn)) && ($rtn =~ /$asymmetry_key\s+(\w+)/)) {
        $asymmetry_value = $1;
        for my $sysi (@{$cfg->{newnodes}}) {
            $sys = Obj::sys($sysi);
            # next if the value is the same as nodeone
            $rtn = $sys->cmd("_cmd_vxtune $asymmetry_key");
            if ((!EDRu::isverror($rtn)) && ($rtn =~ /$asymmetry_key\s+(\w+)/)) {
                next if ($asymmetry_value eq $1);
            } else {
                Msg::log("Get tunable $asymmetry_key failed on $sysi.");
                $status = 0;
                next;
            }

            $rtn = $sys->cmd("_cmd_vxtune $asymmetry_key $asymmetry_value");
            if (EDRu::isverror($rtn)||EDR::cmdexit()) {
                Msg::log("Set tunable $asymmetry_key to $asymmetry_value failed on $sysi.");
                $status = 0;
            }
        }
    } else {
        Msg::log("Get tunable $asymmetry_key failed on $firstnode->{sys}");
        $status = 0;
    }

    # Make newnode cvm online
    if ($activecomponent ne 'SFRAC731') {
        # Make newnode cvm online
        for my $sysi (@{$cfg->{newnodes}}) {
            $sys = Obj::sys($sysi);
            $system = $sys->{vcs_sysname};
            # Run hagrp on one node of cluster
            $rtn = $firstnode->cmd("$vcs->{bindir}/hagrp -online cvm -sys $system");
            if (EDRu::isverror($rtn)) {
                Msg::log("Make cvm online failed on $sysi");
                $status = 0;
            }
            $rtn = $firstnode->cmd("$vcs->{bindir}/hagrp -wait cvm State ONLINE -sys $system -time 120");
            if (EDRu::isverror($rtn)||EDR::cmdexit()) {
                Msg::log("Waiting for cvm state online failed on $sysi.");
                $status = 0;
            }
        }
    } else {
        #
        # Start cvm resources only because the sfrac resources in cvm group
        # are still not configured for new nodes.
        #
        for my $sysi (@{$cfg->{newnodes}}) {
            $sys = Obj::sys($sysi);
            $system = $sys->{vcs_sysname};
            for my $resname (qw/cvm_vxconfigd cvm_clus vxfsckd/ ) {
                $rtn = $firstnode->cmd("$vcs->{bindir}/hares -online $resname -sys $system");
                if (EDRu::isverror($rtn)) {
                    Msg::log("Make $resname online failed on $sysi");
                    $status = 0;
                }
                $rtn = $firstnode->cmd("$vcs->{bindir}/hares -wait $resname State ONLINE -sys $system -time 120");
                if (EDRu::isverror($rtn)||EDR::cmdexit()) {
                    Msg::log("Waiting for $resname state online failed on $sysi.");
                    $status = 0;
                }
            }
        }
    }
    return $status;
}

sub addnode_mount_share_dg {
    my ($ayn,$activemode,$failed,$firstnode,$msg,$mount_info,$mounted,$mount_point,$mount_option,$n,$primary_node,$output,$rtn,$sharedg,$sharevol,@sharevols,$sys,$sysi,$system,$master_node,@lines,@fields,@all_options,@target_options,@ckpt_options);
    my $prod = shift;
    my $cfg = Obj::cfg();
    my $rel = Obj::rel();
    my $activecomponent = $rel->get_activecomponent();
    my $vcs = $prod->prod('VCS731');
    my ($cvmgroupstate, $resnames, @resnames, $depresnames, @depresnames, $restype, $cfsmount_res_in_cvm, $cvmvoldg_res_in_cvm);
    my ($dgstate,$state,@ldisabled_vols);

    $state = 'LDISABLED';
    $dgstate = {};
    $failed = 0;
    my $nodename = ${$cfg->{clustersystems}}[0];
    $firstnode = ($Obj::pool{"Sys::$nodename"}) ? Obj::sys($nodename) : Sys->new($nodename);

    my $ret = $prod->addnode_need_mount_sharedg();
    return $ret if ($ret == 1);
    @sharevols = @{$ret};

    $n = scalar @sharevols;
    unless ($n > 0 ) {
        Msg::log('No share disk resources were found in cluster');
        return 1;
    }
    $msg = Msg::new("The cluster has $n shared volumes mounted\n");
    $msg->print;
    printf("%18s %18s %18s \n",'shared_volumes','mounted_point','shared_disk_group');
    my $web_str = EDRu::fixed_length_str('shared_volumes',30,'L');
    $web_str .= EDRu::fixed_length_str('mounted_point',30,'L');
    $web_str .= EDRu::fixed_length_str('shared_disk_group',30,'R');
    $web_str .= "\\n";
    for my $mount_info (@sharevols) {
        printf("%18s %18s %18s \n",$mount_info->[0], $mount_info->[1],$mount_info->[2]);
        $web_str .= EDRu::fixed_length_str($mount_info->[0],30,'L');
        $web_str .= EDRu::fixed_length_str($mount_info->[1],30,'L');
        $web_str .= EDRu::fixed_length_str($mount_info->[2],30,'R');
        $web_str .= "\\n";
    }
    $web_str =~ s/\s/&nbsp;/mg;
    my $web = Obj::web();
#    $web->web_script_form('alert',$web_str) if (Obj::webui());

    $master_node = $firstnode->cmd("_cmd_vxdctl -c mode 2>/dev/null |_cmd_grep '^master:'| _cmd_awk '{print \$2}'");
    if( ! $master_node) {
        Msg::log("There is no master node. Use $firstnode->{sys} instead.");
        $master_node = $firstnode->{sys};
    }
    for my $sysi (@{$cfg->{newnodes}}) {
        @ldisabled_vols = ();
        $sys = ($Obj::pool{"Sys::$sysi"}) ? Obj::sys($sysi) : Sys->new($sysi);
        for my $mount_info (@sharevols) {
            push (@ldisabled_vols,$mount_info->[0]) if $prod->is_ioship_on_and_ldisabled_sys($sys,$mount_info->[0],$mount_info->[2],$dgstate);
        }

        # ask user whether wants to mount these shared volumes on the new node.
        # if "Y", add them on new node, otherwise, do nothing on new node.
        $cvmgroupstate = $sys->cmd("_cmd_hagrp -state cvm -sys $sys->{vcs_sysname} ");
        if ($activecomponent =~ /^SFRAC\d+/mx && $cvmgroupstate ne 'ONLINE') {
            $msg = Msg::new("The cluster has above shared volumes. Due to service group dependencies, these shared volumes cannot be mounted before cvm service group is online on $sysi. Would you like to configure these shared volumes on $sysi?");
        } else {
            $msg = Msg::new("The cluster has above shared volumes. Would you like to mount these shared volumes on $sysi?");
        }
        if(scalar @ldisabled_vols){
            my $ldisabled_vols = join(',',@ldisabled_vols);
            my $ldisabled_msg = Msg::new(" (Will skip volume(s) which are in $state state: $ldisabled_vols)");
            $msg->{msg}.=$ldisabled_msg->{msg};
        }
        $ayn = $msg->ayny('','',$web_str);
        if( $ayn eq 'N'){
            if ($activecomponent =~ /^SFRAC\d+/mx && $cvmgroupstate ne 'ONLINE') {
                $cfsmount_res_in_cvm = $sys->cmd("_cmd_hares -list Type=CFSMount Group=cvm 2>/dev/null | _cmd_awk '{print \$1}' 2>/dev/null |_cmd_sort -u");
                $cfsmount_res_in_cvm = join("\n\t",split(/\n/,$cfsmount_res_in_cvm));
                $cvmvoldg_res_in_cvm = $sys->cmd("_cmd_hares -list Type=CVMVolDg Group=cvm 2>/dev/null | _cmd_awk '{print \$1}' 2>/dev/null |_cmd_sort -u");
                $cvmvoldg_res_in_cvm = join("\n\t",split(/\n/,$cvmvoldg_res_in_cvm));
                if ($cfsmount_res_in_cvm || $cvmvoldg_res_in_cvm) {
                    if ($cfsmount_res_in_cvm) {
                        if ($cvmvoldg_res_in_cvm) {
                            $msg = Msg::new("As you selected not to configure these shared volumes on $sysi, before cvm service group can be brought online on this system, you must manually update NodeList and MountOpt attributes for the following CFSMount resources configured in cvm service group\n\n\t$cfsmount_res_in_cvm\n\nand CVMActivation attribute for the following CVMVolDg resources configured in cvm service group\n\n\t$cvmvoldg_res_in_cvm");
                        } else {
                            $msg = Msg::new("As you selected not to configure these shared volumes on $sysi, before cvm service group can be brought online on this system, you must manually update NodeList and MountOpt attributes for the following CFSMount resources configured in cvm service group\n\n\t$cfsmount_res_in_cvm");
                        }
                    } else {
                        $msg = Msg::new("As you selected not to configure these shared volumes on $sysi, before cvm service group can be brought online on this system, you must manually update CVMActivation attribute for the following CVMVolDg resources configured in cvm service group\n\n\t$cvmvoldg_res_in_cvm");
                    }
                    if (Obj::webui()) {
                        $web_str = $msg->{msg};
                        $web_str =~ s/\t/&nbsp;&nbsp;&nbsp;&nbsp;/g;
                        $web_str =~ s/\n\n/<br\/>&nbsp;<br\/>/g;
                        $web_str =~ s/\n/<br\/>/g;
                        $web->web_script_form('alert',$web_str);
                        Msg::log($web_str);
                    } else {
                        $msg->print();
                        Msg::prtc();
                    }
                }
            }
            next;
        }
        $system = $sys->{vcs_sysname};
        # Set flag here for SFRAC to mount shared volumes after cvm is online
        $sys->{mount_share_dg} = 1;
        for my $mount_info (@sharevols) {
            $sharevol = $mount_info->[0];
            $mount_point = $mount_info->[1];
            $sharedg = $mount_info->[2];
            next if EDRu::inarr($sharevol, @ldisabled_vols);
            if ($activecomponent =~ /^SFRAC\d+/mx && $cvmgroupstate ne 'ONLINE') {
                $msg = Msg::new("Configuring mountpoint $mount_point for $sysi");
            } else {
                $msg = Msg::new("Mounting $sharevol on $mount_point for $sysi");
            }
            $msg->left;

            $activemode = undef;
            $output = $firstnode->cmd(" _cmd_cfsdgadm display $sharedg | _cmd_sed -e '1d' ");
            @lines = split(/\n/,$output);
            for ( @lines ) {
                @fields = split;
                if ( $fields[0] eq "$master_node" ) {
                    $activemode = $fields[1];
                    last;
                }
            }
            if($activemode) {
                # check whether the new node has added sharedg, if no, add the sharedg for the new node
                $rtn = undef;
                for ( @lines ) {
                    @fields = split;
                    if ( $fields[0] eq "$system" ) {
                        $rtn = $fields[0];
                        last;
                    }
                }
                if (! $rtn) {
                    $rtn = $firstnode->cmd("_cmd_cfsdgadm add $sharedg $system=$activemode");
                    if (EDRu::isverror($rtn)) {
                        Msg::log("Failed to add shared disk group $sharedg on $sysi");
                        Msg::right_failed();
                        $failed++;
                        next;
                    }
                }
            }

            for my $nodename (@{$cfg->{clustersystems}}) {
                my $node = ($Obj::pool{"Sys::$nodename"}) ? Obj::sys($nodename) : Sys->new($nodename);
                $primary_node = $node->cmd("_cmd_fsclustadm -v showprimary $mount_point 2>&1");
                if (EDRu::isverror($primary_node)) {
                    Msg::log("Failed to show primary mount point $mount_point from node $nodename, try next node");
                    next;
                }
                last;
            }
            if (EDRu::isverror($primary_node)) {
                Msg::log("Failed to show primary mount point $mount_point on $sysi: $primary_node");
                Msg::right_failed();
                $failed++;
                next;
            }
            Msg::log("The primary node is $primary_node");
            $mount_option = '';
            $output = $firstnode->cmd("_cmd_cfsmntadm display -v $primary_node | _cmd_sed -e '1,2d'");
            @lines = split(/\n/,$output);
            for ( @lines ) {
                @fields = split;
                if ( $fields[0] eq "$mount_point" && $fields[2] eq "$sharevol" && $fields[3] eq "$sharedg" ) {
                    #e3869620:ckpt=ckpt1 is not needed for checkpoint type
                    @all_options = split(/,/,$fields[5]);
                    @ckpt_options = grep /=/, @all_options;
                    @target_options = @{EDRu::arrdel(\@all_options, @ckpt_options)};
                    $mount_option = join(",",@target_options);
                    last;
                }
            }
            # mount_option could be ""
            if (! $mount_option ) {
                $mount_option = '';
            }
            Msg::log("The mount_options of primary node is $mount_option");
            $rtn = $firstnode->cmd("_cmd_cfsmntadm modify $mount_point add $system=$mount_option");
            if (EDRu::isverror($rtn)) {
                Msg::log("Failed to modify $mount_point/$mount_option on $sysi: $rtn");
                Msg::right_failed();
                $failed++;
                next;
            }
            # make sure CVMActivation of CVMVolDg is added for $mount_point on $sys
            $resnames = $sys->cmd("_cmd_hares -list Type=CFSMount MountPoint='$mount_point' 2>/dev/null |_cmd_awk '{print \$1}' 2>/dev/null | _cmd_sort -u ");
            @resnames = split(/\n/,$resnames);
            if ( scalar @resnames > 0) {
                if (scalar @resnames > 1) {
                    Msg::log("Multiple CFSMount resourses defined for mount point $mount_point");
                }
                for my $cfsmntres (@resnames) {
                    $depresnames =$sys->cmd("_cmd_hares -dep $cfsmntres 2>/dev/null | _cmd_sed '1d' 2>/dev/null | _cmd_awk  '{print \$3}'");
                    @depresnames = split(/\n/,$depresnames);
                    for my $dep (@depresnames) {
                        next if ($dep eq $cfsmntres);
                        $restype = $sys->cmd("_cmd_hares -value $dep Type");
                        chomp $restype;
                        next if ($restype ne 'CVMVolDg');
                        $activemode = $sys->cmd("_cmd_hares -value $dep CVMActivation $sys->{vcs_sysname}");
                        chomp $activemode;
                        if ( ! $activemode ) {
                            $activemode = $sys->cmd("_cmd_hares -value $dep CVMActivation $firstnode->{vcs_sysname}");
                            chomp $activemode;
                            if ($activemode) {
                                $sys->cmd("$vcs->{bindir}/haconf -makerw");
                                $sys->cmd("_cmd_hares -modify $dep CVMActivation $activemode -sys $system");
                                $sys->cmd("$vcs->{bindir}/haconf -dump -makero");
                            } else {
                                Msg::log("No CVMActivation defined for $dep on $firstnode->{vcs_sysname} or $system");
                            }
                        }
                    }
                }
            } else {
                Msg::log("No CFSMount resourse defined for mount point $mount_point");
            }

            if ($activecomponent =~ /^SFRAC\d+/mx && $cvmgroupstate ne 'ONLINE') {
                $rtn = 1; # For SFRAC, deferring mounting shared DGs until SFRAC is fully configured on new nodes.
            #    $failed++;
                Msg::right_done();
                next;
            } else {
                $rtn = $firstnode->cmd("_cmd_cfsmount $mount_point $system");
            }
            if (EDRu::isverror($rtn)) {
                Msg::log("Failed to mount $mount_point on $sysi: $rtn");
                Msg::right_failed();
                $failed++;
                next;
            }

            Msg::right_done();
        }

        if ($failed == 0) {
            Msg::log("Successfully mounted all shared resources on $sysi");
        } elsif ($failed == $n){
            Msg::log("Failed to mount all shared resources on $sysi");
        } else {
            Msg::log("Partially failed to mount shared resources on $sysi with status: $failed");
        }
    }
    return (!$failed);
}

# Get the share volumes to be mounted on the added node
# Abstract to a subroutine for it to be called in SFRAC
sub addnode_need_mount_sharedg {
    my $prod = shift;
    my $cfg = Obj::cfg();
    my ($firstnode, $output, @lines, $mount_point, $sharevol, $sharevols, $sharedg, $mounted, $sysi, $sys, $system);

    $sharevols = [];
    my $nodename = ${$cfg->{clustersystems}}[0];
    $firstnode = ($Obj::pool{"Sys::$nodename"}) ? Obj::sys($nodename) : Sys->new($nodename);
    # get the share volume and its mount point from first node of the cluster
    $output = $firstnode->cmd("_cmd_cfsmntadm display 2>/dev/null | _cmd_grep 'MOUNTED' | _cmd_sort -u ");
    if (EDR::cmdexit()) {
        Msg::log('cfsmntadm display error.');
        return 1;
    }
    @lines = split(/\n/,$output);
    foreach (@lines) {
        ($mount_point,undef,$sharevol,$sharedg,$mounted) = split;
        if($mounted eq 'MOUNTED') {
            $sysi = ${$cfg->{newnodes}}[0];
            $sys = Obj::sys($sysi);
            $system = $sys->{vcs_sysname};
            $output = $firstnode->cmd("_cmd_cfsmntadm display $mount_point");
            if ((!EDR::cmdexit()) && ($output =~ /\n\s*$system\s+MOUNTED/m)) {
                Msg::log("Shared volume $sharevol is already mounted on mount point $mount_point of system $sysi");
                next;
            }
            push (@{$sharevols},[$sharevol,$mount_point,$sharedg]);
        }
    }
    return $sharevols;
}

sub delnode_precheck {
    my $prod=shift;
    $prod->perform_task('delnode_precheck');
    return;
}

sub sfcfsha {
    my $prod=shift;
    return 1 if ($prod->{class} =~ /SFCFSHA/m);
    return 0;
}

# return 1 only if the dg is iostate:on and
sub is_ioship_on_and_ldisabled_sys {
    my ($prod,$sys,$vol,$dg,$dgstate) = @_;
    my ($rtn,$state);

    $state = 'LDISABLED';
    # no need to check the dg's ioship state
#    unless(defined $dgstate->{$dg}{ioship}) {
#        $rtn = $sys->cmd("_cmd_vxdg list $dg 2>/dev/null");
#        if ($rtn =~ /^\s*ioship\s*:\s*(\S+)/m) {
#            $dgstate->{$dg}{ioship} = $1;
#        } else {
#            return 0;
#        }
#    }
    # not fit if the ioship is not on
#    return 0 unless ($dgstate->{$dg}{ioship} =~ /on/);

    #
    $rtn = $sys->cmd("_cmd_vxprint -g $dg 2>/dev/null");
    return 1 if ($rtn =~ /\s+$vol\s+.*$state/m);
    return 0;
}

sub svs {
    my $prod=shift;
    return 1 if ($prod->{class} =~ /SVS/m);
    return 0;
}

sub sfsybasece {
    my $prod=shift;
    return 1 if ($prod->{class} =~ /SFSYBASECE/m);
    return 0;
}

sub sfcfs {
    my $prod=shift;
    return 1 if ($prod->{class} =~ /SFCFS/m);
    return 0;
}

sub get_supported_tunables {
    my ($prod) =@_;
    my ($tunables,$sfha, $sfcfsha);
    $tunables = [];
    push @$tunables, @{$prod->get_tunables};
    if ($prod->svs() || $prod->sfsybasece()) {
        $sfcfsha=$prod->prod('SFCFSHA731');
        push @$tunables, @{$sfcfsha->get_supported_tunables};
    } else {
        $sfha=$prod->prod('SFHA731');
        push @$tunables, @{$sfha->get_supported_tunables};
    }
    return $tunables;
}

package Prod::SFCFSHA731::AIX;
@Prod::SFCFSHA731::AIX::ISA = qw(Prod::SFCFSHA731::Common);

sub init_plat {
    my $prod=shift;
    $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
    $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{proddir}='sfcfsha';
    $prod->{upgradevers}=[qw(6.0 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(5.0.3 5.1 6.0 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTScsocw.rte VRTSdbac731 VRTSdbac.rte VRTSvcswiz731 VRTSvbs731 VRTSvcsea731 VRTScfsdc
        VRTSodm731 VRTSgms731 VRTScavf731 VRTSglm731 VRTScpi.rte
        VRTSd2doc VRTSordoc VRTSfppm VRTSap VRTStep VRTSvcsdb.rte
        VRTSvcsor.rte VRTSgapms.VRTSgapms VRTSmapro VRTSvail.VRTSvail
        VRTSd2gui VRTSorgui VRTSvxmsa VRTSdbdoc VRTSdb2ed VRTSdbed731
        VRTSdbcom VRTSvcsApache VRTScmc VRTSccacm VRTSvcsw.rte
        VRTScspro VRTSvcsdb.rte VRTSvcssy.rte VRTScmccc.rte
        VRTScmcs.rte VRTScscm.rte
        VRTScscw.rte VRTScssim.rte VRTScutil VRTScutil.rte
        VRTSvcs.doc VRTSvcs.man VRTSvcs.msg.en_US VRTSvcsag731
        VRTSvcsag.rte VRTScps731 VRTSvcs731 VRTSvcs.rte VRTSamf731 VRTSvxfen731
        VRTSvxfen.rte VRTSgab731 VRTSgab.rte VRTSllt731 VRTSllt.rte
        VRTSfsmnd VRTSfssdk731 VRTSfsadv731 VRTSfsdoc VRTSfsman VRTSvrdoc
        VRTSvrw VRTSweb.rte VRTSvcsvr VRTSvrpro VRTSddlpr VRTSvdid.rte
        VRTSalloc VRTSvsvc VRTSvmpro VRTSdcli VRTSvmdoc VRTSvmman
        SYMClma VRTSspt731 VRTSaa VRTSmh VRTSccg VRTSobgui VRTSfspro
        VRTSdsa VRTSsfmh731 VRTSob34 VRTSobc33 VRTSaslapm731
        VRTSat50 VRTSat.server VRTSat.client VRTSsmf VRTSpbx
        VRTSicsco VRTSvxfs731 VRTSvxvm731 VRTSveki731 VRTSjre15.rte
        VRTSjre.rte VRTSsfcpi731 VRTSvlic32 VRTSperl520 VRTSperl.rte
    ) ];

    return;
}

sub version_mapping_sys {
    my ($prod,$sys,$vers)=@_;
    my ($subprod);
    # to handle VRTScavf version is 1.0.4 in 4.0MP4 release.
    $vers='4.0.4' if (EDRu::compvers($vers, '1.0.4', 3)==0);
    # 6.1.1 VRTScavf doesn't have a Patch. Hence need to check its sub-product to distinguish 6.1.1/6.1
    if (EDRu::compvers($vers, '6.1.0.0', 4) == 0) {
        $subprod=$prod->prod('SFHA731');
        $vers=$subprod->version_sys($sys,1);
    }
    return $vers;
}

package Prod::SFCFSHA731::HPUX;
@Prod::SFCFSHA731::HPUX::ISA = qw(Prod::SFCFSHA731::Common);

sub init_plat {
    my $prod=shift;
    $prod->{allpkgs}=[ qw(VRTSob34 VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
    $prod->{recpkgs}=[ qw(VRTSob34 VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    $prod->{upgradevers}=[qw(6.0 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw()];
    $prod->{menu_options}=['Volume Replicator','Global Cluster Option'];

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTSdbckp VRTSormap VRTScsocw VRTSdbac731 VRTSvbs731 VRTSvcsea731 VRTScfsdc VRTSodm731
        VRTSgms731 VRTScavf731 VRTSglm731 VRTScpi VRTSd2doc VRTSordoc
        VRTSfsnbl VRTSfppm VRTSap VRTStep VRTSgapms VRTSmapro
        VRTSvail VRTSd2gui VRTSorgui VRTSvxmsa VRTSvrdev VRTSdbdoc VRTSdb2ed
        VRTSdbed731 VRTSdbcom VRTSsydoc VRTSsybed VRTSvcsApache
        VRTScmc VRTSccacm VRTSvcsw VRTScspro VRTSvcsdb VRTSvcsor
        VRTSvcssy VRTScmccc VRTScmcs VRTScscm VRTScscw
        VRTScssim VRTScutil VRTSvcsdc VRTSvcsmn VRTSvcsmg
        VRTSvcsag731 VRTScps731 VRTSvcs731 VRTSamf731 VRTSvxfen731 VRTSgab731
        VRTSllt731 VRTSfsmnd VRTSfssdk731 VRTSfsadv731 VRTSfsdoc VRTSfsman
        VRTSvrdoc VRTSvrw VRTSvrmcsg VRTSweb VRTSvcsvr VRTSvrpro VRTSddlpr
        VRTSvdid VRTSvsvc VRTSvmpro VRTSalloc VRTSdcli VRTSvmdoc
        VRTSvmman SYMClma VRTSspt731 VRTSaa VRTSmh VRTSccg VRTSobgui
        VRTSfspro VRTSdsa VRTSsfmh731 VRTSob34 VRTSobc33 VRTSaslapm731
        VRTSat50 VRTSsmf VRTSpbx VRTSicsco VRTSvxfs731 VRTSvxvm731
        VRTSjre15 VRTSjre VRTSsfcpi731 VRTSvlic32 VRTSperl520 VRTSwl
    ) ];
    return;
}


package Prod::SFCFSHA731::Linux;
@Prod::SFCFSHA731::Linux::ISA = qw(Prod::SFCFSHA731::Common);

sub init_plat {
    my $prod=shift;
    $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
    $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
    $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    $prod->{upgradevers}=[qw(6.0 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(5.0.30 5.1 6.0 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{menu_options}=['Volume Replicator','File Replicator Option','Global Cluster Option'];

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTScsocw VRTSvcswiz731 VRTSvbs731 VRTSvcsvmw731 VRTSvcsea731 VRTScfsdc VRTSodm731 VRTSodm-platform
        VRTSodm-common VRTSgms731 VRTSsvs731 VRTScavf731 VRTSglm731 VRTScpi
        VRTSd2doc VRTSordoc VRTSfsnbl VRTSfppm VRTSap VRTStep
        VRTSgapms VRTSmapro-common VRTSvail VRTSd2gui-common
        VRTSorgui-common VRTSvxmsa VRTSdbdoc VRTSdb2ed-common
        VRTSdbed731 VRTSdbed-common VRTSdbcom-common VRTSsybed-common
        VRTSvcsApache VRTScmc VRTSccacm VRTSvcsw VRTScspro
        VRTSvcsdb VRTSvcsor VRTSvcssy VRTScmccc VRTScmcs
        VRTScscm VRTScscw VRTScssim VRTScutil VRTSvcsdc VRTSvcsmn
        VRTSvcsmg VRTSvcsdr VRTSvcsag731 VRTScps731 VRTSvcs731 VRTSamf731
        VRTSvxfen731 VRTSgab731 VRTSllt731 VRTSveki731 VRTSfsmnd VRTSfssdk731
        VRTSfsadv731 VRTSfsdoc VRTSfsman VRTSvrdoc VRTSvrw VRTSweb VRTSvcsvr
        VRTSvrpro VRTSalloc VRTSdcli VRTSvsvc VRTSvmpro VRTSddlpr
        VRTSvdid VRTSlvmconv VRTSvmdoc VRTSvmman SYMClma
        VRTSspt731 VRTSaa VRTSmh VRTSccg VRTSobgui VRTSfspro
        VRTSdsa VRTSsfmh731 VRTSob34 VRTSobc33 VRTSaslapm731
        VRTSat50 VRTSatClient50 VRTSsmf VRTSpbx VRTSicsco VRTSvxfs731
        VRTSvxfs-platform VRTSvxfs-common VRTSvxvm731 VRTSvxvm-platform
        VRTSvxvm-common VRTSjre15 VRTSjre VRTSsfcpi731 VRTSvlic32 VRTSperl520
    ) ];
    $prod->{obsolete_scripts} = [ qw(installsfsybasece62 uninstallsfsybasece62) ];

    return;
}

package Prod::SFCFSHA731::RHEL5x8664;
@Prod::SFCFSHA731::RHEL5x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731) ];
    }
    return;
}

package Prod::SFCFSHA731::RHEL6x8664;
@Prod::SFCFSHA731::RHEL6x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::RHEL6ppc64;
@Prod::SFCFSHA731::RHEL6ppc64::ISA = qw(Prod::SFCFSHA731::Linux);

package Prod::SFCFSHA731::RHEL7x8664;
@Prod::SFCFSHA731::RHEL7x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::OL6x8664;
@Prod::SFCFSHA731::OL6x8664::ISA = qw(Prod::SFCFSHA731::RHEL6x8664);

package Prod::SFCFSHA731::OL7x8664;
@Prod::SFCFSHA731::OL7x8664::ISA = qw(Prod::SFCFSHA731::RHEL7x8664);

package Prod::SFCFSHA731::CentOS6x8664;
@Prod::SFCFSHA731::CentOS6x8664::ISA = qw(Prod::SFCFSHA731::RHEL6x8664);

package Prod::SFCFSHA731::CentOS7x8664;
@Prod::SFCFSHA731::CentOS7x8664::ISA = qw(Prod::SFCFSHA731::RHEL7x8664);

package Prod::SFCFSHA731::SLES10x8664;
@Prod::SFCFSHA731::SLES10x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::SLES11x8664;
@Prod::SFCFSHA731::SLES11x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::SLES11ppc64;
@Prod::SFCFSHA731::SLES11ppc64::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::SLES12x8664;
@Prod::SFCFSHA731::SLES12x8664::ISA = qw(Prod::SFCFSHA731::Linux);

sub init_padv {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } elsif ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731 SFSYBASECE731) ];
    }
    return ;
}

package Prod::SFCFSHA731::SunOS;
@Prod::SFCFSHA731::SunOS::ISA = qw(Prod::SFCFSHA731::Common);

sub init_plat {
    my $prod=shift;
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } else {
        $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    }

    $prod->{obsoleted_previous_releases_pkgs} = [ qw(
        VRTScsocw VRTSdbac731 VRTSvcswiz731 VRTSvbs731 VRTSvcsea731 VRTScfsdc VRTSodm731
        VRTSgms731 VRTSsvs731 VRTScavf731 VRTSglm731 VRTScpi VRTSd2doc VRTSordoc
        VRTSfsnbl VRTSfppm VRTSap VRTStep VRTSspc VRTSspcq
        VRTSfasdc VRTSfasag VRTSfas VRTSgapms VRTSmapro VRTSvail
        VRTSd2gui VRTSorgui VRTSvxmsa VRTSdbdoc VRTSdb2ed VRTSdbed731
        VRTSdbcom VRTSsydoc VRTSsybed VRTSvcsApache VRTScmc
        VRTSccacm VRTSvcsw VRTScspro VRTSvcsdb VRTSvcsor VRTSvcssy
        VRTScmccc VRTScmcs VRTScscm VRTScscw VRTScssim
        VRTScutil VRTSvcsdc VRTSvcsmn VRTSvcsmg VRTSvcsag731
        VRTScps731 VRTSvcs731 VRTSamf731 VRTSvxfen731 VRTSgab731 VRTSllt731
        VRTSfsmnd VRTSfssdk731 VRTSfsadv731 VRTSfsdoc VRTSfsman VRTSvrdoc
        VRTSvrw VRTSweb VRTSvcsvr VRTSvrpro VRTSddlpr VRTSvdid
        VRTSvsvc VRTSvmpro VRTSalloc VRTSdcli VRTSvmdoc VRTSvmman
        SYMClma VRTSspt731 VRTSaa VRTSmh VRTSccg VRTSobgui VRTSfspro
        VRTSdsa VRTSsfmh731 VRTSob34 VRTSobc33 VRTSaslapm731
        VRTSat50 VRTSsmf VRTSpbx VRTSicsco VRTSvxfs731 VRTSvxvm731
        VRTSjre15 VRTSjre VRTSsfcpi731 VRTSvlic32 VRTSperl520
    ) ];
    return;
}

sub version_mapping_sys {
    my ($prod,$sys,$vers)=@_;
    my ($subprod);
    # 6.1.1 VRTScavf doesn't have a Patch. Hence need to check its sub-product to distinguish 6.1.1/6.1
    if (EDRu::compvers($vers, '6.1.0.0', 4) == 0) {
        $subprod=$prod->prod('SFHA731');
        $vers=$subprod->version_sys($sys,1);
    }
    return $vers;
}

package Prod::SFCFSHA731::Sol11sparc;
@Prod::SFCFSHA731::Sol11sparc::ISA = qw(Prod::SFCFSHA731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{upgradevers}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(6.0.10 6.1 6.2 7.0 7.1 7.2 7.3 7.3.1)];
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } else {
        $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    }

    if ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731) ];
    }
    return;
}

package Prod::SFCFSHA731::Sol12sparc;
@Prod::SFCFSHA731::Sol12sparc::ISA = qw(Prod::SFCFSHA731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{upgradevers}=[qw(7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.2 7.3 7.3.1)];
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } else {
        $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    }

    if ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731) ];
    }
    return;
}

package Prod::SFCFSHA731::Sol11x64;
@Prod::SFCFSHA731::Sol11x64::ISA = qw(Prod::SFCFSHA731::SunOS);

sub init_padv {
    my $prod=shift;
    $prod->{upgradevers}=[qw(6.0.10)];
    $prod->{zru_releases}=[qw(6.0.10)];
    if ($prod->svs()) {
        $prod->{minpkgs}=[ qw(VRTSsfmh731 VRTSglm731 VRTScavf731 VRTSsvs731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSsvs731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='virtualstore';
    } else {
        $prod->{minpkgs}=[ qw(VRTSglm731 VRTScavf731) ] unless (Obj::rel()->{simple});
        $prod->{recpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{allpkgs}=[ qw(VRTSglm731 VRTScavf731 VRTSgms731 VRTSodm731) ] unless (Obj::rel()->{simple});
        $prod->{proddir}='storage_foundation_cluster_file_system_ha';
    }

    if ($prod->sfcfsha) {
        $prod->{licsuperprods}=[ qw(SFRAC731) ];
    }
    return;
}

1;
