#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Prod::STORAGE731::Common;
@Prod::STORAGE731::Common::ISA = qw(Prod);
sub init_common {
    my ($prod) = @_;
    $prod->{prod}="STORAGE";
    $prod->{abbr}=Msg::new("InfoScale Storage")->{msg};
    $prod->{vers}='7.3.1.000';
    $prod->{eula}='EULA.pdf';
    $prod->{name}=Msg::new("Veritas InfoScale Storage")->{msg};
    $prod->{mainpkg}='VRTScavf731';
    $prod->{lic_names}=['Veritas InfoScale Storage'];
    $prod->{lic_keyless_name} = 'STORAGE';
    $prod->{licsuperprods}=[ qw(ENTERPRISE731) ];
    $prod->{upgradevers}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{zru_releases}=[qw(7.0 7.1 7.2 7.3 7.3.1)];
    $prod->{responsefileupgradeok}=1;
    $prod->{subprods}=[qw(SF731 SFCFS731)];
    $prod->{allcomponent}=[qw(SFCFS731)]; #an array include all components without duplicate.
    $prod->{defaultcomponent}='SFCFS731';
    $prod->{has_config} = 1;
    if (Obj::rel()->{simple}) {
        $prod->{mainpkg}='VRTScfs731';
    }
    return;
}

sub description {
    my ($msg);
    
    $msg=Msg::new("Veritas InfoScale Storage enables organizations to provision and manage storage independently of hardware types or locations. InfoScale Storage delivers predictable Quality-of-Service by identifying and optimizing critical workloads. InfoScale Storage increases storage agility enabling you to work with and manage multiple types of storage to achieve better ROI without compromising on performance and flexibility.");

    $msg->print;
    return;
}

sub perform_task_sys {
    my ($prod,$sys,$task) = @_;
    return $prod->perform_component_task_sys($sys,$task);
}

sub perform_task {
    my ($prod,$task) = @_;
    return $prod->perform_component_task($task);
}

sub set_pkgs {
    my($prod,$rel,$sfcfs,$category,@categories,@extrapkgs);
    $prod=shift;

    $rel=$prod->rel;
    $sfcfs=$prod->prod('SFCFS731');

    @categories=qw(allpkgs);
    # The following packages are not supposed to be installed for STORAGE
    @extrapkgs=qw(VRTSvcswiz731 VRTSvcsea731 VRTSvbs731 VRTSfssdk731);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$sfcfs->{$category}});
        $prod->{$category}=EDRu::arrdel($prod->{$category},@extrapkgs);
    }

    @categories=qw(obsoleted_ga_release_pkgs
                   obsoleted_but_need_refresh_when_upgrade_pkgs
                   obsoleted_previous_releases_pkgs
                   obsoleted_but_still_support_pkgs);
    for my $category (@categories) {
        $prod->{$category}=EDRu::arruniq(@{$rel->{$category}},
                                         @{$sfcfs->{$category}});
    }
    push (@{$prod->{allpkgs}}, 'VRTSdbed731') if (Obj::rel()->{simple});
    $prod->{installonupgradepkgs} = $prod->{allpkgs};
    return $prod->{allpkgs};
}

sub upgrade_preinstall_sys {
    my ($prod,$sys) = @_;
    $prod->perform_task_sys($sys,'upgrade_preinstall_sys');
    return;
}

sub licensed_sys {
    my ($prod,$sys)=@_;
    my $rel = Obj::rel();
    return $rel->prod_licensed_sys($sys);
}

sub responsefile_prestart_config {
    my ($prod) = @_;
    my ($cfg);
    $cfg=Obj::cfg();
    if ($cfg->{opt}{activecomponent}) {
        $prod->{activecomponent}=$cfg->{opt}{activecomponent};
    }
    $prod->perform_task('responsefile_prestart_config');
    return;
}

sub verify_responsefile {
    my ($cfg,$msg);

    $cfg=Obj::cfg();

    if ($cfg->{vcs_gco} || $cfg->{vcs_eat_security} || $cfg->{opt}{security}) {
        $msg=Msg::new("GCO or security related items cannot be used with this product.");
        $msg->die();
    }   
    return;
}

sub ask_patchupgrade_configure {
    my ($prod,$sys)=@_;
    return $prod->ask_upgrade_configure();
}

#sub upgrade_prestop_config {
#    my $prod = shift;
#    return $prod->perform_component_task('upgrade_prestop_config', $prod->{previouscomponent});
#}

sub poststart_configure_sys {
    my ($prod,$sys)=@_; 
    return $prod->perform_task_sys($sys, 'poststart_configure_sys'); 
}

sub get_tunables {
    my ($prod)=@_;
    my ($subprodobj,$subprodtunables,@tunables);
    for my $subprod (@{$prod->{subprods}}) {
        $subprodobj=$prod->prod($subprod);
        $subprodtunables=$subprodobj->get_supported_tunables();
        push(@tunables,@{$subprodtunables});
    }
    return EDRu::arruniq(@tunables) || [];
}

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;  
    my ($pkg,$pkgvers,$mpvers,$vers,$prodvers);

    #VRTSllt VRTSvxvm
    #install the 2 packages will be regarded as STORAGE installed.
    my @pkgs=qw(VRTSllt731 VRTSvxvm731);
    @pkgs=qw(VRTSvcsk731 VRTSvxvm731) if (Obj::rel()->{simple} && !Cfg::opt('upgrade'));  
    for my $pkgname (@pkgs) {
        $pkg=$sys->pkg($pkgname);
        $pkgvers=$pkg->version_sys($sys,$force_flag);
        $mpvers=$prod->{vers} if ($prod->check_installed_patches_sys($sys,$pkgvers)); 
        $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag); 
        return '' unless ($pkgvers);
        # pick up the lowest version as the product version
        $vers = ($mpvers || $pkgvers);
        if (!$prodvers || (EDRu::compvers($prodvers,$vers,4)==1)) {
            $prodvers = $vers;
        }
    }
     
    return '' if (2 == EDRu::compvers($prodvers,'7.0'));
    return $prodvers;
}


# When calling Prod::stopprocs, it will retrieve packages from the new product,
# causing more product processes been added into this list.
sub stopprocs {  
    my ($prod)=@_; 
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs($prod);
    return $procs;
}

sub stopprocs_sys { 
    my ($prod,$sys)=@_;
    my ($procs);
    $procs = Prod::ENTERPRISE731::Common::stopprocs_sys($prod,$sys);
    return $procs;
}

sub cli_prod_option {
    my $prod=shift;
    $prod->perform_task('cli_prod_option');
    return;
}

sub precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_sys($prod,$sys);
}

sub configure_precheck_sys {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::configure_precheck_sys($prod,$sys);
}

sub precheck_task {
    my ($prod,$sys)=@_;
    return Prod::ENTERPRISE731::Common::precheck_task($prod);
}

sub addnode_poststart {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::addnode_poststart($prod);
}

sub delnode_precheck {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::delnode_precheck($prod);
}

sub default_systemnames {
    my $prod = shift;
    return Prod::ENTERPRISE731::Common::default_systemnames($prod);
}

package Prod::STORAGE731::AIX;
@Prod::STORAGE731::AIX::ISA = qw(Prod::STORAGE731::Common);

sub memory_size_sys {
    my ($prod,$sys) = @_;
    $prod->SUPER::memory_size_sys($sys);
    return Prod::FS731::AIX::memory_size_sys($prod,$sys);
}

package Prod::STORAGE731::HPUX;
@Prod::STORAGE731::HPUX::ISA = qw(Prod::STORAGE731::Common);

package Prod::STORAGE731::Linux;
@Prod::STORAGE731::Linux::ISA = qw(Prod::STORAGE731::Common);

package Prod::STORAGE731::SunOS;
@Prod::STORAGE731::SunOS::ISA = qw(Prod::STORAGE731::Common);

package Prod::STORAGE731::Sol11sparc;
@Prod::STORAGE731::Sol11sparc::ISA = qw(Prod::STORAGE731::SunOS);

package Prod::STORAGE731::Sol12sparc;
@Prod::STORAGE731::Sol12sparc::ISA = qw(Prod::STORAGE731::SunOS);

package Prod::STORAGE731::Sol11x64;
@Prod::STORAGE731::Sol11x64::ISA = qw(Prod::STORAGE731::SunOS);

sub version_sys {
    my ($prod,$sys,$force_flag) = @_;
    my ($pkg,$pkgvers,$mpvers,$vers);

    #VRTSllt VRTSvxvm
    #install the two packages will be regarded as STORAGE installed.
    for my $pkgname (qw(VRTSllt731 VRTSvxvm731)) {
        $pkg=$sys->pkg($pkgname);
        $pkgvers=$pkg->version_sys($sys,$force_flag);
        $mpvers=$prod->{vers} if ($prod->check_installed_patches_sys($sys,$pkgvers));
        $pkgvers= $prod->revert_base_version_sys($sys,$pkg,$pkgvers,$mpvers,$force_flag);
        return '' unless ($pkgvers);
        return '' if (2 == EDRu::compvers($pkgvers,'7.0'));
    }

    $vers = ($mpvers || $pkgvers);
    return $vers;
}

package Prod::STORAGE731::Sol12x64;
@Prod::STORAGE731::Sol12x64::ISA = qw(Prod::STORAGE731::Sol11x64);

1;
