#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#

package Module::facter::environment;

# Get the following facts for environment
#    user_env
#    user_id
#    user_gid
#    user_homedir
#    user_shell
#    date_time
#    ssh_host_key_rsa_public
#    ssh_host_key_dsa_public
#    mounts


package Module::facter::environment::Linux;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts,$file,$rc,$output,$content,$key,$value);

    $facts={};

    # get date time
    ($rc,$output)=$sys->cmd('_cmd_date +%m%d%y_%H%M%S 2>/dev/null');
    if (!$rc && $output) {
        $facts->{date_time}=$output;
    }

    # get user environments
    ($rc,$output)=$sys->cmd('_cmd_env 2>/dev/null');
    if (!$rc && $output) {
        for my $line (split(/\n+/, $output)) {
            if ($line=~/([\w_]+)=(.*)$/) {
                $key=$1;
                $value=$2;

                next if ($key eq '_' ||
                         $key eq 'LS_COLORS' ||
                         $key eq 'module');

                $facts->{user_env}{"$key"}="$value";
            }
        }

        $value=$facts->{user_env}{USER};
        $facts->{user_id} = $value if $value;

        $value=$facts->{user_env}{HOME};
        $facts->{user_homedir} = $value if $value;

        $value=$facts->{user_env}{SHELL};
        $facts->{user_shell} = $value if $value;
    }

    # get uid/gid
    ($rc,$output)=$sys->cmd('_cmd_id 2>/dev/null');
    if (!$rc && $output && $output=~/uid=(\d+)\(\w+\)\s+gid=(\d+)\(\w+\)/) {
        $facts->{user_uid} = $1;
        $facts->{user_gid} = $2;
    }

    # get ssh public keys
    for my $key_type (qw(rsa dsa)) {
        $file="/etc/ssh/ssh_host_${key_type}_key.pub";
        $content=$sys->readfile($file);
        chomp $content;
        if ($content) {
            $content=~s/^\S+\s+//;
            $content=~s/\s*$//;
            $facts->{"ssh_host_key_${key_type}_public"} = $content;
        }
    }

    # get mounts
    #$file='/etc/mtab';
    #$content=$sys->readfile($file);
    #for my $line (split(/\n+/, $content)) {
    #    if ($line=~/^\/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/) {
    #        my $device="/$1";
    #        my $mount="$2";
    #        my $fstype="$3";
    #        my $options="$4";
    #        $facts->{mounts}{$device}{mount}=$mount;
    #        $facts->{mounts}{$device}{fstype}=$fstype;
    #        $facts->{mounts}{$device}{options}=$options;
    #    }
    #}

    # get df
    ($rc,$output)=$sys->cmd("_cmd_df 2>/dev/null");
    if (!$rc && $output) {
        for my $line (split(/\n+/, $output)) {
            if ($line=~/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\S+\s+(\S+)/m) {
                my $filesystem=$1;
                my $total=$2;
                my $used=$3;
                my $available=$4;
                my $mountpoint=$5;

                next if ($filesystem!~/^\//);

                $total=int($total/1024);
                $available=int($available/1024);
                $facts->{mounts_space}{$mountpoint}{filesystem}="$filesystem";
                $facts->{mounts_space}{$mountpoint}{total_mb}="$total";
                $facts->{mounts_space}{$mountpoint}{available_mb}="$available";
            }
        }
    }

    return $facts;
}

package Module::facter::environment::SunOS;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts);

    $facts={};

    return $facts;
}

package Module::facter::environment::AIX;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts);

    $facts={};

    return $facts;
}

package Module::facter::environment::HPUX;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts);

    $facts={};

    return $facts;
}

1;
