#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#

package Module::facter::hardware;

# Get the following facts for operating system
#    memfree_mb
#    memtotal_mb
#    swapfree_mb
#    swaptotal_mb
#    processor_cores
#    processor_count
#    processor_frequency
#    processor_model_name
#    processor_vendor_id


package Module::facter::hardware::Linux;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts,$content,$output);

    $facts={};

    # get memory information
    $content=$sys->readfile('/proc/meminfo');
    chomp $content;
    if ($content) {
        my @memory_facts=qw(MemTotal SwapTotal MemFree SwapFree);
        for my $key (@memory_facts) {
            if ($content=~/^$key:\s*(\d+)/m) {
                $key=lc($key);
                $facts->{"${key}_mb"}=int($1/1024);
            }
        }
    }

    # get cpu information
    $content=$sys->readfile('/proc/cpuinfo');
    chomp $content;
    if ($content) {
        my $processor_count=0;
        my $processor_cores=0;
        my $processor_frequency=0;
        my $processor_vendor_id='';
        my $processor_model_name='';
        for my $line (split(/\n+/,$content)) {
            if ($line=~/^processor\s*:\s*\d+/m) {
                $processor_count++;
            } elsif ($line=~/^cpu cores\s*:\s*(\d+)/m) {
                $processor_cores=$1 if (!$processor_cores);
            } elsif ($line=~/^cpu MHz\s*:\s*(\S+)/m) {
                $processor_frequency=$1;
            } elsif ($line=~/^vendor_id\s*:\s*(.*)$/m) {
                $processor_vendor_id=$1;
            } elsif ($line=~/^model name\s*:\s*(.*)$/m) {
                $processor_model_name=$1;
            }
        }
        $facts->{processor_count}=$processor_count if ($processor_count);
        $facts->{processor_cores}=$processor_cores if ($processor_cores);
        $facts->{processor_frequency}=int($processor_frequency) if ($processor_frequency);
        $facts->{processor_vendor_id}=$processor_vendor_id if ($processor_vendor_id);
        $facts->{processor_model_name}=$processor_model_name if ($processor_model_name);
    }

    return $facts;
}

package Module::facter::hardware::SunOS;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts,$rc,$output);

    $facts={};

    # get cpu information
    # https://blogs.oracle.com/mandalika/entry/solaris_show_me_the_cpu
    # Counting cores on Solaris can be complicated.
    # Treat 'processor_count' as physical sockets and 'processor_cores' as
    # virtual CPUs visisble to Solaris. Not a true count of cores for modern SPARC as
    # these processors have: sockets -> cores -> threads/virtual CPU.
    ($rc,$output)=$sys->cmd('_cmd_kstat cpu_info 2>/dev/null');
    if (!$rc && $output) {
        my $processor_count=0;
        my $processor_frequency=0;
        my $processor_model_name='';
        my $processor_type='';
        for my $line (split(/\n+/,$output)) {
            if ($line=~/^module: cpu_info\s+/m) {
                $processor_count++;
            } elsif ($line=~/clock_MHz\s*(\S+)/m) {
                $processor_frequency=$1;
            } elsif ($line=~/brand\s*(.*)$/m) {
                $processor_model_name=$1;
            } elsif ($line=~/cpu_type\s*(\S+)/m) {
                $processor_type=$1 if (!$processor_type);
            }
        }
        $facts->{processor_count}=$processor_count if ($processor_count);
        $facts->{processor_frequency}=int($processor_frequency) if ($processor_frequency);
        $facts->{processor_model_name}=$processor_model_name if ($processor_model_name);
        $facts->{processor_type}=$processor_type if ($processor_type);
    }

    return $facts;
}

package Module::facter::hardware::AIX;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts);

    $facts={};

    return $facts;
}

package Module::facter::hardware::HPUX;

sub execute_sys {
    my ($self,$sys) = @_;
    my ($facts);

    $facts={};

    return $facts;
}

1;
