#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#

package Module::facter::veritas;

# Get Veritas Software information:
#     veritas_packages
#     veritas_vxvm_volumes
#     veritas_vxfs_mounts
#     veritas_vcs_service_groups


sub execute_sys {
    my ($self,$sys) = @_;
    my ($rc,$output,$content,$facts,$key,$type,$description);

    $facts={};

    # get veritas licenses
    ($rc,$output) = $sys->cmd("_cmd_vxkeyless display -v 2>/dev/null");
    if (!$rc && $output) {
        for my $line (split(/\n+/,$output)) {
            next if ($line=~/^Product/);
            if ($line=~/(\S+)\s+(.*)$/) {
                $key=$1;
                $description=$2;
                $facts->{veritas_licenses}{$key}{type}="keyless";
                $facts->{veritas_licenses}{$key}{description}="$description";
            }
        }
    }

    ($rc,$output) = $sys->cmd("_cmd_vxlicrep -s 2>/dev/null");
    if (!$rc && $output) {
        $key='';
        $description='';
        $type='';
        for my $line (split(/\n+/,$output)) {
            if ($line=~/License Key\s*=\s*(.*)\s*$/m) {
                $key=$1;
            } elsif ($line=~/Product Name\s*=\s*(.*)\s*$/m) {
                $description=$1;
            } elsif ($line=~/License Type\s*=\s*(.*)\s*$/m) {
                $type=lc($1);
                if ($key) {
                    $facts->{veritas_licenses}{$key}{type}=$type if ($type);
                    $facts->{veritas_licenses}{$key}{description}=$description if ($description);
                }
            }
        }
    }

    # get veritas VxVM information
    ($rc,$output) = $sys->cmd("_cmd_vxdisk -o alldgs list 2>/dev/null");
    if (!$rc && $output) {
        for my $line (split(/\n+/,$output)) {
            if ($line=~/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)/) {
                my $device=$1;
                my $type=$2;
                my $disk=$3;
                my $group=$4;
                my $status=$5;
                next if ($device eq 'DEVICE');
                $facts->{veritas_vxvm_disks}{$device}{type}=$type;
                $facts->{veritas_vxvm_disks}{$device}{diskgroup}=$group if ($group ne '-');
                $facts->{veritas_vxvm_disks}{$device}{status}=$status;
            }
        }
    }

    ($rc,$output) = $sys->cmd("_cmd_vxdg list 2>/dev/null");
    if (!$rc && $output) {
        for my $line (split(/\n+/,$output)) {
            if ($line=~/^(\S+)\s+(\S+)\s+(\S+)/) {
                my $dgname=$1;
                my $dgstate=$2;
                my $dgid=$3;
                next if ($dgname eq 'NAME');
                $facts->{veritas_vxvm_diskgroups}{$dgname}{state}=$dgstate;
                $facts->{veritas_vxvm_diskgroups}{$dgname}{id}=$dgid;
            }
        }
    }

    # get veritas VxFS information
    ($rc,$output) = $sys->cmd("_cmd_mount 2>/dev/null | _cmd_grep 'type vxfs'");
    if (!$rc && $output) {
        for my $line (split(/\n+/,$output)) {
            if ($line=~/^(\S+)\s+on\s+(\S+)\s+type\s+vxfs\s+(\S+)/) {
                my $mount_volume=$1;
                my $mount_point=$2;
                my $mount_option=$3;
                $facts->{veritas_vxfs_mounts}{$mount_point}{volume}=$mount_volume;
                $facts->{veritas_vxfs_mounts}{$mount_point}{option}=$mount_option;
            }
        }
    }
    
    # get veritas VCS information
    ($rc,$output) = $sys->cmd("_cmd_hastatus -summary 2>/dev/null");
    if (!$rc && $output) {
        my ($section,$prefix,$system,$state,$frozen,$group,$probed,$autodisabled);
        $section='';
        for my $line (split(/\n+/,$output)) {
            next if ($line=~/^\s*$/);

            if ($line=~/^-- SYSTEM STATE/) {
                $section='system';
                $facts->{veritas_vcs_cluster}{systems} = {};
                next;
            } elsif ($line =~ /^-- GROUP STATE/) {
                $section='group';
                $facts->{veritas_vcs_cluster}{servicegroups} = {};
                next;
            }

            next unless ($section);

            if ($section eq 'system') {
                $prefix='';
                $system='';
                $state='';
                $frozen='';
                if ($line=~/^-- System/) {
                    next;
                } else {
                    ($prefix,$system,$state,$frozen)=split(/\s+/,$line);
                    $facts->{veritas_vcs_cluster}{systems}{$system}{state}=$state;
                    $facts->{veritas_vcs_cluster}{systems}{$system}{frozen}=$frozen;
                }
            } elsif ($section eq 'group') {
                $prefix='';
                $group='';
                $system='';
                $probed='';
                $autodisabled='';
                $state='';
                if ($line=~/^-- Group/) {
                    next;
                } else {
                    ($prefix,$group,$system,$probed,$autodisabled,$state)=split(/\s+/,$line);
                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{systems}||={};
                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{systems}{$system}{state}=$state;
                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{systems}{$system}{probed}=$probed;
                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{systems}{$system}{autodisabled}=$autodisabled;

                    # get resources for the service group
                    ($rc,$output) = $sys->cmd("_cmd_hares -display -group $group 2>/dev/null");
                    if (!$rc && $output) {
                        my ($resource,$attr,$system1,$value);

                        $facts->{veritas_vcs_cluster}{servicegroups}{$group}{resources}||={};

                        for my $line (split(/\n+/,$output)) {
                            next if ($line=~/^\s*#?$/);

                            #Resource     Attribute             System     Value
                            if ($line=~/(\S+)\s+(\S+)\s+(\S+)\s+(.*$)/) {
                                $resource=$1;
                                $attr=$2;
                                $system1=$3;
                                $value=$4;
                                $facts->{veritas_vcs_cluster}{servicegroups}{$group}{resources}{$resource}||={};
                                if ($attr=~/^(Type|Autostart|Critical|Enabled)$/) {
                                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{resources}{$resource}{$attr}=$value;
                                } elsif ($attr =~ /^(Probed|State|Start)$/) {
                                    $facts->{veritas_vcs_cluster}{servicegroups}{$group}{resources}{$resource}{$attr}{$system1}=$value;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    # Get CFS cluster status
    ($rc,$output) = $sys->cmd("_cmd_cfscluster status 2>/dev/null");
    if (!$rc && $output) {
        my ($system,$state);
        $system='';

        for my $line (split(/\n+/,$output)) {
            next if ($line=~/^\s*$/);

            if ($line =~ /^\s*Node\s+:\s+([\w\.]+)/) {
               $system=$1;
               next unless ($system);

               $facts->{veritas_cfs_cluster}{systems}||={};
               $facts->{veritas_cfs_cluster}{systems}{$system}||={};
            } elsif ($line =~ /^\s*Cluster Manager\s+:\s+(.*)/) {
               $state=$1;
               $facts->{veritas_cfs_cluster}{systems}{$system}{'Cluster Manager'}=$state if ($system);
            } elsif ($line =~ /^\s*CVM state\s+:\s+(.*)/) {
               $state=$1;
               $facts->{veritas_cfs_cluster}{systems}{$system}{'CVM state'}=$state if ($system);
            }
        }
    }

    return $facts;
}

package Module::facter::veritas::Linux;
@Module::facter::veritas::Linux::ISA = qw(Module::facter::veritas);

sub execute_sys {
    my ($self,$sys) = @_;
    my ($pkgs,$rc,$output,$facts);

    # Get common facts
    $facts=$self->SUPER::execute_sys($sys);

    # get Veritas packages
    $pkgs='VRTS* SYMC*';
    ($rc,$output)=$sys->cmd("_cmd_rpm -qa --queryformat '%{NAME} %{VERSION}\n' $pkgs 2>/dev/null");
    if (!$rc && $output) {
        for my $line (split(/\n+/,$output)) {
            if ($line=~/^(\S+)\s+(\S+)/m) {
                my $pkg=$1;
                my $version=$2;
                my $deplist=$sys->cmd("_cmd_rpm -q --whatrequires --queryformat ' %{NAME}' $pkg 2>/dev/null");

                $facts->{vertias_packages}{$pkg}{version} = $version;

                next if (!$deplist || $deplist=~/no package/);
                $deplist=~s/^\s+//;
                $facts->{vertias_packages}{$pkg}{requiredby} = $deplist;
            }
        }
    }

    return $facts;
}

package Module::facter::veritas::SunOS;
@Module::facter::veritas::SunOS::ISA = qw(Module::facter::veritas);

sub execute_sys {
    my ($self,$sys) = @_;
    my ($pkgs,$rc,$output,$facts);

    # Get common facts
    $facts=$self->SUPER::execute_sys($sys);

    # get Veritas packages
    $pkgs='VRTS* SYMC*';

    return $facts;
}

package Module::facter::veritas::AIX;
@Module::facter::veritas::AIX::ISA = qw(Module::facter::veritas);

sub execute_sys {
    my ($self,$sys) = @_;
    my ($pkgs,$rc,$output,$facts);

    # Get common facts
    $facts=$self->SUPER::execute_sys($sys);

    # get Veritas packages
    $pkgs='VRTS* SYMC*';

    return $facts;
}

package Module::facter::veritas::HPUX;
@Module::facter::veritas::HPUX::ISA = qw(Module::facter::veritas);

sub execute_sys {
    my ($self,$sys) = @_;
    my ($pkgs,$rc,$output,$facts);

    # Get common facts
    $facts=$self->SUPER::execute_sys($sys);

    # get Veritas packages
    $pkgs='VRTS* SYMC*';

    return $facts;
}

1;
