package Module::system::group;

# requirements: [ groupadd, groupdel, groupmod ]
# description:
#     - Manage group accounts and group attributes.
# options:
#     name:
#         required: true
#         aliases: [ "group" ]
#         description:
#              - Name of the group to create, remove or modify.
#     gid:
#         required: false
#         description:
#             - Optionally sets the I(GID) of the group.
#     state:
#         required: false
#         default: "present"
#         choices: [ present, absent ]
#         description:
#             - Whether the group should exist or not, taking action if the state is different from what is stated.

sub execute_sys {
    my ($self,$sys,$attrs) = @_;
    my ($return,$plat,$sub,$name,$state);

    $return={ 'changed' => 0 };

    # get distribution
    $plat=$sys->{plat};
    return $return if (!$plat);

    return $return if (!defined $attrs);

	$attrs=eval($attrs) if ref($attrs) eq '';

    $name=$attrs->{name};
    return $return if (!$name);

    $state=$attrs->{state} || 'present';

    if ($state eq 'present') {
        $sub='create_group';
    } elsif ($state eq 'absent') {
        $sub='delete_group';
    }

    $plat=lc($plat);
    $sub="${sub}_${plat}";
    if ($self->can($sub)) {
        $self->$sub($sys,$attrs);
        $return->{changed} = 1;
    }

    return $return;
}

# Linux
sub create_group_linux {
    my ($self,$sys,$attributes) = @_;
    my ($name,$gid,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd='_cmd_groupadd ';

    $gid=$attributes->{gid};
    $cmd.=" -g $gid" if ($gid);

    $cmd.=" $name";
   
    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

sub delete_group_linux {
    my ($self,$sys,$attributes) = @_;
    my ($name,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd="_cmd_groupdel $name";

    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

# Solaris
sub create_group_sunos {
    my ($self,$sys,$attributes) = @_;
    my ($name,$gid,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd='_cmd_groupadd ';

    $gid=$attributes->{gid};
    $cmd.=" -g $gid" if ($gid);

    $cmd.=" $name";
   
    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

sub delete_group_sunos {
    my ($self,$sys,$attributes) = @_;
    my ($name,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd="_cmd_groupdel $name";

    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}


# AIX
sub create_group_aix {
    my ($self,$sys,$attributes) = @_;
    my ($name,$gid,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd='_cmd_groupadd ';

    $gid=$attributes->{gid};
    $cmd.=" -g $gid" if ($gid);

    $cmd.=" $name";
   
    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

sub delete_group_aix {
    my ($self,$sys,$attributes) = @_;
    my ($name,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd="_cmd_groupdel $name";

    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}


# HPUX
sub create_group_hpux {
    my ($self,$sys,$attributes) = @_;
    my ($name,$gid,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd='_cmd_groupadd ';

    $gid=$attributes->{gid};
    $cmd.=" -g $gid" if ($gid);

    $cmd.=" $name";
   
    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

sub delete_group_hpux {
    my ($self,$sys,$attributes) = @_;
    my ($name,$cmd,$rc,$output);

    $name=$attributes->{name};
    return 0 if (!$name);

    $cmd="_cmd_groupdel $name";

    ($rc, $output)=$sys->cmd("$cmd 2>/dev/null");

    return $rc ? 0 : 1;
}

1;
