package Module::system::yum;

# description:
#     - Install/Uninstall packages with YUM
# options:
#     name:
#         required: true
#         description:
#              - Name of the package to install, or uninstall.
#     state:
#         required: false
#         default: "present"
#         choices: [ present, absent ]
#         description:
#             - Whether the package should be installed or uninstalled
#     repository_name:
#         required: false
#         description:
#             - Optionally sets the YUM repository
#     repository_baseurl:
#         required: false
#         description:
#             - Optionally sets the base URL for the YUM repository
#     gpgcheck:
#         required: false
#         description:
#             - Optionally sets if check GPG key
#     gpgkey:
#         required: false
#         description:
#             - Optionally set the GPG key

sub execute_sys {
    my ($self,$sys,$attrs) = @_;
    my ($return,$rpm_name,$rpm_exists,$repository_name,$repository_file,$repository_baseurl,$gpgkey,$gpgcheck,$state,$rc,$output);

    $return={ 'changed' => 0, 'rc' => 0 };

    return $return if (!defined $attrs);

	$attrs=eval($attrs) if ref($attrs) eq '';

    $state=$attrs->{state} || 'present';

    $repository_name=$attrs->{repository_name};
    if ($repository_name) {

        $repository_baseurl=$attrs->{repository_baseurl};
        if (${repository_baseurl}) {
            $repository_file="/etc/yum.repos.d/${repository_name}.repo";
            $gpgcheck=$attrs->{gpgcheck} ? 1 : 0;
            $gpgkey=$attrs->{gpgkey} || '';
            if ($state eq 'present') {
                $sys->cmd("_cmd_echo '[${repository_name}]\nname=${repository_name}\nbaseurl=${repository_baseurl}\nenabled=1\ngpgcheck=${gpgcheck}\ngpgkey=${gpgkey}\n' > ${repository_file}");
            } else {
                $sys->cmd("_cmd_rmr ${repository_file}");
            }
            $return->{changed}=1;
        }
    }

    $rpm_name=$attrs->{name};
    $rpm_exists=0;
    if ($rpm_name) {
        ($rc,$output)=$sys->cmd("_cmd_yum list installed ${rpm_name} 2>/dev/null");
        $rpm_exists=1 if (!$rc);

        if ($rpm_exists==0 && $state eq 'present') {
            ($return->{rc},$return->{results})=$sys->cmd("_cmd_yum install -y ${rpm_name}");
            $return->{changed}=1;
        } elsif ($state eq 'absent') {
            ($return->{rc},$return->{results})=$sys->cmd("_cmd_yum erase -y ${rpm_name}");
            $return->{changed}=1;
        }
    }

    return $return;
}

1;
