package Module::veritas::amf;

# description:
#   - AMF configuration
# options:
#   - state:
#       required: false
#       choices: [present, absent]
#       default: present
#       description:
#           - set enable/disable AMF

$Macros::macros->set("VERITAS_AMF_CONF_SUCCESS",0);
$Macros::macros->set("VERITAS_AMF_UNCONF_SUCCESS",2);
$Macros::macros->set("VERITAS_AMF_NO_ENABLE_FILE",1);

sub execute_sys {
    my ($self, $sys, $attrs) = @_;
    my ( $cmd, $rc,  $return );
    
    $return={ 'changed' => 0 ,'rc'=>0};
	return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

    $attrs->{state} = 'present' unless $attrs->{state};
    if ($attrs->{state} =~ /present/) { 
        
        $rc = $self->enable_amf($sys);
        $return->{changed} ||= $rc->{changed};
        if ($rc->{rc}){
        	$return->{rc} = $rc->{rc};
        } else {
            $return->{rc} = $macros->{VERITAS_AMF_CONF_SUCCESS};		
        }	
    } elsif ($attrs->{state} =~ /absent/) {
       $rc = $self->disable_amf($sys);
       $return->{changed} ||= $rc->{changed};
       if ($rc->{rc}){
           $return->{rc} = $rc->{rc};
       } else {
           $return->{rc} = $macros->{VERITAS_AMF_UNCONF_SUCCESS};  
       }
    }
    return $return;
}

sub enable_amf {
    my ($self, $sys) = @_;
    my ($dir, $return, $conf, $output,$rc);
    $return = {'changed' => 0, 'rc' => 0};
    $dir = "/etc/sysconfig";
    ($rc, $conf) = $sys->cmd("_cmd_cat $dir/amf");
    
    if ($rc){
        $sys->log("can't cat the file $dir/amf","error");
        $return->{rc} = $macros->{VERITAS_AMF_NO_ENABLE_FILE};
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/amf.prev");
    $sys->cmd("_cmd_cp $dir/amf $dir/amf.prev");
    $sys->cmd("_cmd_sed -i 's/AMF_START=0/AMF_START=1/g' $dir/amf");
    $sys->cmd("_cmd_sed -i 's/AMF_STOP=0/AMF_STOP=1/g' $dir/amf");
    ($rc, $output) = $sys->cmd("_cmd_diff $dir/amf $dir/amf.prev");
    $return->{changed}=$rc;    
    return $return;
}

sub disable_amf{
    my ($self, $sys) = @_;
    my ($dir, $rc, $conf, $output,$return);
    $return = {'changed' => 0, 'rc' => 0};
    $dir = "/etc/sysconfig";
    ($rc, $conf) = $sys->cmd("_cmd_cat $dir/amf");

     if ($rc){
        $sys->log("can't cat the file $dir/amf","error");
        $return->{rc} = $macros->{VERITAS_AMF_NO_ENABLE_FILE};
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/amf.prev");
    $sys->cmd("_cmd_cp $dir/amf $dir/amf.prev");
    $sys->cmd("_cmd_sed -i 's/AMF_START=1/AMF_START=0/g' $dir/amf");
    $sys->cmd("_cmd_sed -i 's/AMF_STOP=1/AMF_STOP=0/g' $dir/amf");
    ($rc, $output) = $sys->cmd("_cmd_diff $dir/amf $dir/amf.prev");
    $return->{changed}=$rc;
    return $return;
}
1;
