package Module::veritas::gab;

# description:
#   - GAB configuration
# options:
#   - state:
#       required: false
#       choices: [present, absent]
#       default: present
#       description:
#           - set enable/disable GAB
#   - systems
#       required: true
#       choices [ cluster system hostnames ]

$Macros::macros->set("VERITAS_GAB_CONF_SUCCESS",0);
$Macros::macros->set("VERITAS_GAB_CONF_FAIL",1);
$Macros::macros->set("VERITAS_GAB_UNCONF_SUCCESS",2);
$Macros::macros->set("VERITAS_GAB_UNCONF_FAIL",3);
$Macros::macros->set("VERITAS_GAB_WRITE_GABTAB_FAIL",5);
$Macros::macros->set("VERITAS_GAB_DEL_GABTAB_FAIL",7);
$Macros::macros->set("VERITAS_GAB_NO_ENABLE_FILE",9);

sub execute_sys {
    my ($self, $sys, $attrs) = @_;
    my ($state, $gabtab, $sysnum, @systems, $cmd, $rc, $output, $return);
    
    $return={ 'changed' => 0, 'rc' => 0 };
	return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

    $state = $attrs->{state} || 'present';
    if ($state =~ /present/) { 
        @systems = split(/,/, $attrs->{systems});
        $sysnum = $#systems+1;
        $gabtab = "_cmd_gabconfig -c -n$sysnum";
        $cmd = "_cmd_echo $gabtab > /etc/gabtab";

        ($rc, $output) = $sys->cmd("$cmd 2>/dev/null");
        if ($rc) {
            $return->{rc} = $macros->{VERITAS_GAB_WRITE_GABTAB_FAIL};
            $sys->log("Failed to write file gabtab", 'error');
        } else {
            $return->{changed} = 1;
        }

        $rc = $self->enable_gab($sys, $state);
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};

        if ($return->{rc}) {
            $return->{rc} = $macros->{VERITAS_GAB_CONF_FAIL};
        } else {
            $return->{rc} = $macros->{VERITAS_GAB_CONF_SUCCESS};	
        }
        
    } elsif ($state =~ /absent/) {
        if (-f "/etc/gabtab") {
            ($rc, $output) = $sys->cmd("_cmd_rmr /etc/gabtab 2>/dev/null");
            if ($rc) {
                $return->{rc} = $macros->{VERITAS_GAB_DEL_GABTAB_FAIL};
                $sys->log("Failed to delete the file /etc/gabtab",'error');
            }	
            $return->{changed} = 1;
        }
		$rc = $self->enable_gab($sys, $state);
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};
        if ($return->{rc}) {
            $return->{rc} = $macros->{VERITAS_GAB_UNCONF_FAIL};  		
        } else {
            $return->{rc} = $macros->{VERITAS_GAB_UNCONF_SUCCESS};           		
        }
    }
    return $return;
}

sub enable_gab {
    my ($self, $sys, $state) = @_;
    my ($return, $value, $previous, $dir, $rc, $conf, $output);
    $return = {'changed' => 0, 'rc' => 0};
    $value = 1 if $state =~ /present/;
	$value = 0 if $state =~ /absent/;
	$previous = 1 - $value;

    $dir = "/etc/sysconfig";
    unless (-f "$dir/gab") {
        $return->{rc} = $macros->{VERITAS_GAB_NO_ENABLE_FILE};
        $sys->log("Cannot find file $dir/gab",'error');
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/gab.prev");
    $sys->cmd("_cmd_cp $dir/gab $dir/gab.prev");
    $sys->cmd("_cmd_sed -i \"s/GAB_START=$previous/GAB_START=$value/\" $dir/gab");
    $sys->cmd("_cmd_sed -i \"s/GAB_STOP=$previous/GAB_STOP=$value/\" $dir/gab");
    ($rc, $output) = $sys->cmd("_cmd_diff $dir/gab $dir/gab.prev");
    $return->{changed} = $rc;
    
    return $return;
}

1;
