package Module::veritas::had;

# description:
#   - HAD configuration
# options:
#   - state:
#       required: false
#       choices: [present, absent]
#       default: present
#       description:
#           - set enable/disable HAD
#   - component:
#		required: true
#		choices: [ sf,vcs,sfha,sfcfs,sfcfsha,sfsybase,sfrac ]
#		description:
#			- define which component would be configured
#   - cluster_name:
#       required: true  
#       choices: [ cluster name ]
#       description:
#           - set the cluster name
#   - cluster_uuid:
#       required: true
#       choices: [ cluster uuid ]
#       description:
#           - Each cluster need a distinct UUID which cannot be same with other clusters
#   - vcs_users
#       required: false
#       default: admin:password:Administrators
#       choices: [user:password:priv]
#       description:
#           - VCS user name:user password:user privilege
#   - vcs_user_password
#       required: false
#       default: password
#       choices: [ user password ]
#       description:
#           - VCS user password
#   - systems
#       required: true
#       choices [ cluster system hostnames ]

#  ./installer -module veritas::had -action 'component=vcs cluster_name=vcs cluster_uuid=3332 vcs_users=admin:password:Administrators,user1:passwd1:Operators systems=node1,node2,node3'

$Macros::macros->set("VERITAS_VCS_CONF_SUCCESS",2);
$Macros::macros->set("VERITAS_VCS_CONF_FAIL",3);
$Macros::macros->set("VERITAS_VCS_UNCONF_SUCCESS",4);
$Macros::macros->set("VERITAS_VCS_UNCONF_FAIL",5);
$Macros::macros->set("VERITAS_VCS_NO_ENCRYPT",7);
$Macros::macros->set("VERITAS_VCS_WRITE_MAINCF_FAIL",9);
$Macros::macros->set("VERITAS_VCS_WRITE_SYSNAME_FAIL",11);
$Macros::macros->set("VERITAS_VCS_NO_SYSNAME",13);
$Macros::macros->set("VERITAS_VCS_NO_ENABLE_FILE",15);
$Macros::macros->set("VERITAS_VCS_GEN_UUID_FAIL",17);

sub execute_sys {
    my ($self, $sys, $attrs) = @_;
    my ($cmd, $rc, $return);
    
    $return={ 'changed' => 0, 'rc' => 0 };
    return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

    $attrs->{state} = 'present' unless $attrs->{state};
    
    if ($attrs->{state} =~ /present/) { 
        if (!defined $attrs->{cluster_name}) {
            $sys->log("Attribute 'cluster_name' is not defined!",'error');
			#todo: return code
            return $return;
		}
		if (!defined $attrs->{cluster_uuid}) {
            $sys->log("Attribute 'cluster_uuid' is not defined!",'error');
            return $return;
		}
		if (!defined $attrs->{systems}) {
            $sys->log("Attribute 'systems' is not defined!",'error');
            return $return;
        }
        $rc = $self->update_maincf($sys, $attrs);
        $return->{changed} = 1 if $rc->{changed};
        $return->{rc} = 1 if $rc->{rc};
#        # generate random uuid -- replace it later
#        my @array = (0..9,'a'..'z');
#        # format like this: {500844a6-0034-11e6-a605-cc221266c303}
#        my @uuid=map{$array[int rand @array]} 1..36;
#        $uuid[8]=$uuid[13]=$uuid[18]=$uuid[23]='-';
#        my $uuid="uuid={".join('',@uuid)."}";
#        # generate random uuid -- replace it later
#        $rc=Module::veritas::uuid->execute_sys($sys,$uuid);

        $rc = $self->cluster_uuid($sys, $attrs->{state}, $attrs->{cluster_uuid});
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};

        $rc = $self->enable_vcs($sys);
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};

        if ($return->{rc}) {
            $sys->log("Failed to configure VCS.",'error');
            $return->{rc} = $macros->{VERITAS_VCS_CONF_FAIL};		   
        } else {
            $return->{rc} = $macros->{VERITAS_VCS_CONF_SUCCESS};		
        }

    } elsif ($attrs->{state} =~ /absent/) {
        $rc = $self->disable_vcs($sys);
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};
        $rc = $self->recover_maincf($sys);
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};
        $rc = $self->cluster_uuid($sys, $attrs->{state});
        $return->{changed} ||= $rc->{changed};
        $return->{rc} ||= $rc->{rc};
        if ($return->{rc}) {
            $return->{rc} = $macros->{VERITAS_VCS_UNCONF_FAIL};		
        } else {
            $return->{rc} = $macros->{VERITAS_VCS_UNCONF_SUCCESS};		
        }
    } 
    return $return;
}

sub include_types_cf {
	my ($self, $sys, $attrs) = @_;
	my ( $component, $vcsea_types, $dbac_types, $include_str, $rc, $config_dir, $conf_dir, @tmp_arr);
	$vcsea_types = ['/etc/VRTSagents/ha/conf/Oracle/OracleTypes.cf',
				'/etc/VRTSagents/ha/conf/OracleASM/OracleASMTypes.cf',
				'/etc/VRTSagents/ha/conf/Db2udb/Db2udbTypes.cf',
				'/etc/VRTSagents/ha/conf/Sybase/SybaseTypes.cf'
				];
	$dbac_types = ['/etc/VRTSvcs/conf/PrivNIC.cf',
				'/etc/VRTSvcs/conf/MultiPrivNIC.cf',
				'/etc/VRTSvcs/conf/OracleServiceTypes.cf',
				'/etc/VRTSvcs/conf/CRSResource.cf',
				'/etc/VRTSvcs/conf/CSSD.cf'
				];
	$config_dir = "/etc/VRTSvcs/conf/config";
	$conf_dir = "/etc/VRTSvcs/conf";
	$component = $attrs->{component};
	$include_str = "include \"types.cf\"\n";
	foreach my $type_file (@$vcsea_types){
		next unless  $sys->exists($type_file); 
		$sys->copyfile($type_file, $config_dir);
		$sys->copyfile($type_file, $conf_dir);
		@tmp_arr =  split(/\//, $type_file);
		$include_str .= "include \"$tmp_arr[-1]\"\n";
	}

	if( $component =~ /sfrac/i ){
		foreach my $type_file (@$dbac_types){ 
			next unless  $sys->exists($type_file); 
			$sys->copyfile($type_file, $config_dir);
			@tmp_arr =  split(/\//, $type_file);
			$include_str .= "include \"$tmp_arr[-1]\"\n";
		}
	}

	return $include_str;
}

sub update_maincf {
    my ($self, $sys, $attrs) = @_;
    my ($maincf, $config_dir, $rc, @vcs_user, @users, $encryscript, $password, $n, $users, $admins, $opers, @systems, $sys2, $return, $hostname, $component);
    $return = {'changed' => 0, 'rc' => 0};

    $maincf = $self->include_types_cf($sys, $attrs);
    $maincf .= "\ncluster $attrs->{cluster_name} (";

    $encryscript = "/opt/VRTSvcs/bin/vcsencrypt";
    if (!$sys->exists($encryscript)) {
        $sys->log("Cannot find vcsencrypt binary - skipping add_users", 'warn');
        $return->{rc} = $macros->{VERITAS_VCS_NO_ENCRYPT};
    } else {
        @users = split (/,/, $attrs->{vcs_users});
        @users = ('admin:password:Administrators') unless @users;
        $n = 0;
        for my $user (@users) {
            ($vcs_user[$n]->{user}, $vcs_user[$n]->{password}, $vcs_user[$n]->{priv})
            = split (/:/, $users[$n]);
            $vcs_user[$n]->{password} = 'password' unless $vcs_user[$n]->{password};
            $vcs_user[$n]->{priv} = 'Administrators' unless $vcs_user[$n]->{priv};
            $password = $sys->cmd("$encryscript -vcs $vcs_user[$n]->{password}");
            $vcs_user[$n]->{password} = $password;
            $n++;
        }

        $n = 0;
        for $n (0..$#vcs_user) {
            $users .= ', ' if ($users);
	    	$users .= "\"$vcs_user[$n]->{user}\" = $vcs_user[$n]->{password}";
            if ($vcs_user[$n]->{priv} eq 'Administrators') {
                $admins .= ', ' if ($admins);
                $admins .= "\"$vcs_user[$n]->{user}\"";
            } elsif ($vcs_user[$n]->{priv} eq 'Operators') {
                $opers .= ', ' if ($opers);
                $opers .= "\"$vcs_user[$n]->{user}\"";
            }
        }
        $maincf .= "\n\tUserNames = { $users }" if ($users);
        $maincf .= "\n\tAdministrators = { $admins }" if ($admins);
        $maincf .= "\n\tOperators = { $opers }" if ($opers);
    }
    $maincf .= "\n\t)\n\n";
   
    $n=0;
    @systems = split(/,/,$attrs->{systems});
    for my $sysi (@systems) {
        $sys2 = transform_system_name($sysi);
        $maincf .= "system $sys2 (\n\t)\n\n";
        $n++;
    }
                
    $config_dir = "/etc/VRTSvcs/conf/config";
    $rc = $sys->writefile($maincf, "$config_dir/main.cf");
    if ($rc) {
        $sys->log("Failed to write the main.cf", 'error');
        $return->{rc} = $macros->{VERITAS_VCS_WRITE_MAINCF_FAIL};
    } else {
	    $return->{changed} = 1;
    }
     
    $hostname = $sys->cmd("_cmd_hostname 2>/dev/null");
    $hostname = transform_system_name($hostname);
    if ($hostname) {
        $rc = $sys->cmd("_cmd_echo $hostname > /etc/VRTSvcs/conf/sysname");
        if ($rc) {
            $sys->log("Failed to write the hostname to /etc/VRTSvcs/conf/sysname", 'error');		
            $return->{rc} = $macros->{VERITAS_VCS_WRITE_SYSNAME_FAIL};
        } else {
            $return->{changed} = 1;
        }
    } else {
        $sys->log("Can not get the hostname of the current system!", 'error');	
        $return->{rc} = $macros->{VERITAS_VCS_NO_SYSNAME};
	}

    return $return;
}

sub transform_system_name {
    my $sysname = shift;
    return unless ($sysname);

    #previously, CPI add double quotas for IPv4 IPv6 and FQDN address used in main.cf
    #to make it accepted by VCS enginee, this is an un-documented VCS behavior
    #confirmed with VCS team, change design into:
    #CPI replace IPv4 IPv6 FQDN address with nodename as the system name

    if (EDRu::ip_is_ipv4($sysname) || EDRu::ip_is_ipv6($sysname) || ($sysname =~ /\.\w+/m)) {
        ($sysname,undef)=split(/\./, $sysname, 2);
    } 
    return $sysname;
}

sub enable_vcs {
    my ($self, $sys) = @_;
    my ($return, $dir, $rc, $conf, $output);
    $return = {'changed' => 0, 'rc' => 0};
    $dir = "/etc/sysconfig";
    ($rc, $conf) = $sys->cmd("_cmd_cat $dir/vcs");
    
    if ($rc){
        $sys->log("Cannot find file $dir/vcs", 'error');
        $return->{rc} = $macros->{VERITAS_VCS_NO_ENABLE_FILE};
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/vcs.prev");
    $sys->cmd("_cmd_cp $dir/vcs $dir/vcs.prev");
    $sys->cmd("_cmd_sed -i 's/VCS_START=0/VCS_START=1/g' $dir/vcs");
    $sys->cmd("_cmd_sed -i 's/VCS_STOP=0/VCS_STOP=1/g' $dir/vcs");
    ($rc, $output) = $sys->cmd("_cmd_diff $dir/vcs $dir/vcs.prev");
    $return->{changed} = $rc;
    return $return;
}

sub disable_vcs{
    my ($self, $sys) = @_;
    my ($return, $dir, $rc, $conf, $output);
    $return = {'changed' => 0, 'rc' => 0};
    $dir = "/etc/sysconfig";
    ($rc, $conf) = $sys->cmd("_cmd_cat $dir/vcs");

    if ($rc){
        $sys->log("Cannot find file $dir/vcs", 'error');
        $return->{rc} = $macros->{VERITAS_VCS_NO_ENABLE_FILE};
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/vcs.prev");
    $sys->cmd("_cmd_cp $dir/vcs $dir/vcs.prev");
    $sys->cmd("_cmd_sed -i 's/VCS_START=1/VCS_START=0/g' $dir/vcs");
    $sys->cmd("_cmd_sed -i 's/VCS_STOP=1/VCS_STOP=0/g' $dir/vcs");
    ($rc, $output) = $sys->cmd("_cmd_diff $dir/vcs $dir/vcs.prev");
    $return->{changed} = $rc;

    return $return;
}

sub recover_maincf {
    my ($self, $sys) = @_;
    my ($return, $config_dir, $rc, $changed);
    $return = {'changed' => 0, 'rc' => 0};
    $config_dir = "/etc/VRTSvcs/conf/config";
    if (-e "$config_dir/main.cf") {
        $sys->cmd("_cmd_rmr $config_dir/main.cf.b");
        $sys->cmd("_cmd_cp $config_dir/main.cf $config_dir/main.cf.b");
        $rc = $sys->cmd("_cmd_echo include \"types.cf\" > $config_dir/main.cf");
        if ($rc) { 
            $sys->log("Failed to write the main.cf", 'error');
            $return->{rc} = $macros->{VERITAS_VCS_WRITE_MAINCF_FAIL};
        }
        $rc = $sys->cmd("_cmd_diff $config_dir/main.cf $config_dir/main.cf.b");
        $return->{changed} = 1 if $rc;
    } else {
        $rc = $sys->cmd("_cmd_echo include \"types.cf\" > $config_dir/main.cf");
        if ($rc) {
            $sys->log("Failed to write the main.cf", 'error');		
            $return->{rc} = $macros->{VERITAS_VCS_WRITE_MAINCF_FAIL};
        } else {
            $return->{changed} = 1;
        }
    }
    $rc = $sys->cmd("_cmd_rmr /etc/VRTSvcs/conf/sysname");
    $return->{changed} = 1 if $rc == 0;

    return $return;
}

sub cluster_uuid {
    my ($self, $sys, $state, $uuid) = @_;
    my ($return, $rc);
    $return = {'changed' => 0, 'rc' => 0};
    if ($state =~ /present/) {
        $sys->cmd("_cmd_mkdirp /etc/vx/.uuids");
        $rc = $sys->cmd("_cmd_echo {$uuid} > /etc/vx/.uuids/clusuuid");
        if ($rc) {
            $sys->log("Failed to generate the Cluster UUID", 'error'); 
            $return->{rc} = $macros->{VERITAS_VCS_GEN_UUID_FAIL};
        } else { 
            $return->{changed} = 1;
        }
    }elsif ($state =~ /absent/) {
        $rc = $sys->cmd("_cmd_rmr /etc/vx/.uuids/clusuuid");
        $return->{changed} = 1 if $rc == 0;
    }
    return $return;
}
1;
