package Module::veritas::process;
# description:
#     - Start or stop product process
# options:
#   - state:
#       required: true
#       choices: [present, absent]
#       description:
#           start/stop component       
#	- component:
#		required: true
#       choices: [ sf,vcs,sfha,sfcfsha,sfrac ]
#   - seednode
#       required: true
#       description: 
#			The node to perform cluster level operations.
#
#./installer -module process -action 'state=present component=sfcfsha seednode=rhel65-only-1'

$Macros::macros->set("VERITAS_VCS_START_SUCCESS",6);
$Macros::macros->set("VERITAS_VCS_STOP_SUCCESS",8);
$Macros::macros->set("VERITAS_VCS_START_FAIL",19);
$Macros::macros->set("VERITAS_VCS_STOP_FAIL",21);
$Macros::macros->set("VERITAS_SFCFSHA_START_SUCCESS",2);
$Macros::macros->set("VERITAS_SFCFSHA_STOP_SUCCESS",4);
$Macros::macros->set("VERITAS_SFCFSHA_START_FAIL",3);
$Macros::macros->set("VERITAS_SFCFSHA_STOP_FAIL",5);

sub check_args {
    my ($self, $attrs) = @_; 
    my ($state, @invalid, $arg);
    @invalid = ();	
     
    for $arg (qw/state component seednode/) { 
        next if defined $attrs->{$arg};
        push (@invalid, $arg);
    }
	return sort @invalid;
}

sub execute_sys {
    my ($self,$sys,$attrs) = @_;
    my ($return,$rc, $changed,$plat,$state,$component,$seednode,$ctrl_script);

    $return={ 'changed' => 0, 'rc' => 0 };

    # get distribution
    $plat=$sys->{plat};
    return $return if (!$plat);
    return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

	return $return if ( !$attrs->{state} || !$attrs->{component} || !$attrs->{seednode} );
	$component=$attrs->{component};
	$seednode=$attrs->{seednode};
	$ctrl_script="/opt/VRTSperl/lib/site_perl/UXRT72/CPIR/Module/veritas/$component" . "_ctrl.sh";
	unless (-e $ctrl_script) {
        $sys->log("Cannot find start/stop script for $component.",'error');	
        return $return;
    }
    if ($attrs->{state} eq 'present') {
		$self->prestart($sys,$component,$seednode);
		$rc = $self->start($sys,$ctrl_script);
        $return->{rc} ||= $rc;
		$rc = $self->poststart($sys,$component,$seednode);
        $return->{rc} ||= $rc;
        $component = uc($component);
        if ($return->{rc}) {
            $sys->log("$component startup failed.", 'error');
            $return->{rc} = $macros->{"VERITAS_" . $component . "_START_FAIL"};
        } else {
            $sys->log("$component startup completed successfully.");
            $return->{rc} = $macros->{"VERITAS_" . $component . "_START_SUCCESS"};
        }

        $changed=1;
    } elsif ($attrs->{state} eq 'absent') {
        $self->prestop($sys,$component,$seednode);
        $rc = $self->stop($sys,$ctrl_script);
        $return->{rc} ||= $rc;
		$self->poststop($sys,$component,$seednode);
        $component = uc($component);
        if ($return->{rc}) {
            $sys->log("Stopping $component failed.", 'error');
            $return->{rc} = $macros->{"VERITAS_" . $component . "_STOP_FAIL"};
        } else {
            $sys->log("Stopping $component completed successfully.");
            $return->{rc} = $macros->{"VERITAS_" . $component . "_STOP_SUCCESS"};
        }
		$changed = 1;
    }
    $return->{changed} = 1 if $changed == 1;
    return $return;
}

sub prestart {
	my ($self,$sys,$component,$seednoe) = @_;
	return;
}

sub start {
	my ($self,$sys,$ctrl_script) = @_;
	$sys->cmd("_cmd_chmod +x $ctrl_script");
	my ($rc, $output) = $sys->cmd("$ctrl_script start");
	return $rc;
}

sub poststart {
	my ($self,$sys,$component,$seednode) = @_;
	my ($rc,$hostname,$output);
	($rc,$hostname) = $sys->cmd("_cmd_hostname");
	
	#start sfcfs agents on seednode
	if(($component =~ /sfcfs/i) && ($hostname eq $seednode)){
		($rc,$output) = $sys->cmd("_cmd_cfscluster config -t 200 -s");
	}
	
	# Copy CVMTypes.cf and CFSTypes.cf to /etc/VRTSvcs/conf for Dynamic Upgrade. 
	if((-e "/etc/VRTSvcs/conf/config/CVMTypes.cf") && !(-e "/etc/VRTSvcs/conf/CVMTypes.cf")){
		$sys->cmd("_cmd_cp -p /etc/VRTSvcs/conf/config/CVMTypes.cf /etc/VRTSvcs/conf/CVMTypes.cf");
	}
	if((-e "/etc/VRTSvcs/conf/config/CFSTypes.cf") && !(-e "/etc/VRTSvcs/conf/CFSTypes.cf")){
		$sys->cmd("_cmd_cp -p /etc/VRTSvcs/conf/config/CFSTypes.cf /etc/VRTSvcs/conf/CFSTypes.cf");
	}
	
	return $rc;
}

sub prestop {
	my ($self,$sys,$component,$seednode) = @_;
	#stop sfcfs agents
	if($component =~ /sfcfs/i) {
		my ($port_h,$port_v);
		$port_v = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port v gen' ");
		$port_h = $sys->cmd("_cmd_gabconfig -a 2>/dev/null | _cmd_grep -c 'Port h gen' ");
		if ($port_v >= 1) { 
			if ($port_h == 0) {
				$sys->cmd("_cmd_cfscluster start");
			}
		}
		$sys->cmd("_cmd_cfscluster stop -f");
		$sys->cmd("_cmd_fsclustadm cfsdeinit");
	}

	if( $component =~ /vcs|sfha|sfcfs|sfrac/i ){
		$sys->cmd("_cmd_hastop -local");
	}

	return;
}

sub stop {
	my ($self,$sys,$ctrl_script) = @_;
	$sys->cmd("_cmd_chmod +x $ctrl_script");
	my ($rc, $output) = $sys->cmd("$ctrl_script stop");
	return $rc;
}

sub poststop {
	my ($self,$sys,$component,$seednode) = @_;
	return;
}


1;
