package Module::veritas::sfcfsha_config;

# description:
#     - Configure SFCFSHA
# options:
#   - state:
#       required: true
#       choices: [present, absent]
#       description:
#           enable/disable LLT
#       choice -> present:
#           - method:
#               required: true
#               choices: [ethernet, udp, rdma]
#               description:
#                   1. LLT over ethernet is the fastest way to for communication,
#                   but it requires direct connection between 2 NICs, which is impossible in some scenarios
#                   2. LLT over UDP means the LLT packets are routable and thus able to span longer distance,
#                   hence it requires IP address, port and netmask for routing. 
#
#                   <explain difference among ethernet/udp/rdma>  
#               choice --> udp:
#                   - vip:
#                       required: true
#                       choices: [ Virtual IP address ]
#                   - netmask:
#                       required: true
#                       choices: [ Virtual IP netmask ]
#                   - port:
#                       required: true
#                       choices: [ UDP port ]
#                   - broadcast:
#                       required: false
#                       choices: [true, false]
#                       default: false
#                   For each NIC, it needs a separate VIP/netmask/port                   
#           - vcs_clusterid:
#               required: true
#               choices: [ Cluster ID ]
#           - private_link:
#               required: true
#               choices: [ (NIC name array) ]
#               choice --> only one NIC:
#                   - low_priority_link:
#                       required: true
#                       choices: [ (low priority NIC name array) ]
#   - cluster_name:
#       required: true  
#       choices: [ cluster name ]
#       description:
#           - set the cluster name
#   - vcs_users
#       required: false
#       default: admin:password:Administrators
#       choices: [user:password:priv]
#       description:
#           - VCS user name:user password:user privilege
#   - vcs_user_password
#       required: false
#       default: password
#       choices: [ user password ]
#       description:
#           - VCS user password
#   - systems
#       required: true
#       choices [ cluster system hostnames ]

#./installer -module sfcfsha_config -action 'state=present method=ethernet vcs_clusterid=14852 private_link=eth1,eth2 low_priority_link=eth0 systems=node1,node2,node3  cluster_name=vcs cluster_uuid=1321a-3433b-2334c vcs_users=admin:password:Administrators,user1:passwd1:Operators'

$Macros::macros->set("VERITAS_SFCFSHA_SUCCESS",0);
$Macros::macros->set("VERITAS_SFCFSHA_FAIL",1);

sub check_args {
    my ($self, $args) = @_; 
     
	return Module::veritas::vcs_config->check_args($args);
}

sub execute_sys {
    my ($self,$sys,$attrs) = @_;
    my ($return,@mdrtn,$plat,$state,$result);

    $return={ 'changed' => 0 ,'rc'=>0,'status' => [] };

    # get distribution
    $plat=$sys->{plat};
    return $return if (!$plat);
    return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

    return $return if (!$attrs->{state});
    # we assume all the necessary attributes are set, in future we need to come up with a function to validate for each module.
    if ($attrs->{state} eq 'present') {
        # config vcs
        push @mdrtn, Module::veritas::vcs_config->execute_sys($sys,$attrs);

        # config odm/gms
        push @mdrtn, Module::veritas::odm->execute_sys($sys,$attrs);
        push @mdrtn, Module::veritas::gms->execute_sys($sys,$attrs);

    } elsif ($attrs->{state} eq 'absent') {
        push @mdrtn, Module::veritas::vcs_config->execute_sys($sys,$attrs);
    }   

    for $result (@mdrtn) {
        push @{$return->{status}}, $result->{rc};
        if ($result->{changed} == 1 && $return->{changed} == 0) {
            $return->{changed} = 1;	
        }
        if (($result->{rc} & '255') && $return->{rc} == 0) {
            $return->{rc} = $macros->{VERITAS_SFCFSHA_FAIL};
        }
    }
    if ($return->{rc}) {
        $sys->log("Failed to configure SFCFSHA",'error');	
    } else {
        $sys->log("SFCFSHA configuration completed successfully.");
        $return->{rc} = $macros->{VERITAS_SFCFSHA_SUCCESS}; 
    }
    return $return;
}

1;
