#!/bin/sh
#This script using to control sfcfsha status. (start|stop sfcfsha).


#Usage:
#sfcfsha_ctrl.sh {start|stop}

if [ $# -ne 1 ]
then
	echo "Usage: `basename $0` {start|stop}"
	exit 1
fi

if [ $1 != "stop" ] && [ $1 != "start" ]
then
	echo "Unsupported option"
	echo "Usage: `basename $0` {start|stop}"
	exit 1
fi

platform=$(uname -s)
padv=""
return_code=0
start_proc_list=""
stop_proc_list=""

function kill_process {
	pids=$(ps -ef | grep $1 | grep -v grep | grep -v defunct | awk '{print $2}')
	for pid in $pids
	do
		kill -9 $pid
	done
}

function start_veki {
	case $platform in
	Linux)
		/etc/init.d/veki start &>/dev/null
		lsmod | grep '^veki' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "veki failed to start."
		return_code=1
	fi
}

function stop_veki {
	case $platform in
	Linux)
		/etc/init.d/veki stop &>/dev/null
		lsmod | grep '^veki' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "veki failed to stop."
		return_code=1
	fi
}

function start_vxdmp {
	case $platform in
	Linux)
		modprobe vxdmp &>/dev/null
		lsmod | grep '^vxdmp' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxdmp failed to start."
		return_code=1
	fi
}

function stop_vxdmp {
	case $platform in
	Linux)
		modprobe -r vxdmp &>/dev/null
		lsmod | grep '^vxdmp' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxdmp failed to stop."
		return_code=1
	fi
}

function start_vxio {
	case $platform in
	Linux)
		modprobe vxio &>/dev/null
		lsmod | grep '^vxio' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxio failed to start."
		return_code=1
	fi
}

function stop_vxio {
	case $platform in
	Linux)
		modprobe -r vxio &>/dev/null
		lsmod | grep '^vxio' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxio failed to stop."
		return_code=1
	fi
}

function start_vxspec {
	case $platform in
	Linux)
		modprobe vxspec &>/dev/null
		lsmod | grep '^vxspec' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxspec failed to start."
		return_code=1
	fi
}

function stop_vxspec {
	case $platform in
	Linux)
		modprobe -r vxspec &>/dev/null
		lsmod | grep '^vxspec' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxspec failed to stop."
		return_code=1
	fi
}

function start_vxconfigd {
	voldmode=""
	if [ ! -e /etc/vx/reconfig.d/state.d/install-db ] && [ -e /etc/vx/volboot ]
	then
		voldmode=$(vxdctl mode)
		if [[ $voldmode =~ "not-running" ]]
		then
			vxconfigd -k -r reset -x syslog &>/dev/null
			voldmode=$(vxdctl mode)
		elif [ "$voldmode" != "mode: enabled" ]
		then
			vxdctl enable &>/dev/null
			voldmode=$(vxdctl mode)
		fi
		
		if [ "$voldmode" != "mode: enabled" ]
		then
			echo "vxconfigd failed to start."
			return_code=1
		fi
		return 
	fi
	
	case $platform in
	Linux)
		rm -rf /dev/vx/dmp &>/dev/null
		rm -rf /dev/vx/rdmp &>/dev/null
		mkdir -m 0755 /dev/vx/dmp &>/dev/null
		ln -s /dev/vx/dmp /dev/vx/rdmp &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	
	vxconfigd -k -m disable -x syslog &>/dev/null
	vxdctl init &>/dev/null
	vxdctl enable &>/dev/null
	voldmode=$(vxdctl mode)
	
	if [ "$voldmode" != "mode: enabled" ]
	then
		echo "vxconfigd failed to start."
		return_code=1
	else
		rm -rf /etc/vx/reconfig.d/state.d/install-db &>/dev/null
	fi
}

function stop_vxconfigd {
	vxdctl stop &>/dev/null
	ps -ef | grep 'vxconfigd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		vxdctl -k stop &>/dev/null
	fi
	
	ps -ef | grep 'vxconfigd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		`kill_process 'vxconfigd'`
	fi

	ps -ef | grep 'vxconfigd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "vxconfigd failed to stop."
		return_code=1
	fi
}

function start_vxesd {
	if [ -e /etc/vx/.vxesd.lock ]
	then
		rm -rf /etc/vx/.vxesd.lock &>/dev/null
		touch /etc/vx/.vxesd.lock &>/dev/null	
	fi
	vxddladm start eventsource &>/dev/null
	vxrecover -sn &>/dev/null
	vxrecover -b -o iosize=64k &>/dev/null
	ps -ef | grep 'sbin/vxesd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "vxesd failed to start."
		return_code=1
	fi
}

function stop_vxesd {
	vxddladm stop eventsource &>/dev/null
	timer=0
	while [ $timer -le 60 ]
	do
		ps -ef | grep 'sbin/vxesd' | grep -v grep | grep -v defunct &>/dev/null
		if [ $? -ne 0  ] 
		then
			break
		fi
		sleep 5
		timer=$((timer + 5))
	done

	ps -ef | grep 'sbin/vxesd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		`kill_process 'sbin/vxesd'`
	fi

	ps -ef | grep 'sbin/vxesd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "vxesd failed to stop."
		return_code=1
	fi
}

function start_vxrelocd {
	nohup /usr/lib/vxvm/bin/vxrelocd root > /dev/null 2>&1 < /dev/null &
	sleep 2
	ps -ef | grep 'vxrelocd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxrelocd failed to start."
		return_code=1
	fi
}

function stop_vxrelocd {
	proc="vxrelocd"
	`kill_process $proc`
	ps -ef | grep 'vxrelocd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxrelocd failed to stop."
		return_code=1
	fi
}

function start_vxcached {
	nohup /usr/lib/vxvm/bin/vxcached root > /dev/null 2>&1 < /dev/null &
	sleep 2
	ps -ef | grep 'vxcached' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxcached failed to start."
		return_code=1
	fi
}

function stop_vxcached {
	proc="vxcached"
	`kill_process $proc`
	ps -ef | grep 'vxcached' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxcached failed to stop."
		return_code=1
	fi
}

function start_vxconfigbackupd {
	nohup /etc/vx/bin/vxconfigbackupd > /dev/null 2>&1 < /dev/null &
	sleep 2
	ps -ef | grep 'vxconfigbackupd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxconfigbackupd failed to start."
		return_code=1
	fi
}

function stop_vxconfigbackupd {
	proc="vxconfigbackupd"
	`kill_process $proc`
	ps -ef | grep 'vxconfigbackupd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxconfigbackupd failed to stop."
		return_code=1
	fi
}

function start_vxattachd {
	nohup /usr/lib/vxvm/bin/vxattachd root > /dev/null 2>&1 < /dev/null &
	sleep 2
	ps -ef | grep 'vxattachd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxattachd failed to start."
		return_code=1
	fi
}

function stop_vxattachd {
	proc="vxattachd"
	`kill_process $proc`
	ps -ef | grep 'vxattachd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxattachd failed to stop."
		return_code=1
	fi
}

function start_vvr {
	vxstart_vvr stop &>/dev/null
	vxstart_vvr &>/dev/null
	ps -ef | grep 'in.vxrsyncd' | grep -v grep | grep -v defunct &>/dev/null
	rc1=$?
	ps -ef | grep 'vradmind' | grep -v grep | grep -v defunct &>/dev/null
	rc2=$?
	ps -ef | grep 'vxnetd' | grep -v grep | grep -v defunct &>/dev/null
	rc3=$?
	
	if [ $rc1 -ne 0 ] || [ $rc2 -ne 0 ] || [ $rc3 -ne 0 ] 
	then
		echo "vvr failed to start."
		return_code=1
	fi
}

function stop_vvr {
	vxstart_vvr stop &>/dev/null
	vxdctl_mode=$(vxdctl -c mode 2>/dev/null | grep 'mode:')
	if [[  $vxdctl_mode =~ "MASTER"  ]] || [[  $vxdctl_mode =~ "SLAVE"  ]]
	then
		/opt/VRTS/bin/vxclustadm stopnode &>/dev/null
	fi
	ps -ef | grep 'in.vxrsyncd' | grep -v grep | grep -v defunct &>/dev/null
	rc1=$?
	ps -ef | grep 'vradmind' | grep -v grep | grep -v defunct &>/dev/null
	rc2=$?
	ps -ef | grep 'vxnetd' | grep -v grep | grep -v defunct &>/dev/null
	rc3=$?
	
	if [ $rc1 -eq 0 ] || [ $rc2 -eq 0 ] || [ $rc3 -eq 0 ] 
	then
		echo "vvr failed to stop."
		return_code=1
	fi
}

function start_vxencryptd {
	ps -ef | grep 'vxencryptd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		service vxvm-encrypt restart &>/dev/null
	fi
	
	ps -ef | grep 'vxencryptd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxencryptd failed to start."
		return_code=1
	fi
}

function stop_vxencryptd {
	service vxvm-encrypt stop &>/dev/null
	
	ps -ef | grep 'vxencryptd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		`kill_process 'vxencryptd'`
	fi
	
	ps -ef | grep 'vxencryptd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxencryptd failed to stop."
		return_code=1
	fi
}

function start_vxvvrstatd {
	command=""
	case $platform in
	Linux)
		command="/sbin/vxvvrstatd"
		;;
	AIX)
		command="/usr/sbin/vxvvrstatd"
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	nohup $command root > /dev/null 2>&1 < /dev/null &
	sleep 2
	ps -ef | grep 'vxvvrstatd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ] 
	then
		echo "vxvvrstatd failed to start."
		return_code=1
	fi
}

function stop_vxvvrstatd {
	proc="vxvvrstatd"
	`kill_process $proc`
	ps -ef | grep 'vxvvrstatd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxvvrstatd failed to stop."
		return_code=1
	fi
}

function start_vxgin {
	status=$(systemctl status vxgin | grep 'Active:' )
	if [[ $status =~ "running" ]]
	then
		systemctl stop vxgin &>/dev/null
	fi
	
	rm -rf /etc/systemd/system/multi-user.target.wants/vxgin.service &>/dev/null
	ln -s /usr/lib/systemd/system/vxgin.service /etc/systemd/system/multi-user.target.wants/vxgin.service &>/dev/null
	systemctl enable vxgin &>/dev/null
	systemctl start vxgin &>/dev/null
	
	systemctl status vxgin | grep 'Active:' | grep 'running' &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "vxgin failed to start."
		return_code=1
	fi
}

function stop_vxgin {
	systemctl stop vxgin &>/dev/null
	systemctl disable vxgin &>/dev/null
	
	status=$(systemctl status vxgin | grep 'Active:' )
	if [[ $status =~ "running" ]]
	then
		echo "vxgin failed to stop."
		return_code=1
	fi
}

function start_vxcfsplugin {
	status=$(systemctl status vxcfsplugin | grep 'Active:' )
	if [[ $status =~ "running" ]]
	then
		systemctl stop vxcfsplugin &>/dev/null
	fi
	
	rm -rf /etc/systemd/system/multi-user.target.wants/vxcfsplugin.service &>/dev/null
	ln -s /usr/lib/systemd/system/vxcfsplugin.service /etc/systemd/system/multi-user.target.wants/vxcfsplugin.service &>/dev/null
	systemctl enable vxcfsplugin &>/dev/null
	systemctl start vxcfsplugin &>/dev/null
	
	status=$(systemctl status vxcfsplugin | grep 'Active:' | grep 'running' )
	if [ $? -ne 0 ]
	then
		echo "vxcfsplugin failed to start."
		return_code=1
	fi
}

function stop_vxcfsplugin {
	systemctl stop vxcfsplugin &>/dev/null
	systemctl disable vxcfsplugin &>/dev/null
	
	status=$(systemctl status vxcfsplugin | grep 'Active:' )
	if [[ $status =~ "running" ]]
	then
		echo "vxcfsplugin failed to stop."
		return_code=1
	fi
}

function start_vxinfoscale_docker {
    status=$(/usr/bin/systemctl status vxinfoscale-docker 2>/dev/null | grep 'Active:')
    if [[ $status =~ "running" ]] 
    then 
        /usr/bin/systemctl stop vxinfoscale-docker &>/dev/null
    fi 

    rm -rf /etc/systemd/system/multi-user.target.wants/vxinfoscale-docker.service 
    ln -s /usr/lib/systemd/system/vxinfoscale-docker.service /etc/systemd/system/multi-user.target.wants/vxinfoscale-docker.service &>/dev/null
    /usr/bin/systemctl enable vxinfoscale-docker &>/dev/null
    /usr/bin/systemctl start vxinfoscale-docker &>/dev/null

    status=$(/usr/bin/systemctl status vxinfoscale-docker 2>/dev/null | grep 'Active:' | grep 'running')
    if [ $? -ne 0 ]
    then
        echo "vxinfoscale-docker failed to start."
        return_code=1
    fi 
}

function stop_vxinfoscale_docker {
    /usr/bin/systemctl stop vxinfoscale-docker &>/dev/null
    /usr/bin/systemctl disable vxinfoscale-docker &>/dev/null

    status=$(/usr/bin/systemctl status vxinfoscale-docker 2>/dev/null | grep 'Active:')
    if [[ $status =~ "running" ]]
    then
        echo "vxinfoscale-docker failed to stop."
        return_code=1
    fi
}

function start_vxportal {
	case $platform in
	Linux)
		modprobe vxportal &>/dev/null
		lsmod | grep '^vxportal' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxportal failed to start."
		return_code=1
	fi
}

function stop_vxportal {
	case $platform in
	Linux)
		modprobe -r vxportal &>/dev/null
		lsmod | grep '^vxportal' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxportal failed to stop."
		return_code=1
	fi
}

function start_fdd {
	case $platform in
	Linux)
		modprobe fdd &>/dev/null
		lsmod | grep '^fdd' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "fdd failed to start."
		return_code=1
	fi
}

function stop_fdd {
	case $platform in
	Linux)
		modprobe -r fdd &>/dev/null
		lsmod | grep '^fdd' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "fdd failed to stop."
		return_code=1
	fi
}

function start_vxcafs {
	case $platform in
	Linux)
		modprobe vxcafs &>/dev/null
		lsmod | grep '^vxcafs' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxcafs failed to start."
		return_code=1
	fi
}

function stop_vxcafs {
	case $platform in
	Linux)
		modprobe -r vxcafs &>/dev/null
		lsmod | grep '^vxcafs' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxcafs failed to stop."
		return_code=1
	fi
}

function start_vxglm {
	case $platform in
	Linux)
		/etc/init.d/vxglm start &>/dev/null
		lsmod | grep '^vxglm' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxglm failed to start."
		return_code=1
	fi
}

function stop_vxglm {
	case $platform in
	Linux)
		/etc/init.d/vxglm stop &>/dev/null
		lsmod | grep '^vxglm' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxglm failed to stop."
		return_code=1
	fi
}

function start_vxgms {
	case $platform in
	Linux)
		/etc/init.d/vxgms start &>/dev/null
		lsmod | grep '^vxgms' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxgms failed to start."
		return_code=1
	fi
}

function stop_vxgms {
	case $platform in
	Linux)
		/etc/init.d/vxgms stop &>/dev/null
		lsmod | grep '^vxgms' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxgms failed to stop."
		return_code=1
	fi
}

function start_vxodm {
	case $platform in
	Linux)
		/etc/init.d/vxodm restart &>/dev/null
		lsmod | grep '^vxodm' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "vxodm failed to start."
		return_code=1
	fi
}

function stop_vxodm {
	case $platform in
	Linux)
		/etc/init.d/vxodm stop &>/dev/null
		lsmod | grep '^vxodm' &>/dev/null
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -eq 0 ]
	then
		echo "vxodm failed to stop."
		return_code=1
	fi
}
function start_llt {
	case $platform in
	Linux)
		case $padv in
		RHEL6|SLES11)
			#lsmod | grep '^llt' &>/dev/null
			/etc/init.d/llt start &>/dev/null
			/etc/init.d/llt status | grep -i 'LLT is loaded and configured' &>/dev/null
			;;
		RHEL7|SLES12)
			#lsmod | grep '^llt' &>/dev/null
			/usr/bin/systemctl status llt | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop llt &>/dev/null
			fi
			/usr/bin/systemctl start llt &>/dev/null
			/etc/init.d/llt status | grep -i 'LLT is loaded and configured' &>/dev/null
			;;
		esac
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "llt failed to start."
		return_code=1
	fi
}

function stop_llt {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            lltconfig -oU &>/dev/null
			lsmod | /bin/grep '^llt ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r llt &>/dev/null
			fi
			lsmod | /bin/grep '^llt ' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status llt | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop llt &>/dev/null
            /etc/init.d/llt stop &>/dev/null
            lsmod | /bin/grep '^llt ' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -eq 0 ]
	then
		echo "llt failed to stop."
		return_code=1
	fi	
}

function start_gab {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            #/etc/init.d/gab status &>/dev/null
            /etc/init.d/gab start &>/dev/null
			/etc/init.d/gab status | grep -i 'GAB is loaded and configured' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/etc/init.d/gab status &>/dev/null
            /usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop gab &>/dev/null
			fi
			/usr/bin/systemctl start gab &>/dev/null
            /etc/init.d/gab status | grep -i 'GAB is loaded and configured' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "gab failed to start."
		return_code=1
	fi	
}

function stop_gab {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            gabconfig -U &>/dev/null
			lsmod | /bin/grep '^gab ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r gab &>/dev/null
			fi
			/etc/init.d/gab status | grep -i 'GAB module is not loaded' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop gab &>/dev/null
            /etc/init.d/gab stop &>/dev/null
            /etc/init.d/gab status | grep -i 'GAB module is not loaded' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "gab failed to stop."
		return_code=1
	fi	
}

function start_vxfen {
	if [ -e /etc/vxfenmode ]
	then
		rc=0
		case $platform in
		Linux)  
			case $padv in
			RHEL6|SLES11)
				/etc/init.d/vxfen start &>/dev/null
				
				#wait vxfen status become 'running'
				for (( i=0; i<10; i++ ))
				do	
					sleep 5
					/etc/init.d/vxfen status | grep 'running' &>/dev/null
					rc=$?
					if [ $rc -eq 0 ]
					then
						break
					fi
				done
				;;      
			RHEL7|SLES12)
				/usr/bin/systemctl status vxfen | grep "Active: active" &>/dev/null
				if [ $? -eq 0 ]
				then
					/usr/bin/systemctl stop vxfen &>/dev/null
				fi
				/usr/bin/systemctl start vxfen &>/dev/null
				
				#wait vxfen status become 'running'
				for (( i=0; i<10; i++ ))
				do	
					sleep 5
					/etc/init.d/vxfen status | grep 'running' &>/dev/null
					rc=$?
					if [ $rc -eq 0 ]
					then
						break
					fi
				done
				;;      
			esac    
				;;      
		AIX)    
			;;      
		SunOS)  
			;;      
		*)
			exit 1 
			;;
		esac 
		if [ $? -ne 0 ]
		then
			echo "vxfen failed to start."
			return_code=1
		fi	
		
		
	fi
}

function stop_vxfen {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/vxfen stop &>/dev/null
			lsmod | /bin/grep '^vxfen ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r vxfen &>/dev/null
			fi
			/etc/init.d/vxfen status | grep -i 'not loaded' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop vxfen &>/dev/null
            /etc/init.d/vxfen stop &>/dev/null
            /etc/init.d/vxfen status | grep -i 'not loaded' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "vxfen failed to stop."
		return_code=1
	fi	
}

function start_amf {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/amf start &>/dev/null
            /etc/init.d/amf status | grep 'AMF: Module loaded and configured' &>/dev/null
            ;;      
        RHEL7|SLES12)
			/usr/bin/systemctl status amf | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop amf &>/dev/null
			fi
			/usr/bin/systemctl start amf &>/dev/null
            /etc/init.d/amf status | grep 'AMF: Module loaded and configured' &>/dev/null
            ;;      
        esac    
        ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "amf failed to start."
		return_code=1
	fi	
}

function stop_amf {
	case $platform in
		    Linux)  
				case $padv in
				RHEL6|SLES11)
					/etc/init.d/amf stop &>/dev/null
					/etc/init.d/amf status | grep 'unloaded' &>/dev/null
					;;
	            RHEL7|SLES12)
				    #/usr/bin/systemctl status amf | grep 'Active: active' &>/dev/null
					/usr/bin/systemctl stop amf &>/dev/null
                    /etc/init.d/amf stop &>/dev/null
				    /etc/init.d/amf status | grep 'unloaded' &>/dev/null
				    ;;      
			    esac    
				;;      
			AIX)    
				;;      
			SunOS)  
			    ;;      
			*)
				exit 1 
				;;
	esac 
	
	if [ $? -ne 0 ]
	then
		echo "amf failed to stop."
		return_code=1
	fi
	
}

function start_had {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/vcs start &>/dev/null
                ;;      
        RHEL7|SLES12)
            /usr/bin/systemctl status vcs | grep 'Active: active' &>/dev/null
			if [ $? = 0 ]
			then
				/usr/bin/systemctl stop vcs &>/dev/null
			fi
			/usr/bin/systemctl start vcs &>/dev/null
                ;;      
        esac    
            ;;      
    AIX)    
            ;;      
    SunOS)  
            ;;      
    *)
        exit 1 ;;
    esac
	rc=1
	sleep 5
	#wait had start up.
	for(( i=0; i <= 24; i++ ))
	do
		ps -ef | grep 'bin/had' | grep -v grep | grep -v defunct &>/dev/null
		rc1=$?
		ps -ef | grep 'hashadow' | grep -v grep | grep -v defunct &>/dev/null
		rc2=$?
		if [ $rc1 -ne 0 ]
		then
			echo "had not running"
		fi
		
		if [ $rc2 -ne 0 ]
		then
			echo "hashadow not running"
		fi
		sysname=$(cat /etc/VRTSvcs/conf/sysname)
		/opt/VRTSvcs/bin/hasys -wait $sysname SysState RUNNING -time 60 &>/dev/null
		if [ $? -eq 0 ]
		then
			/opt/VRTSvcs/bin/hasys -state $sysname  &>/dev/null
			if [ $? -eq 0 ]
			then
				#echo "HAD is RUNNING."
				rc=0
				break
			fi
		fi
        sleep 5;
	done
	
	if [ $rc -ne 0 ]
	then
		echo "had failed to start."
		return_code=1
	fi
}

function stop_had {
	/opt/VRTSvcs/bin/hastop -local -force &>/dev/null
	sleep 2
	ps -ef | grep 'bin/had' | grep -v grep | grep -v defunct &>/dev/null
	rc1=$? 
	ps -ef | grep 'hashadow' | grep -v grep | grep -v defunct &>/dev/null
	rc2=$?
	if [ $rc1 -eq 0 ] || [ $rc2 -eq 0 ]
	then
		echo "had failed to stop."
		return_code=1
	fi
}

function start_CmdServer {
    /opt/VRTSvcs/bin/CmdServer &>/dev/null
	ps -ef | grep 'bin/CmdServer' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "CmdServer failed to start."
		return_code=1
	fi
}

function stop_CmdServer {
	/opt/VRTSvcs/bin/CmdServer -stop &>/dev/null
	ps -ef | grep 'bin/CmdServer' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0  ]
	then
		echo "CmdServer failed to stop."
		return_code=1
	fi
}

function start_xprtld {
	/opt/VRTSsfmh/adm/xprtldctrl start 2>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/xprtld' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "xprtld failed to start."
		return_code=1
	fi
}

function stop_sfmh_discovery {
	/opt/VRTSsfmh/adm/vxvmdiscovery-ctrl.sh stop &>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/sfmh-discovery' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "sfmh-discovery failed to stop."
		return_code=1
	fi
}

function stop_vxdclid {
	/opt/VRTSsfmh/bin/vxadm service stop &>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/vxdclid' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "vxdclid failed to stop."
		return_code=1
	fi
}

function stop_vxdbd {
	if [ -e /opt/VRTS/bin/vxdbdctrl ]
	then
		/opt/VRTS/bin/vxdbdctrl stop &>/dev/null
	fi
	
	if [ -e /opt/VRTSdbed/common/bin/vxdbdctrl ]
	then
		/opt/VRTSdbed/common/bin/vxdbdctrl stop &>/dev/null
	elif [ -e /opt/VRTSdbcom/bin/vxdbdctrl ]
	then
		/opt/VRTSdbcom/bin/vxdbdctrl stop &>/dev/null
	fi
	
	ps -ef | grep 'vxdbd' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxvvrstatd failed to stop."
		return_code=1
	fi
}

function stop_vxfs {
	case $platform in
    Linux)  
       
			lsmod | /bin/grep '^vxfs ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r vxfs &>/dev/null
			fi
			lsmod | /bin/grep '^vxfs ' &>/dev/null
        ;;          
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -eq 0 ]
	then
		echo "vxfs failed to stop."
		return_code=1
	fi	
}

function stop_vxsited {
	proc="vxsited"
	`kill_process $proc`
	ps -ef | grep 'vxsited' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxsited failed to stop."
		return_code=1
	fi
}

function stop_vxnotify {
	proc="vxnotify"
	`kill_process $proc`
	ps -ef | grep 'vxnotify' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ] 
	then
		echo "vxnotify failed to stop."
		return_code=1
	fi
}

function start_sfcfsha {
	echo "Starting SFCFSHA.."
	for proc in $start_proc_list
	do
		start_${proc}
	done
}

function stop_sfcfsha {
	echo "Stopping SFCFSHA.."
	for proc in $stop_proc_list
	do
		stop_${proc}
	done
}

#only for Linux
function get_padv {
	if [ -e /etc/oracle-release ] 
	then
		distro="OL"
	elif [ -e /etc/redhat-release ]
	then
		vers=$(uname -r| cut -d 'l' -f2|cut -d '.' -f1)
		case $vers in
		6)
			padv="RHEL6"
			start_proc_list="vxdmp vxio vxspec vxconfigd vxesd vxrelocd vxcached vxconfigbackupd vxattachd vvr vxencryptd vxvvrstatd xprtld vxportal fdd vxcafs llt gab vxfen amf vxglm had CmdServer vxgms vxodm"
			stop_proc_list="vxodm vxgms vxglm vxdbd had CmdServer amf vxfen gab llt vxcafs fdd vxportal vxfs sfmh_discovery vxdclid vxvvrstatd vvr vxencryptd vxconfigbackupd vxsited vxattachd vxcached vxrelocd vxnotify vxesd vxconfigd vxspec vxio vxdmp"
			;;
		7)
			padv="RHEL7"
			start_proc_list="veki vxdmp vxio vxspec vxconfigd vxesd vxrelocd vxcached vxconfigbackupd vxattachd vvr vxencryptd vxvvrstatd xprtld vxportal fdd vxcafs llt gab vxfen amf vxglm had CmdServer vxgms vxodm"
			stop_proc_list="vxodm vxgms vxglm vxdbd had CmdServer amf vxfen gab llt vxcafs fdd vxportal vxfs sfmh_discovery vxdclid vxvvrstatd vvr vxencryptd vxconfigbackupd vxsited vxattachd vxcached vxrelocd vxnotify vxesd vxconfigd vxspec vxio vxdmp"
			;;
		*)
			echo "Unsupported RHEL version."
			exit 1;;
		esac
	elif [ -e /etc/SuSE-release ]
	then 
		distro="SLES"
	fi
}

case $platform in 
Linux)
	get_padv
	if [ $1 = "start" ]
	then
		start_sfcfsha
	elif [ $1 = "stop" ]
	then
		stop_sfcfsha
	fi
	;;
AIX|SunOS)
	;;
*)
	exit 1 
	;;
esac

if [ $return_code -eq 0 ]
then
	echo "Done"
else
	echo "Failed"
fi
exit $return_code
