#!/bin/sh
#This script using to control vcs status. (start|stop vcs).
#Note:
#Now, it's applicable for availability only.
#Use this script to stop enterprise vcs may failed since stop odm,gms,glm are not considered in this script.

#Usage:
#vcs_ctrl.sh {start|stop}

if [ $# -ne 1 ]
then
	echo "Usage: `basename $0` {start|stop}"
	exit 1
fi

if [ $1 != "stop" ] && [ $1 != "start" ]
then
	echo "Unsupported option"
	echo "Usage: `basename $0` {start|stop}"
	exit 1
fi

platform=$(uname -s)
padv=""
return_code=0
function start_llt {
	case $platform in
	Linux)
		case $padv in
		RHEL6|SLES11)
			#lsmod | grep '^llt' &>/dev/null
			/etc/init.d/llt start &>/dev/null
			/etc/init.d/llt status | grep -i 'LLT is loaded and configured' &>/dev/null
			;;
		RHEL7|SLES12)
			#lsmod | grep '^llt' &>/dev/null
			/usr/bin/systemctl status llt | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop llt &>/dev/null
			fi
			/usr/bin/systemctl start llt &>/dev/null
			/etc/init.d/llt status | grep -i 'LLT is loaded and configured' &>/dev/null
			;;
		esac
		;;
	AIX)
		;;
	SunOS)
		;;
	*)
		exit 1 
		;;
	esac
	if [ $? -ne 0 ]
	then
		echo "llt failed to start."
		return_code=1
	fi
}

function stop_llt {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            lltconfig -oU &>/dev/null
			lsmod | /bin/grep '^llt ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r llt &>/dev/null
			fi
			lsmod | /bin/grep '^llt ' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status llt | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop llt &>/dev/null
            /etc/init.d/llt stop &>/dev/null
            lsmod | /bin/grep '^llt ' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -eq 0 ]
	then
		echo "llt failed to stop."
		return_code=1
	fi	
}

function start_gab {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            #/etc/init.d/gab status &>/dev/null
            /etc/init.d/gab start &>/dev/null
			/etc/init.d/gab status | grep -i 'GAB is loaded and configured' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/etc/init.d/gab status &>/dev/null
            /usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop gab &>/dev/null
			fi
			/usr/bin/systemctl start gab &>/dev/null
            /etc/init.d/gab status | grep -i 'GAB is loaded and configured'
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "gab failed to start."
		return_code=1
	fi	
}

function stop_gab {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            gabconfig -U &>/dev/null
			lsmod | /bin/grep '^gab ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r gab &>/dev/null
			fi
			/etc/init.d/gab status | grep -i 'GAB module is not loaded' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop gab &>/dev/null
            /etc/init.d/gab stop &>/dev/null
            /etc/init.d/gab status | grep -i 'GAB module is not loaded' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "gab failed to stop."
		return_code=1
	fi	
}

function start_vxfen {
	if [ -e /etc/vxfenmode ]
	then
		rc=0
		case $platform in
		Linux)  
			case $padv in
			RHEL6|SLES11)
				/etc/init.d/vxfen start &>/dev/null
				
				#wait vxfen status become 'running'
				for (( i=0; i<10; i++ ))
				do	
					sleep 5
					/etc/init.d/vxfen status | grep 'running' &>/dev/null
					rc=$?
					if [ $rc -eq 0 ]
					then
						break
					fi
				done
				;;      
			RHEL7|SLES12)
				/usr/bin/systemctl status vxfen | grep "Active: active" &>/dev/null
				if [ $? -eq 0 ]
				then
					/usr/bin/systemctl stop vxfen &>/dev/null
				fi
				/usr/bin/systemctl start vxfen &>/dev/null
				
				#wait vxfen status become 'running'
				for (( i=0; i<10; i++ ))
				do	
					sleep 5
					/etc/init.d/vxfen status | grep 'running' &>/dev/null
					rc=$?
					if [ $rc -eq 0 ]
					then
						break
					fi
				done
				;;      
			esac    
				;;      
		AIX)    
			;;      
		SunOS)  
			;;      
		*)
			exit 1 
			;;
		esac 
		if [ $? -ne 0 ]
		then
			echo "vxfen failed to start."
			return_code=1
		fi	
		
		
	fi
}

function stop_vxfen {
	case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/vxfen stop &>/dev/null
			lsmod | /bin/grep '^vxfen ' &>/dev/null
			if [ $? -eq 0 ] 
			then
				modprobe -r vxfen &>/dev/null
			fi
			/etc/init.d/vxfen status | grep -i 'not loaded' &>/dev/null
            ;;      
        RHEL7|SLES12)
            #/usr/bin/systemctl status gab | grep "Active: active" &>/dev/null
			/usr/bin/systemctl stop vxfen &>/dev/null
            /etc/init.d/vxfen stop &>/dev/null			
            /etc/init.d/vxfen status | grep -i 'not loaded' &>/dev/null
            ;;      
        esac    
            ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "vxfen failed to stop."
		return_code=1
	fi	
}

function start_amf {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/amf start &>/dev/null
            /etc/init.d/amf status | grep 'AMF: Module loaded and configured' &>/dev/null
            ;;      
        RHEL7|SLES12)
			/usr/bin/systemctl status amf | grep "Active: active" &>/dev/null
			if [ $? -eq 0 ]
			then
				/usr/bin/systemctl stop amf &>/dev/null
			fi
			/usr/bin/systemctl start amf &>/dev/null
            /etc/init.d/amf status | grep 'AMF: Module loaded and configured' &>/dev/null
            ;;      
        esac    
        ;;      
    AIX)    
        ;;      
    SunOS)  
        ;;      
    *)
        exit 1 
		;;
    esac 
	if [ $? -ne 0 ]
	then
		echo "amf failed to start."
		return_code=1
	fi	
}

function stop_amf {
	case $platform in
		    Linux)  
				case $padv in
				RHEL6|SLES11)
					/etc/init.d/amf stop &>/dev/null
					/etc/init.d/amf status | grep 'unloaded' &>/dev/null
					;;
	            RHEL7|SLES12)
				    #/usr/bin/systemctl status amf | grep 'Active: active' &>/dev/null
					/usr/bin/systemctl stop amf &>/dev/null
                    /etc/init.d/amf stop &>/dev/null
				    /etc/init.d/amf status | grep 'unloaded' &>/dev/null
				    ;;      
			    esac    
				;;      
			AIX)    
				;;      
			SunOS)  
			    ;;      
			*)
				exit 1 
				;;
	esac 
	
	if [ $? -ne 0 ]
	then
		echo "amf failed to stop."
		return_code=1
	fi
	
}

function start_had {
    case $platform in
    Linux)  
        case $padv in
        RHEL6|SLES11)
            /etc/init.d/vcs start &>/dev/null
                ;;      
        RHEL7|SLES12)
            /usr/bin/systemctl status vcs | grep 'Active: active' &>/dev/null
			if [ $? = 0 ]
			then
				/usr/bin/systemctl stop vcs &>/dev/null
			fi
			/usr/bin/systemctl start vcs &>/dev/null
                ;;      
        esac    
            ;;      
    AIX)    
            ;;      
    SunOS)  
            ;;      
    *)
        exit 1 ;;
    esac
	rc=1
	sleep 5
	#wait had start up.
	for(( i=0; i <= 24; i++ ))
	do
		ps -ef | grep 'bin/had' | grep -v grep | grep -v defunct &>/dev/null
		rc1=$?
		ps -ef | grep 'hashadow' | grep -v grep | grep -v defunct &>/dev/null
		rc2=$?
		if [ $rc1 -ne 0 ]
		then
			echo "had not running"
		fi
		
		if [ $rc2 -ne 0 ]
		then
			echo "hashadow not running"
		fi
		sysname=$(cat /etc/VRTSvcs/conf/sysname)
		/opt/VRTSvcs/bin/hasys -wait $sysname SysState RUNNING -time 60 &>/dev/null
		if [ $? -eq 0 ]
		then
			/opt/VRTSvcs/bin/hasys -state $sysname  &>/dev/null
			if [ $? -eq 0 ]
			then
				#echo "HAD is RUNNING."
				rc=0
				break
			fi
		fi
        sleep 5;
	done
	
	if [ $rc -ne 0 ]
	then
		echo "had failed to start."
		return_code=1
	fi
}

function stop_had {
	/opt/VRTSvcs/bin/hastop -local -force &>/dev/null
	sleep 2
	ps -ef | grep 'bin/had' | grep -v grep | grep -v defunct &>/dev/null
	rc1=$? 
	ps -ef | grep 'hashadow' | grep -v grep | grep -v defunct &>/dev/null
	rc2=$?
	if [ $rc1 -eq 0 ] || [ $rc2 -eq 0 ]
	then
		echo "had failed to stop."
		return_code=1
	fi
}

function start_CmdServer {
    /opt/VRTSvcs/bin/CmdServer &>/dev/null
	ps -ef | grep 'bin/CmdServer' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "CmdServer failed to start."
		return_code=1
	fi
}

function stop_CmdServer {
	/opt/VRTSvcs/bin/CmdServer -stop &>/dev/null
	ps -ef | grep 'bin/CmdServer' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0  ]
	then
		echo "CmdServer failed to stop."
		return_code=1
	fi
}

function start_xprtld {
	/opt/VRTSsfmh/adm/xprtldctrl start 2>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/xprtld' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -ne 0 ]
	then
		echo "xprtld failed to start."
		return_code=1
	fi
}

function stop_sfmh_discovery {
	/opt/VRTSsfmh/adm/vxvmdiscovery-ctrl.sh stop &>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/sfmh-discovery' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "sfmh-discovery failed to stop."
		return_code=1
	fi
}

function stop_vxdclid {
	/opt/VRTSsfmh/bin/vxadm service stop &>/dev/null
	ps -ef | grep '/opt/VRTSsfmh/bin/vxdclid' | grep -v grep | grep -v defunct &>/dev/null
	if [ $? -eq 0 ]
	then
		echo "vxdclid failed to stop."
		return_code=1
	fi
}


function start_vcs {
	echo "Starting VCS.."
	start_llt
	start_gab
	start_vxfen
	start_amf
	start_had
	start_CmdServer
	start_xprtld
}

function stop_vcs {
	echo "Stopping VCS.."
	stop_sfmh_discovery
	stop_vxdclid
	stop_had
	stop_CmdServer
	stop_amf
	stop_vxfen
	stop_gab
	stop_llt
}

#only for Linux
function get_padv {
	if [ -e /etc/oracle-release ] 
	then
		distro="OL"
	elif [ -e /etc/redhat-release ]
	then
		vers=$(uname -r| cut -d 'l' -f2|cut -d '.' -f1)
		case $vers in
		6)
			padv="RHEL6";;
		7)
			padv="RHEL7";;
		*)
			echo "Unsupported RHEL version."
			exit 1;;
		esac
	elif [ -e /etc/SuSE-release ]
	then 
		distro="SLES"
	fi
}

case $platform in 
Linux)
	get_padv
	if [ $1 = "start" ]
	then
		start_vcs
	elif [ $1 = "stop" ]
	then
		stop_vcs
	fi
	;;
AIX|SunOS)
	;;
*)
	exit 1 
	;;
esac

if [ $return_code -eq 0 ]
then
	echo "Done"
else
	echo "Failed"
fi
exit $return_code
