package Module::veritas::vxfen;
# description:
#   - Set vxfen config files
# options:
#   - state:
#       required: false
#       choices: [present, absent]
#       default: present
#       description:
#           - set enable/disable vxfen
#   - vxfenmode:
#       required: false
#       choices: [disabled, disks, cps, majority,sybase]
#       default: disabled
#       choice-->disks:
#          - vxfendg: 
#               required: true
#               choices: [valid vxfen dg]
#          - disks:
#               required: true
#               choices: [disk1,disk2,disk3]
#          - seednode:
#               required: false
#               default: the first node of systems list
#               choices: [any system in the cluster]
#          - systems:
#               required: true if not define seednode or configure coordinator point agent
#               choices: [systems list]
#          - cpagent:
#               required: false
#               default: 1
#               choices: [1, 0]
#               description:
#                  - 1, choose yes to configure agent for coordination point
#                  - 0, choose no to configure agent for coordination point
#          - vxfengrp:
#               required: false
#               default: vxfen
#               choices: [any valid name for cpagent service groups, not already existent one or VCS keywords]
#          - level2freq:
#               required: false
#               default: 5
#               choices: [0..]
#          - auto_refresh_reg_support:
#               required: false
#               default: 1
#               choices: [1, 0]
#               description:
#                  - 1, choose yes to auto refresh registration
#                  - 0, choose no to auto refresh registration
#
#e.g. ./installer-RHEL7x8664 -m vxfen -a 'state=absent'           
#./installer-RHEL7x8664 -m vxfen -a 'state=present vxfenmode=disk vxfendg=fendg disks=storwizev70000_00000a,storwizev70000_00000b,storwizev70000_00000c systems=ernst,gunter cpagent=0'

$Macros::macros->set("VERITAS_VXFEN_SUCCESS",0);
$Macros::macros->set("VERITAS_VXFEN_UNCONF_FAIL",1);
$Macros::macros->set("VERITAS_VXFEN_DISABLE_FAIL",2);
$Macros::macros->set("VERITAS_VXFEN_DISK_MODE_FAIL",3);
$Macros::macros->set("VERITAS_VXFEN_NO_VXFEND",4);
$Macros::macros->set("VERITAS_VXFEN_NO_ENABLE_FILE",5);
$Macros::macros->set("VERITAS_VXFEN_VXVM_NOT_RUN",6);
$Macros::macros->set("VERITAS_VXFEN_INIT_DISK_FAIL",7);
$Macros::macros->set("VERITAS_VXFEN_CREATE_DG_FAIL",8);
$Macros::macros->set("VERITAS_VXFEN_STOP_VCS_FAIL",9);
$Macros::macros->set("VERITAS_VXFEN_STOP_VXFEN_FAIL",10);
$Macros::macros->set("VERITAS_VXFEN_START_VXFEN_FAIL",11);
$Macros::macros->set("VERITAS_VXFEN_START_VCS_FAIL",12);
$Macros::macros->set("VERITAS_VXFEN_SKIP_CPAGENT",13);
$Macros::macros->set("VERITAS_VXFEN_CONFIG_CPAGENT_FAIL",14);
$Macros::macros->set("VERITAS_VXFEN_DUMP_FAIL",15);
$Macros::macros->set("VERITAS_VXFEN_SKIP_DEL_CPAGENT",16);
$Macros::macros->set("VERITAS_VXFEN_DEL_CPAGENT_FAIL",17);

sub check_args {
    my ($self, $attrs) = @_;   
    my ($state, $vxfenmode, @invalid);

    $state = $attrs->{state};
    $vxfenmode = $attrs->{vxfenmode};
    $attrs->{cpagent} = '1' if !defined $attrs->{cpagent};
    push @invalid, 'state' unless $state;
    if ($state =~ /present/i) {
        push @invalid, 'vxfenmode' unless $vxfenmode;
        if ($vxfenmode =~ /disk/i) {
            for my $arg (qw/disks vxfendg/) {
                next if $attrs->{$arg};
                push @invalid, $arg;
            }
            if ($attrs->{cpagent} == 1) {
                push @invalid, 'systems' unless $attrs->{systems};			
            } else {
                push @invalid, 'seednode' unless $attrs->{seednode} or $attrs->{systems}; 			
            }
        }
    }
    
    return sort @invalid;
}

sub execute_sys {
    my ($self, $sys, $attrs) = @_; 
    my ($return, $result, $vxfenmode, $state, $rc, $modefile, $output, $vxenviron, $vxfendg, $maincf_dir);

    $return={ 'changed' => 0, 'rc' =>0 };
    return $return if (!defined $attrs);
	$attrs=eval($attrs) if ref($attrs) eq '';

    $state = $attrs->{state};
    return $return unless $state;
    $attrs->{vxfenmode} = $attrs->{vxfenmode} || 'disabled';
    $vxfenmode = $attrs->{vxfenmode};
        
    if ($state eq 'present') {
        $result = $self->vxfen_enable($sys, $state);
        $return->{changed} = 1 if $result->{changed};
        $return->{rc} = $result->{rc} if $result->{rc}; 
        return $return if $return->{rc};
        if ($vxfenmode =~ /disable/i) {
            $result = $self->config_disabled_fencing($sys, $attrs);
        } elsif ($vxfenmode =~ /disk/i) {
            $result = $self->config_disk_fencing($sys, $attrs);
        } elsif ($vxfenmode =~ /majority/i) {
            
        } elsif ($vxfenmode =~ /cps/i) {

        }
        $return->{changed} = 1 if $result->{changed};
        if ($result->{rc}) {
            $return->{rc} = $result->{rc};		
        } 
    } elsif ($state eq 'absent') {
        $modefile = "/etc/vxfenmode";
        $vxenviron = "/etc/vxenviron";
        $vxfendg = "/etc/vxfendg";
        $maincf_dir = "/etc/VRTSvcs/conf/config";

        if (-e $modefile) {
            ($rc, undef) = $sys->cmd("_cmd_rmr $modefile");
            $return->{changed} = 1 if $rc == 0;
        }

        $output = $sys->cmd("_cmd_grep UseFence $maincf_dir/main.cf 2> /dev/null");
        if ($output) {
            ($rc, undef) = $sys->cmd("_cmd_sed -i 's#UseFence#//UseFence#g' $maincf_dir/main.cf");
            $return->{changed} = 1 if $rc == 0;
        }
        ($rc, undef) = $self->check_vcs($sys, 'start');
        if ($return->{changed} && $rc) {
            $result = $self->stop_vcsfen($sys);
            $return->{rc} ||= $result->{rc};
            $result = $self->start_vcs($sys);
            $return->{rc} ||= $result->{rc};
        }

        $result = $self->remove_cpagent($sys, $attrs);
        $return->{changed} ||= $result->{changed};
        $return->{rc} ||= $result->{rc};

        if (-e $vxenviron) {
            ($rc, undef) = $sys->cmd("_cmd_rmr $vxenviron"); 
            $return->{changed} = 1 if $rc == 0;
        }
        if (-e $vxfendg) {
            ($rc, undef) = $sys->cmd("_cmd_rmr $vxfendg");
            $return->{changed} = 1 if $rc == 0;
        }

        $result = $self->vxfen_enable($sys, $state);
        $return->{changed} ||= $result->{changed};
        $return->{rc} ||= $result->{rc};

        if ($result->{rc}) {
            $return->{rc} = $macros->{VERITAS_VXFEN_UNCONF_FAIL};    
        } 
    }
    $return->{rc} = $macros->{VERITAS_VXFEN_SUCCESS} unless $return->{rc};
    return $return;
}

sub config_disabled_fencing {
    my ($self, $sys, $attrs) = @_;
	my ($rc, $return, $result, $output, %vxfen, $maincf_dir, $restart);
    $return = {'changed' => 0, 'rc' => 0};
    $maincf_dir = "/etc/VRTSvcs/conf/config";

    %vxfen = $self->parse_vxfen_state($sys);
    $result = $self->update_files($sys, $attrs);
    $return->{changed} ||= $result->{changed};
    $return->{rc} ||= $result->{rc};

   	$output = $sys->cmd("_cmd_grep UseFence $maincf_dir/main.cf 2> /dev/null"); 
    if ($output) {
        ($rc, $output) = $sys->cmd("_cmd_sed -i 's#UseFence#//UseFence#g' $maincf_dir/main.cf");
    	$return->{changed} = 1 if $rc == 0;
    }

    $output = $sys->cmd("_cmd_gabconfig -a 2>&1 | _cmd_grep 'Port a'");
    if (($result->{changed} || $vxfen{fencing_mode} !~ /disable/i) && $output) {
        $result = $self->stop_vcsfen($sys);
        $return->{rc} ||= $result->{rc};
        $result = $self->start_fencing($sys);
        $return->{rc} ||= $result->{rc};
        $result = $self->start_vcs($sys);
        $return->{rc} ||= $result->{rc};
    }
    $result = $self->remove_cpagent($sys, $attrs);
    $return->{changed} ||= $result->{changed};
    $return->{rc} ||= $result->{rc};

    if ($return->{rc}) {
        $sys->log("Failed to configure fencing in disabled mode", 'error');
        $return->{rc} = $macros->{VERITAS_VXFEN_DISABLE_FAIL};
    }

    return $return;    
}

sub config_disk_fencing {
    my ($self, $sys, $attrs) = @_;
	my ($rc, $rc1, $return, $result, $cpagent, %vxfen, $disk_policy);
    $return = {'changed' => 0, 'rc' => 0};
    $attrs->{cpagent} = (defined $attrs->{cpagent}) ? $attrs->{cpagent} : 1;
    $cpagent = $attrs->{cpagent};
    %vxfen = $self->parse_vxfen_state($sys);
    $disk_policy = $vxfen{disk_policy};

	$result = $self->config_fence_dg($sys, $attrs);
	$return->{changed} ||= $result->{changed};
    $return->{rc} ||= $result->{rc};
    $result = $self->update_files($sys, $attrs);
    $return->{changed} ||= $result->{changed};
    $return->{rc} ||= $result->{rc};
    $rc = $self->usefence($sys);
    $return->{changed} ||= $rc;

    if ($return->{changed} || !defined($disk_policy)) {
        $result = $self->stop_vcsfen($sys);
	    $return->{rc} ||= $result->{rc};
	    $result = $self->start_fencing($sys);
	    $return->{rc} ||= $result->{rc};
        $result = $self->start_vcs($sys);
        $return->{rc} ||= $result->{rc};
    }
    if ($cpagent) {
        $result = $self->config_cpagent($sys, $attrs);
    } else {
        $result = $self->remove_cpagent($sys, $attrs);
    }
    $return->{changed} ||= $result->{changed};
    $return->{rc} ||= $result->{rc};
	
    if ($return->{rc}) {
        $sys->log("Failed to configure disk based fencing", 'error');
        $return->{rc} = $macros->{VERITAS_VXFEN_DISK_MODE_FAIL};
    }
	return $return;
}

sub parse_vxfen_state {
    my ($self, $sys) = @_;
    my ($rc, %vxfen, $output);

    ($rc, $output) = $sys->cmd("_cmd_vxfenadm -d 2>/dev/null");
    if ($rc) {
        $vxfen{status} = "vxfen is not started";
        return %vxfen;
    }
    $output =~ m/Fencing Protocol Version: ([^\r\n]+)[\r\n]/;
    $vxfen{protocol_version}=$1;
    $output =~ m/Fencing Mode: ([^\r\n]+)[\r\n]/;
    $vxfen{fencing_mode}=$1;
    if ($output =~ m/Fencing .*? Disk Policy: ([^\r\n]+)[\r\n]/) {
        $vxfen{disk_policy}=$1;
    }   
    if ($output =~ m/Fencing Mechanism:\s*(\S+)\s*/) {
        $vxfen{mechanism}=$1;
    }   

    return %vxfen;
}

sub update_files{
    my ($self, $sys, $attrs) = @_;
    my ($output, $rc, $return, $vxfen_mode, $vxfendg, $fendg_file);
    $return = {'changed' => 0, 'rc' => 0};
    $fendg_file = "/etc/vxfendg";

    if (! -d "/etc/vxfen.d/") {
        $sys->log("Cannot find directory /etc/vxfen.d/.",'error');
        $return->{rc} = $macros->{VERITAS_VXFEN_NO_VXFEND};
        return $return;
    }

    $vxfen_mode = $attrs->{vxfenmode};
    $vxfendg = $attrs->{vxfendg};

    if ($vxfen_mode =~ /disable/i) {
        if (-e $fendg_file) {
            $sys->cmd("_cmd_mv $fendg_file $fendg_file.prev");
        }
        $vxfen_mode = "disabled";
    } elsif ($vxfen_mode =~ /disk/i) {
        return $return unless $vxfendg;
        if (-e $fendg_file) {
            $output = $sys->cmd("_cmd_grep -w $vxfendg $fendg_file");
            if (!$output) {
                $sys->cmd("_cmd_rmr $fendg_file.prev;_cmd_cp $fendg_file $fendg_file.prev");
                ($rc, $output) = $sys->cmd("_cmd_echo $vxfendg > $fendg_file");
                $return->{changed} = 1 if $rc == 0;
            }
        } else {
            ($rc, $output) = $sys->cmd("_cmd_echo $vxfendg > $fendg_file");
            $return->{changed} = 1 if $rc == 0;
        }        

        $vxfen_mode = "scsi3_dmp";
    } elsif ($vxfen_mode =~ /cps/i) {
        $vxfen_mode = "cps";
    } elsif ($vxfen_mode =~ /majority/i) {
        $vxfen_mode = "majority";
    } else {
    }
    
    if (-e "/etc/vxfenmode") {
        $sys->cmd("_cmd_rmr /etc/vxfenmode.prev");
        $sys->cmd("_cmd_cp /etc/vxfenmode /etc/vxfenmode.prev");
        $sys->cmd("_cmd_cp /etc/vxfen.d/vxfenmode_$vxfen_mode /etc/vxfenmode");
        ($rc, $output) = $sys->cmd("_cmd_diff /etc/vxfenmode /etc/vxfenmode.prev");
        $return->{changed} = 1 if $rc;
    } else {
        ($rc, $output) = $sys->cmd("_cmd_cp /etc/vxfen.d/vxfenmode_$vxfen_mode /etc/vxfenmode");
        $return->{changed} = 1 if $rc ==0;
    }
    
    return $return;
}

sub vxfen_enable {
    my ($self, $sys, $state) = @_; 
    my ($return, $value, $previous, $dir, $rc, $conf);
    $return = {'changed' => 0, 'rc' => 0};
    $value = 1 if $state =~ /present/;
    $value = 0 if $state =~ /absent/;
    $previous = 1 - $value;

    $dir = "/etc/sysconfig";
    unless (-f "$dir/vxfen") {
        $return->{rc} = $macros->{VERITAS_VXFEN_NO_ENABLE_FILE};
        $sys->log("Cannot find file $dir/vxfen",'error');
        return $return;
    }
    $sys->cmd("_cmd_rmr $dir/vxfen.prev");
    $sys->cmd("_cmd_cp $dir/vxfen $dir/vxfen.prev");
    $sys->cmd("_cmd_sed -i \"s/VXFEN_START=$previous/VXFEN_START=$value/\" $dir/vxfen");
    $sys->cmd("_cmd_sed -i \"s/VXFEN_STOP=$previous/VXFEN_STOP=$value/\" $dir/vxfen");
    ($rc, undef) = $sys->cmd("_cmd_diff $dir/vxfen $dir/vxfen.prev");
    $return->{changed} = 1 if $rc;
       
    return $return;
}

sub config_fence_dg {
    my ($self, $sys, $attrs) = @_;
    my ($return, $rc, $output, $cmd, $dgname, $disks, @disks, @systems, $sysname, $seednode);
    $return = {'changed' => 0, 'rc' => 0};
	
    $output = $sys->cmd("_cmd_vxdctl mode");
    if ($output !~ /enable/) {
       # VxVM is not running
       $sys->log("Make sure Volume Manager is running.", 'error');
       $return->{rc} = $macros->{VERITAS_VXFEN_VXVM_NOT_RUN};
       return $return; 
    }

	unless ($attrs->{vxfendg}) {
        $sys->log("Not define vxfendg", 'error');
        return $return;
    }

    $dgname = $attrs->{vxfendg};
    $disks = $attrs->{disks};
    $disks =~ s/\s*,\s*/ /g;
    @disks = split(/,/, $attrs->{disks});
    @systems = split (/,/, $attrs->{systems});
    $seednode = $attrs->{seednode};

    unless ($seednode) {
        return $return unless @systems;
        $seednode = "$systems[0]";
    }

    $sysname = $sys->cmd("_cmd_cat /etc/VRTSvcs/conf/sysname"); 
    if ($sysname eq $seednode){
        $output = $sys->cmd("_cmd_vxdisk -o alldgs list | grep -w $dgname");
        if ($output) {
            #the diskgroup already exists
            $sys->log("Disk group already exists", 'info');
            return $return;
        } else {
            return $return unless @disks;
            for my $disk (@disks) {
                my $list = $sys->cmd("_cmd_vxdisk -o alldgs list | _cmd_grep $disk 2>/dev/null | _cmd_grep -v LVM | grep -v cdsdisk");
                if ($list) {
                    ($rc, undef) = $sys->cmd("_cmd_vxdisksetup -i $disk");  
                    if ($rc) {
                        $sys->log("Initializing VxVM disk failed.", 'error');
                        $return->{rc} = $macros->{VERITAS_VXFEN_INIT_DISK_FAIL};
                        return $return;		
                    } else {
                        $return->{changed} = 1;
                    }
                }
            }
            ($rc, undef) = $sys->cmd("_cmd_vxdg init $dgname $disks");	
            if ($rc) {
                $sys->log("Disk group $dgname failed to be created", 'error');
                $return->{rc} = $macros->{VERITAS_VXFEN_CREATE_DG_FAIL};
                return $return;
            }
            $cmd = "_cmd_vxdg deport $dgname;";	
            $cmd .= "_cmd_vxdg -t import $dgname;";
            $cmd .= "_cmd_vxdg -g $dgname set coordinator=on;";
            $cmd .= "_cmd_vxdg deport $dgname";
            $sys->cmd("$cmd 2>/dev/null");
            $return->{changed}= 1;
        }
    }
    return $return;
}

sub usefence {
    my ($self, $sys) = @_;
    my ($change, $maincf_dir, $maincf, $tmpmaincf);
    $change = 0;
    $maincf_dir='/etc/VRTSvcs/conf/config';
    if(!$sys->cmd("_cmd_grep UseFence $maincf_dir/main.cf 2> /dev/null")) {
        $sys->cmd("_cmd_cp $maincf_dir/main.cf $maincf_dir/main.cf.bak");
		$maincf=$sys->readfile("$maincf_dir/main.cf");
		for my $line (split(/^/m,$maincf)) {
		    $tmpmaincf .= $line;
			$tmpmaincf .= "\tUseFence=SCSI3\n" if ($line =~ /^cluster/m);
		}
        $sys->writefile($tmpmaincf,"$maincf_dir/main.cf");		
		if ($sys->cmd("_cmd_grep UseFence $maincf_dir/main.cf 2> /dev/null")) {
		    $change = 1;
		}
    }
    return $change;
}

sub stop_vcsfen {
    my ($self, $sys) = @_;
    my ($output, $return);
    $return = {'changed' => 0, 'rc' => 0};

    if ($self->check_vcs($sys, 'stop')) {
	    $sys->cmd("_cmd_hastop -local");
        $sys->cmd('_cmd_hasys -wait localsys SysState EXITED -time 600');
        if ($self->check_vcs($sys, 'stop')) {
            $sys->log("Stopping VCS failed.",'error');
            $return->{rc} = $macros->{VERITAS_VXFEN_STOP_VCS_FAIL};
		} else {
            $return->{changed} = 1;
        }
	}
    sleep 5;
    $output = $sys->cmd("_cmd_lsmod | _cmd_grep vxfen");
	if ($output) {
	    $sys->cmd("_cmd_vxfen stop");
		$output = $sys->cmd("_cmd_lsmod | _cmd_grep vxfen");
        if ($output) {
            $sys->log("Stopping fencing failed", 'error');
            $return->{rc} = $macros->{VERITAS_VXFEN_STOP_VXFEN_FAIL};
        } else {
            $return->{changed} = 1;
        }
	}
	
    return $return;
}

sub start_fencing {
    my ($self, $sys) = @_;
	my ($return, $running, $sleep);
    $return = {'changed' => 0, 'rc' => 0};

    $running = $self->check_fencing($sys);
    unless ($running) {
        $return = $self->vxfen_enable($sys, 'present');
        return $return if $return->{rc};
	    $sys->cmd("_cmd_vxfen start");
        $running = $self->check_fencing($sys);
        if ($running) {
            $return->{changed} = 1;
        } else {
            $sleep = 0;
            while ($sleep < 25) {
                sleep 5;
                $sys->cmd("_cmd_vxfenconfig -c");
                $running = $self->check_fencing($sys);
                last if $running;
                $sleep++;
            }
            if ($running) {
                $return->{changed} = 1;
            } else {
                # failed to start fencing
                $sys->log("Starting fencing failed.", 'error');
                $return->{rc} = $macros->{VERITAS_VXFEN_START_VXFEN_FAIL};
            }
        }
    }

    return $return;
}

sub check_fencing {
    my ($self, $sys) = @_;
    my $output;

    $output = $sys->cmd("_cmd_lsmod | _cmd_grep vxfen");
    if ($output eq '') {
        # vxfen not loaded
        $sys->log("vxfen not loaded", 'info');
        return 0;
    }
    $output = $sys->cmd("_cmd_vxfenadm -d");
    if ($output =~ /running/m) {
        return 1;
    } else {
        return 0;
    }
}

sub start_vcs {
    my ($self, $sys) = @_;
	my ($running, $return, $sleep) ;
    $return = {'changed' => 0, 'rc' => 0};
	
	$running = $self->check_vcs($sys, 'start');
	unless ($running) {
	    # start the process
	    $sys->cmd("_cmd_vcs start");	
		$running = $self->check_vcs($sys, 'start');
		if ($running) {
		    $return->{changed} = 1;
		} else {
            $sleep = 0;
            while ($sleep < 4) {
                sleep 5;
                $running = $self->check_vcs($sys, 'start');
                last if $running;
                $sleep++;
            }
            if ($running) {
                $return->{changed} = 1;
            } else {
                # failed to start vcs
                $sys->log("Starting VCS failed.", 'error');
                $return->{rc} = $macros->{VERITAS_VXFEN_START_VCS_FAIL};
            }

		}
	}
	
	return $return;
}

sub check_vcs {
    my ($self, $sys, $state) = @_;
	my ($had, $hashadow, $rc, $output, $sysname);
	$had = $sys->cmd("_cmd_ps -ef | _cmd_grep 'bin/had' | _cmd_grep -v grep | _cmd_grep -v defunct");
	$hashadow = $sys->cmd("_cmd_ps -ef | _cmd_grep hashadow | _cmd_grep -v grep | _cmd_grep -v defunct");
	if ($state =~ /start/m) {
	    #When in startup phase,
		#regard had not running if had not running or hashadow not running
	    if (!$had || !$hashadow) {
            $sys->log("had is not running.",'info') if (!$had);
            $sys->log("hashadow is not running.",'info') if (!$hashadow);
		    return 0;
		}
	} elsif ($state =~ /stop/m) {
	    # When in stop phase,
		# regard had running if had running or hashadow running
		# regard had not running if both had and hashadow not running
		if (!$had && !$hashadow) {
            $sys->log("both had and hashadow are not running.",'info');
		    return 0;
		} else { 
		    return 1;
		}
	}
    $sys->log("both had and hashadow are running, checking VCS state further",'info');
	$sysname = $sys->cmd("_cmd_cat /etc/VRTSvcs/conf/sysname");
	($rc, undef) = $sys->cmd("_cmd_hasys -wait $sysname SysState RUNNING -time 60");
    if ($rc == 0) {
        $sys->log("VCS state is RUNNING", 'info');
        return 1;
    }   

    $output = $sys->cmd("_cmd_hasys -state $sysname");
    $sys->log("VCS is not in RUNNING state.",'warn');
    return 0 if ($output = ~/ERROR/m);
    if ($output = ~/STALE/m) {
        $sys->log("State is $output, means that had failed to start up because of main.cf error",'warn');
        return 0;
    }   

	return 1;
}

sub config_cpagent {
    my ($self, $sys, $attrs) = @_;
	my (@systems, $vxfengrp, $level2freq, $auto_refresh_reg_support, $cpgrp, $return, $seednode, $sysname, $rc, $output, $sysnamelist, $index, $cmd, @add_cpagent_steps);
    $return = {'changed' => 0, 'rc' => 0};
    
    @systems = split (/,/, $attrs->{systems});
    return unless @systems;
    $seednode = $attrs->{seednode};
    $seednode ||= $systems[0];

    $sysname = $sys->cmd("_cmd_cat /etc/VRTSvcs/conf/sysname");
    if ($seednode eq $sysname) {
        if (!$self->check_vcs($sys, 'start')) {
            $sys->log("VCS is not running, exit to configure agent of coordination point",'error');
            $return->{rc} = $macros->{VERITAS_VXFEN_SKIP_CPAGENT};
            return $return;
        }
    	$cpgrp = $sys->cmd("_cmd_hares -display -attribute Group -type CoordPoint 2>/dev/null | _cmd_grep -v '^#' | awk '{print \$1}'");
        unless ($cpgrp) {
            $vxfengrp = $attrs->{vxfengrp} || 'vxfen';
            $level2freq = (defined $attrs->{level2freq}) ? $attrs->{level2freq} : 5;
            $auto_refresh_reg_support = (defined $attrs->{auto_refresh_reg_support}) ? $attrs->{auto_refresh_reg_support} : 1;
            
    	    $rc = $self->haconf_makerw($sys);
    		$return->{changed} =1 if $rc;
    
            $sysnamelist = '';
    		$index = 0;
    		for my $sys (@systems) {
    		    $sysnamelist .= $sys . " $index ";
    			$index++;
    		}
            
            $cmd = "_cmd_hagrp -add $vxfengrp";
            push (@add_cpagent_steps, $cmd);

    		$cmd = "_cmd_hagrp -modify $vxfengrp SystemList $sysnamelist";
    		push (@add_cpagent_steps, $cmd);
    		
    		$cmd = "_cmd_hagrp -modify $vxfengrp AutoFailOver 0";
    		push (@add_cpagent_steps, $cmd);
    		
    		$cmd = "_cmd_hagrp -modify $vxfengrp Parallel 1";
    		push (@add_cpagent_steps, $cmd);
    		
    		# Add CoordPoint resource
    		$cmd = "_cmd_hares -add coordpoint CoordPoint $vxfengrp";
    		push (@add_cpagent_steps, $cmd);
    		
    		if ($level2freq) {
    		    $cmd = "_cmd_hares -override coordpoint LevelTwoMonitorFreq";
    			push (@add_cpagent_steps, $cmd);
    			$cmd = "_cmd_hares -modify coordpoint LevelTwoMonitorFreq $level2freq";
                push (@add_cpagent_steps, $cmd);
    		}
    		
    		if ($auto_refresh_reg_support) {
    		    $cmd = "_cmd_hares -modify coordpoint ActionOnCoordPointFault RefreshRegistrations";
                push (@add_cpagent_steps, $cmd);
    		}
    		
    		$cmd = "_cmd_hares -modify coordpoint Enabled 1";
    		push (@add_cpagent_steps, $cmd);
    		
    		$cmd = "_cmd_hares -add RES_phantom_$vxfengrp Phantom $vxfengrp";
    		push (@add_cpagent_steps, $cmd);
    		
    		$cmd = "_cmd_hares -modify RES_phantom_$vxfengrp Enabled 1";
    		push (@add_cpagent_steps, $cmd);
    		
    		for my $step (@add_cpagent_steps) {
    		    ($rc, $output) = $sys->cmd("$step");
                if ($rc) {
                    $sys->log("Failed to configure CoordPoint Agent. Configure the Coordination Point Agent manually",'error');
                    $return->{rc} = $macros->{VERITAS_VXFEN_CONFIG_CPAGENT_FAIL};
                    last;
                } elsif ($return->{changed} == 0) {
    			    $return->{changed} = 1;
    			}
    		}
#            $rc = $self->haconf_dumpmakero($sys);
            $rc = $sys->cmd("_cmd_haconf -dump -makero");
            if ($rc) {
                $sys->log("Cannot dump the VCS configuration into main.cf and make it read-only. Configure the Coordination Point Agent manually.",'error');
                $return->{rc} = $macros->{VERITAS_VXFEN_DUMP_FAIL};
            } else {
                $return->{changed} = 1;
            }
        }
        $sys->log("Group name $vxfengrp for Coordination Point Agent already exists,skipping.",'info');
    }
	
    return $return;
}

sub remove_cpagent {
    my ($self, $sys, $attrs) = @_;
    my ($return, $seednode, $sysname, @systems, $vxfengrp, $vxfenres);
    $return = {'changed' => 0, 'rc' => 0};

    @systems = split (/,/, $attrs->{systems});
    $seednode = $attrs->{seednode};
    unless ($seednode) {
        return $return unless @systems;
        $seednode = $systems[0];
    }

	$sysname = $sys->cmd("_cmd_cat /etc/VRTSvcs/conf/sysname");
	if ($seednode eq $sysname) {
        if (!$self->check_vcs($sys, 'start')) {
            $sys->log("VCS is not running, exit to remove agent of coordination point",'error');
            $return->{rc} = $macros->{VERITAS_VXFEN_SKIP_DEL_CPAGENT};
            return $return;
        }
	    $vxfengrp = $sys->cmd("_cmd_hares -display -attribute Group -type CoordPoint 2> /dev/null | _cmd_grep -v '^#' | awk '{print \$4}'");
	    return $return if ($vxfengrp eq '');
        $self->haconf_makerw($sys);
	    $vxfenres = $sys->cmd("_cmd_hagrp -resources $vxfengrp -localclus 2> /dev/null");
		for my $res (split(/\n/, $vxfenres)) {
		    $sys->cmd("_cmd_hares -delete $res");
		}
		$sys->cmd("_cmd_hagrp -delete $vxfengrp");
#        $self->haconf_dumpmakero($sys);
        $sys->cmd("_cmd_haconf -dump -makero");
		$vxfengrp = $sys->cmd("_cmd_hares -display -attribute Group -type CoordPoint 2> /dev/null | _cmd_grep -v '^#' | awk '{print \$4}'");
		if ($vxfengrp eq '') {
		    $return->{changed} = 1;
		} else {
		    #failed to delete vxfen agent group
            $sys->log("Failed to delete Coordination Point Agent Service Group $vxfengrp",'error');
            $return->{rc} = $macros->{VERITAS_VXFEN_DEL_CPAGENT_FAIL};
		}
    }
	
	return $return;   
}

sub haconf_makerw {
    my ($self, $sys) = @_;
    my ($rc, $change);
    $change = 0;

    $rc = $sys->cmd("_cmd_haclus -value ReadOnly");
    if ($rc eq '1') {
        ($rc, undef) = $sys->cmd("_cmd_haconf -makerw");
        $change = 1 if $rc == 0;
        sleep 3;
    }
    return $change;
}

sub haconf_dumpmakero {
    my ($self, $sys) = @_;
    my ($rtn, $change, $done, $count);
    $change = 0;
    $rtn = $sys->cmd("_cmd_haclus -value ReadOnly");
    if ($rtn eq '0') {
        $done = 0;
        $count = 0;
        # We should try again after some times if one haconf -dump fails
        while ($count < 7) {
            sleep 5;
            $rtn = $sys->cmd("_cmd_haconf -dump -makero");
            if ($rtn) {
                sleep 5;
                $count++;
            } else {
                if ($self->haconf_dump_ok($sys)) {
                    $done = 1;
                    $change = 1;
                    last;
                }
                sleep 5;
                $count++;
            }
        }
#        $sys->log("The command haconf -dump -makero failed",'error') unless $done;
    }
    return $change;
}

# if the dump is ok, it will return '0' for <31 nodes(eg, '5' stands for node0/2 is dumping), '0 0' for >31 nodes
sub haconf_dump_ok {
    my ($self, $sys) = @_;
    my $rtn;
    $rtn = $sys->cmd("_cmd_haclus -value DumpingMembership");

    if ($rtn) {
        return 0;
    } else {
        return 1 if ($rtn =~ /^\s*0\s*$/) || ($rtn =~ /^\s*0\s+0\s*$/);
    }
    return 0;
}


1;

