#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
package EDR;
use strict;
use LWP::UserAgent;
use JSON;


sub update_vom_task_status {
    my ($edr) = shift;
    my (%task_parent, @task_children, $web, @hosts, $cfg, $error_num, $errors, %post_data, $taskid, $cms_url, $localsys, $sys, $error_flag, $localErrors);
    return '' if($edr->{vom_task_finished});
    $cms_url = Cfg::opt('cmsurl') if(Cfg::opt('cmsurl'));
    $taskid = Cfg::opt('taskid') if(Cfg::opt('taskid'));
    if (!defined($cms_url) || !defined($taskid)) {
        Msg::log("Failed to send task status to VOM since no specified Task ID or CMS URL.");
        return 0;
    }
    $cfg = Obj::cfg();
    @hosts = @{$cfg->{systems}};

    #$task_parent{'name'} = "";

    # Note that hostname should be get from responsefile and should be the same with 
    # thd displayed hostname in VOM web page
    for my $hostname (@hosts) {
        my %child;
        my @output;

        $child{'hostname'} = $hostname;
        $child{'name'} = '';
        $sys=Obj::sys($hostname);
        $errors = $edr->check_exit_errors_sys($sys);
        if ($sys->{sys} eq $edr->{localsys}->{'fqdn'} || $sys->{sys} eq $edr->{localsys}->{'ip'}) {
            $localErrors = $edr->check_exit_errors_sys($edr->{localsys});
            if ($localErrors) {
                if($errors) {
                    push(@{$errors}, @{$localErrors}) if (defined $localErrors && @{$localErrors});
                } else {
                    $errors = $localErrors; 
                }
            }
        }
        if ($errors && (scalar @{$errors}) > 0) {
            $error_flag = 1;
            $child{'state'} = 'FINISHED';
            $child{'result'} = 1;
            for my $err (@{$errors}) {
                push(@output, $err);
            }
        } else {
            $child{'state'} = 'FINISHED';
            $child{'result'} = 0;
            push(@output, "Task completed successfully");
        }
        $child{'output'} = \@output;
        push(@task_children, \%child);
    }

    #incident:3866473 if any nodes failed, other nodes should also show failed
    if ($error_flag) {
        for my $task_child (@task_children) {
            if($task_child->{'result'} == 0) {
                $task_child->{'result'} = 1;
                my @output;
                push(@output, "Task completed failed due to the failures on other nodes");
                $task_child->{'output'} = \@output;
            }
        }
    }

    $edr->update_child_task_status(\%task_parent, \@task_children);

    $post_data{'taskid'} = $taskid;
    $edr->update_parent_task_status(\%post_data, $cms_url);

    $edr->refresh_vom_hosts();
    $edr->vom_unsetup_ssh();
    $edr->{vom_task_finished} = 1;
}

sub update_parent_task_status {
    my ($edr, $data_hash, $cms_url) = @_;
    my ($json_str, $cmd_str, $exit_code, $stdout, $localsys);
    $json_str = encode_json $data_hash;
    $json_str =~ s/"/\\"/g;

    $cmd_str = "/opt/VRTSsfmh/bin/xprtlc"
    .' -l https://'
    .$cms_url
    .'/agent/cgi-bin/deploy_task_update.pl -d json="'
    .$json_str
    .'"';
    $localsys = EDR::get('localsys');
    $stdout = $localsys->cmd($cmd_str);
    $exit_code = EDR::cmdexit();

    if ($exit_code == 0) {
        Msg::log("Successfully update parent task status to VOM: ".$cmd_str);
        return 1;
    } else {
        Msg::log("Failed to update parent task status to VOM: ".$cmd_str);
        return 0;
    }
}

sub update_child_task_status{
    my ($edr, $task_parent, $task_children) = @_;
    my (%status, $syslist, $json_str, $cmd_url, $taskid, $cmd_str, $localsys, $stdout, $cms_url, $exit_code);
    $cms_url = Cfg::opt('cmsurl') if(Cfg::opt('cmsurl'));
    $taskid = Cfg::opt('taskid') if(Cfg::opt('taskid'));
    if (!defined($cms_url) || !defined($taskid)) {
        Msg::log("Failed to send task status to VOM since no specified Task ID or CMS URL.");
        return 0;
    }

    $status{'job'} = 'update';
    $status{'taskid'} = $taskid;

    $status{'parent'} = $task_parent;
    $status{'children'} = $task_children;

    $json_str = encode_json \%status;
    $json_str =~ s/"/\\"/g;

    $cmd_str = "/opt/VRTSsfmh/bin/xprtlc"
    .' -l https://'
    .$cms_url
    .'/agent/cgi-bin/deploy_status.pl -d json="'
    .$json_str
    .'"';
    $localsys = EDR::get('localsys');
    $stdout = $localsys->cmd($cmd_str);
    $exit_code = EDR::cmdexit();
    
    if ($exit_code == 0) {
        Msg::log("Successfully update child task status to VOM: ".$cmd_str);
        return 1;
    } else {
        Msg::log("Failed to update child task status to VOM: ".$cmd_str);
        return 0;
    }
}

# Check installer error messages for vom
sub check_exit_errors_sys {
    my ($edr,$sys) = @_;
    my ($error_num, @errors);

    #return "" unless($sys->{errors});

    for my $err (@{$sys->{errors}}) {
        push (@errors, $err);
    }
    for my $err (@{$sys->{installfailmsg}}) {
        push (@errors, ((ref($err) =~ m/^Msg/) ? $err->{msg} : $err));
    }
    for my $err (@{$sys->{uninstallfailmsg}}) {
        push (@errors, ((ref($err) =~ m/^Msg/) ? $err->{msg} : $err));
    }
    for my $err (@{$sys->{startfailmsg}}) {
        push (@errors, ((ref($err) =~ m/^Msg/) ? $err->{msg} : $err));
    }
    for my $err (@{$sys->{stopfailmsg}}) {
        push (@errors, ((ref($err) =~ m/^Msg/) ? $err->{msg} : $err));
    }

    return if(scalar(@errors) < 1);
    return \@errors;
}

sub refresh_vom_hosts {
    my $edr = shift;
    for my $sys (@{$edr->{systems}}) {
        $sys->cmd("/opt/VRTSsfmh/bin/mh_ctl.pl --family HOST --refresh");
    }
}

sub vom_unsetup_ssh {
  my $edr = shift;
  my ($sshkey, @cmds, $cmd, $remote_home, $authorized_keys_file, $usr, $localsys);

  $usr||='root';
  $localsys=EDR::get('localsys');
  $sshkey=$usr.'@'.$localsys->{sys};

  return '' unless ($sshkey);
  for my $sys (@{$edr->{systems}}) {
      next if ($sys->{islocal} || $sys->{sys} eq $edr->{localsys}->{'fqdn'} || $sys->{sys} eq $edr->{localsys}->{'ip'});
      $remote_home=$sys->{root_home} || $sys->get_root_home();
      $authorized_keys_file="$remote_home/.ssh/authorized_keys";
      @cmds = (
              "_cmd_cp -f $authorized_keys_file $authorized_keys_file.bak 2>/dev/null",
              #only remove the last line if matched
              "_cmd_sed '\${/$sshkey/d}' $authorized_keys_file.bak 2>/dev/null > $authorized_keys_file",
              "_cmd_chmod 0600 $authorized_keys_file 2>/dev/null",
              );  
      $cmd=join(';',@cmds);
      $sys->cmd($cmd);
  }

  return '';
}



1;
