#
# VT50-2000-2016-84-28-4
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#

package Playbook;
@Playbook::ISA = qw(Obj);

use strict;

sub init {
    my ($playbook,$pname,%params) = @_;

    $playbook->{name}=$pname;

    return $playbook unless (keys %params);

    set_description($playbook, $params{description});

    my $file=$params{file};
    load_file($playbook,$file) if ($file && -f $file);

    return $playbook;
}

# Set playbook description.
sub set_description {
    my ($playbook,$desc)=@_;
    Obj::reset_value($playbook, 'description', $desc) if ($desc);
    return 1;
}

sub load_file {
    my ($playbook,$file)=@_;
    my ($yaml_str,$plays,@reserved,$tasks,$vars,$var,$value,$replace,$replaceby);

    $yaml_str=EDRu::readfile($file);
    $plays = eval { require CPAN::Meta::YAML; CPAN::Meta::YAML::Load($yaml_str) };
    if ($@) {
        print "Parse YAML error: $@\n";
        return $playbook;
    }

    # replace "{{ var }}"
    @reserved=qw(name when ignore_errors with_items);

    for my $play (@{$plays}) {
        $tasks=$play->{tasks};
        next if (!defined $tasks || !@{$tasks});

        $vars=$play->{vars};

        for my $task (@{$tasks}) {
            for my $key (keys %$task) {
                my $value=$task->{$key};
                chomp $value;

                if (defined $vars && keys %{$vars}) {
                    $value=~s/\{\{\s*item\s*}}/__ITEM__/g;
                    while ($value=~/(\{\{\s*(\S+)\s*}})/m) {
                        $var=$2;
                        $replace=$1;
                        $replaceby=$vars->{$var} || $var;
                        $value=~s/$replace/$replaceby/g;
                    }
                    $value=~s/__ITEM__/{{ item }}/g;
                    $task->{$key}=$value;
                }

                next if (EDRu::inarr($key, @reserved));

                $task->{module_name}=$key;
                $task->{module_params}=$value;
            }

        }
    }

    $playbook->{plays}=$plays;

    return $playbook;
}

1;
