#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Pkg;

sub verify_media {
    my $pkg = shift;
    my ($cpic,$padv,$pkgpath,$pkgi,$rel,$edr,$msg,$mediapath,$cfg);
    return if ($pkg->nopadvpkg);
    $cpic = Obj::cpic();
    $edr = Obj::edr();
    $rel = $pkg->rel;
    $cfg = Obj::cfg();
    # initiate the padv release object to define file, vers, size, and space
    if ($cpic->{fromdisk} || (!$rel->{pkgs} && !$pkg->{pkgonly})) {
        $pkg->{vers}=$pkg->{gavers};
        $pkg->{vers}||=$rel->{gavers};
        return '';
    } elsif ($rel->{padv} eq 'Common') {
        $pkg->{vers}=$pkg->version();
        return '';
    }
    if ($pkg->{file}) {
        $pkg->{vers} ||= $pkg->padv->media_pkg_version($pkg,EDRu::dirname($pkg->{file}));
        ####add for et3875921
        $pkg->{mpok} = 0 if($pkg->{mpok});
        ####
        return '';
    }

    $pkgpath=Cfg::opt('pkgpath');
    # only hanldles packages in base_path at the moment
    # should be improved to dynamically define packages at all three levels
    #for pkgonly packages, we will get the $pkgpath from mr release first
    if ($pkg->{pkgonly} && Cfg::opt('base_path')) {
        $mediapath=$cpic->{mediapath};
    } else {
        $mediapath=$cpic->{basemediapath};
    }
    if ($cfg->{opt}{ignorepkgs}) {
        $pkg->{ignore}=1 if (EDRu::inarr($pkg->{pkg}, @{$cfg->{opt}{ignorepkgs}}));
    }
    $pkgpath||=$rel->pkgs_patches_dir('pkgpath', $mediapath);
    $padv=$pkg->padv;
    $pkg->{file}=$padv->media_pkg_file($pkg, $pkgpath);
    if ($pkg->{file}) {
        # common version definitions also defined at the $rel level now
        $pkg->{vers} = $padv->media_pkg_version($pkg,$pkgpath);
        EDR::die("Cannot determine version of $pkg->{padv} $pkg->{pkg}")
            if (!$pkg->{vers} && !$pkg->{batchinstall});
        $pkg->{size}=$cpic->pkg_patch_size($pkg->{file});
        $pkg->{space}=$pkg->space();
        $pkg->{mpok} = 0 if($pkg->{mpok});
    } elsif (Cfg::opt('mpok')) {
        $pkg->{mpok}=1;
    } elsif (!$rel->{verify_media}) {
        Msg::log("Cannot find $pkg->{pkg} for padv $pkg->{padv} in $pkgpath");
    } elsif (Cfg::opt('pkgpath')) {
        $pkgi =Msg::get('pdfr')." ".$pkg->{pkg};
        $msg = Msg::new("Cannot find $pkgi in $pkgpath");
        $msg->die;
    } elsif ($pkg->{mpok} || Cfg::opt('base_path')){
        Msg::log("$pkg->{pkg} is not in media\n");
    } else {
        EDR::die("Cannot find $pkg->{pkg} for padv $pkg->{padv} in $pkgpath");
    }
    return;
}
package CPIC;

sub hotfix_release { 'VRTSvcsea731P100' }
sub maintenance_requires { '7.3.1' }

sub set_pkgs {
    my $cpic=shift;
    my $prod=$cpic->prod;

    Cfg::set_opt('silent',1) if (($prod->{silent_installconfig}) && (Cfg::opt('install')));
#do not uninstall lp pkgs during patchupgrade and hotfixupgrade
    $cpic->set_uninstall_language_pkgs() if (Cfg::opt(qw(install uninstall upgrade)));
    $cpic->set_uninstall_pkgs() if (Cfg::opt(qw(install patchupgrade hotfixupgrade uninstall rollback upgrade)));
    $cpic->set_install_pkgs() if (Cfg::opt(qw(precheck install patchupgrade hotfixupgrade upgrade)));
    $cpic->analyse_and_display_pkgs();

    return '';
}

sub prod_preremove { return 1; }

sub filter_args {
    my ($cpic,$args_hash)=@_;
    my ($script,$plat,$rel,$mediapath, @unused_args);

    $script=$cpic->{script};
    $plat=Padv::plat($cpic->{padv});
    

    if ($script=~/^uninstall/) {
        if ($script =~ /^uninstallrp/ || $script =~ /^uninstallmr/) {
            $args_hash->{args_task}=[ qw(rollback) ];
        } elsif ($script =~ /^uninstall\w+720P\d+$/) {
            # this is for post-release hotfix/patch rollback only
            # like uninstall<pkg>720P\d{3}
            $args_hash->{args_task}=[ qw(rollback) ];
        } else {
            $args_hash->{args_task}=[ qw(uninstall rollback) ];
        }
        #do not show -uninstall for uninstallxxx script
        $args_hash->{uninstall}->{undocumented}=1;
        @unused_args=qw(tunablesfile pkgpath patchpath nolic tunables
                        settunables upgrade_kernelpkgs upgrade_nonkernelpkgs
                        rolling_upgrade rollingupgrade_phase1 rollingupgrade_phase2
                        installminpkgs installrecpkgs installallpkgs
                        minpkgs recpkgs allpkgs repkgs
                        pkgset pkgtable pkginfo
                        jumpstart flash_archive kickstart yumgroupxml nim ignite
                        security securityonenode securitytrust fips addnode fencing configcps
                        ignorepatchreqs hotfix_path hotfix2_path hotfix3_path hotfix4_path hotfix5_path
                        patch_path patch2_path patch3_path patch4_path patch5_path
                        base_path disable_dmp_native_support online_upgrade
                        );
        #have to delete both args_opt and args_def.
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    } elsif ($script eq 'install_lp') {
        $args_hash->{args_task}=[ qw(install) ];
        @unused_args=qw(installminpkgs installrecpkgs installallpkgs
                        minpkgs recpkgs allpkgs);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
    } elsif ($script eq 'showversion') {
        $args_hash->{args_task}=[];
        $args_hash->{args_def}=[qw(hostfile keyfile sshpath scppath rootpath)];
        $args_hash->{args_opt}=[qw(rsh version history)];

        # Change description for -hostfile
        $args_hash->{hostfile}->{description}=Msg::new("The -hostfile option specifies the location of a file containing the system names for version check");

        # Change description for -rootpath
        $args_hash->{rootpath}->{description}=Msg::new("The -rootpath option is used to specify the alternative root path for version check. On Solaris, -rootpath passes -R <root_path> to pkginfo. The -rootpath option is not supported on AIX, HPUX or Linux.");

        # Do not show -version for showversion script
        $args_hash->{version}->{undocumented}=1;
    } elsif ($script eq 'installmr') {
        if (Cfg::opt('base_path')) {
            $args_hash->{args_task}=[ qw(patchupgrade precheck install upgrade) ];
        } elsif ($plat ne 'Linux') {
            $args_hash->{args_task}=[ qw(patchupgrade precheck rollback postcheck) ];
        } else {
            $args_hash->{args_task}=[ qw(patchupgrade precheck postcheck) ];
        }
        @unused_args=qw(security securityonenode securitytrust fips addnode fencing configcps
                        installminpkgs installrecpkgs installallpkgs
                        pkginfo minpkgs recpkgs allpkgs settunables pkgpath tunablesfile tunables);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    } elsif ($script=~/^install\w+731P\d+$/) {
        # this is for post-release hotfix/patch installer only
        $args_hash->{args_task}=[ qw(hotfixupgrade precheck postcheck) ];
        @unused_args=qw(security securityonenode securitytrust fips addnode fencing configcps
                        jumpstart flash_archive kickstart yumgroupxml nim ignite
                        installminpkgs installrecpkgs installallpkgs uninstall
                        pkginfo minpkgs recpkgs allpkgs settunables pkgpath tunablesfile tunables
                        online_upgrade rolling_upgrade rollingupgrade_phase1 rollingupgrade_phase2 prod component);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    } elsif ($script=~/^install/ && $script!~/^installer/) {
        $args_hash->{install}->{undocumented}=1;
        $args_hash->{uninstall}->{undocumented}=1;
    } 

    #get the mediapath
    if (Cfg::opt('base_path')) {
        $mediapath = $cpic->{basemediapath};
    } else {
        my $edr=$Obj::pool{'EDR'};
        $mediapath = $edr->{mediapath};
    }
    #remove ru options for sfb
    if ($mediapath && (-d "$mediapath/storage_foundation_basic")) {
        @unused_args=qw(rolling_upgrade rollingupgrade_phase1 rollingupgrade_phase2);
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, @unused_args);
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, @unused_args);
    }
    # Only SunOS need -rootpath option
    if ($plat ne 'SunOS') {
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def},qw(rootpath));
    }

    # Only SunOS and HPUX need -ignorepatchreqs option
    if ($plat ne 'SunOS' && $plat ne 'HPUX') {
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt},qw(ignorepatchreqs));
    }

    # Only RP/HF releases need -listpatches/-patchpath options
    $rel=$cpic->rel;
    if (!$rel->{patches}) {
        $args_hash->{args_def}=EDRu::arrdel($args_hash->{args_def}, qw(patchpath));
        $args_hash->{args_opt}=EDRu::arrdel($args_hash->{args_opt}, qw(listpatches));
    }

    #e3528259:make patchpath ,pkgpath and hotfix*_path options hidden
    $args_hash->{patchpath}->{undocumented}=1 if ($args_hash->{patchpath});
    $args_hash->{pkgpath}->{undocumented}=1 if ($args_hash->{pkgpath});
    $args_hash->{hotfix_path}->{undocumented}=1 if ($args_hash->{hotfix_path});
    $args_hash->{hotfix2_path}->{undocumented}=1 if ($args_hash->{hotfix2_path});
    $args_hash->{hotfix3_path}->{undocumented}=1 if ($args_hash->{hotfix3_path});
    $args_hash->{hotfix4_path}->{undocumented}=1 if ($args_hash->{hotfix4_path});
    $args_hash->{hotfix5_path}->{undocumented}=1 if ($args_hash->{hotfix5_path});

    return 1;
}

package Rel;

sub self_healing {
    return 1;
}

sub postremove_sys {
    my ($rel,$sys) = @_;

    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    my $cpic=$sys->cpic;
    for my $hotfix (@{$cpic->{hotfixobj}}) {
        $hotfix->postremove_sys($sys) if ($rel->{type} =~ /H/);
    }
    return 1;
}

sub pre_platvers {
    my ($rel)=@_;
    my ($sort_padv,$matrix,$level);
    my ($msg,$common_sys,$cpic);
    $cpic=Obj::cpic();

    return 1 if (Padv::plat($rel->{padv}) ne 'Linux');
    return 1 if (Cfg::opt('ignorechecks'));
    $sort_padv=$rel->{sort_padv};
    if ($rel->{type}=~/M/) {
        $level='patch';
    } elsif ($rel->{type} eq 'H') {
        if ($rel->{currentrelease} =~ /^UXRT\d{4}/) {
            $level='patch';
        } else {
            $level='ga';
        }
    } else {
        $level='ga';
    }
    $matrix=$rel->{release_matrix}{$sort_padv}{$level};
    while ( my ($id, $val) = each %{$matrix} ) {
        if (EDRu::compvers($val->{release_version},$rel->{vers},3)==0) {
            $rel->set_value('os_update_min', $val->{os_update_min});
            $rel->set_value('os_update_max', $val->{os_update_max});
            $rel->set_value('os_update_err', $val->{os_update_err});
            while ( my ($required_patch_update, $required_patch_id) = each %{$val->{required_patch}} ) {
                $rel->set_value("os_update_patchid_mapping,$required_patch_update", $required_patch_id);
            }
            last;
        }
    }
    Msg::log("Minimum OS update level supported: $rel->{os_update_min}");
    Msg::log("Maximum OS update level supported: $rel->{os_update_max}");
    Msg::log("Unsupported OS update level: $rel->{os_update_err}") if ($rel->{os_update_err});
    
    unless (defined($rel->{os_update_min}) && defined($rel->{os_update_max})) {
        $common_sys=@{$cpic->{systems}}[0];
        $msg=Msg::new("Failed to retrieve supported $rel->{padv} update level from the release matrix, contact our technical support for further troubleshooting.");
        $common_sys->push_error($msg);
        for my $sys (@{$cpic->{systems}}) {
            $sys->set_value('stop_checks',1);
        }
    }

    return 1;
}

package Hotfix;

sub postremove_sys {
    my ($hotfix,$sys) = @_;
    my ($rel, $patch_ver, $hotfix_uninstalled);

    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    $rel=$sys->rel;
    $hotfix_uninstalled=1;
    for $patch_ver (@{$hotfix->{patches}}) {
        if (($patch_ver =~ /^\d{7}_\d{2}$/) && ($sys->{padv}=~ /Sol10/)) {
            $patch_ver =~s/_/-/;
        }
        $hotfix_uninstalled=0 if ($sys->{patchuninstallfail} &&  EDRu::inarr($patch_ver,@{$sys->{patchuninstallfail}}));
    }
    return unless ($hotfix_uninstalled);
    $hotfix->hotfix_copy_cpi_sys($sys) if ($rel->{type} =~ /H/);

    return 1;
}

sub postinstall_sys {
    my ($hotfix,$sys) = @_;
    my ($rel, $patch_ver, $hotfix_installed,$hotfix_installed_partial);

    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');

    $rel=$sys->rel;
    $hotfix_installed=1;
    for $patch_ver (@{$hotfix->{patches}}) {
        if (($patch_ver =~ /^\d{7}_\d{2}$/) && ($sys->{padv}=~ /Sol10/)) {
            $patch_ver =~s/_/-/;
        }
        # If one included patch failed install, we take it as
        # not install for the hotfix as a whole
        $hotfix_installed=0 unless ($sys->{patchinstallpass} && EDRu::inarr($patch_ver,@{$sys->{patchinstallpass}}));
        $hotfix_installed_partial=1 if ($sys->{patchinstallpass} && EDRu::inarr($patch_ver,@{$sys->{patchinstallpass}}));
    }
    return unless ($hotfix_installed || $hotfix_installed_partial);
    $hotfix->hotfix_copy_cpi_sys($sys) if ($rel->{type} =~ /H/);

    return 1;
}

# hotfix-specific modules/rollback script changes
# for hotfixupgrade/rollback
# it's a mimic of $cpic->copy_cpi_sys, limited to HF modules/scripts
sub hotfix_copy_cpi_sys {
    my ($hotfix,$sys) = @_;
    my ($cpic,$rootpath,$rel,$bmcdomaindir,$installscript_prod,$installscript_name,$cpidir,$installscript,$prod,$localsys,$msg);
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $cpic=Obj::cpic();
    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($cpic->{prod} && $sys->prod->{nocpionsys});
    $localsys=$cpic->localsys;
    $rel=$sys->rel;

    $rootpath=Cfg::opt('rootpath');
    $cpidir=$rootpath.$cpic->{perlpath}.'/lib/site_perl/'.$cpic->{release};

    # copy CPI libraries
    if (($rel->{type} eq 'H') ||
        (($rel->{type} =~ /H/) && Cfg::opt(qw(patch_path patch2_path patch3_path patch4_path patch5_path)))) {
        # hotfixupgrade or 'installer -hotfix_path'
        $bmcdomaindir=$cpic->{mediapath} .'/scripts';
        $bmcdomaindir=Cfg::opt($hotfix->{arg}).'/scripts' if (defined Cfg::opt($hotfix->{arg}) && Cfg::opt($hotfix->{arg}));
        $msg = Msg::new("Copy modules for $rel->{hotfixname} to $cpidir/CPIC");
        $msg->log;
        $sys->mkdir("$cpidir/CPIC");    # okay if it already exists
        $localsys->copy_to_sys($sys, "$bmcdomaindir/*.pm", "$cpidir/CPIC");
        unless ($sys->linux() || $sys->{padv} =~ /Sol11/) {
            $installscript_prod = '';
            $installscript_name = $hotfix->{hotfix};
            $installscript = "$rootpath/opt/VRTS/install/uninstall" . $installscript_name;
            $hotfix->create_task_script_sys($sys,'rollback',$installscript_prod,$installscript);
        }
    } elsif (($rel->{type} =~ /H/) && Cfg::opt("rollback")) {
        # rollback has $rel->{type} as "BH" ;
        $installscript = "$cpidir/CPIC/". $rel->{hotfixname} .".pm";
        if ($sys->exists($installscript)) {
            $sys->cmd("_cmd_rmr $installscript");
            $msg = Msg::new("Removing $rel->{hotfixname} module $installscript");
            $msg->log;
        }
        unless ($sys->linux() || $sys->{padv} =~ /Sol11/) {
            $installscript = "$rootpath/opt/VRTS/install/uninstall" . $hotfix->{hotfix};
            $sys->cmd("_cmd_rmr $installscript");
            $msg = Msg::new("Removing $rel->{hotfixname} rollback script $installscript");
            $msg->log;
        }
    }

    $sys->chown_root("$rootpath$cpic->{installpath}");
    return 1;
}

sub create_task_script_sys {
    my ($hotfix,$sys,$task,$prodname,$filename) = @_;
    my ($script,$cpic,$rel,$edr,$padvs);
    $rel=$sys->rel;
    $edr=Obj::edr();
    $cpic=Obj::cpic();
    $padvs=join(' ',@{$edr->{padvs}});
    $script=<<'SCRIPT';
#!/bin/sh

#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $

#
PDIR=/opt/VRTSperl
SDIR=/opt/VRTSperl/lib/site_perl/RELEASE
PERLINFO=`$PDIR/bin/perl -v 2>/dev/null`
PERLVERS=`echo "$PERLINFO" | grep This.is | sed 's/.*v\([0-9.]*\).*/\1/'`
PERLFOR=`echo "$PERLINFO"  | grep This.is | sed 's/.*for \([a-zA-Z0-9._-]*\).*/\1/'`
IOPT="-I$PDIR/lib/$PERLVERS/$PERLFOR -I$PDIR/lib/$PERLVERS -I$PDIR/lib/site_perl/$PERLVERS/$PERLFOR -I$PDIR/lib/site_perl/$PERLVERS -I$PDIR/lib/site_perl"
IOPT="-I$SDIR/EDR -I$SDIR/CPIC -I$SDIR/CPIP -I$SDIR/Deploy $IOPT"

if [ -z "$PDIR" ]; then
        echo "Cannot find perl to execute $0"
        exit 1
fi

eval 'exec $PDIR/bin/perl -x -S $IOPT $0 ${1+"$@"}'
        if 0;
#!$PDIR/bin/perl

use RELEASE;
use PATCHNAME;

CPIC::main( "RELEASE",
            "PADV",
            "",
            "TASK",
            "PROD");
SCRIPT

    $script=~s/RELEASE/$cpic->{release}/mg;
    if ($rel->{type} =~ /H/) {
        # to cover both hotfixupgrade and Base+hotfix_path
        $script=~s/PATCHNAME/$hotfix->{hotfix}/mg;
    } else {
        $script=~s/'use PATCHNAME;'//mg;
    }
    $script=~s/PADV/$padvs/mg;
    $script=~s/TASK/$task/mg;
    $script=~s/PROD/$prodname/mg;

    $sys->cmd("_cmd_rmr $filename");
    $sys->writefile($script, $filename);
    $sys->chmod($filename, '+x');
    return '' if(EDR::cmdexit());
    return 1;
}

# we need the mpok setting for aslapm for HF installer
package Pkg::VRTSaslapm731::Common;
@Pkg::VRTSaslapm731::Common::ISA = qw(Pkg);

sub init_common {
    my $pkg=shift;
    $pkg->{pkg}='VRTSaslapm';
    $pkg->{name}=Msg::new("Volume Manager - ASL/APM")->{msg};
    $pkg->{pkgonly}=1;
    $pkg->{mpok}=1;
    return;
}

package Prod::SFCFSHA731::Common;
@Prod::SFCFSHA731::Common::ISA = qw(Prod);

# this is to make sure all VCS modules/daemons are covered during postremove configure
sub rollback_precheck_sys {
    my ($prod, $sys) = @_;

    $prod->perform_task_sys($sys,"rollback_precheck_sys");
    return;
}

package Prod::SFHA731::Common;
@Prod::SFHA731::Common::ISA = qw(Prod);

sub rollback_precheck_sys { my ($prod, $sys) = @_; $prod->perform_task_sys($sys,"rollback_precheck_sys"); }

package Prod::SFRAC731::Common;
@Prod::SFRAC731::Common::ISA = qw(Prod);

# this is to make sure all VCS modules/daemons are covered during postremove configure
sub rollback_precheck_sys {
    my ($prod, $sys) = @_;

    $prod->perform_task_sys($sys,"rollback_precheck_sys");
    return;
}

package Prod::VCS731::Common;
@Prod::VCS731::Common::ISA = qw(Prod);

sub hotfixupgrade_precheck_sys {
    my ($prod, $sys) = @_;
    return $prod->patchupgrade_precheck_sys ($sys);
}

# this is to make sure all VCS modules/daemons are covered during postremove configure
sub rollback_precheck_sys {
    my ($prod, $sys) = @_;

    # set $cfg->{vcs_allowcomms}
    $prod->set_vcs_allowcomms_sys($sys) if ($sys->system1);
    return;
}

sub upgrade_postinstall_sys {
    my ($prod,$sys) = @_;
    my ($filename,$old,$rootpath,$tmpdir,$filecontent,$value,$initfile_content);

    $prod->add_namespace_vcs_sys($sys);
    $prod->configure_sso_sys($sys);
    $sys->cmd("/opt/VRTSvcs/portal/vcs_wizards/admin/synchronize_guest_config.pl CreateAppSGMap=Yes 2>/dev/null");

    $tmpdir=EDR::tmpdir();
    $rootpath = Cfg::opt('rootpath');

    for my $file (@{$prod->{upgrade_wizard_backup}}) {
        $filename = EDRu::basename($file);
        $old = "$tmpdir/VCS-CFG-BAK/$filename";
        $file = $rootpath.$file if ($rootpath);
        next unless ($sys->exists("$old"));
        $sys->cmd("_cmd_cp -pf $old $file");
    }
    $sys->cmd("_cmd_rmr $tmpdir/VCS-CFG-BAK");
    $prod->register_extra_types() if($sys->system1 && !Cfg::opt("hotfixupgrade")); 
    $prod->restore_vcs_lock_files_sys($sys) if ($prod->can('restore_vcs_lock_files_sys'));
    $prod->move_NFS_triggers_sys($sys) if ($prod->can('move_NFS_triggers_sys'));

    # restore init files for llt/gab/vxfen/vcs/amf
    if ( ! $rootpath ) {
        $rootpath = '';
    }
    $initfile_content = {};
    if ($sys->{initfile_content}) {
        $initfile_content = JSON::from_json($sys->{initfile_content});
    }
    for my $initfile (keys %{$initfile_content}) {
        Msg::log('Restoring '.$rootpath.$prod->{initfile}{$initfile});
        if ( $sys->exists($rootpath.$prod->{initfile}{$initfile})) {
            Msg::log('Reading '.$rootpath.$prod->{initfile}{$initfile});
            $filecontent = $sys->readfile($rootpath.$prod->{initfile}{$initfile});
        } else {
            Msg::log('Creating '.$rootpath.$prod->{initfile}{$initfile});
            $filecontent = '';
        }
        for my $key (sort keys %{$initfile_content->{$initfile}}) {
            $value = $initfile_content->{$initfile}{$key};
            Msg::log('Adding '."$key=$value".' into '.$rootpath.$prod->{initfile}{$initfile});
            if (!($filecontent =~ s/^\s*$key=.*$/$key=$value/mxg)) {
                Msg::log('Appending '."$key=$value".' into '.$rootpath.$prod->{initfile}{$initfile});
                $filecontent .= "\n$key=$value";
            }
        }
        $filecontent .= "\n";
        $sys->writefile($filecontent, $rootpath.$prod->{initfile}{$initfile});
    }

    # remove obsolete cps key and apply new cps key after upgrade if existed.
    $prod->replace_obsolete_cps_key_sys($sys);

    return;
}

sub upgrade_postremove_sys { my ($prod,$sys)=@_; return $prod->perform_task_sys($sys, 'upgrade_postremove_sys'); }

package Hotfix::VRTSvcsea731P100::Common;
@Hotfix::VRTSvcsea731P100::Common::ISA = qw(Hotfix);

sub init_common {
    my ($hotfix) = @_;
    my $rel=$hotfix->rel;
    $rel->{currentrelease}="UXRT731";
    $rel->{patches}=1;
    $rel->{type}.='H' if ($rel->{type}!~/H/);
    
    if ((CPIC::get('task')) eq 'rollback') {
        # this line is for module/script backout in postremove_sys
        $rel->{hotfixname}="VRTSvcsea731P100";
        # without the following line, we need to change rel->cli_installer_menu
        # this reduces the size of this hotfix.template
        delete $rel->{pkgs};
    }
    return if ((CPIC::get('task')) ne 'hotfixupgrade');
    $rel->{type}='H';
    $hotfix->init_release;
}

# must define release values when hotfix is run independently
# use defined values when hotfix is bundled using -maintenance_path or -hotfix_path
sub init_release {
    my ($hotfix) = @_;
    my $rel=$hotfix->rel;
    # figure out how to handle hotfix only bundles later
    return if ($rel->{hotfixname});
    $rel->{hotfixname}="VRTSvcsea731P100";
    $rel->{vers}='7.3.1.100';
    $rel->{basevers}=[ "7.3.1" ];
    $rel->{baseversions}=[ "7.3.1" ];
    $rel->{gavers}='7.3.1';
    $rel->{titlevers}='VRTSvcsea731P100';
    $rel->{args_task}=[ qw(hotfixupgrade) ];
    delete $rel->{pkgs};
}

package Padv::Linux;

# determine whether an RPM installation has succeeded
# return string signifying success if passed, null if error
sub pkg_install_success_sys {
    my ($padv,$sys,$pkg)=@_;
    my ($iof,$iv,$pf,$pobj);

    # can't see anything yet, verbose isn't verbose, exit code is rsh?
    $pobj=(ref($pkg)=~/^Patch/m) ? "$pkg->{patchname}" :
          (ref($pkg)=~/^Pkg/m) ? $pkg->{pkg} : $pkg;
    $pf=(ref($pkg)=~/^Patch/m) ? "patch.$pkg->{patch_vers}" : $pobj;
    $iof=EDRu::readfile(EDRu::outputfile('install', $sys->{sys}, $pf));
    $iv=$padv->pkg_version_sys($sys, $pkg);
    if (!$iv) {
        Msg::log("$pobj is not installed on $sys->{sys}\n$iof");
        return 0;
    }
    if ($iv ne $pkg->{vers}) {
        Msg::log("$pobj version on $sys->{sys} $iv is not $pkg->{vers}\n$iof");
        return 0;
    }

    #Don't check error message if ignore_log_error set on $pkg
    #add condition ref($pkg) =~ /^Pkg/m in hotfix template to fix ET3437237, only ignore error on pkg install
    if ((ref($pkg) =~ /^Pkg/m) && ($pkg->{ignore_log_error})) {
        Msg::log("Skip error message checking for $pobj package installation on $sys->{sys}");
        return 1;
    }
    return 0 if ($iof=~/error:/mi);
    # check if reboot is required
    if(!$pkg->{donotcheckinstalllog} && $iof =~ /\breboot\b/mi && !EDRu::inarr($pkg->{pkg}, @{$sys->{requirerebootpkgs}})) {
        $sys->set_value('requirerebootpkgs', 'push', $pkg->{pkg});
    }
    return 1;
}

package Hotfix::VRTSvcsea731P100::AIX;
@Hotfix::VRTSvcsea731P100::AIX::ISA = qw(Hotfix::VRTSvcsea731P100::Common);
sub define_patches {
    my $hotfix=shift;
    $hotfix->define_hf_patches("VRTSvcsea731", "all", "UXRT731", "VRTSvcsea_7_3_1_100");
}

package Hotfix::VRTSvcsea731P100::OL7x8664;
@Hotfix::VRTSvcsea731P100::OL7x8664::ISA = qw(Hotfix::VRTSvcsea731P100::Common);
sub define_patches {
}

package Hotfix::VRTSvcsea731P100::OL6x8664;
@Hotfix::VRTSvcsea731P100::OL6x8664::ISA = qw(Hotfix::VRTSvcsea731P100::Common);
sub define_patches {
}

package Prod::FS731::Common;
@Prod::FS731::Common::ISA = qw(Prod);

sub hotfixupgrade_precheck_sys {    
    my ($prod,$sys) = @_;
    $prod->check_vxfs_sys($sys,'upgrade') unless(Cfg::opt('upgrade_nonkernelpkgs'));
    return '';
}

package Prod::VM731::Common;
@Prod::VM731::Common::ISA = qw(Prod);
sub hotfixupgrade_precheck_sys {
    my ($prod,$sys)= @_;
	my ($rel);
	$rel = $sys->rel;
    if ($prod->is_encapsulated_bootdisk_sys($sys)) {
        return '' unless ($prod->encap_upgrade_precheck_sys($sys));
        $prod->set_flag_encap_upgrade_sys($sys);
    }    
    $prod->cache_areas_sys($sys);
    $prod->open_volumes_sys($sys, 'upgrade') unless (Cfg::opt(qw(upgrade_nonkernelpkgs)));
    $prod->vvr_upgrade_precheck_sys($sys) if ($rel->vr_licensed_sys($sys));
    return;
}

package Rel::UXRT731::RHEL7x8664;
@Rel::UXRT731::RHEL7x8664::ISA = qw(Rel::UXRT731::Linux);

sub init_padv {
    my ($rel) = @_;

    $rel->{upgradeprods}=EDRu::arrdel($rel->{upgradeprods}, qw(SVS731 SFSYBASECE731));
    $rel->{prods}=EDRu::arrdel($rel->{prods},qw(SFSYBASECE731));
    $rel->{pkgsetprods}=EDRu::arrdel($rel->{pkgsetprods}, qw(SFSYBASECE731));
    $rel->{menuprods}=EDRu::arrdel($rel->{menuprods}, qw(SFSYBASECE731));
    $rel->{platreqs} = ['RHEL7.1 (3.10.0-229.el7)','RHEL7.2 (3.10.0-327.el7)',
        'OL7.1 (3.10.0-229.el7)','OL7.2 (3.10.0-327.el7)',
        'OL7.1 (3.8.13-55.1.6.el7uek)','OL7.2 (3.8.13-98.7.1.el7uek)'
            ];
    $rel->{minorvers_for_fallback}='RHEL7.3';
    $rel->{oslibs}{all}={
        "/usr/bin/netstat"  =>  "net-tools.x86_64",
    };
    $rel->{commands_need_be_checked}=[ qw(arch ethtool getconf hostid modprobe modunload nm) ];

    $rel->{pkgsspace}=<<"_PKGSPACE_";
VRTSglm:75,0,0,742
VRTSveki:0,0,0,153
VRTSgab:21400,735,0,79
VRTSdbed:282383,0,0,4
VRTSvlic:88,0,0,1614
VRTSvxfs:4076,26233,0,16170
VRTSvxfen:21050,114,0,825
VRTSllt:46872,80,0,383
VRTSdbac:13695,38,0,381
VRTSspt:41747,0,0,0
VRTScps:63222,0,0,4
VRTSsfcpi:15555,0,0,0
VRTSgms:6,0,0,76
VRTScavf:846,95,0,0
VRTSsfmh:178728,0,0,0
VRTSvcsea:1380,0,0,61
VRTSvxvm:302215,75267,0,63380
VRTSvcswiz:11441,0,0,0
VRTSperl:77633,0,0,0
VRTSodm:185,449,0,603
VRTSamf:18957,457,0,17
VRTSaslapm:28205,0,0,1576
VRTSvbs:75953,0,0,0
VRTSvcsag:90799,0,0,72
VRTSfsadv:16183,0,0,8
VRTSvcs:217438,16370,0,281
_PKGSPACE_
return;
}


package Rel::UXRT731::OL7x8664;
@Rel::UXRT731::OL7x8664::ISA = qw(Rel::UXRT731::Linux);

sub init_padv {
    my ($rel) = @_;

    $rel->{prods} = [ qw(VCS731) ];
    $rel->{upgradeprods} = [ qw(VCS731) ];
    $rel->{pkgsetprods}= [ qw(VCS731) ];
    $rel->{menuprods}= [ qw(VCS731) ];
    $rel->{platreqs} = [ 'OL7 (3.8.13-35.3.1.el7uek)' ];
    $rel->{pkgs}=1;

    # If install on RHEL 7.2, fallback to check OS libraries of RHEL 7.1
    $rel->{minorvers_for_fallback}='RHEL7.1';
    $rel->{oslibs}{all}={
        "/usr/bin/netstat"  =>  "net-tools.x86_64",
    };

    $rel->{commands_need_be_checked}=[ qw(arch ethtool getconf hostid modprobe modunload nm) ];

    $rel->{pkgsspace}=<<"_PKGSPACE_";
VRTSglm:75,0,0,742
VRTSveki:0,0,0,153
VRTSgab:21400,735,0,79
VRTSdbed:282383,0,0,4
VRTSvlic:88,0,0,1614
VRTSvxfs:4076,26233,0,16170
VRTSvxfen:21050,114,0,825
VRTSllt:46872,80,0,383
VRTSdbac:13695,38,0,381
VRTSspt:41747,0,0,0
VRTScps:63222,0,0,4
VRTSsfcpi:15555,0,0,0
VRTSgms:6,0,0,76
VRTScavf:846,95,0,0
VRTSsfmh:178728,0,0,0
VRTSvcsea:1380,0,0,61
VRTSvxvm:302215,75267,0,63380
VRTSvcswiz:11441,0,0,0
VRTSperl:77633,0,0,0
VRTSodm:185,449,0,603
VRTSamf:18957,457,0,17
VRTSaslapm:28205,0,0,1576
VRTSvbs:75953,0,0,0
VRTSvcsag:90799,0,0,72
VRTSfsadv:16183,0,0,8
VRTSvcs:217438,16370,0,281
_PKGSPACE_
    return;
}

package CPIC;

sub prod_prerequisite_oslibs_sys {
    my ($cpic,$sys) = @_;
    my ($rel,$prod,$pkg,$padv,$pdfrs,$minorvers);
    my ($ospkg,$ospkgs,$msg,$rtn,$rtns,%all_oslibs,$oslibs,$uniq_oslibs,@oslibs,@missing_oslibs,@missing_oslibpkgs);

    $rel=$sys->rel;
    $prod=$sys->prod;
    $padv=$sys->padv;

    $pdfrs=Msg::get('pdfrs');
    $minorvers=$sys->{minorvers};

    %all_oslibs=();
    for my $pkgi (@{$prod->get_pkgs}) {
        $pkg=$sys->pkg($pkgi);

        $oslibs=$pkg->{oslibs}{all};
        @all_oslibs{ keys %{$oslibs} } = values %{$oslibs} if (defined $oslibs);

        $oslibs=$pkg->{oslibs}{$sys->{platvers}};
        @all_oslibs{ keys %{$oslibs} } = values %{$oslibs} if (defined $oslibs);

        if ($minorvers) {
            if (defined $pkg->{oslibs}{$minorvers}) {
                $oslibs=$pkg->{oslibs}{$minorvers};
                @all_oslibs{ keys %{$oslibs} } = values %{$oslibs};
            } 
        }
    }

    # Add release level information
    if (defined $rel) {
        $oslibs=$rel->{oslibs}{all};
        @all_oslibs{ keys %{$oslibs} } = values %{$oslibs} if (defined $oslibs);

        $oslibs=$rel->{oslibs}{$sys->{platvers}};
        @all_oslibs{ keys %{$oslibs} } = values %{$oslibs} if (defined $oslibs);

        if ($minorvers) {
            if (defined $rel->{oslibs}{$minorvers}) {
                $oslibs=$rel->{oslibs}{$minorvers};
                @all_oslibs{ keys %{$oslibs} } = values %{$oslibs};
            }
        }
    }

    @oslibs= keys %all_oslibs;
    return 1 unless (@oslibs);

    # To check library dependencies (mainly for Linux)
    @missing_oslibs=();

    $uniq_oslibs=EDRu::arruniq(@oslibs);
    if ($padv->can('oslibraries_sys')) {
        $rtns=$padv->oslibraries_sys($sys,@{$uniq_oslibs});
        for my $i (0..$#$rtns) {
            push(@missing_oslibs,$uniq_oslibs->[$i]) unless ($rtns->[$i]);
        }
    } elsif ($padv->can('oslibrary_sys')) {
        for my $oslib (@{$uniq_oslibs}) {
            $rtn=$padv->oslibrary_sys($sys,$oslib);
            push(@missing_oslibs,$oslib) unless ($rtn);
        }
    }

    if (@missing_oslibs) {
        @missing_oslibpkgs=();

        for my $oslib (@missing_oslibs) {
            $ospkg=$all_oslibs{$oslib};
            ($ospkg) = split(/\s+/,$ospkg);
            push (@missing_oslibpkgs,$ospkg) unless EDRu::inarr($ospkg,@missing_oslibpkgs);
        }

        if (@missing_oslibpkgs) {
            $sys->set_value('missing_ospkgs_for_product', 'push', @missing_oslibpkgs);
            $ospkgs=join(' ',@missing_oslibpkgs);
            $msg=Msg::new("The following required OS $pdfrs were not found on $sys->{sys}:\n\t$ospkgs");
        } else {
            $oslibs=join(' ',@missing_oslibs);
            $msg=Msg::new("The following required OS libraries were not found on $sys->{sys}:\n\t$oslibs");
		}
        $sys->push_error($msg);
        return 0;
    }

    return 1;
}

# To install missing OS packages
sub prod_prerequisite_error_handler {
    my $cpic=shift;
    my ($ret,$sys0,$native_method,$pdfrs,@missing_ospkgs_systems,$pkgs,$rtn,$msg,$webmsg);

    $sys0=$cpic->{systems}->[0];
    $native_method=$sys0->padv->{native_install_tool};

    # need not automatically install the missing OS packages if it is not supported.
    return 1 if (!$native_method);

    $pdfrs=Msg::get('pdfrs');
    @missing_ospkgs_systems =(); 
    $webmsg='';

    for my $sys(@{CPIC::get('systems')}) {
        if (defined $sys->{missing_thirdpartypkgs_for_product}) {
            # install third party packages
            Msg::n();
            $pkgs=join ' ', @{$sys->{missing_thirdpartypkgs_for_product}};
            $msg=Msg::new("The following required third party $pdfrs (or higher version) were not found on $sys->{sys}:\n\t$pkgs");
            $msg->print;
            $ret = $cpic->prod->install_thirdparty_pkgs_sys($sys,$sys->{missing_thirdpartypkgs_for_product});
            $rtn ||= $ret;
        }    
        if (defined $sys->{missing_ospkgs_for_product}) {
            # get if any systems is missing OS required packages
            Msg::n();
            $pkgs=join ' ', @{$sys->{missing_ospkgs_for_product}};
            $msg=Msg::new("The following required OS $pdfrs were not found on $sys->{sys}:\n\t$pkgs");
            $msg->print;
            $webmsg .=$msg->{msg}."\\n";
            push @missing_ospkgs_systems, $sys;
        }    
    }    

    if (Obj::webui()) {
        my $web = Obj::web();
        $web->web_script_form('alert',$webmsg) if (@missing_ospkgs_systems);
    }    

    # Try to install the missing OS packages with native install method
    $rtn=$cpic->install_missing_os_pkgs(\@missing_ospkgs_systems) if (@missing_ospkgs_systems);
    if ($rtn) {
        $cpic->{retry_verify_checks}=1;
        EDR::reset_errors_warnings();
    }    

    return 1;
}

package Pkg::VRTSvxvm731::RHEL7x8664;
@Pkg::VRTSvxvm731::RHEL7x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

sub preinstall_sys {
    my ($pkg,$sys)=@_;
    $sys->cmd("_cmd_rmr $pkg->{vminstall} 2>/dev/null");
    $sys->cmd("_cmd_touch $pkg->{vminstall} 2>/dev/null") if ($sys->{encap});

    if (Cfg::opt('patch_path') && ($sys->{updatelevel} > 7.2)) {
        $pkg->{ignore_log_error} = 1;
    }

    return;
}

package Pkg::VRTSvxfs731::RHEL7x8664;
@Pkg::VRTSvxfs731::RHEL7x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);
sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
        "coreutils"  =>  "coreutils.x86_64",
        "ed"  =>  "ed.x86_64",
        "findutils"  =>  "findutils.x86_64",
        "libc.so.6()(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.14)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3.3)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3.4)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.4)(64bit)"  =>  "glibc.x86_64",
        "libdl.so.2()(64bit)"  =>  "glibc.x86_64",
        "libdl.so.2(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libgcc_s.so.1()(64bit)"  =>  "libgcc.x86_64",
        "libgcc_s.so.1(GCC_3.0)(64bit)"  =>  "libgcc.x86_64",
        "libgcc_s.so.1(GCC_3.3.1)(64bit)"  =>  "libgcc.x86_64",
        "libm.so.6()(64bit)"  =>  "glibc.x86_64",
        "libm.so.6(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libncurses.so.5()(64bit)"  =>  "ncurses-libs.x86_64",
        "libnsl.so.1()(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0()(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.3.2)(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.3.3)(64bit)"  =>  "glibc.x86_64",
        "librt.so.1()(64bit)"  =>  "glibc.x86_64",
        "librt.so.1(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libtinfo.so.5()(64bit)"  =>  "ncurses-libs.x86_64",
        "policycoreutils"  =>  "policycoreutils.x86_64",
    };
    return;
}
sub preinstall_sys {
    my ($pkg, $sys) = @_;

    if (Cfg::opt('patch_path') && ($sys->{updatelevel} > 7.2)) {
        $pkg->{ignore_log_error} = 1;
    }
    
    return;
}

package Pkg::VRTSglm731::SLES12x8664;
@Pkg::VRTSglm731::SLES12x8664::ISA = qw(Pkg::VRTSglm731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
        "libc.so.6()(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.7)(64bit)"  =>  "glibc.x86_64",
    };
    return;
}

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        $pkg->{ignore_log_error} = 1;
    }
    return;
}

package Pkg::VRTSaslapm731::SLES12x8664;
@Pkg::VRTSaslapm731::SLES12x8664::ISA = qw(Pkg::VRTSaslapm731::Linux);

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        $pkg->{ignore_log_error} = 1;
    }
    return;
}

package Pkg::VRTScavf731::SLES12x8664;
@Pkg::VRTScavf731::SLES12x8664::ISA = qw(Pkg::VRTScavf731::Linux);

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        $pkg->{ignore_log_error} = 1;
    }
    return;
}

package Pkg::VRTSveki731::SLES12x8664;
@Pkg::VRTSveki731::SLES12x8664::ISA = qw(Pkg::VRTSveki731::Linux);

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        $pkg->{ignore_log_error} = 1;
    }
    return;
}

package Pkg::VRTSvxvm731::SLES12x8664;
@Pkg::VRTSvxvm731::SLES12x8664::ISA = qw(Pkg::VRTSvxvm731::Linux);

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();
    $pkg->SUPER::preinstall_sys($sys);

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        #skip both preinstall and postinstall scripts
        $pkg->{iopt}=" --noscripts";
    }
    return;
}

package Pkg::VRTSvxfs731::SLES12x8664;
@Pkg::VRTSvxfs731::SLES12x8664::ISA = qw(Pkg::VRTSvxfs731::Linux);

sub init_padv {
    my $pkg=shift;
    $pkg->{oslibs}{all}={
        "coreutils"  =>  "coreutils.x86_64",
        "ed"  =>  "ed.x86_64",
        "findutils"  =>  "findutils.x86_64",
        "libc.so.6()(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.14)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3.3)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3.4)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.3)(64bit)"  =>  "glibc.x86_64",
        "libc.so.6(GLIBC_2.4)(64bit)"  =>  "glibc.x86_64",
        "libdl.so.2()(64bit)"  =>  "glibc.x86_64",
        "libdl.so.2(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libm.so.6()(64bit)"  =>  "glibc.x86_64",
        "libm.so.6(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libnsl.so.1()(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0()(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.3.2)(64bit)"  =>  "glibc.x86_64",
        "libpthread.so.0(GLIBC_2.3.3)(64bit)"  =>  "glibc.x86_64",
        "librt.so.1()(64bit)"  =>  "glibc.x86_64",
        "librt.so.1(GLIBC_2.2.5)(64bit)"  =>  "glibc.x86_64",
        "modutils"  =>  "kmod-compat.x86_64",
    };
    return;
}

sub preinstall_sys {
    my ($pkg, $sys) = @_;
    my $cpic = Obj::cpic();

    if (Cfg::opt('patch_path') && ($sys->{patchlevel} >= 2)) {
        $pkg->{ignore_log_error} = 1;
    }
    return;
}
1;
