#
# $Copyright: Copyright (c) 2018 Veritas Technologies LLC. All rights reserved.
# 
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF VERITAS
# TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED WITHOUT THE
# PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
# 
# The Licensed Software  and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software  -  Restricted Rights"
# and DFARS 227.7202,  Rights  in  "Commercial Computer Software or  Commercial
# Computer Software Documentation" as applicable, and any successor regulations,
# whether  delivered  by  Veritas  as  on premises or hosted services.  Any use,
# modification, reproduction release, performance, display or disclosure of the
# Licensed Software and Documentation by the U.S. Government shall be solely in
# accordance with the terms of this Agreement. $
#
use strict;

package Deploy;
use filetest 'access';
no if $] >= 5.018, warnings => 'experimental::smartmatch';

@Deploy::ISA = qw(CPIP);

sub set_deploy_sfha_menu_msgs {
    my ($deploy)=@_;
    my ($edr,$msg);

    # menu task selection
    $edr=Obj::edr();
    $edr->{msg}{key_manage_repository}='R';
    $edr->{msg}{key_version_check}='V';
    $edr->{msg}{key_install_upgrade}='I';
    $edr->{msg}{key_update_metadata}='M';
    $edr->{msg}{key_set_preferences}='S';
    $edr->{msg}{key_terminology}='U';
    $edr->{msg}{key_define_install_bundle}='B';
    $edr->{msg}{key_create_install_templates}='T';
    $edr->{msg}{key_deploy_sfha_help}='?';
    $edr->{msg}{key_quit}='Q';

    # set preference key
    $edr->{msg}{key_back}='B';

    # Update Menu
    $msg=Msg::new("Manage Repository Images");
    $msg->msg('menu_manage_repository');
    $msg=Msg::new("Version Check Systems");
    $msg->msg('menu_version_check');
    $msg=Msg::new("Install/Upgrade Systems");
    $msg->msg('menu_install_upgrade');
    $msg=Msg::new("Define/Modify Install Bundles");
    $msg->msg('menu_define_install_bundle');
    $msg=Msg::new("Create Install Templates");
    $msg->msg("menu_create_install_templates");
    $msg=Msg::new("Update Metadata");
    $msg->msg('menu_update_metadata');
    $msg=Msg::new("Set Preferences");
    $msg->msg('menu_set_preferences');
    $msg=Msg::new("Terminology and Usage");
    $msg->msg('menu_terminology');
    $msg=Msg::new("Help");
    $msg->msg('menu_deploy_sfha_help');
    $msg=Msg::new("Quit");
    $msg->msg('menu_quit');
    $msg=Msg::new("deploy_sfha script");
    $msg->msg('deploy_sfha_script');

    # set preference message
    $msg=Msg::new("Back to Main Menu");
    $msg->msg('menu_back');
    return 1;
}

sub cli_deploy_sfha_menu {
    my $deploy = shift;
    my ($edr,@tasks,$cpic,$task_k);
    $edr = Obj::edr();
    $cpic=Obj::cpic();
    $edr->set_padv_msgs;
    @tasks=qw(manage_repository update_metadata version_check
              set_preferences install_upgrade terminology define_install_bundle deploy_sfha_help create_install_templates quit);
    $deploy->set_deploy_sfha_menu_msgs();

    $deploy->create_deploy_sfha_pid_file($edr->{tmpdir},$edr->{script});
    return $deploy->update_metadata() if (Cfg::opt(qw(load_metadata download_metadata)));
    while(1) {
        $deploy->read_deploy_config();
        $deploy->cleanup_stale_deploy_sfha_locks();
        Msg::title();
        $task_k=$deploy->cli_select_task_menu(@tasks);
        EDR::set_value('savelog', 1);
        EDR::set_value('not_display_summary_file', 1);
        if ($task_k eq Msg::get('key_manage_repository')) {
            $deploy->manage_repository();
        } elsif ($task_k eq Msg::get('key_version_check')) {
            $cpic->version_check("deploy_sfha");
        } elsif ($task_k eq Msg::get('key_install_upgrade')) {
            $deploy->upgrade_install();
        } elsif ($task_k eq Msg::get('key_define_install_bundle')) {
            $deploy->define_install_bundle();
        } elsif ($task_k eq Msg::get('key_create_install_templates')) {
            $deploy->install_templates();
        } elsif ($task_k eq Msg::get('key_update_metadata')) {
            $deploy->update_metadata();
        } elsif ($task_k eq Msg::get('key_set_preferences')) {
            $deploy->set_preferences();
        } elsif ($task_k eq Msg::get('key_terminology')) {
            $deploy->terminology();
        } elsif ($task_k eq Msg::get('key_deploy_sfha_help')) {
            $deploy->deploy_sfha_help();
        }
        # quit exits via Msg::menu();
    }
    return 1;
}

sub cli_select_task_menu {
    my ($deploy,@tasks)=(@_);
    my(@mk,@mo,$buf,$be,$bs,$cpic,$edr,$m,$mkey,$mm1,$mm2,$msg,$task_k,$tsl);
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    @mo=@tasks;
    $bs=EDR::get2('tput','bs');
    $be=EDR::get2('tput','be');

    $msg=Msg::new("\nTask Menu:\n");
    $msg->bold;
    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$bs$mkey$be) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$bs$mkey$be) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-50s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task_k,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl");
        $task_k=$msg->ask;
        $task_k=uc($task_k);
    }
    return $task_k;
}

# _START_Manage_Repository
sub init_repository {
    my ($deploy)=@_;
    my ($part1,$part2,$part3,$part4);
    my ($msg1,$msg2,$msg3);

    # should eventually compute this based on our lowest supported version which is currently 5.1
    $deploy->{repo}{padvs} = $deploy->{preference}{Platform}{value}||$deploy->{preference}{Platform}{default};
    $deploy->{repo}{levels} = [qw(ga patch hotfix)];
    $msg1=Msg::new("Base");
    $msg2=Msg::new("Maintenance");
    $msg3=Msg::new("Patch");
    $deploy->{repo}{leveltitles} = [$msg1->{msg},$msg2->{msg},$msg3->{msg}];
    $deploy->{repo}{details_to_view}= [qw(release_version release_name release_type file_name release_date downloaded autoinstall install_path upload_location repository_location readme)];
    $deploy->{repo}{details_to_view_obsolete} = [qw(obsoletes obsoleted_by)];
    $part1 = ['release_version',[qw(release_name SORT_release_name)]];
    $part2 = [[qw(downloaded DL)]];
    $part3 = [[qw(obsoleted_repo OBS)],[qw(auto_install AI)]];
    $part4 = [[qw(release_date rel_date)],[qw(upload_size_kb size_KB)]];

    $deploy->{repo}{header_ga_available} = [@$part1,@$part2,@$part4];
    $deploy->{repo}{header_patch_available} = [@$part1,@$part2,@$part3,@$part4];
    $deploy->{repo}{header_hotfix_available}= $deploy->{repo}{header_patch_available};

    $deploy->{repo}{header_ga_local} = [@$part1];
    $deploy->{repo}{header_patch_local} = [@$part1,@$part3];
    $deploy->{repo}{header_hotfix_local}= $deploy->{repo}{header_patch_local};
    $deploy->update_release_matrix();
    return 1;
}

# Modify/Add some values in memory after retrieving release matrix information
sub update_release_matrix {
    my ($deploy)=@_;
    my ($rel,$padvs,$levels,$matrix);
    $rel=Obj::rel();
    # set the obsoleted value to Y
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            while (my ($key,$value) = each %$matrix) {
                $value->{obsoleted_repo}='Y' if $value->{obsoleted_by};
                $value->{upload_size_kb}=sprintf("%d",$value->{upload_size} / 1024) if($value->{upload_size});
                $value->{auto_install}='Y' if $value->{install_path};
                # For Patch version < 6.1.0 we block Deployment Server not allow auto installation.
                undef $value->{auto_install} if (($level eq 'hotfix') && ($value->{vers_level} lt '610'));
            }
        }
    }

    return 1;
}

sub manage_repository {
    my ($deploy) = @_;
    my ($msg,$opt1,$opt2,$opt3,$done,$help,$menuopt,$menu,$rtn);

    return unless ($deploy->repository_set_home);
    $deploy->init_repository;
    $deploy->image_repository_scan();
    $msg=Msg::new("View/Download Available Releases");
    $opt1=$msg->{msg};
    $msg=Msg::new("View/Remove Repository Images");
    $opt2=$msg->{msg};
    $msg=Msg::new("Load a Release Image File");
    $opt3=$msg->{msg};
    $help=Msg::new("Choose one menu item to manage the release images in your repository.");
    while (!$done) {
        Msg::title();
        $menuopt=[$opt1,$opt2,$opt3];
        $msg=Msg::new("Select the option you would like to perform");
        $menu=$msg->menu($menuopt,'',$help,'back');
        return $menu if (EDR::getmsgkey($menu,'back'));

        $rtn = $deploy->repository_view_download_available if ($menu==1);
        $rtn = $deploy->repository_view_remove_image       if ($menu==2);
        $rtn = $deploy->repository_load_image              if ($menu==3);
        # return to main menu only if function return 1.
        $done=1 if ($rtn);
    }

    return 1;
}

sub repository_set_home {
    my ($deploy) = @_;
    my ($msg,$dir,$localsys,$result,$repository);
    my $cpic=Obj::cpic();
    my $edr=Obj::edr();
    $repository=$deploy->get_image_repository_dir();
    unless ($repository) {
        while (1) {
            Msg::n();
            $msg=Msg::new("In which directory would you like to store release images?");
            $dir=$msg->ask($deploy->{preference}{Repository}{default},0,1);
            return 0 if (EDR::getmsgkey($dir,'back'));
            if ($dir !~ m{^/}) {
                $dir=EDRu::abs_path($dir);
                $msg=Msg::new("Would you like to create repository directory under $dir?");
                next if ($msg->ayny eq 'N');
            }
            $dir=$cpic->path($dir);
            $localsys=$edr->{localsys};
            last if ($deploy->verify_repository($localsys,$dir));
        }
        # set to $deploy->{preference}
        $deploy->{preference}{Repository}{value}=$dir;
        $deploy->write_deploy_config();
    }
    return 1;
}

sub image_repository_scan {
    my ($deploy,$repository) = @_;
    my ($cfg,$repos,$image_repo,$repo_dir,$repo_info,$rtn,
        $targz,$padv,$rel_id,$install_path,$version);
    Msg::log("Checking Repository");
    $cfg=Obj::cfg();
    $repository||=$deploy->get_image_repository_dir();
    return 1 if (!$repository || !-d $repository);
    $repository=~s/\/$//;

    $deploy->repository_reset_matrix();
    # unset $deploy->{image_repository} before setting any value.
    # in case the value still exists during rescan
    undef ($deploy->{image_repository});
    for my $matrix (sort keys %{$deploy->{image_repository_defaults}}) {
        $repo_dir="$repository/$deploy->{image_repository_defaults}{$matrix}";
        $rtn=EDR::cmd_local("_cmd_ls -l $repo_dir/images");
        next if ($?);
        for my $image (split /\n/,$rtn) {
            # all the images should be a directory, not a file
            next unless ($image =~ /^d\w+/);
            $image=(split /\s+/,$image)[-1];
            $targz=$image . ".tar.gz";
            $repo_info=$deploy->image_repository_sanity_check($matrix,$repo_dir,$image,$targz);
            for my $item (@$repo_info) {
                next unless ($item =~ /:/);
                ($padv,$rel_id,$version,$install_path)=(split /:/, $item);
                $deploy->{image_repository}{$padv}{$version}{image} ="$repo_dir/images/$image";
                $deploy->{image_repository}{$padv}{$version}{targz} ="$repo_dir/targz/$targz" if (-f "$repo_dir/targz/$targz");
                $deploy->{image_repository}{$padv}{$version}{type}  ="$deploy->{image_repository_defaults}{$matrix}";
                $deploy->{image_repository}{$padv}{$version}{rel_id}=$rel_id;
                $deploy->{image_repository}{$padv}{$version}{install_path}=$install_path;
            }
        }
    }
    return 1;
}

# basic sanity check of an image repository per release
sub image_repository_sanity_check {
    my ($deploy,$matrix,$repo_dir,$release,$targz) = @_;
    my ($rel,$file,@padv_rel_id);
    my $rel=Obj::rel();
    @padv_rel_id=();
    return \@padv_rel_id unless (-d "$repo_dir/images/$release");
    return \@padv_rel_id unless ($rel->{$matrix});
    for my $padv (sort keys %{$rel->{$matrix}}) {
        for my $rel_id (sort keys %{$rel->{$matrix}{$padv}}) {
            if ($rel->{$matrix}{$padv}{$rel_id}{filename} &&
                $rel->{$matrix}{$padv}{$rel_id}{filename} eq $targz) {
                $deploy->download_cksum_verification($matrix,$padv,$rel_id,"$repo_dir/targz");
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            } elsif ( $rel->{$matrix}{$padv}{$rel_id}{release_name} &&
                     ($release =~ /^$rel->{$matrix}{$padv}{$rel_id}{release_name}(|-patches|-rpms)/)) {
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            } elsif ( $rel->{$matrix}{$padv}{$rel_id}{upload_location} &&
                     ($rel->{$matrix}{$padv}{$rel_id}{upload_location} =~ /$targz$/)) {
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            }
        }
    }
    return \@padv_rel_id;
}

sub download_cksum_verification {
    my ($deploy,$matrix,$padv,$rel_id,$repo_dir) = @_;
    my ($correct_cksum,$filename,@cksum_size,$cksum,$size);

    return;
    $correct_cksum="";
    @cksum_size=($deploy->{$matrix}{$padv}{$rel_id}{upload_cksum},
                 $deploy->{$matrix}{$padv}{$rel_id}{upload_size},
                 $deploy->{$matrix}{$padv}{$rel_id}{gunzip_cksum},
                 $deploy->{$matrix}{$padv}{$rel_id}{gunzip_size});
    $filename=$deploy->{$matrix}{$padv}{$rel_id}{filename};
    $filename="$repo_dir/$filename";

    if ((-f $filename) && ($#cksum_size > 0)) {
        $cksum=EDRu::cksum($filename);
        $size=EDR::cmd_local("_cmd_ls -l $filename");
        $size=(split /\s+/, $size)[4];
        if ($filename =~ /\.tar.gz$/) {
            $correct_cksum="Y" if (($cksum eq $cksum_size[0]) &&
                                   ($size  eq $cksum_size[1]));
        } elsif ($filename =~ /\.tar$/) {
            $correct_cksum="Y" if (($cksum eq $cksum_size[2]) &&
                                   ($size  eq $cksum_size[3]));
        }
    }
    $deploy->{$matrix}{$padv}{$rel_id}{downloaded}="Y" if ($correct_cksum);

    return;
}

sub find_repository_padv_installer {
    my ($deploy,$file)=@_;
    my ($list,$installer);

    if (-d $file) {
        $list=EDR::cmd_local("_cmd_find $file -maxdepth 1 -type f -name \"install*\"");
        $installer=(split /\n/,$list)[0];
    } elsif (-f $file) {
        $installer=$file;
    }

    return $installer;
}

sub repository_reset_matrix {
    my ($deploy) = @_;
    my ($rel,$padvs,$levels,$matrix);
    $rel = Obj::rel();
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            while (my ($key,$value) = each %$matrix) {
                delete $value->{downloaded};
            }
        }
    }
    return 1;
}

# In View/Download Available option, all Padvs/Levels will be shown
# In View/Remove Repository option, only Padvs/Levels that have images downloaded will be shown
sub repository_ask_padv_level {
    # if $action eq 'repo' -> Only show Padvs/Levels with images downloaded
    # if $action eq 'all'  -> Show all Padv/Levels retrieved from $padvs/$levels
    # if $padv or $level specified, only $level or $padv will be asked
    my ($deploy,$action,$padvs,$levels,$padv,$level) = @_;
    my ($msg,$menu,$userpadvs,$padvmsg,$repoleveltitles);

    unless ($padv) {
        if($action eq 'repo') {
            $padvs = $deploy->repository_downloaded_padvs($padvs);
            unless(scalar @$padvs) {
                $msg=Msg::new("No downloaded images");
                $msg->print;
                Msg::prtc();
                return '__back__';
            }
        }
        if ($deploy->{preference}{Platform}{value} && scalar(@{$deploy->{preference}{Platform}{value}})==1) {
            # only one element defined in padv list
            $padv=${$padvs}[0];
            $padvmsg=Msg::get($padv);
            $msg=Msg::new("$padvmsg has been selected as the platform type because $padvmsg is the only platform defined in the Preference menu.\n");
            $msg->bold();
        } else {
            $userpadvs = [];
            @$userpadvs = map {Msg::get($_)} @$padvs;

            if ($action eq 'repo') {
                $msg=Msg::new("Select the platform of the release to view/remove");
            } else {
                $msg=Msg::new("Select the platform of the release to view/download");
            }

            $menu=$msg->menu($userpadvs,scalar(@$userpadvs)==1,'','back',0,0,2);
            return $menu if (EDR::getmsgkey($menu,'back'));
            $padv = $padvs->[$menu - 1];
            Msg::n();
        }
    }
    $padvmsg=Msg::get($padv);
    unless ($level) {
        if($action eq 'repo') {
            $levels = $deploy->repository_downloaded_levels($levels,$padv);
            unless(scalar @$levels) {
                Msg::n();
                $msg=Msg::new("No downloaded images");
                $msg->print;
                sleep 2;
                return '__back__';
            }
        }
        if ($action eq 'repo') {
            $msg=Msg::new("Select the level of the $padvmsg release to view/remove");
        } else {
            $msg=Msg::new("Select the level of the $padvmsg release to view/download");
        }
        my $help=Msg::new("Base releases are major releases that allow you to perform fresh installations and select products, including new features, and are installable over most prior versions. You can use base releases to install package based updates and perform automated installations. You can obtain base releases from FileConnect (https://fileconnect.symantec.com).\n\n");
        $help->{msg} .= Msg::new("Maintenance releases are patch release updates that include multiple fixes and limited new features that are installable over any product of one specific base release version. Maintenance releases allow you to install patch-based updates (on native-patch based platforms) and perform automated installations. You can obtain maintenance releases from SORT (https://sort.symantec.com).\n\n")->{msg};
        $help->{msg} .= Msg::new("Patch releases are product release updates that include fixes only, typically for one specific issue, and are installable over specific products of one specific base or maintenance release version. Patch releases allow you to install patch-based updates (on native-patch based platforms). You can obtain Patch releases from SORT (https://sort.symantec.com).")->{msg};
        $repoleveltitles=$deploy->repo_level_titles($levels);
        $menu=$msg->menu($repoleveltitles,scalar(@$repoleveltitles)==1,$help,'back');
        return $menu  if (EDR::getmsgkey($menu,'back'));
        $level = $levels->[$menu - 1];
    }

    # Ensure the $padv,$level matrix file is the latest
    $deploy->repository_update_matrixfile($padv, $level);

    return ($padv,$level);
}

sub repo_level_titles {
    my ($deploy,$levels) = @_;
    my ($leveltitles,$i);

    for my $level (@$levels) {
        for ($i=0; $i<=$#{$deploy->{repo}{levels}};$i++) {
            next unless ($level eq $deploy->{repo}{levels}[$i]);
            push (@{$leveltitles},$deploy->{repo}{leveltitles}[$i]);
        }
    }
    return $leveltitles;
}

sub repository_update_matrixfile {
    my ($deploy,$padv,$level) = @_;
    my ($response,$url,$dir,$jsonhash,$filename,$matrix,$msg,$levelmsg);
    my ($time1,$time2,$totaltime,$clocktime);
    my $rel=Obj::rel();
    my $cpic=Obj::cpic();

    return 0 if EDRu::inarr('-noipc', @ARGV);
    return 1 if ($deploy->{updated_release_information}{$padv}{$level});
    # Connecting to SORT, print message.
    $levelmsg=$deploy->level_msg($level);
    $msg=Msg::new("\nRetrieving updated $padv $levelmsg release information from SORT");
    $msg->left();
    $time1 = time;
    $url = CPIP::get_hotfix_sort_url();
    $url .= "?product=SFHA&pretty=1&padv=$padv&level=$level";
    Msg::log("Downloading content from $url");
    $response = EDRu::download_page($url);
    # No need to adjust the result since 'pretty=1' provided in the $url
    # $response =~ s/\n+|\r+|\t+|\f+//mg;
    # $response =~ s/^.*<pre>//mg;
    # $response =~ s/<\/pre>.*$//mg;
    # $response =~ s/(.*)}.*$/$1}/mg;
    if ($response) {
        # Msg::log("RESULT is :$response");
        $cpic->{sortready}=1;
        $jsonhash = EDRu::eval_json($response);
    } else {
        $cpic->{sortready}=0;
        Msg::log("Encounter error during contacting SORT");
        $msg->right_failed();
        return 0;
    }
    $time2 = time;
    $totaltime = $time2 - $time1;
    $clocktime=EDRu::clocktime($totaltime);
    Msg::log("Downloading json file for $padv-$level took $totaltime seconds($clocktime)");
    if (!%{$jsonhash} || $jsonhash->{errors}) {
        Msg::log("Encounter error reading retrieved release information");
        $msg->right_failed();
        return 0;
    }
    $deploy->{updated_release_information}{$padv}{$level}=1;
    $matrix = (values %$jsonhash)[0];
    if ($level eq 'ga') {
        $rel->{vc_release_matrix_data}{$padv} = $matrix;
    } elsif ($level eq 'patch') {
        $rel->{vc_patch_matrix_data}{$padv}   = $matrix;
    } elsif ($level eq 'hotfix') {
        $rel->{vc_hotfix_matrix_data}{$padv}  = $matrix;
    } else {
        Msg::log("$level is not recongnized");
        $msg->right_failed();
        return 0;
    }

    $dir = '/opt/VRTS/install/.matrix';
    EDRu::mkdir_local_nosys($dir) unless -d $dir;
    $filename = "$dir/$padv-$level.json";
    EDRu::writefile("$response\n","$filename");
    
    if ($deploy->{preference}{Repository}{value}) {
        $dir = "$deploy->{preference}{Repository}{value}/.matrix";
        # if the repository reference is set, also need to update the json file in $dir
        EDRu::mkdir_local_nosys($dir) unless -d $dir;
        $filename = "$dir/$padv-$level.json";
        EDRu::writefile("$response\n","$filename");
    }

    # Todo: Currently image_repository_scan will scan the whole local repository that stores releases.
    # But actually only one json file is changed, we can add some function to only scan specific $level && $padv
    # Same goes to function update_release_matrix.
    $deploy->image_repository_scan();
    $deploy->update_release_matrix();
    $msg->right_done();
    return 1;
}

sub refresh_repository_table_content {
    my ($deploy,$padv,$level,$action)=@_;
    my ($rel,$matrix,$titles,$contents);
    $rel=Obj::rel();
    $matrix=$rel->get_release_matrix($padv,$level);
    $contents = $deploy->filter_matrix($matrix);
    $deploy->image_repository_scan();
    if ($action eq 'repo') {
        $titles=$deploy->get_titles_local($level);
        @$contents = grep {$_->{downloaded} =~ /y/i} @$contents;
    } else {
        $titles=$deploy->get_titles_available($level);
        @$contents = grep {$_->{in_development} != 1} @$contents;
    }
    return ($contents,$titles);
}

# return 1 if user agree to go to upgrade/install
# Todo: We need to figure out exact when should we prompt this message
# based on release version, padv ...
sub repository_download_to_upgrade_install {
    my ($deploy,$target)=@_;
    my ($msg);
    Msg::n();
    $msg=Msg::new("Would you like to go to the U) option to perform Install/Upgrade on your systems?");
    if ($msg->ayny eq 'Y') {
        $deploy->upgrade_install();
        return 1;
    }
    return 0;
}

sub repository_view_download_available {
    my ($deploy) = @_;
    my ($action,$ayn,$cfg,$padvs,$levels,$msg,$done,$padv,$padvmsg,$level,$titlemsg,$contents,$titles,$rtn,$details,$details_obs,$targets,$help,$levelmsg);
    my $cpic=Obj::cpic();
    $cfg = Obj::cfg();
    $action = 'all';
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $details= $deploy->{repo}{details_to_view};
    $details_obs = $deploy->{repo}{details_to_view_obsolete};

    $titlemsg = Msg::new("To view or download available releases, the platform type and release level type must be selected.\n");

    while (!$done) {
        Msg::title();
        $titlemsg->print;
        ($padv,$level) = $deploy->repository_ask_padv_level($action,$padvs,$levels);
        for my $op ($padv,$level) {
            return 0 if (EDR::getmsgkey($op,'back'));
        }
        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
        $msg=$deploy->available_releases_msg($level,$padv);
        $msg->bold;
        Msg::table($contents,$titles);
        unless (scalar @$contents) {
            Msg::n();
            $msg=Msg::new("Would you like to download another release?");
            next if ($msg->ayny eq 'Y');
            return 0;
        }

        while (1) {
            Msg::n();
            $msg=Msg::new("Enter the release_version to view the details about a release or press 'Enter' to continue");
            $help=Msg::new("release_version: The version of the release.\n\nSORT_release_name: The name of the release, used when accessing SORT (https://sort.symantec.com).\n\nDL: An indicator that the release is present in your repository.\n\nOBS: An indicator that the release has been obsoleted by another higher release.\n\nAI: An indicator that the release has scripted install capabilities. All base and maintenance releases have auto install capabilities. Patch releases with auto-install capabilities are available beginning with version 6.1. Otherwise the Patch will require a manual installation.\n\nrel_date: The date the release is available.\n\nsize_KB: The file size of the release in kilobytes.");
            $rtn = $msg->ask('',$help,'back','nullok');
            last if (EDR::getmsgkey($rtn,'back'));
            # if user press Enter
            if ($rtn=~/^\s*$/) {
                # $rtn=0 means due to some reason not success, back to manage repository menu
                # $rtn=1 means downloaded, let user press Enter and see the new available release info
                # $rtn=2 means back to Main Menu
                return 0 unless ($deploy->allow_release_download);
                $rtn=$deploy->repository_download_image($padv,$level);
                if ($rtn == 1) {
                    $msg=$deploy->available_releases_msg($level,$padv);
                    $msg->bold;
                    ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                    Msg::table($contents,$titles);
                    next;
                } elsif ($rtn == 2) {
                    return 1;
                } else {
                    return 0;
                }
            }
            $targets = [];
            @$targets = grep {$_->{release_version} eq $rtn} @$contents;
            if (scalar @$targets) {
                $msg=$deploy->repository_detail_release_msg($padv,$level,$targets,$details,$details_obs);
                Msg::print($msg);

                if(${$targets}[0]->{downloaded} !~ /y/i && $level ne 'ga') {
                    Msg::n();
                    return 0 unless ($deploy->allow_release_download);
                    $levelmsg=$deploy->level_msg($level);
                    $msg = Msg::new("Would you like to download this $levelmsg release?");
                    $ayn=$msg->ayny;
                    if($ayn eq 'Y') {
                        if ($deploy->repository_download($padv,$level,${$targets}[0])) {
                            # download successfully
                            #return 1 if((${$targets}[0]->{auto_install} eq 'Y') && $deploy->repository_download_to_upgrade_install($targets[0]));

                            $msg=$deploy->available_releases_msg($level,$padv);
                            $msg->bold;
                            ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                            Msg::table($contents,$titles);
                        } else {
                            Msg::n();
                            $msg=Msg::new("The $targets->[0]{release_name} download did not complete successfully. Please retry");
                            $msg->warning();
                            Msg::prtc();
                            return 0;
                        }
                    }
                }
            } else {
                $msg=Msg::new("No matching release.");
                $msg->print;
            }
         }
    }
    return 1;
}

sub allow_release_download {
    my ($deploy) = @_;
    my ($cpic,$cfg,$msg);
    $cpic=Obj::cpic();
    $cfg=Obj::cfg();
    
    if ($cfg->{opt}{repository}) {
        $msg=Msg::new("Downloading available releases is not allowed while the -repository option is being used.");
        Msg::n();
        $msg->bold();
        Msg::prtc();
        return 0;
    }
    if ($cfg->{opt}{noipc}) {
        $msg=Msg::new("Downloading available releases is not allowed while the -noipc option is being used.");
        Msg::n();
        $msg->bold();
        Msg::prtc();
        return 0;
    }
    unless ($cpic->{sortready}) {
        $msg=Msg::new("Downloading available releases is not allowed because SORT (https://sort.symantec.com) is not available.");
        Msg::n();
        $msg->bold();
        Msg::prtc();
        return 0;
    }
    return 1;
}

sub available_releases_msg {
    my ($deploy,$level,$padv)=(@_);
    if ($level eq 'ga') {
        return Msg::new("\nAvailable Base releases for $padv:\n");
    } elsif ($level eq 'patch') {
        return Msg::new("\nAvailable Maintenance releases for $padv:\n");
    }
    return Msg::new("\nAvailable Patch releases for $padv:\n");
}

sub get_titles_available {
    my ($deploy,$level) = @_;

    return $deploy->{repo}{header_ga_available}     if ($level eq 'ga');
    return $deploy->{repo}{header_patch_available}  if ($level eq 'patch');
    return $deploy->{repo}{header_hotfix_available};
}

sub get_titles_local {
    my ($deploy,$level) = @_;

    return $deploy->{repo}{header_ga_local}     if ($level eq 'ga');
    return $deploy->{repo}{header_patch_local}  if ($level eq 'patch');
    return $deploy->{repo}{header_hotfix_local};
}

sub filter_matrix {
    my ($deploy,$matrix) = @_;
    my (@releases);
    # Filter the releases with:
    # 1. version > 5.1.0 -- supported by Deployment Server
    # 2. release_type != 'HF' -- only show public HFs(pHF) instead of all HFs
    # return an array reference sorted by release version
    @releases = sort {$a->{vers_level} cmp $b->{vers_level}} values %$matrix;
    @releases = grep {(defined $_->{release_type}) && ($_->{release_type} ne 'HF')} @releases;
    @releases = grep {$_->{vers_level} ge '510'} @releases;
    return \@releases;
}

sub repository_download_image {
    my ($deploy,$padv,$level) = @_;
    my ($rel,$msg,$ayn,@menu,$matrix,$targets,$all_non,$all_releases,$ret,$download,$release_name,$levelmsg);
    $rel=Obj::rel();
    if ($level eq 'ga') {
        Msg::n();
        $msg=Msg::new("Base level release images must be downloaded from https://fileconnect.symantec.com/ and loaded using the R) 3) Load a Release Image File option");
        $msg->print();
        Msg::prtc();
        return 0;
    }
    return 0 if ($deploy->{sortIsDown});

    $all_non = "All non-obsoleted releases";
    $all_releases = "All releases";

    $deploy->image_repository_scan();
    # move the following section above the user prompt for Etrack 3086758
    $matrix=$rel->get_release_matrix($padv,$level);
    $targets = $deploy->filter_matrix($matrix);

    return unless scalar @{$targets};

    @{$targets} = grep {$_->{downloaded} !~ /y/i} @{$targets};
    @{$targets} = grep {$_->{in_development} != 1} @{$targets};

    # add the following check for Etrack 3086758
    unless (scalar @{$targets}) {
        $msg=Msg::new("\nAll releases are in the repository.");
        $msg->print;
        Msg::prtc();
        return 0;
    }

    if ($level eq 'ga') {
        $msg=Msg::new("\nWould you like to download a $padv Base Release Image?");
    } elsif ($level eq 'patch') {
        $msg=Msg::new("\nWould you like to download a $padv Maintenance Release Image?");
    } else {
        $msg=Msg::new("\nWould you like to download a $padv Patch Release Image?");
    }

    $ayn=$msg->aynn();
    return 0 unless ($ayn eq 'Y');
    Msg::n();

    @menu = (map({$_->{release_version}} @{$targets}), $all_non, $all_releases);
    $levelmsg=$deploy->level_msg($level);
    $msg=Msg::new("Select the $levelmsg release to download, '$all_non' to download all non-obsoleted releases, or '$all_releases' to download all releases");
    $ret=$msg->menu(\@menu,scalar(@menu)==1,'','back');
    return 0 if (EDR::getmsgkey($ret,'back'));
    # $download flag is for user downloading all non-obsolete releases.
    # if $download=0, means no download action take place because all the required images have been downloaded.
    # if $downloaded=1, means at least 1 image have been downloaded.
    $download=0;
    if ($ret < scalar(@menu)-1) {
        # user choose a regular release
        if ($deploy->repository_download($padv,$level,${$targets}[$ret-1])) {
            #return 2 if ((${$targets}[$ret-1]->{auto_install} ne 'Y') && $deploy->repository_download_to_upgrade_install(${$targets}[$ret-1]));
            return 1;
        } else {
            Msg::n();
            $release_name=${$targets}[$ret-1]->{release_name};
            $msg=Msg::new("The $release_name download failed. Please retry.");
            $msg->warning();
            Msg::prtc();
            return 0;
        }
    } elsif ($ret == scalar(@menu)-1) {
        # user choose $all_non
        for my $item (@{$targets}) {
            next if ($item->{obsoleted_by});
            $deploy->repository_download($padv,$level,$item);
            $download=1;
        }
    } else {
        # user choose $all_releases
        for my $item (@{$targets}) {
            $deploy->repository_download($padv,$level,$item);
            $download=1;
        }
    }

    unless ($download) {
        # there's no download action. notify user nothing has been dwnloaded.
        # If all releases have been downloaded, the previous code will deal with it.
        # If all non-obsoleted releases have been downloaded, it goes here.
        $msg=Msg::new("\nAll required releases have already been downloaded in the repository.");
        $msg->print;
    }

    Msg::prtc();
    return 1;
}

sub repository_download {
    my ($deploy,$padv,$level,$item) = @_;
    my ($ayn,$msg,$url,$filename,$targz_dir,$image_dir,$image,$free,$filesize,$conflict);
    my $cpic=Obj::cpic();
    my $edr=Obj::edr();

    unless (defined $item->{upload_location}) {
        Msg::n();
        $msg=Msg::new("Could not download $item->{release_name} since the upload_location is not set.");
        $msg->print();
        return 0;
    }
    $url = $item->{upload_location};
    $image = $filename = (split /\//,$url)[-1];
    $image =~ s/\.tar\.gz\s*$//;
    $targz_dir = "$deploy->{preference}{Repository}{value}/$level/targz";
    # $image is ok here because at the moment we only download Maintenance/Patch
    $image_dir = "$deploy->{preference}{Repository}{value}/$level/images/$image";
    $filename = "$targz_dir/$filename";
    EDR::cmd_local("_cmd_rmr $filename");
    EDR::cmd_local("_cmd_rmr $image_dir");
    EDRu::mkdir_local_nosys($targz_dir) unless -d $targz_dir;

    #Check the available space
    $free = $cpic->volumespace_sys($edr->{localsys},$deploy->{preference}{Repository}{value},1) || $cpic->volumespace_sys($edr->{localsys},'/');
    # need a write check here and alternate message if it fails
    $filesize = $item->{upload_size_kb} * 3;
    if ($free < $filesize) {
        $msg=Msg::new("Downloading and expanding $item->{release_name} require $filesize KB in the repository and only $free is available");
        $msg->print;
        Msg::prtc();
        return 0;
    }
    $conflict=$deploy->add_deploy_sfha_lock($image,"download");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflicts for repository download of $image:\n");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        return 0;
    }

    $msg=Msg::new("\nDownloading $item->{release_name} from SORT - https://sort.symantec.com");
    $msg->print();
    my $ret = EDRu::download_file($url, $filename, 1, 20);
    if (!$ret) {
        #Error occurred
        my $download_errors=EDR::get('download_errors');
        $msg=Msg::new("Could not download $item->{release_name}\n$download_errors\n");
        $msg->print;
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }
    unless ($deploy->is_cksum_match($filename,$item->{upload_cksum})) {
        $msg=Msg::new("The cksum of file $filename is not correct");
        $msg->print;
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }

    EDRu::mkdir_local_nosys($image_dir) unless -d $image_dir;
    EDR::cmd_local("cd $image_dir; _cmd_gunzip -c $filename | _cmd_tar xvf - 2> /dev/null");
    $msg=Msg::new("Untarring $item->{release_name}");
    $msg->left();
    if (EDR::cmdexit()) {
        $msg=Msg::new("Failed to un-zip and un-tar the file $filename");
        $msg->print;
        EDR::cmd_local("_cmd_rmr $image_dir");
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }
    $msg->right_done();

    if ((defined $deploy->{preference}{SaveTar}{value}) && ($deploy->{preference}{SaveTar}{value} eq 'N')) {
        $msg=Msg::new("Deleting $filename");
        $msg->left();
        EDR::cmd_local("_cmd_rmr $filename");
        $msg->right_done();
    }

    $item->{downloaded} = 'Y';

    $msg=Msg::new("$item->{release_name} has been downloaded successfully.");
    $msg->nprint;

    $deploy->clear_deploy_sfha_lock($image,"download");
    $deploy->image_repository_scan();

    return 1;
}

sub repository_view_remove_image {
    my ($deploy) = @_;
    my ($padvs,$levels,$msg,$done,$padv,$level,$titlemsg,$contents,$titles,$rtn,$details,$details_obs,$targets,$help,$ayn,$action,$cfg);

    my $cpic=Obj::cpic();
    $cfg = Obj::cfg();
    $action = 'repo';
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $details= $deploy->{repo}{details_to_view};
    $details_obs = $deploy->{repo}{details_to_view_obsolete};

    $titlemsg = Msg::new("To view or remove repository images, the platform type and release level type must be selected.\n");

    while (!$done) {
        Msg::title();
        $titlemsg->print;
        ($padv,$level) = $deploy->repository_ask_padv_level($action,$padvs,$levels);
        for my $op ($padv,$level) {
            return 0 if (EDR::getmsgkey($op,'back'));
        }

        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
        $msg=Msg::new("\nStored Repository Releases:\n");
        $msg->bold;
        Msg::table($contents,$titles);
        while (1) {
            Msg::n();
            $msg=Msg::new("Enter the release_version to view details about a release including its stored location, or press 'Enter' to continue");
            $help=Msg::new("release_version: The version of the release.\n\nSORT_release_name: The name of the release, used when accessing SORT (https://sort.symantec.com).\n\nOBS: An indicator that the release has been obsoleted by another higher release.\n\nAI: An indicator that the release has scripted install capabilities. All base and maintenance releases have auto install capabilities. Patch releases with auto-install capabilities are beginning with version 6.1. Otherwise the Patch will require manual installation.");
            $rtn = $msg->ask('',$help,'back','nullok');
            last if (EDR::getmsgkey($rtn,'back'));

            if ($rtn =~ /^\s*$/) {
                # $rtn=0 means due to some reason not success, back to manage repository menu
                # $rtn=1 means removed, let user press Enter and see the new available release info
                # $rtn=2 means go back, redo the previous stuff
                if ($cfg->{opt}{repository}) {
                    $msg=Msg::new("Removing repository images is not allowed while the -repository option is being used.");
                    Msg::n();
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }
                $rtn=$deploy->repository_remove_image($padv,$level);

                if ($rtn == 1) {
                    Msg::prtc();
                    $msg=Msg::new("\nStored Repository Releases:\n");
                    $msg->bold;
                    ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                    Msg::table($contents,$titles);
                    unless(@$contents) {
                        Msg::prtc();
                        return 0;
                    }
                    next;
                } elsif ($rtn == 2) {
                    next;
                }
                else {
                    return 0;
                }
            }

            $targets = [];
            @$targets = grep {$_->{release_version} eq $rtn} @$contents;
            if (scalar @$targets) {
                $msg=$deploy->repository_detail_release_msg($padv,$level,$targets,$details,$details_obs);
                Msg::printn($msg);

                if ($cfg->{opt}{repository}) {
                    $msg=Msg::new("Removing repository images is not allowed while the -repository option is being used.");
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }

                if (${$targets}[0]->{downloaded} =~/y/i) {
                    $msg=Msg::new("Would you like to remove this release?");
                    # default value is not to remove image.
                    $ayn=$msg->aynn;
                    if($ayn eq 'Y') {
                        # user want to remove this release
                        $deploy->repository_remove($padv,$level,${$targets}[0]);
                        Msg::prtc();
                        $msg=Msg::new("\nStored Repository Releases:\n");
                        $msg->bold;
                        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                        Msg::table($contents,$titles);
                        unless(@$contents) {
                            Msg::prtc();
                            return 0;
                        }
                    }
                }
            } else {
                $msg=Msg::new("No matching release.");
                $msg->print;
            }
        }
    }
    return 0;
}

sub repository_remove {
    my ($deploy,$padv,$level,$item) = @_;
    my ($msg,$url,$filename,$targz_dir,$image_dir,$image,$conflict);
    my (@padvs,$padvlist,$release_version);

    my $cpic=Obj::cpic();

    if ($level eq 'ga') {
        $image = $filename = $item->{filename};
    } else {
        unless (defined $item->{upload_location}) {
            Msg::n();
            $msg=Msg::new("Could not remove $item->{release_name} since the upload_location is not set.");
            $msg->print();
            return 0;
        }
        $url = $item->{upload_location};
        $image = $filename = (split /\//,$url)[-1];
    }

    $image =~ s/\.tar\.gz\s*$//;
    $targz_dir = "$deploy->{preference}{Repository}{value}/$level/targz";
    $image_dir = "$deploy->{preference}{Repository}{value}/$level/images/$image";
    $filename = "$targz_dir/$filename";

    $release_version=$item->{release_version};
    # Check if there're other padv share the same image.
    for my $padvi (@{$deploy->{repo}{padvs}}) {
        next if ($padv eq $padvi);
        for my $rel_ver(keys%{$deploy->{image_repository}{$padvi}}) {
            if (EDRu::basename($deploy->{image_repository}{$padvi}{$rel_ver}{image}) eq $image) {
                push(@padvs,$padvi);
            }
        }
    }

    if(@padvs) {
        push(@padvs,$padv);
        $padvlist=join(', ',@padvs);
        Msg::n();
        $msg=Msg::new("The $release_version release image applies to the following platforms:\n$padvlist\nRemoving $release_version release removes it for all platforms.");
        $msg->printn();
        $msg=Msg::new("Are you sure you would like to remove the $release_version release?");
        return if ($msg->ayny eq 'N');
    }

    $conflict=$deploy->add_deploy_sfha_lock($image,"remove");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflicts for repository remove of $image:");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        return 0;
    }

    $msg=Msg::new("\nRemoving $image");
    $msg->left();
    EDR::cmd_local("_cmd_rmr $filename 2> /dev/null");
    EDR::cmd_local("_cmd_rmr $image_dir 2> /dev/null");

    $msg=Msg::new("Done");
    $msg->right();

    delete $item->{downloaded};

    $msg=Msg::new("$image has been removed successfully.");
    $msg->nprint;
    $deploy->clear_deploy_sfha_lock($image,"remove");

    $deploy->image_repository_scan();
    return 0;
}

sub is_cksum_match {
    my ($deploy,$filename,$aim) = @_;

    # Consider the file OK if there is no value set in the matrix
    return 1 unless $aim;
    my $cksum=EDRu::cksum($filename);
    return ($cksum eq $aim) ? 1 : 0;
}

# the decision has been made that 
# GA->Base
# Patch->Maintence
# Hotfix->Patch
sub level_msg {
    my ($deploy,$level)=@_;
    if ($level =~ /^ga$/mi) {
        return Msg::new("Base")->{msg};
    } elsif ($level =~ /^patch$/mi) {
        return Msg::new("Maintenance")->{msg};
    } elsif ($level =~ /^hotfix$/mi) {
        return Msg::new("Patch")->{msg};
    }
    return $level;
}

sub get_item_by_size_cksum {
    my ($deploy,$padvs,$levels,$filename)=@_;
    my ($rel,$matrix,$local_file_size,$local_file_cksum);
    my ($gzipped_cksum,$gzipped_size,$gunzipped_cksum,$gunzipped_size);
    $rel=Obj::rel();
    $local_file_size=Obj::edr()->{localsys}->filesize($filename);
    $local_file_cksum=EDRu::cksum($filename);

    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            for my $item (values %$matrix) {
                if ($local_file_size == $item->{gunzip_size} && $local_file_cksum == $item->{gunzip_cksum}) {
                    return ($padv,$level,$item,0,1);
                } elsif($local_file_size == $item->{upload_size} && $local_file_cksum == $item->{upload_cksum}) {
                    return ($padv,$level,$item,1,0);
                }
            }
        }
    }
    return 0;
}

sub repository_remove_image {
    my ($deploy,$padv,$level)=@_;
    my ($rel,$matrix,$targets,@menu,$msg,$ret,$ayn,$levelmsg);
    $rel=Obj::rel();
    $deploy->image_repository_scan();

    $matrix=$rel->get_release_matrix($padv,$level);
    $targets = $deploy->filter_matrix($matrix);
    return unless scalar @{$targets};

    @{$targets} = grep {$_->{downloaded} =~ /y/i } @{$targets};
    @menu = map({$_->{release_version}} @{$targets});

    Msg::n();
    if ($level eq 'ga') {
        $msg=Msg::new("Would you like to remove a $padv Base Release Image?");
    } elsif ($level eq 'patch') {
        $msg=Msg::new("Would you like to remove a $padv Maintenance Release Image?");
    } else {
        $msg=Msg::new("Would you like to remove a $padv Patch Release Image?");
    }

    $ayn=$msg->aynn();
    return 0 if ($ayn eq 'N');

    Msg::n();
    $levelmsg=$deploy->level_msg($level);
    $msg=Msg::new("Select the $levelmsg release to remove");
    $ret=$msg->menu(\@menu,scalar(@menu)==1,'','back');
    return 0 if (EDR::getmsgkey($ret,'back'));

    $deploy->repository_remove($padv,$level,${$targets}[$ret-1]);
    return 1;
}

sub repository_load_image {
    my ($deploy) = @_;
    my ($cpic,$padvs,$levels,$titlemsg,$rtn,$msg,$list,$help,$matrix,$titles,$localsys,$cfg);
    my ($image_padv,$image_level,$image_item,$image,$image_match,$targz_dir,$image_dir,$filename,$istar,$isgz,$filegzname,$conflict,$free,$filesize);

    $cfg=Obj::cfg();
    $cpic = Obj::cpic();
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $localsys = Obj::edr()->{localsys};

    Msg::n();
    if ($cfg->{opt}{repository}) {
        $titlemsg = Msg::new("Loading repository is not allowed while the -repository option is being used.");
        $titlemsg->bold();
        Msg::prtc();
        return 0;
    }
    $titlemsg = Msg::new("Enter the file path of the release image (Example: /tmp/VRTS_SF_HA_Solutions_6.0.1_RHEL.tar.gz)");

    while (1) {
        $image_padv = $image_level = '';
        $rtn = $titlemsg->ask('','','back','nullok');
        return 0 if (EDR::getmsgkey($rtn,'back'));
        next if ($rtn=~/^\s*$/);

        unless (-f $rtn) {
            $msg=Msg::new("\nThe file $rtn does not exist. Please retry.\n");
            $msg->print;
            next;
        }
        unless (-r $rtn) {
            $msg=Msg::new("\nCurrent user does not have the read permission for $rtn. Fix this issue and retry.\n");
            $msg->print;
            next;
        }
        $isgz = $istar = 0;
        $filegzname = $image = $filename = EDRu::basename($rtn);

        Msg::n();
        $msg=Msg::new("Verifying the contents of this file");
        $msg->left;

        # Even though a file name is changed, it may still be a tar file or gzip file.
        # So we use funtion &get_item_by_size_cksum to determine if it's a tar file or a gzip file.
        ($image_padv,$image_level,$image_item,$isgz,$istar)=$deploy->get_item_by_size_cksum($padvs,$levels,$rtn);

        Msg::log("Padv:$image_padv,Level:$image_level");

        $image_match=0;
        if ($image_padv) {
            # It's possible that user renamed the tar.gz file.
            # use $image to track the real release name.
            if ($image_level eq 'ga') {
                $image = $filename = $image_item->{filename};
            } else {
                $image = $filename = (split /\//,($image_item->{upload_location}))[-1];
            }
            if ($image) {
                $image_match=1;
            }
        }
        if (!$image_match) {
            $msg=Msg::new("Failed");
            $msg->right;
            if ($deploy->{preference}{Platform}{showform}) {
                $msg=Msg::new("\nCannot match $filegzname to a release of the Selected Platforms: $deploy->{preference}{Platform}{showform}.\n");
            } else {
                $msg=Msg::new("\nCannot match $filegzname to a release.\n");
            }
            $msg->print;
            next;
        }

        Msg::log("Image:$image");

        $msg=Msg::new("Done");
        $msg->right;

        $image =~ s/\.tar\.gz\s*$//;
        $targz_dir = "$deploy->{preference}{Repository}{value}/$image_level/targz";
        $image_dir = "$deploy->{preference}{Repository}{value}/$image_level/images/$image";
        $filename = "$targz_dir/$filename";

        $conflict=$deploy->add_deploy_sfha_lock($image,"load");
        if (@$conflict) {
            $msg=Msg::new("\nFound the following conflicts for repository load of $image:\n");
            $msg->print();
            $deploy->display_deploy_sfha_conflicts($conflict);
            Msg::prtc();
            return 0;
        }

        if ($image_item->{downloaded} =~ /y/i || $localsys->is_dir($image_dir)) {
            # Since the file have already been loaded. Ask user a question if he want to reload
            Msg::n();

            # User may loading it from $targz_dir, if so, only need to remove $image_dir when reloading it.
            # If $filename exists and $filename have the same cksum with $image on SORT. Consider it's a complete file
            if ($localsys->exists($filename) && $deploy->is_cksum_match($filename, $image_item->{upload_cksum})) {
                $list=$image_dir;
            } else {
                $list="$image_dir\n$filename";
            }

            $msg=Msg::new("$image has already been loaded, would you like to reload?");
            $help=Msg::new("By reloading $image, the following file or directory will be removed and recreated:\n$list\n\nYou can reload the image when the existing one is potentially corrupted.");
            if ($msg->aynn($help) eq 'N') {
                Msg::n();
                $deploy->clear_deploy_sfha_lock($image,"load");
                next;
            }
            Msg::n();
            $msg=Msg::new("Removing existing $image");
            $msg->left();

            # When the $filename exists and it's cksum not matching the one in SORT, remove this broken file
            EDR::cmd_local("_cmd_rmr $filename") if (($list ne $image_dir) && $localsys->exists($filename));
            EDR::cmd_local("_cmd_rmr $image_dir");

            $msg->right_done();
        } else {
            # if image is not loaded and $image_dir does NOT exist,
            # but $filename exists, there're 2 possible explanation:
            # 1. $filename have the same cksum with the SORT one, we just need to untar it.
            # 2. $filename do NOT have the same cksum with the SORT one, we need to remove it.
            if($localsys->exists($filename)) {
                if($deploy->is_cksum_match($filename, $image_item->{upload_cksum})) {
                    $rtn = $filename;
                } else {
                    EDR::cmd_local("_cmd_rmr $filename");
                }
            }
        }

        # Check free space
        $msg=Msg::new("Checking free space on $localsys->{sys}");
        $msg->left();
        $free = $cpic->volumespace_sys($localsys,$deploy->{preference}{Repository}{value},1) || $cpic->volumespace_sys($localsys,'/');
        $filesize = $image_item->{upload_size_kb} * 3;
        if ($free < $filesize) {
            $msg->right_failed();
            $msg=Msg::new("\nLoading and expanding $image require $filesize KB in the repository and only $free is available");
            $msg->print;
            $deploy->clear_deploy_sfha_lock($image,"load");
            Msg::prtc();
            Msg::n();
            next;
        }
        $msg->right_done();

        $msg=Msg::new("Loading $image");
        $msg->left();
        EDRu::mkdir_local_nosys($targz_dir) unless -d $targz_dir;
        EDRu::mkdir_local_nosys($image_dir) unless -d $image_dir;

        ## if user set not to save tar file, do NOT need to copy it to tar file.
        if ($isgz) {
            if ($deploy->{preference}{SaveTar}{value} && ($deploy->{preference}{SaveTar}{value} eq 'N')) {
                EDR::cmd_local("_cmd_rmr $filename") if ($localsys->exists($filename));
            } else {
                EDR::cmd_local("_cmd_cp -rf $rtn $filename") unless ($localsys->exists($filename));
            }
            EDR::cmd_local("cd $image_dir; _cmd_gunzip -c $rtn | _cmd_tar xvf -");
        } elsif ($istar) {
            EDR::cmd_local("cd $image_dir; _cmd_tar -xvf $rtn");
        }
        if (EDR::cmdexit()) {
            $msg=Msg::new("Failed");
        } else {
            $image_item->{downloaded} = 'Y';
            $msg=Msg::new("Done");
        }
        $msg->right();
        $msg=Msg::new("\n$image has been loaded successfully.\n");
        $msg->print;
        $deploy->clear_deploy_sfha_lock($image,"load");
        $deploy->image_repository_scan();
    }

    return;
}

sub repository_downloaded_padvs {
    my ($deploy,$padvs) = @_;
    my ($rel,@padvs,$levels,$match,$matrix);
    $rel = Obj::rel();
    $levels = $deploy->{repo}{levels};
    @padvs = ();
    for my $padv (@$padvs) {
        $match = 0;
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            my @match = grep {$_->{downloaded} =~ /y/i} values %$matrix;
            @match= grep {$_->{vers_level} ge '510'} @match;
            if (scalar @match) {
                $match = 1;
                last;
            }
        }
        push(@padvs,$padv) if $match;
    }
    return \@padvs;
}

sub repository_downloaded_levels {
    my ($deploy,$levels,$padv) = @_;
    my ($rel,@levels,$matrix);
    $rel=Obj::rel();
    for my $level (@$levels) {
        $matrix=$rel->get_release_matrix($padv,$level);
        my @match = grep {$_->{downloaded} =~ /y/i} values %$matrix;
        @match= grep {$_->{vers_level} ge '510'} @match;
        push(@levels,$level) if scalar @match;
    }
    return \@levels;
}

sub repository_detail_release_msg {
    my ($deploy,$padv,$level,$targets,$details,$details_obs)=@_;
    my ($rel,$matrix_ga,$matrix_patch,$matrix_hotfix,$msg,$obs_release,%details_obs);
    $rel=Obj::rel();
    $matrix_ga=$rel->get_release_matrix($padv,'ga');
    $matrix_patch=$rel->get_release_matrix($padv,'patch');
    $matrix_hotfix=$rel->get_release_matrix($padv,'hotfix');
    if (scalar @$targets) {
        $msg = "\n";
        for my $key (@$details) {
            $msg .= "$key: $targets->[0]{$key}\n" if defined $targets->[0]{$key};
        }
        for my $key (@$details_obs) {
            my @obs_details;
            next unless (defined $targets->[0]{$key});
            for my $element (split(/,/,$targets->[0]{$key})) {
                $obs_release="$matrix_ga->{$element}->{release_version}" || "$matrix_patch->{$element}->{release_version}" || "$matrix_hotfix->{$element}->{release_version}";
                if($obs_release) {
                    push (@obs_details, $obs_release);
                } else {
                    Msg::log("$padv Base/Maintenance/Patch id $element can not be found in release matrix");
                }
            }
            if (@obs_details) {
                @obs_details=sort @obs_details;
                $details_obs{$key}=\@obs_details;
            }
        }
        for my $key (@$details_obs) {
            next unless (defined $targets->[0]{$key});
            $msg .= "$key: ";
            if ($details_obs{$key} && @{$details_obs{$key}}) {
                $msg .= join(',', @{$details_obs{$key}});
            } else {
                $msg .= Msg::new("None")->{msg};
            }
            $msg .="\n";
        }
        # Remove the last "\n"
        chop($msg);
    }
    return $msg;
}
# _END_Manage_Repository

# _START_Set_Preference
sub init_preference_hash {
    my $deploy = shift;
    
    $deploy->{preference} = {
        'Repository' => {
            'name' => Msg::new("Repository"),
            'type' => 'edit',
            'default' => '/opt/VRTS/repository',
            'verify_sub'  => \&Deploy::verify_repository,
            'set_handler' => \&Deploy::set_handler_repository,
            'sub_dirs'=>["patch","hotfix","ga",".cpihotfix",".matrix",".lock"],
        },
        'Platform' => {
            'name' => Msg::new("Selected Platforms"),
            'type' => 'list',
            'options' => [qw(aix53 aix61 aix71 hpux1131 rhel5_x86_64 rhel6_x86_64 rhel7_x86_64 sles10_x86_64 sles11_x86_64 sles12_x86_64 sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64)], 
            'default' => [qw(aix53 aix61 aix71 hpux1131 rhel5_x86_64 rhel6_x86_64 rhel7_x86_64 sles10_x86_64 sles11_x86_64 sles12_x86_64 sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64)],
            'verify_sub'  => \&Deploy::verify_platform,
            'set_handler' => \&Deploy::set_handler_platform,
        },
        'Year' => {
            'name' => Msg::new("Year"),
            'type' => 'int',
            'max_value' => 2099,
            'default' => 2014,
            'verify_sub'  => \&Deploy::verify_year,
            'set_handler' => \&Deploy::set_handler_year,
        },
        'SaveTar' => {
            'name' => Msg::new("Save Tar Files"),
            'type' => 'yn',
            'default' => 'y',
            'verify_sub'  => \&Deploy::verify_savetar,
            'set_handler' => \&Deploy::set_handler_savetar,
        },
    };
    $deploy->{pref_elements} = ["Repository","Platform","SaveTar"];
    $deploy->{config_file}="/opt/VRTS/.deploy_config";
    $deploy->{image_repository_defaults} = {
        'vc_release_matrix_data'=> "ga",
        'vc_patch_matrix_data'  => "patch",
        'vc_hotfix_matrix_data'=> "hotfix",
    };
    $deploy->{bundle_elements} = [ "Name", "Platform", "Base", "Maintenance", "Patch" ];
    
    $deploy->{template_elements} = [ "Name", "Platform", "Product", "Base", "Maintenance", "Patch", 
                                     "Installed Packages", "Missing Packages", "Extra Packages" ];
}

sub set_preferences {
    my ($deploy)=@_;
    my @tasks=qw(set_preferences back);
    # As suggested by Judy and approved by Steve,
    # We hide the main preference menu for now.
    $deploy->cli_set_preference_menu();
#    while(1) {
#        my $task_k=$deploy->cli_set_preference_task_menu(@tasks);
#        if ($task_k eq Msg::get('key_set_preferences')) {
#            # create a default menu
#            $deploy->cli_set_preference_menu();
#        } elsif ($task_k eq Msg::get('key_back')) {
#            return EDR::get2('msg','back');
#        }
#    }
    return 1;
}

sub cli_set_preference_menu {
    my ($deploy)=@_;
    my ($msg,$menu,@prefmenu,$conflict,$help);
    my $cfg=Obj::cfg();
    # Show preference to user
    $deploy->cli_show_preference();

    if ($cfg->{opt}{repository}) {
        $deploy->{preference}{Repository}{value} = $cfg->{opt}{repository};
        $msg=Msg::new("\nPreference values may not be modified while the -repository option is being used.");
        $msg->bold();
        Msg::prtc();
        return 0;
    }
    $msg=Msg::new("\nPreference List:\n");
    $msg->bold;

    for my $key(@{$deploy->{pref_elements}}) {
        push(@prefmenu, $deploy->{preference}->{$key}->{name}->{msg});
    }

    $msg=Msg::new("Select a preference to set:");
    $help=Msg::new("Repository – Define the directory where release images are stored.\n\nSelected Platforms – Define the a of platforms displayed in selection menus.  The Selected Platforms list is used to filter the list of all platforms to the specific platforms used in your environment.\n\nSave Tar Files – Define whether to save the tar files in the Repository folder after they are expanded.");
    $menu=$msg->menu(\@prefmenu,'',$help,1);
    if (EDR::getmsgkey($menu,'back')) {
        return 0;
    }

    my $item=$deploy->{pref_elements}[$menu-1];

    # Execute set handler function
    if (&{$deploy->{preference}{$item}{set_handler}}) {
        # Only if set handler function return 1 will preference value be reset
        $deploy->write_deploy_config();
    }
    return $deploy->cli_set_preference_menu();
}

sub cli_set_preference_task_menu {
    my ($deploy,@tasks)=@_;
    my ($msg,@mo,$m,$mm1,$mm2,$bs,$be,$mkey,@mk,$buf,$tsl,$task_k);
    my $cfg=Obj::cfg();
    
    # Show preference to user
    $deploy->cli_show_preference();
    if ($cfg->{opt}{repository}) {
        $deploy->{preference}{Repository}{value} = $cfg->{opt}{repository};
        $msg=Msg::new("Preference values may not be modified while the -repository option is being used.");
        $msg->bold();
        Msg::prtc();
        return 'B';
    }

    # List task
    $msg=Msg::new("\nTask Menu:\n");
    $msg->bold;
    @mo=@tasks;
    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$bs$mkey$be) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$bs$mkey$be) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-39s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task_k,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl");
        $task_k=$msg->ask;
        $task_k=uc($task_k);
    }
    return $task_k;
}

# Perform the following tasks:
# 1. Verify dir
# 2. Create dir
# 3. Change dir permission to 755
sub verify_repository {
    my ($deploy,$localsys,$dir) = @_;
    my ($result,$msg);
    my ($files,@files);

    if ($localsys->is_dir($dir)) {
        $msg=Msg::new("The directory $dir already exists on $localsys->{sys}");
        $msg->log;
        $files=$localsys->lsfile($dir);
        @files=split(/\n/,$files);
        $files=EDRu::arrdel(\@files, @{$deploy->{preference}{Repository}{sub_dirs}});
        if (@$files > 0) {
            # There're irrelevant files in the repository dir
            # print warning message.
            $msg=Msg::new("The directory $dir already exists and contains non-repository files on $localsys->{sys}.  It is recommended to create a new repository directory that does not contain any unrelated files.");
            $msg->warning;
            $msg=Msg::new("Do you want to continue?");
            if ($msg->aynn() eq 'Y') {
                # User still want to use this dir
                $localsys->chmod($dir,'755');
                return 1;
            } else {
                return 0;
            }
        }
    } else {
        $result=$localsys->mkdir($dir) if (!-d $dir);
        if (!$localsys->is_dir($dir)) {
            $msg=Msg::new("Cannot create $dir directory on $localsys->{sys}");
            $msg->error();
            return 0;
        }
    }
    $localsys->chmod($dir,'755');
    return 1;
}

sub set_handler_repository {
    my ($deploy)=@_;
    my ($edr,$msg,$dir,$localsys,$ayn,@files,$files,$conflict);

    $edr=Obj::edr();
    $localsys=$edr->{localsys};
    
    $deploy->cli_show_preference();
    Msg::n();
    
    while (1) {
        $msg=Msg::new("Input a path for the repository directory:");
        if (defined($deploy->{preference}{Repository}{value})) {
            $dir=$msg->ask('',0,1);
        } else {
            $dir=$msg->ask($deploy->{preference}{Repository}{default},0,1);
        }
        return 0 if (EDR::getmsgkey($dir,'back'));
        unless ($localsys->filename_is_absolute($dir)) {
            $dir=$deploy->path(EDRu::abs_path($dir));
            if ($localsys->is_dir($dir)) { # if the dir already exists, Skip creating the dir
                $msg=Msg::new("Would you like to use repository directory under $dir?");
            } else {
                $msg=Msg::new("Would you like to create repository directory under $dir?");
            }
            next if ($msg->ayny eq 'N');
        }
        $dir=$deploy->path($dir);
        # If user input the same path, skip below steps
        return if ($dir eq $deploy->{preference}{Repository}{value});
        last if ($deploy->verify_repository($localsys,$dir));
    }

    $conflict=$deploy->add_deploy_sfha_lock("repository","change_repository");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflict(s) for repository change:\n");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        Msg::prtc();
        return 0;
    }

    # if we already have a repo path, merge the old repo path into the new repo path
    if ($deploy->{preference}{Repository}{value}) {
        $msg=Msg::new("Would you like to merge the old repository files into the new directory?");
        if ($msg->ayny eq 'Y') {
            for my $repo(@{$deploy->{preference}{Repository}{sub_dirs}}) {
                $deploy->merge_repository($localsys,"$deploy->{preference}{Repository}{value}/$repo", "$dir/$repo");
            }
            # After merge, ask user if he would like to remove the old repository dir
            $msg=Msg::new("Would you like to remove the old repository directory?");
            if ($msg->ayny() eq 'Y') {
                # remove the old repository
                $files=$localsys->lsfile($deploy->{preference}{Repository}{value});
                @files=split(/\n/,$files);
                $files=EDRu::arrdel(\@files, @{$deploy->{preference}{Repository}{sub_dirs}});
                if (@$files > 0) {
                    # there're irrelevant files in the old repository, only remove @{$deploy->{preference}{Repository}{sub_dirs}}
                    for my $repo(@{$deploy->{preference}{Repository}{sub_dirs}}) {
                        $localsys->rm("$deploy->{preference}{Repository}{value}/$repo");
                    }
                } else {
                    # remove $deploy->{preference}{Repository}{value}
                    $localsys->rm($deploy->{preference}{Repository}{value});
                }
            }
        }
    }
    # Clear the deploy sfha lock first because it clear the lock file under $deploy->{preference}{Repository}{value}
    $deploy->clear_deploy_sfha_lock("repository","change_repository");
    # set to $deploy->{preference}
    $deploy->{preference}{Repository}{value}=$dir;
    # Ensure both of the repositories' locks are cleared
    $deploy->clear_deploy_sfha_lock("repository","change_repository");
    return 1;
}

sub verify_platform {}

sub set_handler_platform {
    my ($deploy)=@_;
    my ($add_platform_msg, $delete_platform_msg);
    my ($msg,$menuitems,$menu,@userplats);
    $add_platform_msg = Msg::new("Add a Platform")->{msg};
    $delete_platform_msg = Msg::new("Delete a Platform")->{msg};

    while (1) {
        $deploy->cli_show_preference();
        Msg::n();
        $msg=Msg::new("Select a Task:");
        $msg->bold();
        Msg::n();

        if ($deploy->{preference}{Platform}{showform}) {
            @userplats=split(',\s*',$deploy->{preference}{Platform}{showform});
        }
        $msg=Msg::new("Select an action to perform on the platform list:");
        if ($deploy->{preference}{Platform}{showform}) {
            if (scalar(@userplats) == scalar(@{$deploy->{preference}{Platform}{options}})) {
                $menuitems=[$delete_platform_msg];
            } else {
                $menuitems=[$add_platform_msg,$delete_platform_msg];
            }
        } else {
            $menuitems=[$add_platform_msg];
        }
        $menu=$msg->menu($menuitems,'',0,'back');
        return 1 if (EDR::getmsgkey($menu,'back'));
        if ($menu == 2) {
            $deploy->delete_platform();
        } else {
            if (@userplats && (scalar(@userplats) == scalar(@{$deploy->{preference}{Platform}{options}}))) {
                $deploy->delete_platform();
            } else {
                $deploy->add_platform();
            }
        }
    }
    return 1;
}

sub add_platform {
    my ($deploy)=@_;
    my (@platlist,$filterlist,@showlist);
    my ($msg,$menu);
    $deploy->read_deploy_config();
    @platlist=@{$deploy->{preference}{Platform}{options}};
    Msg::n();
    if ($deploy->{preference}{Platform}{value} && @{$deploy->{preference}{Platform}{value}}) {
        $filterlist=EDRu::arrdel(\@platlist, @{$deploy->{preference}{Platform}{value}});
    } else {
        $filterlist=\@platlist;
    }
    @showlist=map {Msg::get($_)} @{$filterlist};
    $msg=Msg::new("Select a Platform to add to Platform Preference List:");
    $menu=$msg->menu(\@showlist,'', 0,'back');
    return 1 if (EDR::getmsgkey($menu,'back'));
    push (@{$deploy->{preference}{Platform}{value}}, ${$filterlist}[$menu-1]);
    @{$deploy->{preference}{Platform}{value}}=sort(@{$deploy->{preference}{Platform}{value}});
    $deploy->{preference}{Platform}{showform}=join(',', map {Msg::get($_)} @{$deploy->{preference}{Platform}{value}});
    $deploy->write_deploy_config();
    return 1;
}

sub delete_platform {
    my ($deploy)=@_;
    my (@userplat,$msg,$menu,@showlist);
    $deploy->read_deploy_config();
    $msg=Msg::new("Select a Platform to remove from Platform Preference List:");
    Msg::n();
    @showlist=map {Msg::get($_)} @{$deploy->{preference}{Platform}{value}};
    $menu=$msg->menu(\@showlist,'', 0,'back');
    return 1 if (EDR::getmsgkey($menu,'back'));
    splice(@{$deploy->{preference}{Platform}{value}},$menu-1,1);
    if (@{$deploy->{preference}{Platform}{value}}) {
        $deploy->{preference}{Platform}{showform}=join(',',map {Msg::get($_)} @{$deploy->{preference}{Platform}{value}});
    } else {
        delete $deploy->{preference}{Platform}{value};
        delete $deploy->{preference}{Platform}{showform};
    }
    $deploy->write_deploy_config();
    return 1;
}

# Example of an query that only allows a value < $deploy->{preference}{Year}{max_value}
sub verify_year {
    my ($deploy,$value)=@_;
    my ($msg);
    
    if ($value >= $deploy->{preference}{Year}{max_value}) {
        $msg=Msg::new("The year you set exceeds the maximum value($deploy->{preference}{Year}{max_value}). Reset the value.");
        $msg->error();
        return 0;
    } else {
        return 1;
    }
}

# Allow user input a int value
sub set_handler_year {
    my ($deploy)=@_;
    my ($value,$msg);
    
    $deploy->cli_show_preference();
    Msg::n();
    
    $msg=Msg::new("Set year: ");
    do {
        $value=$msg->ask($deploy->{preference}{Year}{default},0,1);
        return 0 if (EDR::getmsgkey($value,'back'));
    } while (!$deploy->verify_year($value));

    # set to $deploy->{preference}
    $deploy->{preference}{Year}{value}=$value;
    return 1;
}

sub verify_savetar{}

sub set_handler_savetar {
    my ($deploy)=@_;
    my ($msg,$value);
    
    $deploy->cli_show_preference();
    Msg::n();
    
    $msg=Msg::new("Do you want to save the image tarfiles?");
    $value=$msg->ayn($deploy->{preference}{SaveTar}{default},0,1);
    return 0 if (EDR::getmsgkey($value,'back'));
    $deploy->{preference}{SaveTar}{value}=$value;
    return 1;
}

sub cli_show_preference {
    my ($deploy)=@_;
    my ($msg,$max,$width,$format);
    
    Msg::title();
    $msg=Msg::new("Current Preferences:\n");
    $msg->bold;
    # Read preference file
    $deploy->read_deploy_config();

    $max = 0;
    for my $key(@{$deploy->{pref_elements}}) {
        $width=Msg::string_width($deploy->{preference}{$key}{name}{msg});
        $max = $max > $width? $max : $width
   }
    # 4 spaces between key and value
    $max += 4;
    $format = "%-${max}s%s";
    for my $key(@{$deploy->{pref_elements}}) {
        if ($deploy->{preference}{$key}{type} eq 'list') {
            if (defined($deploy->{preference}{$key}{showform})) {
                $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, $deploy->{preference}{$key}{showform});
            } else {
                $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, "N/A");
            }
        } else {
            if (defined($deploy->{preference}{$key}{value})) {
                $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, $deploy->{preference}{$key}{value});
            } else {
                $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, "N/A");
            }
        }
        Msg::print($msg);
    }
    return 1;
}

# Deployment Server config file is a json file that contains all the configurations including:
# 1. Preferences
# 2. Install Bundle Definition
sub read_deploy_config {
    my ($deploy) = @_;
    my ($config_file,$content,$localsys);
    my ($deploy_config_hash,$bundle);
    $config_file = $deploy->{config_file};
    $localsys = Obj::localsys();

    # initiate bundles and templates
    undef $deploy->{bundle};
    $deploy->{bundle}=[];
    undef $deploy->{template};
    $deploy->{template}=[];

    # If the old preference file format has not been converted
    # and the current configure file format does not exist
    # we convert the old configure file to new format.
    if (!$deploy->{config_converted} && !$localsys->exists($config_file)) {
        $deploy->convert_deploy_config();
        $deploy->{config_converted}=1;
    }
    $config_file = $deploy->{config_file};
    $localsys = Obj::localsys();
    $content = $localsys->catfile($config_file);
    return 0 unless ($content);
    $deploy_config_hash = EDRu::eval_json($content);
    return 0 unless ($deploy_config_hash);
    # Read preference file into $deploy object
    for my $key(@{$deploy->{pref_elements}}) {
        if ($deploy->{preference}{$key}{type} eq 'list') {
            if (defined($deploy_config_hash->{preference}{$key}{showform})) {
                $deploy->{preference}{$key}{value} = $deploy_config_hash->{preference}{$key}{value};
                $deploy->{preference}{$key}{showform} = $deploy_config_hash->{preference}{$key}{showform};
            }
        } else {
            if (defined($deploy_config_hash->{preference}{$key})) {
                $deploy->{preference}{$key}{value} = $deploy_config_hash->{preference}{$key};
            } else {
                undef ($deploy->{preference}{$key}{value});
            }
        }
    }
    # Read bundle information into $deploy object
    for my $bundle_name (keys%{$deploy_config_hash->{bundle}}) {
        $bundle=$deploy->bundle_structure();
        for my $key (@{$deploy->{bundle_elements}}) {
            if ($deploy_config_hash->{bundle}{$bundle_name}{$key} && %{$deploy_config_hash->{bundle}{$bundle_name}{$key}}) {
                $bundle->{$key}{value}=$deploy_config_hash->{bundle}{$bundle_name}{$key}{value};
                $bundle->{$key}{showform}=$deploy_config_hash->{bundle}{$bundle_name}{$key}{showform};
            }
        }
        push(@{$deploy->{bundle}},$bundle);
    }
    # Read template information into $deploy object
    for my $template_name (keys%{$deploy_config_hash->{template}}) {
        my $template; # Need new $template variable in each loop
        for my $key (@{$deploy->{template_elements}}) {
            if ($deploy_config_hash->{template}{$template_name}{$key} && %{$deploy_config_hash->{template}{$template_name}{$key}}) {
                $template->{$key}{value}=$deploy_config_hash->{template}{$template_name}{$key}{value};
                $template->{$key}{showform}=$deploy_config_hash->{template}{$template_name}{$key}{showform};
            }
        }
        push (@{$deploy->{template}}, $template);
    }
    return 1;
}

sub write_deploy_config {
    my ($deploy)=@_;
    my ($config_file,$content);
    my (%deploy_config_hash,$bundle_name,$template_name);
    $config_file = $deploy->{config_file};

    # Save preference information
    for my $key(@{$deploy->{pref_elements}}) {
        if ($deploy->{preference}{$key}{type} eq 'list') {
            if (defined($deploy->{preference}{$key}{showform})) {
                $deploy_config_hash{preference}{$key}{showform}=$deploy->{preference}{$key}{showform};
                $deploy_config_hash{preference}{$key}{value} = $deploy->{preference}{$key}{value};
            }
        } else {
            if (defined($deploy->{preference}{$key}{value})) {
                $deploy_config_hash{preference}{$key} = $deploy->{preference}{$key}{value};
            }
        }
    }
    # Save bundle information
    if ($deploy->{bundle}) {
        for my $bundle(@{$deploy->{bundle}}) {
            $bundle_name=$bundle->{Name}{showform};
            for my $key(@{$deploy->{bundle_elements}}) {
                if ($bundle->{$key}{value} && $bundle->{$key}{showform}) {
                    $deploy_config_hash{bundle}{$bundle_name}{$key}{value}=$bundle->{$key}{value};
                    $deploy_config_hash{bundle}{$bundle_name}{$key}{showform}=$bundle->{$key}{showform};
                }
            }
        }
    }
    # Save template information
    if ($deploy->{template}) {
        for my $template(@{$deploy->{template}}) {
            $template_name=$template->{Name}{showform};
            for my $key(@{$deploy->{template_elements}}) {
                if ($template->{$key}{value} && $template->{$key}{showform}) {
                    $deploy_config_hash{template}{$template_name}{$key}{value}=$template->{$key}{value};
                    $deploy_config_hash{template}{$template_name}{$key}{showform}=$template->{$key}{showform};
                }
            }
        }
    }
    $content=JSON::to_json(\%deploy_config_hash,{pretty=>1});
    EDRu::writefile($content,$config_file);
    return 1;
}

# Convert old Deployment Server preference file to current configure file
sub convert_deploy_config {
    my ($deploy)=@_;
    my ($preference_file,$content,$localsys);
    $localsys = Obj::localsys();
    $preference_file = '/opt/VRTS/.install_pref';
    return 1 unless ($localsys->exists($preference_file));
    $deploy->read_default_values($preference_file);
    $localsys->rm($preference_file);
    for my $key(@{$deploy->{pref_elements}}) {
        if (defined($deploy->{default}{$key})) {
            $deploy->{preference}{$key}{value} = $deploy->{default}{$key};
        } else {
            undef ($deploy->{preference}{$key}{value});
        }
    }
    $deploy->write_deploy_config();
    return 1;
}

# Current algorithm:
# If a dir is in $srcdir but not in $destdir, copy it.
# If a dir is in $srcdir and in $destdir, merge it.
# If a file is in $srcdir but not in $destdir, copy it.
# If a file is in $srcdir and in $destdir, ignore it.
sub merge_repository {
    my ($deploy,$localsys,$src,$dest) = @_;
    return unless ($localsys->exists($src));

    if ($localsys->is_dir($src)) {
        if ($localsys->exists($dest)) {
            if ($localsys->is_dir($dest)) {
                for my $repo(split(/\n/,$localsys->lsfile($src))) {
                    $deploy->merge_repository($localsys,"$src/$repo","$dest/$repo");
                }
            }
        } else {
            # Copy the $src to $dest
            $localsys->copyfile($src, $dest);
        }
    } else {
        unless ($localsys->exists($dest)) {
            # Copy the $src to $dest
            $localsys->copyfile($src, $dest);
        }
    }
    return;
}
# _END_Set_Preference

# _START_Define_Install_bundle
sub bundle_structure {
    my $deploy = shift;
    # Initiate Bundle structure
    # Each element have the following attributes:
    # 1. name: The element type official name
    # 2. type: The type of this element(scalar,array,hash) stored inside
    # 3. value: The value of this element stored inside
    # 4. showform: The official name of the value
    my $bundle = {
        'Name' => {
            'name' => Msg::new("Install Bundle Name"),
            'type' => 'scalar',
        },
        'Platform' => {
            'name' => Msg::new("Platform"),
            'type' => 'scalar',
        },
        'Base' => {
            'name' => Msg::new("Base Release"),
            'type' => 'scalar',
        },
        'Maintenance' => {
            'name' => Msg::new("Maintenance Release"),
            'type' => 'scalar',
        },
        'Patch' => {
            'name' => Msg::new("Patch Releases"),
            'type' => 'array',
        },
    };
    return $bundle;
}

sub define_install_bundle {
    my ($deploy)=@_;
    my ($titlemsg,$msg,$opt1,$opt2,$opt3,$menu,$menuopt,$done);
    return unless ($deploy->repository_set_home);
    $titlemsg=Msg::new("An Install Bundle is a combination of Storage Foundation Base, Maintenance, and Patch releases that can be defined and installed together.");
    $opt1=Msg::new("Create a new Install Bundle")->{msg};
    $opt2=Msg::new("Modify an Install Bundle")->{msg};
    $opt3=Msg::new("Delete an Install Bundle")->{msg};
    $deploy->image_repository_scan();
    while (!$done) {
        Msg::title();
        $titlemsg->printn;
        $msg=Msg::new("Install Bundles can only be created from versions 6.1 or higher with the exception of Base release 6.0.1, 6.0.2 or 6.0.4 and Maintenance release 6.0.5 or higher.\n");
        $msg->bold();
        if ($deploy->list_bundles) {
            $menuopt=[$opt1,$opt2,$opt3];
        } else {
            $menuopt=[$opt1];
        }
        Msg::new("Select a Task:\n")->bold();
        $msg=Msg::new("Select the task you would like to perform");
        $menu=$msg->menu($menuopt,'',0,'back');
        return $menu if (EDR::getmsgkey($menu,'back'));

        $deploy->create_install_bundle if ($menu==1);
        $deploy->modify_install_bundle if ($menu==2);
        $deploy->delete_install_bundle if ($menu==3);
    }
    return 1;
}

sub create_install_bundle {
    my ($deploy)=@_;
    my ($name,$padv,$padvmsg,$msg,$padvs,$userpadvs,$menu,$bundle,$invalid,$length);

    $padvs=$deploy->{preference}{Platform}{value}||$deploy->{preference}{Platform}{default};
    $invalid=1;
    Msg::n();
    $msg=Msg::new("Enter the name of the Install Bundle you would like to define: (press [Enter] to go back)");
    while ($invalid) {
        # First ask bundle name
        $name=$msg->ask('__back__','','','nullok');
        unless ($name =~ /^\w+$/) {
            Msg::new("\nAn Install Bundle name can only include numbers, letters and underscores.\n")->print;
            next;
        }
        $length = length($name);
        if ($length > 255) {
            Msg::new("\nAn Install Bundle name cannot exceed 255 letters.\n")->print;
            next;
        }
        return 1 if (EDR::getmsgkey($name,'back'));
        $invalid=0;
        # There shouldn't be duplicate name as the bundle name is the unique identifier
        for my $bundle(@{$deploy->{bundle}}) {
            if ($bundle->{Name}{value} eq $name) {
                Msg::new("\nThere's already an Install Bundle named $name on $bundle->{Platform}{showform} platform, enter another name\n")->print;
                $invalid=1;
                last;
            }
        }
    }
    Msg::n();
    if (scalar(@{$padvs}) == 1) {
        # only one element defined in padv list
        $padv=${$padvs}[0];
        $padvmsg=Msg::get($padv);
        $msg=Msg::new("$padvmsg has been selected as the platform type because $padvmsg is the only platform defined in the Preference menu.\n");
        $msg->bold(1);
        Msg::prtc() unless (EDRu::inarr($padv, qw/aix53 sol9_sparc/));
    } else {
        # Ask padv
        $msg=Msg::new("To create an Install Bundle, the platform type must be selected:\n");
        $msg->print;
        $msg=Msg::new("Select the platform of the release for the Install Bundle $name:");
        @$userpadvs = map {Msg::get($_)} @$padvs;
        $menu=$msg->menu($userpadvs,scalar(@$userpadvs)==1,'','back',0,0,2);
        return $menu if (EDR::getmsgkey($menu,'back'));
        $padv = $padvs->[$menu - 1];
    }
    if (EDRu::inarr($padv, qw/aix53 sol9_sparc/)) {
        # Give warning message that cannot create bundle on the selected platform
        $padvmsg=Msg::get($padv);
        $msg=Msg::new("\nNo release on $padvmsg platform supports Install Bundle");
        $msg->print;
        Msg::prtc();
        return 1;
    }
    # Update json file
    for my $level (qw/ga patch hotfix/) {
        $deploy->repository_update_matrixfile($padv,$level);
    }
    $bundle = $deploy->bundle_structure;
    $bundle->{Name}{showform}=$bundle->{Name}{value}=$name;
    $bundle->{Platform}{showform}=Msg::get($padv);
    $bundle->{Platform}{value}=$padv;
    $deploy->modify_install_bundle_page($bundle);
    return 1;
}

sub modify_install_bundle_page {
    my ($deploy,$bundle)=@_;
    my ($max,$width,$format,$done,$menu_function_array,@menuitems);
    my ($menuopt,$bundle_release,$msg,$menu);
    Msg::title();
    $max = 0;
    for my $element(@{$deploy->{bundle_elements}}) {
        $width=Msg::string_width($bundle->{$element}{name}->{msg});
        $max=$max > $width? $max : $width;
    }
    $max+=4;
    $format="%-${max}s%s";
    # Do not prompt Save $bundle when user first enter the menu
    $bundle->{Saved}=1;
    # Switch to next page
    while (!$done) {
        Msg::title();
        $msg=Msg::new("Details of the Install Bundle: $bundle->{Name}{showform}\n");
        $msg->bold();
        for my $element (@{$deploy->{bundle_elements}}) {
            if ($bundle->{$element}{showform}) {
                $msg=Msg::string_sprintf($format, $bundle->{$element}{name}{msg}, $bundle->{$element}{showform});
            } else {
                $msg=Msg::string_sprintf($format, $bundle->{$element}{name}{msg}, "N/A");
            }
            Msg::print($msg);
        }
        Msg::n();
        @menuitems=();
        $menuopt=[];
        $menu_function_array = [
        {
            'index' => 0,
            'msg' => Msg::new("Add a Base Release")->{msg},
            'function' => \&Deploy::add_base_release
        },
        {
            'index' => 1,
            'msg' => Msg::new("Add a Maintenance Release")->{msg},
            'function' => \&Deploy::add_mr_release 
        },
        {
            'index' => 2,
            'msg' => Msg::new("Add a Patch Release")->{msg},
            'function' => \&Deploy::add_patch_release 
        },
        {
            'index' => 3,
            'msg' => Msg::new("Remove Base Release $bundle->{Base}{showform}")->{msg},
            'function' => \&Deploy::remove_base_release 
        },
        {
            'index' => 4,
            'msg' => Msg::new("Remove Maintenance Release $bundle->{Maintenance}{showform}")->{msg},
            'function' => \&Deploy::remove_mr_release 
        },
        {
            'index' => 5,
            'msg' => Msg::new("Remove a Patch Release")->{msg},
            'function' => \&Deploy::remove_patch_release 
        },
        {
            'index' => 6,
            'msg' => Msg::new("Save Install Bundle $bundle->{Name}{showform}")->{msg},
            'function' => \&Deploy::save_bundle
        },
        {   'index' => 7,
            'msg' => Msg::new("Change Install Bundle Name")->{msg},
            'function' => \&Deploy::change_install_bundle_name
        }];
        # Generate bundle menu based on a complicated scenario
        if ($bundle->{Base}{showform}) {
            push (@menuitems, 3);
        } else {
            push (@menuitems, 0);
        }
        if ($bundle->{Maintenance}{showform}) {
            push (@menuitems, 4);
        } else {
            push (@menuitems, 1);
        }
        if ($bundle->{Patch}{showform}) {
            push (@menuitems, 5);
            push (@menuitems, 2) if (scalar(@{$bundle->{Patch}{value}})<5);
        } else {
            push (@menuitems, 2);
        }
        $bundle_release=0;
        if ($bundle->{Base}{showform}) {
            $bundle_release++;
        }
        if ($bundle->{Maintenance}{showform}) {
            $bundle_release++;
        }
        if ($bundle->{Patch}{value} && @{$bundle->{Patch}{value}}) {
            $bundle_release+=scalar(@{$bundle->{Patch}{value}});
        }
        # If there're more than 1 releases defined(including 2 Patch release), we offer Save Bundle option
        if ($bundle_release > 1 && !$bundle->{Saved}) {
            push (@menuitems, 6);
        }
        push (@menuitems, 7);
        for my $item (@menuitems) {
            push(@$menuopt, ${$menu_function_array}[$item]->{msg});
        }
        $msg=Msg::new("Select an action to perform on the Install Bundle $bundle->{Name}{showform}");
        $menu=$msg->menu($menuopt,'',0,'back');
        if (EDR::getmsgkey($menu,'back')) {
            return if($deploy->create_bundle_back($bundle, $bundle_release));
            next;
        }
        &{${$menu_function_array}[$menuitems[$menu-1]]->{function}}($deploy,$bundle);
    }
}

# Assumption:
# 1. When a GA release and a Patch release are bundled, it's not possible to bundle a MR release between.
# 2. Each patch only have one installed_over release, which means multiple patches must have same installed_over release id.
sub add_base_release {
    my ($deploy,$bundle)=@_;
    my (%matrix,$installed_over,@ids,$menu,$msg);
    my ($rel,$releases,@release_id,@supported_release_id,@release_version,$release,$unsupported_release);
    $rel=Obj::rel();
    Msg::n();
    for my $level (qw/ga patch hotfix/) {
        $matrix{$level}=$rel->get_release_matrix($bundle->{Platform}{value},$level);
    }
    # Check if we have MR release
    if ($bundle->{Maintenance}{value}) {
        # we have MR release defined, only show the releases this MR release can install on
        $installed_over=$matrix{patch}{$bundle->{Maintenance}{value}}{installed_over};
        if ($installed_over) {
            @ids=split(',',$installed_over);
            for my $id(@ids) {
                # only push base release into the array
                # Todo: Filter the illegal releases for GA release.
                next unless ($matrix{ga}{$id});
                push(@release_id,$id);
            }
        }
    } elsif ($bundle->{Patch}{value} && @{$bundle->{Patch}{value}}) {
        # we only have Patch release defined, only show the releases this Patch release can install on
        $installed_over=$matrix{hotfix}{$bundle->{Patch}{value}->[0]}{installed_over};
        if ($installed_over && $matrix{ga}{$installed_over}) {
            push(@release_id,$installed_over);
        }
    } else {
        # both MR and Patch releases are not defined, show all the GA release supported
        $releases = $deploy->filter_matrix($matrix{ga});
        # 6.0.X install bundle do NOT support any-any, need to filter them
        if ((Padv::sort_to_cpi_padv($bundle->{Platform}{value}) ne Obj::localpadv()) && !($bundle->{Platform}{value}=~/^aix/i && Obj::localpadv()=~/^aix/i)) {
            @$releases = grep {$_->{vers_level} ge '610'} @$releases;
        }
        # For GA release, we also need some extra filtering
        @$releases = grep {$_->{install_path}} @$releases;
        @release_id = map({$_->{release_id}} @$releases);
    }
    # For GA release, the release version must >=6.0.1 so that it could support Install Bundle
    for my $id(@release_id) {
        if ($matrix{ga}{$id}{vers_level} ge '601') {
            push (@supported_release_id, $id);
        }
    }
    for my $id(@supported_release_id) {
        push (@release_version, $matrix{ga}{$id}{release_version});
    }
    # It's possible that a specific hotfix do NOT have a GA base release
    unless (scalar(@supported_release_id)) {
        if ($bundle->{Maintenance}{showform} && $bundle->{Patch}{showform}) {
            $msg=Msg::new("No Base releases are supported in the selected release combination.");
        } else {
            $unsupported_release=$bundle->{Maintenance}{showform}||$bundle->{Patch}{showform};
            if ($unsupported_release) {
                $msg=Msg::new("There are no Base releases for $unsupported_release on $bundle->{Platform}{showform} at this time");
            } else {
                $msg=Msg::new("There are no Base releases on $bundle->{Platform}{showform} at this time");
            }
        }
        $msg->print;
        $msg->prtc();
        return 1;
    }
    $msg=Msg::new("Select the Base Release version to add to the Install Bundle $bundle->{Name}{showform}");
    $menu=$msg->menu(\@release_version,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));
    $release=$matrix{ga}{$supported_release_id[$menu-1]};
    $bundle->{Base}{value}=$release->{release_id};
    $bundle->{Base}{showform}=$release->{release_version};
    $bundle->{Saved}=0;
    Msg::n();
    return 1;
}

sub add_mr_release {
    my ($deploy,$bundle)=@_;
    my (%matrix,$installed_over,@base_releases,@ids,$menu,$msg);
    my ($rel,$releases,@release_id,@supported_release_id,@release_version,$release,$unsupported_release);
    $rel=Obj::rel();
    Msg::n();
    for my $level (qw/ga patch hotfix/) {
        $matrix{$level}=$rel->get_release_matrix($bundle->{Platform}{value},$level);
    }
    $releases = $deploy->filter_matrix($matrix{patch});
    # 6.0.X install bundle do NOT support any-any, need to filter them
    if ((Padv::sort_to_cpi_padv($bundle->{Platform}{value}) ne Obj::localpadv()) && !($bundle->{Platform}{value}=~/^aix/i && Obj::localpadv()=~/^aix/i)) {
        @$releases = grep {$_->{vers_level} ge '610'} @$releases;
    }
    @$releases = grep {$_->{install_path}} @$releases;

    # Check if we have GA release
    if ($bundle->{Base}{value} && $bundle->{Patch}{value} && @{$bundle->{Patch}{value}}) {
        # Todo: For each Patch release, find the MR release that will obsolete this Patch
    } elsif ($bundle->{Patch}{value} && @{$bundle->{Patch}{value}}) {
        # Only Patch release defined
        $installed_over=$matrix{hotfix}{$bundle->{Patch}{value}->[0]}{installed_over};
        if ($installed_over && $matrix{patch}{$installed_over}) {
            push(@release_id,$installed_over);
        }
    } elsif ($bundle->{Base}{value}) {
        # Only Base release defined 
        for my $mr (@$releases) {
            @base_releases=split(',',$mr->{installed_over});
            for my $baseid (@base_releases) {
                if ($baseid == $bundle->{Base}{value}) {
                    next if (EDRu::inarr($mr->{release_id}, @release_id));
                    push(@release_id,$mr->{release_id}); last;
                }
            }
        }
    } else {
        # Both base release and patch release are not defined, list all the MR releases supported
        @release_id = map({$_->{release_id}} @$releases);
    }        
    # For MR release, it must be >=6.0.5 to support Install Bundle
    for my $id(@release_id) {
        if ($matrix{patch}{$id}{vers_level} ge '605') {
            push (@supported_release_id, $id);
        }
    }
    for my $id(@supported_release_id) {
        push (@release_version, $matrix{patch}{$id}{release_version});
    }
    # It's possible that a specific GA+Hotfix combination do not need a MR
    unless (scalar(@supported_release_id)) {
        if ($bundle->{Base}{showform} && $bundle->{Patch}{showform}) {
            $msg=Msg::new("No Maintenance Releases are supported in the selected release combination.");
        } else {
            $unsupported_release=$bundle->{Base}{showform}||$bundle->{Patch}{showform};
            if ($unsupported_release) {
                $msg=Msg::new("There are no Maintenance Releases for $unsupported_release on $bundle->{Platform}{showform} at this time");
            } else {
                $msg=Msg::new("There are no Maintenance Releases on $bundle->{Platform}{showform} at this time");
            }
        }
        $msg->print;
        $msg->prtc();
        return 1;
    }
    $msg=Msg::new("Select the Maintenance Release version to add to the Install Bundle $bundle->{Name}{showform}");
    $menu=$msg->menu(\@release_version,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));
    $release=$matrix{patch}{$supported_release_id[$menu-1]};
    $bundle->{Maintenance}{value}=$release->{release_id};
    $bundle->{Maintenance}{showform}=$release->{release_version};
    $bundle->{Saved}=0;
    return 1;
}

sub add_patch_release {
    my ($deploy,$bundle)=@_;
    my (%matrix,$installed_over,@base_mr_releases,@ids,$menu,$msg);
    my ($rel,$releases,@release_id,@supported_release_id,@release_version,$release,$unsupported_release,$obsoleted_releases);
    $rel=Obj::rel();
    Msg::n();
    for my $level (qw/ga patch hotfix/) {
        $matrix{$level}=$rel->get_release_matrix($bundle->{Platform}{value},$level);
    }
    $releases = $deploy->filter_matrix($matrix{hotfix});
    # 6.0.X install bundle do NOT support any-any, need to filter them
    if ((Padv::sort_to_cpi_padv($bundle->{Platform}{value}) ne Obj::localpadv()) && !($bundle->{Platform}{value}=~/^aix/i && Obj::localpadv()=~/^aix/i)) {
        @$releases = grep {$_->{vers_level} ge '610'} @$releases;
    }
    @$releases = grep {$_->{install_path}} @$releases;

    if ($bundle->{Patch}{value} && @{$bundle->{Patch}{value}}) {
        # It is possible that one hotfix obsoletes the other hotfix.
        # Use obsoleted_by and obsoletes keyword to avoid 2 hotfixes have obsolete relationship.
        # Multiple Patch scenario
        $installed_over=$matrix{hotfix}{${$bundle->{Patch}{value}}[0]}{installed_over};
        for my $patch (@$releases) {
            if (($patch->{installed_over} == $installed_over) &&
                !EDRu::inarr($patch->{release_id}, @release_id) && !EDRu::inarr($patch->{release_id},@{$bundle->{Patch}{value}})) {
                push(@release_id,$patch->{release_id});
            }
        }
        # For any patch in @release_id, it cannot be obsoleted by any patch in @{$bundle->{Patch}{value}}
        # When a patch in @release_id obsoletes a patch in @{$bundle->{Patch}{value}}, the obsoleted patch in @{$bundle->{Patch}{value}} will be removed.
        for my $patchid (@{$bundle->{Patch}{value}}) {
            for my $obsolete_patch (split(/,/, $matrix{hotfix}{$patchid}{obsoletes})) {
                if (EDRu::inarr($obsolete_patch, @release_id)) {
                    @release_id=@{EDRu::arrdel(\@release_id, $obsolete_patch)};
                }
            }
        }
    } elsif ($bundle->{Maintenance}{value}) {
        for my $patch (@$releases) {
            @base_mr_releases=split(',',$patch->{installed_over});
            for my $base_mr_id(@base_mr_releases) {
                if ($matrix{patch}{$base_mr_id} && $base_mr_id == $bundle->{Maintenance}{value}) {
                    next if (EDRu::inarr($patch->{release_id}, @release_id));
                    push(@release_id,$patch->{release_id}); last;
                }
            }
        }
    } elsif ($bundle->{Base}{value}) {
        for my $patch (@$releases) {
            @base_mr_releases=split(',',$patch->{installed_over});
            for my $base_mr_id(@base_mr_releases) {
                if ($matrix{ga}{$base_mr_id} && $base_mr_id == $bundle->{Base}{value}) {
                    next if (EDRu::inarr($patch->{release_id}, @release_id));
                    push(@release_id,$patch->{release_id}); last;
                }
            }
        }
    } else {
        @release_id = map({$_->{release_id}} @$releases);
    }
    # For Patch release, it must be >=6.1.0 to support Install Bundle
    for my $id(@release_id) {
        if ($matrix{hotfix}{$id}{vers_level} ge '610') {
            push (@supported_release_id, $id);
        }
    }
    for my $id(@supported_release_id) {
        push (@release_version, $matrix{hotfix}{$id}{release_version});
    }
    # It's possible that a GA/MR release do NOT have a patch
    unless (scalar(@supported_release_id)) {
        $unsupported_release=$bundle->{Maintenance}{showform} || $bundle->{Base}{showform};
        if ($unsupported_release) {
            # Since 6.0.x's patch release doesn't support Install Bundle
            # We need to stress that there's no patch release supports "Install Bundle" at this time.
            $msg=Msg::new("There are no patch releases for $unsupported_release on $bundle->{Platform}{showform} that support Install Bundle at this time");
        } else {
            $msg=Msg::new("There are no more patch releases on $bundle->{Platform}{showform} at this time");
        }
        $msg->print;
        $msg->prtc();
        return 1;
    }
    $msg=Msg::new("Select the Patch Release version to add to the Install Bundle $bundle->{Name}{showform}");
    $menu=$msg->menu(\@release_version,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));

    $release=$matrix{hotfix}{$supported_release_id[$menu-1]};
    
    if ($bundle->{Patch}{value}) {
        # check if this $release_id obsolete anything in $bundle->{Patch}{value}
        undef @release_version;
        for my $obsolete_patch (split(/,/,$release->{obsoletes})) {
            if (EDRu::inarr($obsolete_patch, @{$bundle->{Patch}{value}})) {
                $bundle->{Patch}{value}=EDRu::arrdel($bundle->{Patch}{value},$obsolete_patch);
                push (@release_version, $matrix{hotfix}{$obsolete_patch}{release_version});
            }
        }
        if (@release_version) {
            $obsoleted_releases=join(', ',@release_version);
            Msg::n();
            $msg=Msg::new("The following releases are removed from Install Bundle $bundle->{Name}{showform} because they're obsoleted by $release->{release_version}:\n\n$obsoleted_releases");
            $msg->bold();
            Msg::prtc();
        }
        push (@{$bundle->{Patch}{value}},$release->{release_id});
    } else {
        $bundle->{Patch}{value}=[$release->{release_id}];
    }
    
    # Reset the $bundle->{Patch}{showform}
    undef @release_version;
    for my $release_id (@{$bundle->{Patch}{value}}) {
        push (@release_version, $matrix{hotfix}{$release_id}{release_version});
    }
    $bundle->{Patch}{showform}=join (', ', @release_version);
    $bundle->{Saved}=0;
    return 1;
}

sub remove_base_release {
    my ($deploy,$bundle)=@_;
    delete $bundle->{Base}{value};
    delete $bundle->{Base}{showform};
    $bundle->{Saved}=0;
    return 1;
}

sub remove_mr_release {
    my ($deploy,$bundle)=@_;
    my ($msg,$ayn);
    # When a MR release is removed, the Patch release will be invalid also.
    # Currently we'll give a warning about this.
    if ($bundle->{Base}{showform} && $bundle->{Patch}{showform}) {
        Msg::n();
        $msg=Msg::new("Removing a Maintenance Release will also lead to remove all the Patch Releases defined on top of the Install Bundle $bundle->{Maintenance}{showform}");
        $msg->warning();
        Msg::n();
        $msg=Msg::new("Are you sure you want to continue?");
        return if ($msg->aynn() eq 'N');
        delete $bundle->{Patch}{value};
        delete $bundle->{Patch}{showform};
    }
    delete $bundle->{Maintenance}{value};
    delete $bundle->{Maintenance}{showform};
    $bundle->{Saved}=0;
    return 1;
}

sub remove_patch_release {
    my ($deploy,$bundle)=@_;
    my (@showform,$msg,$menu);
    Msg::n();
    @showform=split(/, /,$bundle->{Patch}{showform});
    $msg=Msg::new("Select the Patch Release version to remove from the Install Bundle $bundle->{Name}{showform}");
    # List the patches in $bundle->{Patch}{value}
    $menu=$msg->menu(\@showform,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));
    splice(@{$bundle->{Patch}{value}}, $menu-1, 1);
    splice(@showform, $menu-1, 1);
    $bundle->{Patch}{showform}=join(', ',@showform);
    $bundle->{Saved}=0;
    return 1;
}

sub save_bundle {
    my ($deploy,$bundle)=@_;
    my ($length,$msg);
    $deploy->read_deploy_config();
    if ($deploy->{bundle} && @{$deploy->{bundle}}) {
        $length=scalar(@{$deploy->{bundle}})-1;
        for my $index (0..$length) {
            if (${$deploy->{bundle}}[$index]->{Name}{showform} eq $bundle->{Name}{showform} || 
                ${$deploy->{bundle}}[$index]->{Name}{showform} eq $bundle->{obsoleted_bundle_name}) {
                delete ${$deploy->{bundle}}[$index]; 
                last; # only one of the above condition can happen:
                # Either ${$deploy->{bundle}}[$index]->{Name}{showform} eq $bundle->{Name}{showform} (in that case user modified the Install Bundle but he definiately didn't change Install Bundle name)
                # or ${$deploy->{bundle}}[$index]->{Name}{showform} eq $bundle->{obsoleted_bundle_name} (in that case user changed Install Bundle name, and Install Bundle name could not be the same)
            }
        }
        push (@{$deploy->{bundle}}, $bundle);
    } else {
        $deploy->{bundle}=[$bundle];
    }
    undef ($bundle->{obsoleted_bundle_name});
    $deploy->write_deploy_config();
    Msg::n();
    $msg=Msg::new("Install Bundle $bundle->{Name}{showform} has been saved successfully");
    $msg->print;
    Msg::prtc();
    $bundle->{Saved}=1;
    $deploy->download_required_releases($bundle);
    return 1;
}

sub change_install_bundle_name {
    my ($deploy,$bundle)=@_;
    my ($msg,$invalid,$name,$length);
    $invalid=1;
    Msg::n();
    $msg=Msg::new("Enter the name of the Install Bundle you would like to redefine: (press [Enter] to go back)");
    while ($invalid) {
        $name=$msg->ask('__back__','','','nullok');
        unless ($name =~ /^\w+$/) {
            Msg::new("\nAn Install Bundle name can only include numbers, letters and underscores.\n")->print;
            next;
        }
        $length = length($name);
        if ($length > 255) {
            Msg::new("\nAn Install Bundle name cannot exceed 255 letters.\n")->print;
            next;
        }
        return 1 if (EDR::getmsgkey($name,'back') || $bundle->{Name}{value} eq $name);
        $invalid=0;
        for my $bd (@{$deploy->{bundle}}) {
            if ($bd->{Name}{value} eq $name) {
                Msg::new("\nThere's already an Install Bundle named $name on $bd->{Platform}{showform} platform, enter another name\n")->print;
                $invalid=1;
                last;
            }
        }
    }

    # Record the obsoleted bundle name, so that it will be removed when saving Install Bundle
    $bundle->{obsoleted_bundle_name}=$bundle->{Name}{showform} unless ($bundle->{obsoleted_bundle_name});
    # Change the name of the bundle
    $bundle->{Name}{showform}=$bundle->{Name}{value}=$name;
    $bundle->{Saved}=0;
    return 1;
}

sub create_bundle_back {
    my ($deploy,$bundle,$bundle_release)=@_;
    my ($msg);
    if (!$bundle->{Saved} && $bundle_release > 1) {
        Msg::n();
        $msg=Msg::new("Install Bundle $bundle->{Name}{showform} has been modifed, would you like to save the changes?");
        if ($msg->ayn() eq 'Y') {
            $deploy->save_bundle($bundle);
        }
    } elsif (!$bundle->{Saved}) {
        Msg::n();
        if ($bundle_release == 0) {
            $msg=Msg::new("Install bundle $bundle->{Name}{showform} currently has zero releases defined. Would you like to delete this bundle?");
        } else {
            $msg=Msg::new("Install bundle $bundle->{Name}{showform} currently has only one release defined. Would you like to delete this bundle?");
        }
        if ($msg->ayn() eq 'Y') {
            # delete the bundle
            $deploy->delete_bundle($bundle->{Name}{showform});
        } else {
            return 0;
        }
    }
    return 1;
}

sub modify_install_bundle {
    my ($deploy)=@_;
    my ($msg,$menu,@bundle_name_array);
    my (@padvs,$padv_preference_set);
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $bd (@{$deploy->{bundle}}) {
        # If the platform of the $bd is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($bd->{Platform}{value},@padvs));
        push (@bundle_name_array, $bd->{Name}{showform});
    }
    Msg::n();
    $msg=Msg::new("Select an Install Bundle to modify");
    $menu=$msg->menu(\@bundle_name_array,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));
    for my $bd (@{$deploy->{bundle}}) {
        if ($bd->{Name}{showform} eq $bundle_name_array[$menu-1]) {
            # Update json file on the corresponding padv
            for my $level (qw/ga patch hotfix/) {
                $deploy->repository_update_matrixfile($bd->{Platform}{value},$level);
            }
            $deploy->modify_install_bundle_page($bd);
            last;
        }
    }
    return 1;
}

sub delete_install_bundle {
    my ($deploy)=@_;
    my ($msg,$menu,@bundle_name_array);
    my (@padvs,$padv_preference_set);
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $bd (@{$deploy->{bundle}}) {
        # If the platform of the $bd is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($bd->{Platform}{value},@padvs));
        push (@bundle_name_array, $bd->{Name}{showform});
    }
    Msg::n();
    $msg=Msg::new("Select an Install Bundle to delete");
    $menu=$msg->menu(\@bundle_name_array,'',0,'back');
    return $menu if (EDR::getmsgkey($menu,'back'));
    $deploy->delete_bundle($bundle_name_array[$menu-1]);
    Msg::n();
    $msg=Msg::new("Install Bundle $bundle_name_array[$menu-1] has been removed successfully");
    $msg->print;
    Msg::prtc();
    return 1;
}

sub delete_bundle {
    my ($deploy,$bundle_name)=@_;
    my ($length);
    # delete the bundle from $deploy->{bundle} && deploy_config
    $length=scalar(@{$deploy->{bundle}})-1;
    for my $index (0..$length) {
        if (${$deploy->{bundle}}[$index]->{Name}{showform} eq $bundle_name) {
            delete ${$deploy->{bundle}}[$index];
            last;
        }
    }
    $deploy->write_deploy_config();
    return 1;
}

sub list_bundles {
    my ($deploy)=@_;
    my ($msg,@bundle_array);
    my (@padvs,$padv_preference_set);
    $msg=Msg::new("Defined Install Bundles:\n");
    $msg->bold;
    # Read deploy_config file
    $deploy->read_deploy_config();
    unless ($deploy->{bundle} && @{$deploy->{bundle}}) {
        $msg=Msg::new("None\n");
        $msg->bold();
        return 0;
    }
    @{$deploy->{bundle}} = sort {
        $a->{Platform}{showform} cmp $b->{Platform}{showform}
        or $a->{Name}{showform} cmp $b->{Name}{showform} 
    } @{$deploy->{bundle}};
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $bd (@{$deploy->{bundle}}) {
        # If the platform of the $bd is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($bd->{Platform}{value},@padvs));
        my %bundle;
        for my $element (@{$deploy->{bundle_elements}}) {
            $bundle{$element}=$bd->{$element}{showform};
        }
        $bundle{'I'}='Y' if ($deploy->check_installable($bd));
        push (@bundle_array,\%bundle);
    }
    my $bundle_table_elements=[ "Name", "I", "Platform", "Base", "Maintenance", "Patch" ];
    Msg::table(\@bundle_array, $bundle_table_elements);
    Msg::n();
    return 1 if (@bundle_array);
    return 0;
}

sub list_templates {
    my ($deploy)=@_;
    my ($msg,@template_array);
    my (@padvs,$padv_preference_set);
    $msg=Msg::new("Defined Install Templates:\n");
    $msg->bold;
    # Read deploy_config file
    $deploy->read_deploy_config();
    unless ($deploy->{template} && @{$deploy->{template}}) {
        $msg=Msg::new("None\n");
        $msg->bold();
        return 0;
    }
    @{$deploy->{template}} = sort {
        $a->{Platform}{showform} cmp $b->{Platform}{showform}
        or $a->{Name}{showform} cmp $b->{Name}{showform}
    } @{$deploy->{template}};
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $tp (@{$deploy->{template}}) {
        # If the platform of the $tp is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($tp->{Platform}{value},@padvs));
        my %template;
        for my $element (@{$deploy->{template_elements}}) {
            $template{$element}=$tp->{$element}{showform};
        }
        $template{'I'}='Y' if ($deploy->check_installable($tp));
        # As Steve suggested, we show the product abbreviation
        # we have to use the {value} attribute and remove the tailing version
        $template{'Product'}=uc($tp->{'Product'}{value});
        $template{'Product'}=~s/\d+$//mx;
        push (@template_array, \%template);
    }
    my $template_table_elements=[ "Name", "I", "Platform", "Base", "Maintenance", "Patch", "Product" ];
    Msg::table(\@template_array, $template_table_elements);
    Msg::n();
    return 1 if (@template_array);
    return 0;
}

sub check_installable {
    my ($deploy,$target)=@_;
    my (%matrix,$rel,$release);
    $rel=Obj::rel();
    $deploy->image_repository_scan();
    for my $level (qw/ga patch hotfix/) {
        $matrix{$level}=$rel->get_release_matrix($target->{Platform}{value},$level);
    }
    if ($target->{Base}{showform}) {
        $release=$matrix{ga}{$target->{Base}{value}};
        return 0 unless ($release->{downloaded});
    }
    if ($target->{Maintenance}{showform}) {
        $release=$matrix{patch}{$target->{Maintenance}{value}};
        return 0 unless ($release->{downloaded});
    }
    if ($target->{Patch}{showform}) {
        for my $patchid (@{$target->{Patch}{value}}) {
            $release=$matrix{hotfix}{$patchid};
            return 0 unless ($release->{downloaded});
        }
    }
    return 1;
}
# _END_Define_Install_bundle

# _START_Install_Templates
sub install_templates {
    my ($deploy) = @_;
    my ($msg,$titlemsg,$opt1,$opt2,$opt3,$menu,$menuopt,$done);
    return unless ($deploy->repository_set_home);
    $titlemsg=Msg::new("Install Templates are a set of packages discovered from a running system used to install the same product/version/packages on to other systems.");
    $deploy->image_repository_scan();
    $opt1=Msg::new("Create a new Install Template")->{msg};
    $opt2=Msg::new("View Install Templates")->{msg};
    $opt3=Msg::new("Delete an Install Template")->{msg};
    while (!$done) {
        Msg::title();
        $titlemsg->printn();
        $msg=Msg::new("Install templates will not be created from a running system when:\n");
        $msg->bold();
        $msg=Msg::new("The release version is lower or equal to 6.1");
        $msg->bullet();
        $msg=Msg::new("Multiple products installed on the system such as SF and AppHA");
        $msg->bullet();
        $msg=Msg::new("An MR release is partially installed");
        $msg->bullet();
        Msg::n();
        if ($deploy->list_templates) {
            $menuopt=[$opt1,$opt2,$opt3];
        } else {
            $menuopt=[$opt1];
        }
        Msg::new("Select a Task:\n")->bold();
        $msg=Msg::new("Select the task you would like to perform");
        $menu=$msg->menu($menuopt,'',0,'back');
        return $menu if (EDR::getmsgkey($menu,'back'));

        $deploy->create_install_template if ($menu==1);
        $deploy->view_install_template if ($menu==2);
        $deploy->delete_install_template if ($menu==3);
    }
    return 1;
}

sub create_install_template {
    my ($deploy)=@_;
    my ($edr, $cfg, $msg, $padv, $rel);
    my ($sys, $localsys, $systems, $inputsystems, $sysname, $localsysname, $localsys_backup);
    my ($pseudo_hotfix, $noipc, $ayn, $rootpath, %updated_padvs, $ident_systems, $invalid, @mpkgs,$mpatch);
    my (%template_candidate, @menuopts, $menu, @target_sys, @prodinfo, $prod, $template_sys);
    
    my $cpic = Obj::cpic();
    $edr = Obj::edr();
    $cfg = Obj::cfg();
    $rel = Obj::rel();
    $rel->require_versionmatrix_lib();
    # Use $cpic->{sortready} from Rel::self_healing to retrieve internet connection

    $rel->{sort_ready_for_vc} = $cpic->{sortready};
    $msg = Msg::new("\nEnter the system names separated by spaces for creating an Install Template: (press [Enter] to go back)");
    $invalid=1;
    while ($invalid) {
        $inputsystems = $msg->ask('__back__','','','nullok');
        return 1 if (EDR::getmsgkey($inputsystems,'back'));
        @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
        if ($edr->validate_systemnames(@{$systems})) {
            next;
        } else {
            # System validation passed, check the communication
            @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
            $rel->{vc_sys} = [];
            $cfg->{systems} = \@{$systems};
            $edr->init_sys_objects();
            Msg::n();
            if ($edr->check_and_setup_transport($edr->{systems}) != -1) {
                $invalid=0;
            }
        }
    }
    $rel->{vc_matrix_path_update}="$deploy->{preference}{Repository}{value}/.matrix/";
    # Since Deployment Server does not support -rootpath, $edr->{installpath} will always be /opt/VRTS/install/
    $rel->{matrix_path_in_installpath}="$edr->{installpath}/.matrix/";
    for my $sysname (@{$systems}) {
        $sys = Obj::sys($sysname);
        $msg = Msg::new("Checking installed products on $sysname");
        $msg->left();
        $sys->pkgs_patches();

        $padv=$rel->get_padv_for_vc($sys);
        $rel->{padv_managed_host_for_vc} = $padv;
        $rel->{vc_host_padv}{$sysname} = $padv;
        push @{$rel->{vc_sys}}, $sysname;

        #one padv only need to be updated for one time.
        unless ($deploy->{updated_release_information}{$padv}{'ga'} && 
            $deploy->{updated_release_information}{$padv}{'patch'} && 
            $deploy->{updated_release_information}{$padv}{'hotfix'}) {
            # If any of the release matrix are not updated, we should call update_matrix_data to update 'ga', 'patch' and 'hotfix'
            $rel->update_matrix_data($pseudo_hotfix, $noipc);
            $deploy->{updated_release_information}{$padv}{'ga'}=1;
            $deploy->{updated_release_information}{$padv}{'patch'}=1;
            $deploy->{updated_release_information}{$padv}{'hotfix'}=1;
        }
        # For Create Template, $rel->init_releasematrix_vc is used to retrieve $rel->{releases}
        $rel->init_releasematrix_vc();
        # Determine what product is actually installed and set mainpkgs
        $rel->set_inst_vrtsprods_sys($sys);
        $rel->check_prods_install_status_sys($sys);
        # Check the extra packages that not belongs to the product
        $rel->check_extra_pkgs($sys);
        $msg->right_done();
    }
    for my $sysname (@{$systems}) {
        $sys = Sys->new($sysname);
        $rel->output_prods_install_status_sys($sys);
        $rel->output_extra_pkgs($sys);
    }
    $rel->reorganize_systems();
    
    my %invalid_template;
    my $invalid_reason1=Msg::new("an MR release is partially installed")->{msg};
    my $invalid_reason2=Msg::new("the version of the installed product is lower than 6.1.1")->{msg};
    my $invalid_reason3=Msg::new("the Deployment Server detects multiple products are installed")->{msg};
    my $invalid_reason4=Msg::new("the Deployment Server cannot detect any product installed")->{msg};
    my $invalid_reason5=Msg::new("the target platform(s) are not in the Platform Preference List: $deploy->{preference}{Platform}{showform}")->{msg}; 
    my $padvs=$deploy->{preference}{Platform}{value}||$deploy->{preference}{Platform}{default};

    # Filter the product-system list for Install Templates
    # The following scenarioes are not supported:
    # 1. Releases prior to 6.1, including the 6.0.1+6.0.5 combination where Install Bundles are supported
    # 2. Partially Upgraded System (SF at one level, VCS at another) or any uninstallable combination (AppHA and SF)
    # 3. Partially installed MRs
    # 4. Packages defined in the product with different versions(CURRENTLY WE DON'T HAVE A WAY TO CHECK THAT)
    # 5. The target platform is not in the platform preference
    for my $key (keys %{$rel->{prodsys}}) {
        @prodinfo=split('-',$key);
        # $prodinfo[0] must be the padv, even without product installed.
        unless (EDRu::inarr($prodinfo[0], @{$padvs})) {
            # the target platforms are not in the Platform Preference list.
            push (@{$invalid_template{$invalid_reason5}}, @{$rel->{prodsys}{$key}});
            next;
        }
        # Should only have 3 elements
        if (scalar(@prodinfo) < 3) {
            # No product installed
            push (@{$invalid_template{$invalid_reason4}}, @{$rel->{prodsys}{$key}});
            next;
        } elsif (scalar(@prodinfo) > 3) {
            # Multiple products installed
            push (@{$invalid_template{$invalid_reason3}}, @{$rel->{prodsys}{$key}});
            next;
        }
        # For RPx/PRx/MPx we don't allow them to create generate template files
        # Because their version are definitely lower than 6.1
        # For MR release, they will not have capital letter, so I think the below condition is necessary and sufficient condition
        if ((@prodinfo[2]=~/[A-Z]/) || (EDRu::compvers('6.1',@prodinfo[2]) < 2)) {
            push (@{$invalid_template{$invalid_reason2}}, @{$rel->{prodsys}{$key}});
            next;
        }
        $prod=$prodinfo[1];
        my @syslist;
        for my $sysname (@{$rel->{prodsys}{$key}}) {
            $mpatch=0;
            $sys = Sys->new($sysname);
            if ($sys->{iprod}{$prod}{mpatches} && %{$sys->{iprod}{$prod}{mpatches}}) {
                @mpkgs=();
                if ($sys->{iprod}{$prod}{mopkgs} && @{$sys->{iprod}{$prod}{mopkgs}}) {
                    push(@mpkgs, @{$sys->{iprod}{$prod}{mopkgs}});
                }
                if ($sys->{iprod}{$prod}{mrpkgs} && @{$sys->{iprod}{$prod}{mrpkgs}}) {
                    push(@mpkgs, @{$sys->{iprod}{$prod}{mrpkgs}});
                }
                # Need to check if the missing patches belongs to missing packages or not
                for my $mpatchpkg (values %{$sys->{iprod}{$prod}{mpatches}{$sys->{iprod}{$prod}{mprpvers}}}) {
                    # each $mpatchpkg is an array reference, assume each patch can only apply to one package, pick up the first one as it's package.
                    unless (EDRu::inarr(${$mpatchpkg}[0], @mpkgs)) {
                        $mpatch=1;
                        last;
                    }
                }
            }
            if ($mpatch) {
                push (@{$invalid_template{$invalid_reason1}}, $sysname);
            } else {
                push (@syslist,$sysname);
            }
        }
        $template_candidate{$key}=\@syslist if (@syslist);
    }

    for my $key (keys%invalid_template) {
        if (@{$invalid_template{$key}}) {
            Msg::n();
            my $syslist=join(', ', @{$invalid_template{$key}});
            $msg=Msg::new("Cannot create an Install Template from the discovered package set on $syslist because $key.");
            $msg->warning();
        }
    }

    if (%template_candidate) {
        Msg::n();
        $msg=Msg::new("Would you like to generate a template file based on the above release information?");
        if ($msg->ayny() eq 'Y') {
            Msg::n();
            # for each array in $template_candidate->{key}, we will not tolerate the following difference:
            # 1. Same product, same release level, different package set installed (such as A with allpkgs and B with recpkgs)
            # 2. Different Patch level, including different version of same patch installed on the systems.
            # The above situation will be regarded as differnet template candidate.
            for my $key (keys %template_candidate) {
                my $candidate_array=$deploy->seperate_template_candidates($template_candidate{$key});
                for my $candidates (@{$candidate_array}) {
                    push (@menuopts,join(',',@{$candidates}));
                }
            }
            # Todo: If there's only one machine detected, do not prompt this question.
            $msg=Msg::new("Select a machine to generate the template file");
            $menu=$msg->menu(\@menuopts,'',0,'back');
            if (!EDR::getmsgkey($menu,'back')) {
                @target_sys=split(',',$menuopts[$menu-1]);
                # the machines in @target_sys should have same package/patch list except the extrapkgs list
                # since we're not able to install extrapkgs, we consider they're the same.
                Msg::log("Picking up $target_sys[0] as the template target");
                $template_sys=Sys->new($target_sys[0]);
                $deploy->save_template($template_sys);
            }
        }
    } else {
        Msg::prtc();
    }
    Msg::n();
    # Do com cleanup here
    $edr->cleanup_com_setup();
    # Clean up all the info
    for my $sysname (@{$systems}) {
        delete $Obj::pool{"Sys::$sysname"};
    }
    delete $cfg->{systems};
    delete $rel->{prodsys};
    # Clean up the systems that has SSH/RSH established 
    undef $systems;
    for my $key (qw/prodsys relname_to_vers_level relid_to_relname relname_to_relid hotfixname_to_vers_level hotfixid_to_hotfixname hotfixname_to_hotfixid/) {
        delete $rel->{$key};
    }
    return 1;
}

# The following information needs to be exactly the same:
# $template_info->{irpkgs}  => installed required packages
# $template_info->{iopkgs}  => installed optional packages
# $template_info->{mrpkgs}  => missing required packages
# $template_info->{mopkgs}  => missing optional packages
# $template_info->{ipatches} => installed patches
# $template_info->{mpatches} => missing patches
# $template_info->{ihotfixes} => installed patches
# Since missing patch is not acceptable. No need to check patches.
sub seperate_template_candidates {
    my ($deploy, $template_candidates) =@_;
    my (@template_info_array, @candidate_array, $in_array, $idx, $sys);
    my (@prods, $prod);
    for my $candidate (@{$template_candidates}) {
        $sys=Sys->new($candidate);
        $in_array=0;
        @prods=keys %{$sys->{iprod}};
        $prod=$prods[0];
        for ($idx=0; $idx<=$#template_info_array; $idx++) {
            if (@{$template_info_array[$idx]->{irpkgs}} ~~ @{$sys->{iprod}{$prod}{irpkgs}} &&
                @{$template_info_array[$idx]->{iopkgs}} ~~ @{$sys->{iprod}{$prod}{iopkgs}} &&
                @{$template_info_array[$idx]->{mrpkgs}} ~~ @{$sys->{iprod}{$prod}{mrpkgs}} &&
                @{$template_info_array[$idx]->{mopkgs}} ~~ @{$sys->{iprod}{$prod}{mopkgs}} &&
                @{$template_info_array[$idx]->{ihotfixes}} ~~ @{$sys->{iprod}{$prod}{ihotfixes}}) {
                push(@{$candidate_array[$idx]},$candidate);
                $in_array=1;
                last;
            }
        }
        if (!$in_array) {
            $template_info_array[$idx]->{irpkgs}=$sys->{iprod}{$prod}{irpkgs};
            $template_info_array[$idx]->{iopkgs}=$sys->{iprod}{$prod}{iopkgs};
            $template_info_array[$idx]->{mrpkgs}=$sys->{iprod}{$prod}{mrpkgs};
            $template_info_array[$idx]->{mopkgs}=$sys->{iprod}{$prod}{mopkgs};
            $template_info_array[$idx]->{ihotfixes}=$sys->{iprod}{$prod}{ihotfixes};
            push(@{$candidate_array[$idx]},$candidate);
        }
    }
    return \@candidate_array;
}

sub save_template {
    my ($deploy, $sys)= @_;
    my ($msg,$template,$invalid,$name,$length,$prod,@prods,@patch_showform,@installpkgs,@missingpkgs,@extrapkgs,$suffix);
    my $rel=Obj::rel();
    $deploy->read_deploy_config();
    # Save the selected template information:
    Msg::n();
    @prods=keys%{$sys->{iprod}};
    if (scalar(@prods)!=1) {
        $msg=Msg::new("Mutiple products found on $sys->{sys}. Saving an Install Template with multiple products is not supported.\n");
        $msg->die;
    }
    $invalid=1;
    
    # Ask Template Name:
    $msg=Msg::new("Enter the name of the Install Template you would like to define: (press [Enter] to go back)");
    while ($invalid) {
        $name=$msg->ask('__back__','','','nullok');
        unless ($name =~ /^\w+$/) {
            Msg::new("\nAn Install Template name can only include numbers, letters and underscores.\n")->print;
            next;
        }
        $length = length($name);
        if ($length > 255) {
            Msg::new("\nAn Install Template name can not exceed 255 letters.\n")->print;
            next;
        }
        return 1 if (EDR::getmsgkey($name,'back'));
        $invalid=0;
        # The Install Template name must be unique
        for my $template (@{$deploy->{template}}) {
            if ($template->{Name}{value} eq $name) {
                Msg::new("\nThere's already an Install Template named $name on $template->{Platform}{showform} platform, enter another name\n")->print;
                $invalid=1;
                last;
            }
        }
    }
    $prod=$prods[0];
    $template->{Name}{showform}=$template->{Name}{value}=$name;
    $template->{Platform}{showform}=$sys->{padv};
    $template->{Platform}{value}=$rel->get_padv_for_vc($sys);
    $template->{Product}{showform}=$sys->{iprod}{$prod}{name};

    $suffix=$sys->{iprod}{$prod}{mjvers};
    $suffix=~s/\.//mg;
    $template->{Product}{value}=uc($prod).$suffix;
    $template->{Base}{showform}=$sys->{iprod}{$prod}{mjvers};
    $template->{Base}{value}=$rel->{relname_to_relid}{$sys->{iprod}{$prod}{mjvers}}{relid};
    # if mjvers ne mprpvers, then we have MR release
    if ($sys->{iprod}{$prod}{mjvers} ne $sys->{iprod}{$prod}{mprpvers}) {
        $template->{Maintenance}{showform}=$sys->{iprod}{$prod}{mprpvers};
        $template->{Maintenance}{value}=$rel->{relname_to_relid}{$sys->{iprod}{$prod}{mprpvers}}{relid};
    }
    if ($sys->{iprod}{$prod}{ihotfixes} && @{$sys->{iprod}{$prod}{ihotfixes}}) {
        for my $patch (@{$sys->{iprod}{$prod}{ihotfixes}}) {
            push (@patch_showform,$rel->{vc_hotfix_matrix_data}{$template->{Platform}{value}}{$patch}{release_name});
        }
        $template->{Patch}{showform}="\n\n\t".join("\n\t", @patch_showform);
        $template->{Patch}{value}=$sys->{iprod}{$prod}{ihotfixes};
    }
    
    if ($sys->{iprod}{$prod}{irpkgs} && @{$sys->{iprod}{$prod}{irpkgs}}) {
        @installpkgs=@{$sys->{iprod}{$prod}{irpkgs}};
    }
    if ($sys->{iprod}{$prod}{iopkgs} && @{$sys->{iprod}{$prod}{iopkgs}}) {
        push (@installpkgs, @{$sys->{iprod}{$prod}{iopkgs}});
    }
    @installpkgs = sort @installpkgs;

    if ($sys->{iprod}{$prod}{mrpkgs} && @{$sys->{iprod}{$prod}{mrpkgs}}) {
        @missingpkgs=@{$sys->{iprod}{$prod}{mrpkgs}};
    }
    if ($sys->{iprod}{$prod}{mopkgs} && @{$sys->{iprod}{$prod}{mopkgs}}) {
        push (@missingpkgs, @{$sys->{iprod}{$prod}{mopkgs}});
    }
    @missingpkgs = sort @missingpkgs;

    if ($sys->{extrapkgs} && @{$sys->{extrapkgs}}) {
        @extrapkgs = sort (@{$sys->{extrapkgs}});
    }
        
    $template->{"Installed Packages"}{showform}="\n\n\t".join("\n\t", @installpkgs);
    $template->{"Installed Packages"}{value}=\@installpkgs;
    if (@missingpkgs) {
        $template->{"Missing Packages"}{showform}="\n\n\t".join("\n\t", @missingpkgs);
        $template->{"Missing Packages"}{value}=\@missingpkgs;
    }
    # We store extrapkgs here but we will not use it in 6.2 release.
    if (@extrapkgs) {
        $template->{"Extra Packages"}{showform}="\n\n\t".join("\n\t", @extrapkgs);
        $template->{"Extra Packages"}{value}=\@extrapkgs;
    }
    if ($deploy->{template} && @{$deploy->{template}}) {
        $length=scalar(@{$deploy->{template}})-1;
        for my $index (0..$length) {
            if (${$deploy->{template}}[$index]->{Name}{showform} eq $template->{Name}{showform}) {
                delete ${$deploy->{template}}[$index];
                last;
            }
        }
        push (@{$deploy->{template}},$template);
    } else {
        $deploy->{template}=[$template];
    }
    $deploy->write_deploy_config();
    Msg::n();
    $msg=Msg::new("Install Template $template->{Name}{showform} has been saved successfully");
    $msg->print;
    Msg::prtc();
    $deploy->download_required_releases($template);
    return 1;
}

sub download_required_releases {
    my ($deploy,$target)=@_;
    my (%matrix,$base_release,$mr_release,@patch_releases);
    my ($rel, $msg, @not_downloaded, $not_downloaded_msg);
    $rel=Obj::rel();
    $deploy->image_repository_scan();

    for my $level (qw/ga patch hotfix/) {
        $matrix{$level}=$rel->get_release_matrix($target->{Platform}{value},$level);
    }

    $base_release=$matrix{ga}{$target->{Base}{value}} if ($target->{Base}{value});
    $mr_release=$matrix{patch}{$target->{Maintenance}{value}} if ($target->{Maintenance}{value});
    if ($target->{Patch}{value} && @{$target->{Patch}{value}}) {
        for my $patch (@{$target->{Patch}{value}}) {
            push (@patch_releases,$matrix{hotfix}{$patch});
        }
    }
    if ($base_release && !$base_release->{downloaded}) {
        push (@not_downloaded, $base_release->{release_version});
    }
    if ($mr_release && !$mr_release->{downloaded}) {
        push (@not_downloaded, $mr_release->{release_version});
    }
    if (@patch_releases) {
        for my $patch (@patch_releases) {
            if (!$patch->{downloaded}) {
                push (@not_downloaded, $patch->{release_version});
            }
        }
    }
    return 1 unless (@not_downloaded);

    $not_downloaded_msg=join(', ', @not_downloaded);
    if (@not_downloaded) {
        Msg::n();
        $msg=Msg::new("The following release(s) are not downloaded:\n$not_downloaded_msg");
        $msg->print();
    }

    # At least one release not downloaded
    if ($deploy->allow_release_download) {
        if ($base_release && !$base_release->{downloaded}) {
            $msg=Msg::new("$base_release->{release_version} has not been downloaded.");
            $msg->warning();
            $msg=Msg::new("\nBase level release images must be downloaded from https://fileconnect.symantec.com/ and loaded using the R) 3) Load a Release Image File option");
            $msg->print;
        }
        if ($mr_release && !$mr_release->{downloaded}) {
            if ($deploy->repository_download($target->{Platform}{value},'patch',$mr_release)) {
                @not_downloaded=@{EDRu::arrdel(\@not_downloaded,$mr_release->{release_version})};
            } else {
                # Failed to download $mr_release
                Msg::n();
                $msg=Msg::new("Failed to download $mr_release->{release_version}");
                $msg->warning;
            }
        }
        if (@patch_releases) {
            for my $patch (@patch_releases) {
                if (!$patch->{downloaded}) {
                    if ($deploy->repository_download($target->{Platform}{value},'hotfix',$patch)) {
                        @not_downloaded=@{EDRu::arrdel(\@not_downloaded,$patch->{release_version})};
                    } else {
                        # Failed to download $patch
                        Msg::n();
                        $msg=Msg::new("Failed to download $patch->{release_version}");
                        $msg->warning;
                    }
                }
            }
        }
        Msg::prtc();
    }
    $deploy->image_repository_scan();
    return 0 if (@not_downloaded);
    return 1;
}

sub view_install_template {
    my ($deploy)=@_;
    my ($msg, $menu, @template_name_array);
    my (@padvs, $padv_preference_set, $length);
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $template (@{$deploy->{template}}) {
        # If the platform of the $bd is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($template->{Platform}{value},@padvs));
        push (@template_name_array, $template->{Name}{showform});
    }
    @template_name_array=sort @template_name_array;
    Msg::n();
    $msg=Msg::new("Select an Install Template to view the details");
    $menu=$msg->menu(\@template_name_array,'',0,'back');
    return $menu if(EDR::getmsgkey($menu,'back'));

    $length=scalar(@{$deploy->{template}})-1;
    for my $index (0..$length) {
        if (${$deploy->{template}}[$index]->{Name}{showform} eq $template_name_array[$menu-1]) {
            $deploy->show_template_details(${$deploy->{template}}[$index]);
            last;
        }
    }
    Msg::prtc();
    return 1;
}

sub show_template_details {
    my ($deploy,$template)=@_;
    my ($msg);
    Msg::n();
    $msg=Msg::new("Details of Install Template $template->{Name}{showform}:\n");
    $msg->bold();
    for my $item (@{$deploy->{template_elements}}) {
        if ($template->{$item}{showform}) {
            Msg::bold("$item: ",1);
            Msg::printn($template->{$item}{showform});
        }
    }
    return 1;
}

sub delete_install_template {
    my ($deploy)=@_;
    my ($msg, $menu, @template_name_array);
    my (@padvs, $padv_preference_set, $length);
    if ($deploy->{preference}{Platform}{value}) {
        @padvs=@{$deploy->{preference}{Platform}{value}};
        $padv_preference_set=1;
    }
    for my $template (@{$deploy->{template}}) {
        # If the platform of the $bd is not in platform preference list, do not list them
        next if ($padv_preference_set && !EDRu::inarr($template->{Platform}{value},@padvs));
        push (@template_name_array, $template->{Name}{showform});
    }
    @template_name_array=sort @template_name_array;
    Msg::n();
    $msg=Msg::new("Select an Install Template to delete");
    $menu=$msg->menu(\@template_name_array,'',0,'back');
    return $menu if(EDR::getmsgkey($menu,'back'));

    $length=scalar(@{$deploy->{template}})-1;
    for my $index (0..$length) {
        if (${$deploy->{template}}[$index]->{Name}{showform} eq $template_name_array[$menu-1]) {
            delete ${$deploy->{template}}[$index];
            last;
        }
    }
    $deploy->write_deploy_config();
    Msg::n();
    $msg=Msg::new("Install Template $template_name_array[$menu-1] has been removed successfully");
    $msg->print;
    Msg::prtc();
    return 1;
}
# _END_Install_Templates

# _START_Upgrade_Install
sub upgrade_install {
    my ($deploy) = @_;
    my (@image_padvs,$supported_padvs,@padvs,$padv,$msg);
    my (@bundles,@templates,$opt1,$opt2,$opt3,$menuopts,$menu,$padvmsg,$back);
     
    $deploy->image_repository_scan();

    # All the padvs in the repository.
    @image_padvs=keys %{$deploy->{image_repository}};
    # All the padvs defined in the bundle.
    for my $bundle (@{$deploy->{bundle}}) {
        if (!EDRu::inarr($bundle->{Platform}{value},@image_padvs)) {
            push (@image_padvs, $bundle->{Platform}{value});
        }
    }
    # All the padvs defined in the bundle.
    for my $template (@{$deploy->{template}}) {
        if (!EDRu::inarr($template->{Platform}{value},@image_padvs)) {
            push (@image_padvs, $template->{Platform}{value});
        }
    }
    @image_padvs=sort @image_padvs;

    $supported_padvs=$deploy->{preference}{Platform}{value}||$deploy->{preference}{Platform}{default};
    for my $element (@image_padvs) {
        push (@padvs, $element) if (EDRu::inarr($element,@{$supported_padvs}));
    }
    
    # @padvs is empty mean on the selected preference platform:
    # 1. No downloaded images to install
    # 2. No Bundles defined
    # 3. No Template defined
    unless (@padvs) {
        Msg::n();
        if ($deploy->{preference}{Platform}{value}) {
            $msg=Msg::new("No downloaded images, Bundles, Templates found under the selected platform(s): $deploy->{preference}{Platform}{showform}");
            $msg->bold();
        } else {
            $msg=Msg::new("No downloaded images, Bundles, Templates found.");
            $msg->bold();
        }
        Msg::prtc();
        return 0;
    }
        
    # @padvs can not be empty from now on
    # When $back eq 'Select Platform', when user choose to go back, it will back to 'Select Platform' menu.
    # When $back eq 'Select Install Method', when user choose to go back, it will skip asking platform, directly ask for upgrade method.
    while (1) {
        if ($deploy->{preference}{Platform}{value} && scalar(@{$deploy->{preference}{Platform}{value}})==1) {
            return 0 if ($back eq 'Select Platform');
            Msg::title();
            $padv=$padvs[0];
            $padvmsg=Msg::get($padv);
            $msg=Msg::new("$padvmsg has been selected as the platform type because $padvmsg is the only platform defined in the Preference menu.\n");
            $msg->bold(1);
        } else {
            # If !$back (Enter the first time) or back to Select Platforms
            if (!$back || $back eq 'Select Platform') {
                $padv=$deploy->upgrade_install_ask_padv(\@padvs);
                return 0 if (EDR::getmsgkey($padv,'back'));
            }
        }
        $padvmsg=Msg::get($padv);
        @bundles=();
        @templates=();
        
        # Check if we have Install Bundles on this padv
        for my $bundle (@{$deploy->{bundle}}) {
            if ($bundle->{Platform}{value} eq $padv) {
                push (@bundles, $bundle);
            }
        }
        # Check if we have Install Templates on this padv
        for my $template (@{$deploy->{template}}) {
            if ($template->{Platform}{value} eq $padv) {
                push (@templates, $template);
            }
        }

        # Always show the complete list of the 3 menu items
        if ($deploy->{preference}{Platform}{value} && scalar(@{$deploy->{preference}{Platform}{value}})==1) {
            Msg::n();
        } else {
            Msg::title();
        }
        $opt1=Msg::new("Install/Upgrade systems using a single release")->{msg};
        $opt2=Msg::new("Install/Upgrade systems using an Install Bundle")->{msg};
        $opt3=Msg::new("Install systems using an Install Template")->{msg};
        $menuopts=[$opt1,$opt2,$opt3];
        $msg=Msg::new("Select the method by which you want to Install/Upgrade your systems");
        $menu=$msg->menu($menuopts,'',0,'back');
        if (EDR::getmsgkey($menu,'back')) {
            $back = 'Select Platform';
            next;
        }
        if ($menu == 1) {
            $deploy->upgrade_install_release($padv);
            $back = 'Select Install Method';
            next;
        } elsif ($menu == 2) {
            if (!@bundles) {
                $msg=Msg::new("\nNo Install Bundles defined on platform $padvmsg");
                $msg->print();
                Msg::prtc();
                $back = 'Select Install Method';
                next;
            }
            $deploy->upgrade_install_bundle($padv,\@bundles);
            $back = 'Select Install Method';
            next;
        } elsif ($menu == 3) {
            if (!@templates) {
                $msg=Msg::new("\nNo Install Templates defined on platform $padvmsg");
                $msg->print();
                Msg::prtc();
                $back = 'Select Install Method';
                next;
            }
            $deploy->upgrade_install_template($padv,\@templates);
            $back = 'Select Install Method';
            next;
        }
    }
    return 1;
}

sub upgrade_install_bundle {
    my ($deploy,$padv,$bundles)=@_;
    my ($envlang,@list,$menu,$msg,$bundle,$cmd,$cmd_hash,$rel_id,$target,$length);
    for my $bd (@{$bundles}) {
        push(@list, $bd->{Name}{showform});
    }
    @list=sort @list;
    Msg::n();
    $msg=Msg::new("Select the bundle to be upgraded/installed");
    $menu=$msg->menu(\@list,'',0,'back');
    return 0 if (EDR::getmsgkey($menu,'back'));
    
    $length=scalar(@{$bundles})-1;
    for my $index (0..$length) {
        if (${$bundles}[$index]->{Name}{showform} eq $list[$menu-1]) {
            $bundle=${$bundles}[$index];
        }
    }
    # Check if all the releases are in repository.
    return 0 unless ($deploy->upgrade_install_check_downloaded($padv,$bundle));

    # Each release is in it's place, need to ask for the target machines
    $target=$deploy->upgrade_install_ask_hostname($padv);
    if ((!$target) || EDR::getmsgkey($target,'back')) {
        return 0;
    } else {
        $cmd_hash=$deploy->upgrade_install_command_option_hash($padv,$bundle);
	if ($padv =~ /^sol/) {
	   $envlang=EDR::get('envlang');
           $cmd="LC_ALL=$envlang; export LC_ALL; cd $cmd_hash->{dirname}; ./$cmd_hash->{basename} ";
	}else{
           $cmd="cd $cmd_hash->{dirname}; ./$cmd_hash->{basename} ";
	}
        for my $key (qw/base_path hotfix_path hotfix2_path hotfix3_path hotfix4_path hotfix5_path 
                                   patch_path patch2_path patch3_path patch4_path patch5_path/) {
            if ($cmd_hash->{$key}) {
                $cmd.="-$key $cmd_hash->{$key} ";
            }
        }
        $rel_id=$bundle->{Base}{value} || $bundle->{Maintenance}{value} || ${$bundle->{Patch}{value}}[0];
        $cmd .= "-noipc " if ($deploy->set_noipc_option($padv,$rel_id));
        $cmd .= "-ds " if ($deploy->support_shikra_features($padv,$rel_id)); # indicates it is executed by the Deployment Server
        $cmd .= "$target";
        # Check Deployment Server Lock:
        return 0 unless ($deploy->add_deploy_sfha_bundle_lock($bundle,"install_bundle_$bundle->{Name}{showform}",$target));
        # all the needed information are here. Let's Install!
        $msg=Msg::new("\nExecuting the following command to start installation of Install Bundle $bundle->{Name}{showform} on $padv:\n\n\t$cmd\n");
        $msg->bold();
        system("$cmd");
        $deploy->clear_deploy_sfha_bundle_lock($bundle,"install_bundle_$bundle->{Name}{showform}",$target);
        $msg->prtc();
    }
    return 1;
}

sub upgrade_install_template {
    my ($deploy,$padv,$templates)=@_;
    my (@list,$menu,$msg,$rel,$template,$target,$cmd_hash,$cmd,$rel_id,$response,$responsefile,$ignorepkgs,$prod);
    my ($edr,@sysnames,$sysobj,@systems,$length);
    $rel=Obj::rel();
    $edr=Obj::edr();
    for my $template (@{$templates}) {
        push (@list, $template->{Name}{showform});
    }
    @list=sort @list;
    Msg::n();
    $msg=Msg::new("Select the template to be installed");
    $menu=$msg->menu(\@list,'',0,'back');
    return 0 if (EDR::getmsgkey($menu,'back'));

    $length=scalar(@{$templates})-1;
    for my $index (0..$length) {
        if (${$templates}[$index]->{Name}{showform} eq $list[$menu-1]) {
            $template=${$templates}[$index];
        }
    }
    # Check if all the releases are in repository.
    return 0 unless ($deploy->upgrade_install_check_downloaded($padv,$template));
    # Each release is in it's place, need to ask for the target machines
    $target=$deploy->upgrade_install_ask_hostname($padv);
    if ((!$target) || EDR::getmsgkey($target,'back')) {
        return 0;
    } else {
        # Check Deployment Server Lock:
        return 0 unless ($deploy->add_deploy_sfha_bundle_lock($template,"install_template_$template->{Name}{showform}",$target));
        # Check communication based on incident 3543559
        @sysnames = @{EDRu::arruniq(split(/\s+/m, $target))};
        for my $sysname (@sysnames) {
            $sysobj=$Obj::pool{"Sys::$sysname"} || Sys->new($sysname);
            push (@systems,$sysobj);
        }
        Msg::n();
        if ($edr->check_and_setup_transport(\@systems) == -1) {
            # Finally give up setup communication, installation will not be possible, abort
            return 0;
        }
        $cmd_hash=$deploy->upgrade_install_command_option_hash($padv,$template);
        $rel_id=$template->{Base}{value} || $template->{Maintenance}{value} || ${$template->{Patch}{value}}[0];
        $response=<<'RESPONSE_FILE';
our %CFG;

$CFG{accepteula}=1;
$CFG{opt}{configure}=0;
$CFG{opt}{nolic}=1;
$CFG{opt}{install}=1;
$CFG{opt}{installallpkgs}=1;
RESPONSE_FILE
        # Fix the discrepancy between APPHA in json file but APPLICATIONHA in responsefile
        $prod=$template->{Product}{value};
        $prod=~s/APPHA/APPLICATIONHA/mgx;
        $response.="\$CFG{opt}{prod}=\"$prod\";\n";
        $response.="\$CFG{systems}=[ qw($target) ];\n";
        for my $key (qw/base_path hotfix_path hotfix2_path hotfix3_path hotfix4_path hotfix5_path
                                  patch_path patch2_path patch3_path patch4_path patch5_path/) {
            if ($cmd_hash->{$key}) {
                $response.="\$CFG{opt}{$key}=\"$cmd_hash->{$key}\";\n";
            }
        }
        # Missing packages
        if ($template->{"Missing Packages"}{value} && @{$template->{"Missing Packages"}{value}}) {
            $ignorepkgs=join(' ',@{$template->{"Missing Packages"}{value}});
            $response.="\$CFG{opt}{ignorepkgs}=[ qw($ignorepkgs) ];\n";
        }
        if ($deploy->set_noipc_option($padv,$rel_id)) {
            $response.="\$CFG{opt}{noipc}=1;\n";
        }
        if ($deploy->support_shikra_features($padv,$rel_id)) {
            $response.="\$CFG{opt}{ds}=1;\n"
        }
        Msg::log("temporary responsefile:\n$response");
        $responsefile= EDR::tmpdir() . "/$template->{Name}{showform}";
        EDRu::writefile($response, $responsefile);
	if ($padv =~ /^sol/) {
            $cmd="LC_ALL=$edr->{envlang}; export LC_ALL; cd $cmd_hash->{dirname}; ./$cmd_hash->{basename} -responsefile $responsefile";
	}else{
            $cmd="cd $cmd_hash->{dirname}; ./$cmd_hash->{basename} -responsefile $responsefile";
	}
        if ($template->{'Extra Packages'}{showform}) {
            $msg=Msg::new("\nThe following Extra Packages stored in Install Template $template->{Name}{showform} will not be installed:$template->{'Extra Packages'}{showform}");
            $msg->print();
        }
        # all the needed information are here. Let's Install!
        $msg=Msg::new("\nExecuting the following command to start installation of Install Template $template->{Name}{showform} on $padv:\n\n\t$cmd\n");
        $msg->bold();
        system("$cmd");
        $edr->cleanup_com_setup();
        $deploy->clear_deploy_sfha_bundle_lock($template,"install_template_$template->{Name}{showform}",$target);
        $msg->prtc();
    }
    return 1;
}

sub upgrade_install_release {
    my ($deploy,$padv)=@_;
    my (@releases,$msg,$release,$target);
    # find the available releases for selected $padv
    @releases=();
    $deploy->image_repository_scan();
    for my $rel_id (sort keys %{$deploy->{image_repository}{$padv}}) {
        push (@releases,$rel_id) if ( $deploy->{image_repository}{$padv}{$rel_id}{install_path}  &&
                              (-f $deploy->{image_repository}{$padv}{$rel_id}{install_path}) &&
                             $deploy->deployment_server_supported_installable_hotfix($padv,$rel_id) &&
                             $deploy->deployment_server_supported_installable_release($padv,$rel_id));
    }

    if ($#releases < 0) {
        # if no releases found for $padv, then we need to point out possibly filtered out releases
        $padv=Msg::get($padv);
        Msg::n();
        $msg=Msg::new("Repository images for $padv must be executed from a $padv system. Releases from 6.1 onwards can be installed from any supported platform.");
        $msg->bold();
        Msg::prtc();
        return 0;
    } else {
        $release=$deploy->upgrade_install_ask_release(\@releases);
        return 0 if (EDR::getmsgkey($release,'back'));
        $target=$deploy->upgrade_install_ask_hostname($padv);
        if ((!$target) || EDR::getmsgkey($target,'back')) {
            return 0;
        } else {
            $deploy->upgrade_install_padv_release($padv,$release,$target);
            Msg::prtc();
        }
    }
    return 1;
}

sub upgrade_install_check_downloaded {
    my ($deploy,$padv,$target)=@_;
    my (@not_downloaded,$not_downloaded_string,$matrix_hotfix);
    my ($rel,$msg);
    $rel=Obj::rel();
    $deploy->image_repository_scan();
    for my $key (qw/Base Maintenance/) {
        # If we have Base release defined but install_path not defined or install_path doesn't exist,
        # Consider it is not downloaded
        if ($target->{$key}{showform} && (!$deploy->{image_repository}{$padv} || !$deploy->{image_repository}{$padv}{$target->{$key}{showform}} ||
                                          !$deploy->{image_repository}{$padv}{$target->{$key}{showform}}{install_path} ||
                                          !(-f $deploy->{image_repository}{$padv}{$target->{$key}{showform}}{install_path}))) {
            push (@not_downloaded, $target->{$key}{showform});
        }
    }
    if ($target->{Patch}{showform}) {
        $matrix_hotfix=$rel->get_release_matrix($padv,'hotfix');
        for my $key (@{$target->{Patch}{value}}) {
            unless ($deploy->{image_repository}{$padv} && $deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}} &&
                   (-f $deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path})) {
                    push (@not_downloaded, $matrix_hotfix->{$key}{release_version});
            }
        }
    }
    if (@not_downloaded) {
        Msg::n();
        $not_downloaded_string=join(', ',@not_downloaded);
        $msg=Msg::new("The following release(s) are not downloaded:\n$not_downloaded_string");
        $msg->printn();
        $msg=Msg::new("Ensure all the releases defined are downloaded and try again");
        $msg->print();
        Msg::prtc();
        return 0;
    }
    return 1;
}

sub upgrade_install_command_option_hash {
    my ($deploy,$padv,$target)=@_;
    my ($install_path,$cmd,$basename,$dirname,$counter,$rel,$matrix_hotfix);
    my ($rel_id,$option);
    $rel=Obj::rel();
    $matrix_hotfix=$rel->get_release_matrix($padv,'hotfix');
    # Check if this bundle/template version is >= 6.2
    # If so, we need to use -patch_path instead of -hotfix_path
    $rel_id = $target->{Base}{value} || $target->{Maintenance}{value} || ${$target->{Patch}{value}}[0];
    if ($deploy->support_shikra_features($padv,$rel_id)) {
        $option="patch";
    } else {
        $option="hotfix";
    }
    if ($target->{Maintenance}{showform}) {
        $basename=EDRu::basename($deploy->{image_repository}{$padv}{$target->{Maintenance}{showform}}{install_path});
        $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$target->{Maintenance}{showform}}{install_path});
        $cmd->{"dirname"}=$dirname;
        $cmd->{"basename"}=$basename;
        if ($target->{Base}{showform}) {
            $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$target->{Base}{showform}}{install_path});
            $cmd->{"base_path"}=$dirname;
        }
        if ($target->{Patch}{showform}) {
            $counter=0;
            for my $key (@{$target->{Patch}{value}}) {
                $counter++;
                $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path});
                if ($counter == 1) {
                    $cmd->{"$option"."_path"}=$dirname;
                } else {
                    $cmd->{"$option"."$counter"."_path"}=$dirname;
                }
            }
        }
    } elsif ($target->{Base}{showform}) {
        $basename=EDRu::basename($deploy->{image_repository}{$padv}{$target->{Base}{showform}}{install_path});
        $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$target->{Base}{showform}}{install_path});
        $cmd->{"dirname"}=$dirname;
        $cmd->{"basename"}=$basename;
        if ($target->{Patch}{showform}) {
            $counter=0;
            for my $key (@{$target->{Patch}{value}}) {
                $counter++;
                $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path});
                if ($counter == 1) {
                    $cmd->{"$option"."_path"}=$dirname;
                } else {
                    $cmd->{"$option"."$counter"."_path"}=$dirname;
                }
            }
        }
    } else {
        $counter=0;
        for my $key (@{$target->{Patch}{value}}) {
            if ($cmd) {
                $counter++;
                $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path});
                if ($counter == 1) {
                    $cmd->{"$option"."_path"}=$dirname;
                } else {
                    $cmd->{"$option"."$counter"."_path"}=$dirname;
                }
            } else {
                $basename=EDRu::basename($deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path});
                $dirname=EDRu::dirname($deploy->{image_repository}{$padv}{$matrix_hotfix->{$key}{release_version}}{install_path});
                $cmd->{"dirname"}=$dirname;
                $cmd->{"basename"}=$basename;
            }
        }
    }
    return $cmd;
}

sub upgrade_install_padv_release {
    my ($deploy,$padv,$release,$target) = @_;
    my ($envlang,$rel_id,$msg,$error,$install_path,$basename,$image,$cmd);

    $msg="platform," if (!$padv);
    $msg.="release," if (!$release);
    $msg=~s/,$//g;
    if ($msg) {
        $error=Msg::new("No $msg defined. Cannot proceed without proper release information provided");
        $error->error();
    } elsif (!$deploy->{image_repository}{$padv}{$release}{install_path} ) {
        # should not hit this
        $error=Msg::new("No install_path defined for $release on $padv.");
        $error->error();
    } else {
        $install_path=$deploy->{image_repository}{$padv}{$release}{install_path};
        if (!-f $install_path) {
            # should not hit this either
            $error=Msg::new("Non-existing install_path directory $install_path found");
            $error->error();
        } else {
            $install_path=$deploy->{image_repository}{$padv}{$release}{install_path};
            if ($deploy->{image_repository}{$padv}{$release}{rel_id}) {
                $rel_id=$deploy->{image_repository}{$padv}{$release}{rel_id};
                $deploy->get_repository_release_info($padv,$rel_id);
            }
            $basename=EDRu::basename($install_path);
            $install_path=~s/$basename$//;
            $image=(split /\//,$deploy->{image_repository}{$padv}{$release}{image})[-1];
            $error=$deploy->add_deploy_sfha_lock($image,"install",$target);
            if (@$error) {
                $msg=Msg::new("Found the following conflicts for installation of $release on $target:\n");
                $msg->print();
                $deploy->display_deploy_sfha_conflicts($error);
                sleep 3;
                return 0;
            }
            $basename.=" -noipc" if ($deploy->set_noipc_option($padv,$rel_id));
            $basename.=" -ds" if ($deploy->support_shikra_features($padv,$rel_id));
	    if ($padv =~ /^sol/) {
	        $envlang=EDR::get('envlang');
                $cmd="LC_ALL=$envlang; export LC_ALL; cd $install_path; ./$basename $target";
	    }else{
                $cmd="cd $install_path; ./$basename $target";
	    }
            $msg=Msg::new("\nExecuting the following command to start installation of $release on $padv:\n\n\t$cmd\n");
            $msg->bold();
	    system("$cmd");
            $error=$deploy->clear_deploy_sfha_lock($image,"install",$target);
        }
    }
    sleep 6;
    return;
}

sub upgrade_install_ask_padv {
    my ($deploy,$padvs)=@_;
    my ($msg,@menu,$help,$ret);
    Msg::title();
    $msg=Msg::new("Select the platform of the available release(s) to be upgraded/installed");
    $help=Msg::new("Select a platform of images within the available repository");
    @menu = map {Msg::get($_)} @$padvs;
    $ret=$msg->menu(\@menu,'',$help,'back');
    return $ret if (EDR::getmsgkey($ret,'back'));
    return $padvs->[$ret-1];
}

sub upgrade_install_ask_hostname {
    my ($deploy,$padv)=@_;
    my ($msg,$padvmsg,@menu,$option,$help,$ret,$default);
    Msg::n();
    $padvmsg=Msg::get($padv);
    $msg=Msg::new("Enter the $padvmsg target system name(s) separated by spaces: (press [Enter] to go back)");
    $help=Msg::new("Enter the names of the system(s) where you would like to install or upgrade the selected platform release");
    $ret=$msg->ask('__back__','','','nullok');
    return $ret if (EDR::getmsgkey($ret,'back'));
    $ret=~s/\s+/ /g;
    return $ret;
}

sub upgrade_install_ask_release {
    my ($deploy,$releases)=@_;
    my ($msg,$help,$ret,$default);
    Msg::n();
    $msg=Msg::new("Select the release image of the available release(s) to be upgraded/installed");
    if ($#$releases==0) {
        $msg=Msg::new("Only one release found");
        $default='1';
    }
    $help=Msg::new("Select a release of images within the available repository");
    $ret=$msg->menu($releases,$default,$help,'back');
    return $ret if (EDR::getmsgkey($ret,'back'));
    return $releases->[$ret-1];
}

sub get_repository_release_info {
    my ($deploy,$padv,$rel_id) = @_;

    my $ret="";
    for my $key (sort keys %{$deploy->{image_repository}{$padv}{$rel_id}}) {
        next if (ref($deploy->{image_repository}{$padv}{$rel_id}{$key}) ne 'SCALAR');
        $ret.="\t$key\t: $deploy->{image_repository}{$padv}{$rel_id}{$key}\n";
    }
    return $ret;
}

# only 6.1 hotfixes are installable through deployment server
sub deployment_server_supported_installable_hotfix {
    my ($deploy,$padv,$rel_id) = @_;
    my ($path,$base_release);
    $base_release='6.1';
    $path = $deploy->{image_repository}{$padv}{$rel_id}{install_path};
    return 1 if (($path !~ /\/hotfix\//) || (EDRu::compvers($rel_id, $base_release) <2));
    return 0;
}

# 6.1 onwards releases support any-to-any
# and older releases on the same platform can be installed on deployment server
sub deployment_server_supported_installable_release {
    my ($deploy,$padv,$rel_id) = @_;
    my ($rel,$localsys,$localpadv,$base_release,$target_release);
    $rel=Obj::rel();
    $localsys=Obj::localsys();
    $localpadv=Padv::cpi_to_sort_padv($localsys->{padv});
    $base_release="610-000";

    $rel_id=$deploy->{image_repository}{$padv}{$rel_id}{'rel_id'};
    $target_release=$rel->{vc_release_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_patch_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_hotfix_matrix_data}{$padv}{$rel_id}{vers_level};
    return 1 if ($target_release ge $base_release);
    return 1 if ($padv eq $localpadv);
    return 1 if (($padv =~ /^AIX\d+/i) && ($localpadv =~ /^AIX\d+/i));
    return 0;
}

sub set_noipc_option {
    my ($deploy,$padv,$rel_id)=@_;
    my ($base_release,$target_release,$rel);
    
    return 0 unless (Cfg::opt('noipc'));
    $base_release="610-000";
    $rel=Obj::rel();
    $target_release=$rel->{vc_release_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_patch_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_hotfix_matrix_data}{$padv}{$rel_id}{vers_level};
    return 1 if ($target_release ge $base_release);
    return 0;
}

sub support_shikra_features {
    my ($deploy,$padv,$rel_id)=@_;
    my ($base_release,$target_release,$rel);

    $base_release="620-000";
    $rel=Obj::rel();
    $target_release=$rel->{vc_release_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_patch_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_hotfix_matrix_data}{$padv}{$rel_id}{vers_level};
    return 1 if ($target_release ge $base_release);
    return 0;
}
# _END_Upgrade_Install

# _START_Update_Metadata
sub update_metadata {
    my ($deploy)=@_;
    my ($cpic,$rel,$dir,$edr,$menu,$msg,$response,$ret,$url);
    my ($double_quote,$link);
    $cpic = Obj::cpic();
    $edr  = Obj::edr();
    $rel  = Obj::rel();
    $rel->require_versionmatrix_lib();
    if (!$deploy->{preference}{Repository}{value}) {
        Msg::log("The repository directory is not set. The directory can be set using the 'Set Preferences' task");
    }
    $double_quote="\"";
    $link="https://sort.symantec.com/support/related_links/offline-release-updates";
    $deploy->{install_dir} = '/opt/VRTS/install';
    $deploy->{supported_padvs} = [qw(aix53 aix61 aix71 hpux1123 hpux1131 rhel4_x86_64 rhel5_x86_64 rhel6_x86_64 rhel7_x86_64 sles9_x86_64 sles10_x86_64 sles11_x86_64 sles12_x86_64 sol8_sparc sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64)];
    $deploy->{default_metadata_file} = 'deploy_sfha.tar';
    $deploy->{default_metadata_name} = 'deploy_sfha';
    $dir = "$edr->{tmpdir}/metadata/$deploy->{default_metadata_name}";
    $edr->{localsys}->mkdir($dir) if (!-d $dir);
    $edr->{savelog} = 1;
    $url = CPIP::get_hotfix_sort_url();
    # Need to fill in level,padv,product for a correct page.
    $url.='?level=ga&padv=sol10_sparc&product=SFHA';
    if (Cfg::opt('load_metadata')) {
        # If specified -load_metadata, do NOT check for SORT connectivity, just expand the deploy_sfha.tar file
        my $metadata=Cfg::opt('load_metadata');
        $deploy->expand_and_load_tarfile(EDRu::dirname($metadata), EDRu::basename($metadata));
    } elsif (Cfg::opt('noipc')) {
        $msg=Msg::new("Download release matrix and installer patches is not allowed while -noipc option is being used.");
        Msg::n();
        $msg->bold();
        $ret = $deploy->load_re_matrix_or_install_hotfixes();
        return if EDR::getmsgkey($ret,'back');
        Msg::prtc();
        return;
    } else {
        Msg::n() unless (Cfg::opt('download_metadata'));
        $msg = Msg::new("Checking SORT connectivity");
        $msg->left();
        $response = EDRu::download_page($url);
        if (!$response) {
            $msg->right_failed();
            $cpic->{sortready}=0;
            Msg::n();
            $msg = Msg::new("Cannot connect with SORT (https://sort.symantec.com) to download update files");
            $msg->printn;
            return if (Cfg::opt('download_metadata'));
            $msg = Msg::new("When you install the Deployment Server, the installation includes the product metadata with up-to-date release matrices. The release matrices include real-time information about the latest product updates and installer patches available across all products and platforms. If your system has Internet access, the release matrix information is automatically updated from the SORT site with each use. If your system is behind a firewall, the matrices cannot be updated and they will eventually get out of date.");
            $msg->printn;
            $msg = Msg::new("To update release information on systems without Internet access, you can download a .tar file (deploy_sfha.tar) containing all the latest release matrices. Then, load it on to your Deployment Server.  Go to $link and click on ${double_quote}deploy_sfha.tar [Download]${double_quote}, and save it to your desktop. Then, copy the file to a location accessible to the Deployment Server, and load it by entering the file location at the prompt below:");
            $msg->printn;
            $ret = $deploy->load_re_matrix_or_install_hotfixes();
            return if EDR::getmsgkey($ret,'back');
            Msg::prtc();
            return;
        }
        $msg->right_done();
        $cpic->{sortready}=1;
        $msg=Msg::new("The Update Metadata option is used to load release matrix updates on to systems that do not have an Internet connection with SORT (https://sort.symantec.com).  Your system has a connection with SORT and is able to receive updates.  No action is necessary unless you would like to create a file to update another Deployment Server system.");
        if (Cfg::opt('download_metadata')) {
            $deploy->download_and_tar_re_matrix_and_install_hotfixes();
        } else {
            while (1) {
                Msg::title();
                $msg->printn();
                $menu = $deploy->metadata_menu();
                last if (EDR::getmsgkey($menu,'back'));
            }
        }
    }
    return;
}

sub metadata_menu {
    my ($deploy)=@_;
    my ($backopt,$def,$help,$menuopts,$msg,$ret,$menu);
    $backopt = 1;
    $help = Msg::new("Release matrix files are reference files used for storing all Storage Foundation release information.");
    $msg = Msg::new("Download release matrix updates and installer patches");
    push (@$menuopts,$msg->{msg});
    $msg = Msg::new("Load an update tar file");
    push (@$menuopts,$msg->{msg});
    $msg = Msg::new("Select the option:");
    $menu = $msg->menu($menuopts, $def, $help, $backopt);
    return $menu if EDR::getmsgkey($menu,'back');
    Msg::n();
    if ($menu == 1) {
        $deploy->download_and_tar_re_matrix_and_install_hotfixes();
        Msg::prtc();
    } elsif ($menu == 2) {
        $ret = $deploy->load_re_matrix_or_install_hotfixes();
        return if EDR::getmsgkey($ret,'back');
        Msg::prtc();
    }
    return;
}

sub download_re_matrix_meta_files {
    my ($deploy)=@_;
    my ($rel,$msg,$noipc,$pseudo_hotfix,$ret,$tmpdir,@padvs);
    $rel=Obj::rel();
    $tmpdir = EDR::tmpdir();
    $rel->{sort_ready_for_vc} = 1;
    $rel->{vc_matrix_path_update} = "$tmpdir/metadata/$deploy->{default_metadata_name}/matrix/";
    @padvs=@{$deploy->{preference}{Platform}{value}||$deploy->{supported_padvs}};
    for my $padv(@padvs) {
        $rel->{padv_managed_host_for_vc} = $padv;
        $msg = Msg::new("Downloading release matrix updates for platform $padv");
        $msg->left;
        $ret = $rel->update_matrix_data($pseudo_hotfix, $noipc);
        ($ret) ? $msg->right_failed : $msg->right_done;
    }
    return;
}

sub download_install_hotfixes {
    my ($deploy)=@_;
    my ($content,$cpic,$all_hotfix,$msg,$release,$sort_status,$tmpdir,@padvs);
    $cpic = Obj::cpic();
    $tmpdir = EDR::tmpdir();
    @padvs=@{$deploy->{preference}{Platform}{value}||$deploy->{supported_padvs}};
    for my $padv(@padvs) {
        ($sort_status,$all_hotfix) = $deploy->get_all_install_hotfix_via_sort_webservice($padv);
        if (!$sort_status) {
            $msg = Msg::new("Failed to download installer patches from the SORT website (https://sort.symantec.com)");
            $msg->error();
            last;
        }
        if (!@$all_hotfix) {
            Msg::log("Cannot get the install patches for padv $padv");
            next;
        }
        $cpic->{hotfixdir} = "$tmpdir/metadata/$deploy->{default_metadata_name}/cpihotfix";
        for my $install_hotfix(@$all_hotfix) {
            $release = $install_hotfix->{release_name};
            $msg = Msg::new("Downloading installer patch for release $release on platform $padv");
            $msg->left;
            $content = Rel::get_hotfix_content_from_gzfile($install_hotfix);
            if (!$content) {
                $msg->right_failed;
                next;
            }
            $deploy->save_install_hotfix($install_hotfix,$content,$padv,$release);
            $msg->right_done;
        }
    }
    return;
}

sub get_all_install_hotfix_via_sort_webservice {
    my ($deploy,$padv)=@_;
    my ($rel,$all_hotfix,$ahotfix,$json,$response);
    $rel=Obj::rel();
    $all_hotfix = [];

    ($response,$json) = Rel::get_hotfix_json_via_sort($padv);
    if (($response) && (ref($json->{$padv}) eq 'HASH')) {
        for my $id (keys %{$json->{$padv}}) {
            $ahotfix = $json->{$padv}{$id};
            push (@$all_hotfix,$ahotfix);
        }
    }
    return ($response,$all_hotfix);
}

sub download_and_tar_re_matrix_and_install_hotfixes {
    my ($deploy)=@_;
    my ($msg,$ret,$edr,$tmpdir);
    $edr = Obj::edr();
    $tmpdir = "$edr->{tmpdir}/metadata/$deploy->{default_metadata_name}";
    EDR::cmd_local("_cmd_rmr $tmpdir");
    $deploy->download_re_matrix_meta_files();
    $deploy->download_install_hotfixes();
    $ret = $deploy->tar_metadata_files();
    if ($ret) {
        $msg = Msg::new("File $deploy->{default_metadata_file} was not successfully created.");
        $msg->error();
    } else {
        $msg = Msg::new("File $deploy->{default_metadata_file} is saved at $deploy->{install_dir}. It must be copied to a file system accessible by the Deployment Server.");
        $msg->nprint;
    }
    return $ret;
}

sub tar_metadata_files {
    my ($deploy)=@_;
    my($dest_dir,$tarfile,$ret,$tmpdir);
    my $localsys = Obj::localsys();
    $tmpdir = EDR::tmpdir();
    $dest_dir = $deploy->{install_dir};
    $tarfile  = $deploy->{default_metadata_file};
    $localsys->mkdir($dest_dir) if (!-d $dest_dir);
    EDR::cmd_local("cd $tmpdir/metadata; _cmd_tar -cf $tarfile $deploy->{default_metadata_name}");
    $ret ||= EDR::cmdexit();
    EDR::cmd_local("_cmd_cp -pf $tmpdir/metadata/$tarfile $dest_dir");
    $ret ||= EDR::cmdexit();
    return $ret;
}

sub save_install_hotfix {
    my ($deploy,$install_hotfix,$content,$padv,$release) = (@_);
    my ($file,$tmpdir);
    $file = "${release}_$padv.pl";
    $tmpdir = EDR::tmpdir();
    $content = "#HOTFIX_VERSION:$install_hotfix->{release_name}\n$content";
    EDRu::writefile($content,"$tmpdir/metadata/$deploy->{default_metadata_name}/cpihotfix/$file");
    return;
}

sub load_re_matrix_or_install_hotfixes {
    my ($deploy)=@_;
    my ($answer,$backopt,$defopt,$dir,$file,$msg);
    $backopt = 1;
    $defopt = "$deploy->{install_dir}/$deploy->{default_metadata_file}";
    $msg = Msg::new("Enter the location of the meta-data tar file:");
    $answer = $msg->ask($defopt,'',$backopt);
    return $answer if EDR::getmsgkey($answer,'back');
    if (-f $answer) {
        $dir = EDRu::dirname($answer);
        $file = EDRu::basename($answer);
        $deploy->expand_and_load_tarfile($dir,$file);
    } else {
        $msg =Msg::new("The tar file does not exist. Input again");
        $msg->print;
        $deploy->load_re_matrix_or_install_hotfixes();
    }
    return;
}

sub expand_and_load_tarfile {
    my ($deploy,$src_dir,$tarfile) = (@_);
    my (@dest_dirs,$cpic,$msg,$repo_dir,$ret,$tmpdir);
    my $localsys = Obj::localsys();
    $cpic = Obj::cpic();
    $repo_dir = $deploy->{preference}{Repository}{value};
    $repo_dir =~ s/\/+$//g;

    push (@dest_dirs,$deploy->{install_dir});
    if ($repo_dir && ($repo_dir ne $deploy->{install_dir})) {
        push (@dest_dirs,$repo_dir);
    }

    $tmpdir = EDR::tmpdir()."/metadata";
    $localsys->mkdir($tmpdir) if (!-d $tmpdir);
    if ($tarfile =~/\.tar$/g) {
        # untar this file
        EDR::cmd_local("_cmd_cp -pf $src_dir/$tarfile $tmpdir; cd $tmpdir; _cmd_tar xf $tarfile 2>/dev/null");
    } elsif ($tarfile =~/\.tar\.gz$/g) {
        # unzip and the untar this file
        EDR::cmd_local("_cmd_cp -pf $src_dir/$tarfile $tmpdir; cd $tmpdir; _cmd_gunzip -c $tarfile | _cmd_tar xf - 2>/dev/null");
    } else {
        $msg = Msg::new("File name must end with .tar or .tar.gz");
        $msg->print;
        return;
    }
    if (EDR::cmdexit() || (!-d "$tmpdir/$deploy->{default_metadata_name}")) {
        $msg = Msg::new("Failed to expand the tar file $tarfile");
        $msg->print;
        return;
    }
    $tmpdir .= "/$deploy->{default_metadata_name}";
    $ret = $deploy->verify_and_update_metadata($tmpdir,@dest_dirs);
    if ($ret) {
        if (@dest_dirs > 1) {
            $msg = Msg::new("Metadata files in directories $repo_dir and $deploy->{install_dir} are updated");
        } else {
            $msg = Msg::new("Metadata files in directory $deploy->{install_dir} are updated");
        }
    } else {
        if (@dest_dirs > 1) {
            $msg = Msg::new("None of the metadata files in the tar file $tarfile needs to be loaded to directories $repo_dir or $deploy->{install_dir}");
        } else {
            $msg = Msg::new("None of the metadata files in the tar file $tarfile needs to be loaded to directory $deploy->{install_dir}");
        }
    }
    (Cfg::opt('load_metadata')) ? $msg->print : $msg->nprint;
    return;
}

sub verify_and_update_metadata {
    my ($deploy,$src_dir,@dest_dirs) = (@_);
    my ($files,$hdest_dir,$hsrc_dir,$mdest_dir,$msrc_dir,$rel,$updated);
    my $localsys = Obj::localsys();
    $rel=Obj::rel();
    $msrc_dir = "$src_dir/matrix";
    $hsrc_dir = "$src_dir/cpihotfix";

    $files = '';
    $files = EDR::cmd_local("cd $msrc_dir 2>/dev/null; _cmd_ls *.json 2>/dev/null") if (-d $msrc_dir);
    for my $dest_dir(@dest_dirs) {
        $mdest_dir = "$dest_dir/.matrix";
        $localsys->mkdir($mdest_dir) if (!-d $mdest_dir);
        for my $file(split(/\n/,$files)) {
            if ($deploy->verify_metadata_file_name($file,$msrc_dir) && $deploy->verify_metadata_contents($file,$msrc_dir,$mdest_dir)) {
                EDR::cmd_local("_cmd_cp -pf $msrc_dir/$file $mdest_dir");
                Msg::log("File $file is loaded to $mdest_dir");
                $updated = 1;
            } else {
                Msg::log("File $file does not have a valid file name or more recent contents, skip to load it to $mdest_dir");
            }
        }
    }
    # Re-init release matrix because lots of matrixes has been updated
    $rel->init_releasematrix($rel->{matrix_path});
    $files = '';
    $files = EDR::cmd_local("cd $hsrc_dir 2>/dev/null; _cmd_ls *.pl 2>/dev/null") if (-d $hsrc_dir);
    for my $dest_dir(@dest_dirs) {
        $hdest_dir = "$dest_dir/.cpihotfix";
        $localsys->mkdir($hdest_dir) if (!-d $hdest_dir);
        for my $file(split(/\n/,$files)) {
            if ($deploy->verify_metadata_file_name($file,$hsrc_dir) && $deploy->verify_metadata_contents($file,$hsrc_dir,$hdest_dir)) {
                EDR::cmd_local("_cmd_cp -pf $hsrc_dir/$file $hdest_dir");
                Msg::log("File $file is loaded to $hdest_dir");
                $updated = 1;
            } else {
                Msg::log("File $file does not have a valid file name or more recent contents, skip to load it to $hdest_dir");
            }
        }
    }
    return $updated;
}

sub verify_metadata_file_name {
    my ($deploy,$file,$src_dir) = (@_);
    my ($filename,$padv,$release,$version,@padvs);
    @padvs=@{$deploy->{preference}{Platform}{value}||$deploy->{supported_padvs}};
    chomp($file);
    if ($file =~ /^(\S+)-(ga|patch|hotfix)\.json$/) {
        $padv = $1;
        return 1 if (EDRu::inarr($padv,@padvs));
    } elsif ($file =~ /\.pl$/) {
        $filename = $file;
        $filename =~ s/\.pl$//g;
        ($release,$padv)=split(/_/,$filename,2);
        $version = EDR::cmd_local("_cmd_grep '^#HOTFIX_VERSION' $src_dir/$file 2>/dev/null");
        chomp($version);
        return 1 if ($version =~ /:${release}$/ && (EDRu::inarr($padv,@padvs)));
    }
    Msg::log("File $file does not have a valid file name");
    return 0;
}

sub verify_metadata_contents {
    my ($deploy,$file,$src_dir,$dest_dir) = (@_);
    my ($content,$padv,$update,$update_hash);
    my ($is_hotfix,$is_matrix,$dest_file,$matrix,$matrix_hash,$src_file);
    chomp($file);
    return 0 if ($file !~ /\.(pl|json)$/);
    $src_file = "$src_dir/$file";
    $dest_file = "$dest_dir/$file";
    $update = EDRu::readfile($src_file);
    if ($file =~ /\.pl$/) {
        EDR::cmd_local("$^X -c $src_file 2>/dev/null");
        return 0 if (EDR::cmdexit()!=0);
        $is_hotfix = 1;
    } elsif ($file =~ /\.json$/) {
        $update_hash = EDRu::eval_json($update);
        return 0 if (!$update_hash);
        $is_matrix = 1;
    }
    return 1 if (!-f "$dest_file");
    # check if update contents if more recent than existing files
    if (EDRu::cksum($src_file) eq EDRu::cksum($dest_file)) {
        Msg::log("$dest_file has the same checksum as $src_file, no need to update");
        return 0;
    }
    if ($is_hotfix) {
        my ($update_vers,$vers);
        $content = EDRu::readfile($dest_file);
        $update_vers = $1 if ($update =~ /^#HOTFIX_VERSION:\S+-\S+P(\d+)$/m);
        $vers = $1 if ($content =~ /^#HOTFIX_VERSION:\S+-\S+P(\d+)$/m);
        return 1 if ($update_vers >= $vers);
    } elsif ($is_matrix) {
        $matrix = EDRu::readfile($dest_file);
        $matrix_hash = EDRu::eval_json($matrix);
        return 1 if (!$matrix_hash);
        $padv = $file;
        $padv =~ s/-\S*$//g;
        return $deploy->verify_matrix_release_date($update_hash,$matrix_hash,$padv);
    }
    return 0;
}

sub verify_matrix_release_date {
    my ($deploy,$update_hash,$matrix_hash,$padv) = (@_);
    my ($max_mdate,$max_mid,$max_udate,$max_uid,$mdate,$mid,$udate,$uid);
    for my $id(keys %{$update_hash->{"$padv"}}) {
        $uid = $update_hash->{"$padv"}{$id}{release_id};
        $udate = $update_hash->{"$padv"}{$id}{release_date};
        $max_uid = $uid if ($uid > $max_uid);
        $max_udate = $udate if ($udate gt $max_udate);
    }
    for my $id(keys %{$matrix_hash->{"$padv"}}) {
        $mid = $matrix_hash->{"$padv"}{$id}{release_id};
        $mdate = $matrix_hash->{"$padv"}{$id}{release_date};
        $max_mid = $mid if ($mid > $max_mid);
        $max_mdate = $mdate if ($mdate gt $max_mdate);
    }
    Msg::log("Update file latest release date: $max_udate, highest release id: $max_uid; Matrix file latest release date: $max_mdate, highest release id: $max_mid.");
    return 1 if (($max_uid >= $max_mid) || ($max_udate ge $max_mdate));
    return 0;
}
# _END_Update_Metadata

# _START_Terminology
sub terminology {
    my ($deploy)=@_;
    Msg::title();
    my $edr=Obj::edr();
    my $title=Msg::new("$edr->{msg}{deploy_sfha_script} Terminology\n");
    $title->bold;

    my $ri=Msg::new("Release Image");
    my $rid=Msg::new("A directory that contains the package, patch, documentation, and scripted components of a Storage Foundation release.");
    $ri->bold_bullet($rid);

    my $rep=Msg::new("Repository");
    my $repd=Msg::new("A designated directory where release images are stored in an organized manner. Sharing the repository directory makes the stored release images accessible from other systems in your environment.");
    $rep->bold_bullet($repd);

    my $ds=Msg::new("Deployment Server");
    my $dsd=Msg::new("The system hosting the repository.  With ssh communication enabled between systems, the Deployment Server can act as a central point of installation and upgrade of other systems in your environment.");
    $ds->bold_bullet($dsd);

    my $rl=Msg::new("Release Levels");
    my $rld=Msg::new("Each release image is defined to be one of three release levels:");
    $rl->bold_bullet($rld);

    my $ga=Msg::new("Base Releases");
    my $gad=Msg::new("Major releases that:");
    my $ga1=Msg::new("allow fresh installation and product selection");
    my $ga2=Msg::new("include new features");
    my $ga3=Msg::new("are installable over most prior versions");
    my $ga4=Msg::new("install package-based updates");
    my $ga5=Msg::new("offer automated installations");
    my $ga6=Msg::new("are available from FileConnect (https://fileconnect.symantec.com)");
    $ga->bold_bullet($gad,1);
    $ga1->bullet(2);
    $ga2->bullet(2);
    $ga3->bullet(2);
    $ga4->bullet(2);
    $ga5->bullet(2);
    $ga6->bullet(2);

    my $patch=Msg::new("Maintenance Releases");
    my $patchd=Msg::new("Patch release updates that:");
    my $patch1=Msg::new("include multiple fixes and limited new features");
    my $patch2=Msg::new("are installable over any product of one specific Base Release version");
    my $patch3=Msg::new("install patch based updates (on native-patch based platforms)");
    my $patch4=Msg::new("offer automated installations");
    my $patch5=Msg::new("are available from SORT (https://sort.symantec.com)");
    $patch->bold_bullet($patchd,1);
    $patch1->bullet(2);
    $patch2->bullet(2);
    $patch3->bullet(2);
    $patch4->bullet(2);
    $patch5->bullet(2);

    my $hf=Msg::new("Patch Releases");
    my $hfd=Msg::new("Product release updates that:");

    my $hf1=Msg::new("include fixes only, typically for one specific issue");
    my $hf2=Msg::new("are installable over specific products of one specific Base or Maintenance release version");
    my $hf3=Msg::new("multiple Patches may potentially be installed");
    my $hf4=Msg::new("install patch based updates (on native-patch based platforms)");
    my $hf5=Msg::new("offer automated installation from 6.0.1 level forward. Prior levels must be manually installed");
    my $hf6=Msg::new("are available from SORT (https://sort.symantec.com)");
    $hf->bold_bullet($hfd,1);
    $hf1->bullet(2);
    $hf2->bullet(2);
    $hf3->bullet(2);
    $hf4->bullet(2);
    $hf5->bullet(2);
    $hf6->bullet(2);

    my $ib=Msg::new("Install Bundle");
    my $ibd=Msg::new("A defined set of compatible Base, Maintenance, or Patch releases that are installed together in order to directly install or upgrade to higher release levels with a single execution.  Install Bundles are supported from version 6.1 forward with the exception of bundling 6.0.1, 6.0.2, or 6.0.4 Base Releases with 6.0.5 or higher Maintenance Releases.");
    $ib->bold_bullet($ibd);

    my $it=Msg::new("Install Template");
    my $itd=Msg::new("A set of packages discovered from a running system.  Once discovered, the Install Template is used to install the same product/version/packages on to other systems.  The release image(s) for the Install Template must be stored in the repository in order for it to be installed.  Install Templates may be a single release image (Base only) or an Install Bundle, however only supported Install Bundle versions can be installed using Install Templates.  Install Templates do not replicate any configuration information at this time.");
    $it->bold_bullet($itd);

    my $fc=Msg::new("FileConnect");
    my $fcd=Msg::new("The site where Base images can be downloaded - https://fileconnect.symantec.com/");
    $fc->bold_bullet($fcd);

    my $sort=Msg::new("SORT");
    my $sortd=Msg::new("The location where Maintenance and Patch release images can be downloaded - https://sort.symantec.com.  Release images on SORT can be automatically downloaded by the Deployment Script as long as the system has an Internet connection.");
    $sort->bold_bullet($sortd);

    my $obs=Msg::new("obsoleted");
    my $obsd=Msg::new("A term describing patches that should typically be avoided because a more recent update exists.");
    $obs->bold_bullet($obsd);

    my $curr=Msg::new("current");
    my $currd=Msg::new("A term describing patches that are currently of the most recent version and therefore are not obsoleted.");
    $curr->bold_bullet($currd);

    my $rm=Msg::new("release matrix");
    my $rmd=Msg::new("Reference files that store all Storage Foundation release information.  These files are periodically updated in order to provide the most current release information.");
    $rm->bold_bullet($rmd);

    my $push=Msg::new("push installation");
    my $pushd=Msg::new("Installing and updating remote systems from the Deployment Server using enabled ssh.");
    $push->bold_bullet($pushd);

    my $pull=Msg::new("direct installation");
    my $pulld=Msg::new("Installing and updating systems directly by accessing Release Images stored within a shared repository.");
    $pull->bold_bullet($pulld);

    Msg::prtc();
}
# _END_Terminology

# _START_Deploy_SFHA_Help
sub deploy_sfha_help {
    my ($deploy)=@_;
    Msg::title();
    my $edr=Obj::edr();
    my $help=Msg::new("$edr->{msg}{deploy_sfha_script} Usage\n");
    $help->bold;

    my $rmsg=Msg::new("The [$edr->{msg}{key_manage_repository}] Manage Repository Images option is used to:");
    my $r1=Msg::new("View All Available Storage Foundation Releases");
    my $r2=Msg::new("Download a Release Image into the repository");
    my $r3=Msg::new("View Available Release Images that are currently stored in the repository");
    my $r4=Msg::new("Remove a Release Image");
    my $r5=Msg::new("Load a previously downloaded Release Image into the repository");
    $rmsg->print;
    $r1->bullet;
    $r2->bullet;
    $r3->bullet;
    $r4->bullet;
    $r5->bullet;
    Msg::n();

    my $vmsg=Msg::new("The [$edr->{msg}{key_version_check}] Version Check Systems option is used to:");
    my $v1=Msg::new("Discover what Storage Foundation products are installed on a system including version and Patches");
    my $v2=Msg::new("Discover what higher Base, Maintenance, and Patch release versions are available for the system to upgrade to");
    my $v3=Msg::new("Conveniently download available Maintenance and Patch level Release Images into the repository");
    my $v4=Msg::new("Install any available release currently stored in the repository that is applicable to the system");
    $vmsg->print;
    $v1->bullet;
    $v2->bullet;
    $v3->bullet;
    $v4->bullet;
    Msg::n();

    my $umsg=Msg::new("The [$edr->{msg}{key_install_upgrade}] Install/Upgrade Systems option is used to install any available release currently stored in the repository on to the local or remote systems");
    $umsg->print;
    Msg::n();

    my $dmsg=Msg::new("The [$edr->{msg}{key_define_install_bundle}] Define/Modify Install Bundles option is used to define sets of compatible releases, referred to as Install Bundles, to be Installed together.  Defining and utilizing Install Bundles helps install or upgrade systems directly to Maintenance or Patch release levels with only one execution.");
    $dmsg->print;
    Msg::n();

    my $tmsg=Msg::new("The [$edr->{msg}{key_create_install_templates}] Create Install Templates option is used to create a template based on a system that already has Storage Foundation HA software installed.  The product, version, and package list will be discovered and an Install Template will be created with that information.  Once corresponding release images are downloaded and stored in the repository, the [$edr->{msg}{key_install_upgrade}] Install/Upgrade Systems option can be used to install or upgrade systems in an identical manner to the discovered system.");
    $tmsg->print;
    Msg::n();

    my $mmsg=Msg::new("The [$edr->{msg}{key_update_metadata}] Update Meta-Data option is required only when a Deployment Server is behind a firewall and does not have access to the SORT website for receiving updates.  This option performs two tasks:");
    my $m1=Msg::new("When run from a system with Internet access, downloads a tar file containing release matrix and Patch updates so that the most up-to-date release information is available");
    my $m2=Msg::new("Loads the contents of the tar file on to a Deployment Server running behind a firewall");
    $mmsg->print;
    $m1->bullet;
    $m2->bullet;
    Msg::n();

    my $smsg=Msg::new("The [$edr->{msg}{key_set_preferences}] Set Preferences option is used to view and modify various program settings.");
    $smsg->print;
    Msg::n();

    my $tmsg=Msg::new("The [$edr->{msg}{key_terminology}] Terminology and Usage option is used to define frequently referenced terms and describe usage.");
    $tmsg->print;
    Msg::prtc();
}
# _END_Deploy_SFHA_Help

# _START_Deploy_SFHA_Lock
# return 0 if the lock is added successfully
# return 1 otherwise
sub add_deploy_sfha_lock {
    my ($deploy,$image,$action,$target) = @_;
    my $cfg=Obj::cfg();
    return [] if($cfg->{opt}{repository});
    return $deploy->manage_deploy_sfha_lock("add",$image,$action,$target);
}

# return 0 if the lock is cleared successfully
# return 1 otherwise
sub clear_deploy_sfha_lock {
    my ($deploy,$image,$action,$target) = @_;
    my $cfg=Obj::cfg();
    return [] if($cfg->{opt}{repository});
    return $deploy->manage_deploy_sfha_lock("clear",$image,$action,$target);
}

# A bundle can be either Install Bundle or Install Template
sub add_deploy_sfha_bundle_lock {
    my ($deploy,$bundle,$action,$target) = @_;
    my (@errors,$error,$image,$msg);
    if ($bundle->{Base}{showform}) {
        $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$bundle->{Base}{showform}}{image})[-1];
        $error=$deploy->add_deploy_sfha_lock($image, $action, $target);
        push (@errors, @$error);
    }
    if (!@errors && $bundle->{Maintenance}{showform}) {
        $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$bundle->{Maintenance}{showform}}{image})[-1];
        $error=$deploy->add_deploy_sfha_lock($image, $action, $target);
        push (@errors, @$error);
    }
    if (!@errors && $bundle->{Patch}{showform}) {
        for my $patch (split(/, /, $bundle->{Patch}{showform})) {
            $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$patch}{image})[-1];
            $error=$deploy->add_deploy_sfha_lock($image, $action, $target);
            push (@errors, @$error);
        }
    }
    if (@errors) {
        $msg=Msg::new("Found the following conflicts for installation of $bundle->{Name}{showform} on $target:\n");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts(\@errors);
        # After displaying the conflicts, we clear the locks of that bundle, in case Base/Maintenance image already been locked.
        # Since the condition of clearing deploy_sfha lock must ensure all the information are the same, it won't clear up the wrong lock
        $deploy->clear_deploy_sfha_bundle_lock($bundle,$action,$target);
        Msg::prtc();
        return 0;
    }
    return 1;
}

sub clear_deploy_sfha_bundle_lock {
    my ($deploy,$bundle,$action,$target) = @_;
    my ($image);
    if ($bundle->{Base}{showform}) {
        $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$bundle->{Base}{showform}}{image})[-1];
        $deploy->clear_deploy_sfha_lock($image, $action, $target);
    }
    if ($bundle->{Maintenance}{showform}) {
        $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$bundle->{Maintenance}{showform}}{image})[-1];
        $deploy->clear_deploy_sfha_lock($image, $action, $target);
    }
    if ($bundle->{Patch}{showform}) {
        for my $patch (split(/, /, $bundle->{Patch}{showform})) {
            $image=(split /\//,$deploy->{image_repository}{$bundle->{Platform}{value}}{$patch}{image})[-1];
            $deploy->clear_deploy_sfha_lock($image, $action, $target);
        }
    }
    return 1;
}

# return 0 if successful
# return 1 if failed
# when writing to the lockfile, the following fields are added in one single line:
# field[0]	hostname that this action is taking place
# field[1]	PID of this process
# field[2]	deploy_sfha script name of the above PID
# field[3]	image to be opterated on
# field[4]	epoch time this action happens, not the PID creating time
# field[5]	action of this entry, right now can be install/delete
# field[6]	target hostname(s) for install only
#
sub manage_deploy_sfha_lock {
    my ($deploy,$action_type,$image,$action,$target) = @_;
    my ($lockfile,$msg,@lines,@fields,$conflict,%seen);

    $deploy->get_deploy_sfha_process_info();
    $conflict=[];

    # $lockfile always be $deploy->{preference}{Repository}{value}/.lock/$image
    $lockfile=$deploy->get_deploy_sfha_lockfile($image);
    if (!$lockfile) {
        $deploy->{repo}{read_only}=1;
        return $conflict;
    }
    if ($target) {
        @lines=(split /\s+|,/, $target);
        $target=join ",", @lines;
    }
    $msg=$deploy->read_deploy_sfha_locks();
    @lines=(split /\n/,$msg);
    @lines = grep { ! $seen{ $_ }++ } @lines;
    $deploy->cleanup_stale_deploy_sfha_locks(\@lines,$deploy->{proc_info}{hostname},$deploy->{proc_info}{name});
    # no further action if $action_type is unset, taken as cleanup only.
    return $conflict unless ($action_type);

    if ($action_type eq "add") {
        $conflict=$deploy->find_conflict_deploy_sfha_lock(\@lines,$action_type,$action,$image,$target);
        if ($#$conflict<0) {
            $msg="$deploy->{proc_info}{hostname} $deploy->{proc_info}{pid} $deploy->{proc_info}{name} $image $deploy->{proc_info}{time} $action ";
            $msg.="$target" if ($target);
            push (@lines, $msg);
        }
    } elsif ($action_type eq "clear") {
        for (my $i=$#lines+1; $i>=0; $i--) {
            next if ($lines[$i] =~ /^(#|\s*$)/);
            @fields=(split /\s+/, $lines[$i]);
            next unless (($fields[0] eq $deploy->{proc_info}{hostname}) &&
                         ($fields[1] eq $deploy->{proc_info}{pid}) &&
                         ($fields[2] eq $deploy->{proc_info}{name}) &&
                         ($fields[3] eq $image) &&
                         ($fields[5] eq $action) );
            next unless ($target eq $fields[6]);
            delete $lines[$i];
        }
    }
    %seen = ();
    @lines = grep { ! $seen{ $_ }++ } @lines;
    $msg = join "\n", @lines;
    if ($#$conflict<0) {
        $deploy->write_deploy_sfha_locks($msg,$lockfile);
    }

    return $conflict;
}

sub cleanup_stale_deploy_sfha_locks {
    my ($deploy,$lines,$hostname,$proc_name)=@_;
    my ($pids,@fields,$content,$repo_dir,$files);

    $pids=$deploy->get_deploy_sfha_pids();
    return 0 if ($#$pids < 0);

    $deploy->get_deploy_sfha_process_info() unless ($deploy->{proc_info});
    $hostname ||=$deploy->{proc_info}{hostname};
    $proc_name||=$deploy->{proc_info}{name};
    if ($#$lines <0 ) {
        $repo_dir=$deploy->get_deploy_sfha_lock_dir();
        $files=EDR::cmd_local("_cmd_ls $repo_dir");
        for my $file (split /\n/,$files) {
            next unless (-f "$repo_dir/$file");
            @fields=(split /\n/, EDRu::readfile("$repo_dir/$file"));
            next if ($#fields < 0);
            $deploy->cleanup_stale_deploy_sfha_locks(\@fields,$hostname);
            $content=join "\n",@fields;
            $deploy->write_deploy_sfha_lockfile($content,"$repo_dir/$file");
        }
    } else {
        for (my $i=$#$lines; $i>=0; $i--) {
            if ($lines->[$i] =~ /^(#|\s*$)/) {
                delete $lines->[$i];
                next;
            }
            @fields=(split /\s+/, $lines->[$i]);
            next unless ($fields[0] eq $hostname);
            delete $lines->[$i] unless (EDRu::inarr($fields[1],@{$pids}));
        }
    }
    return 0;
}

sub duplicate_actions_on_host {
    my ($deploy,$targets,$hosts)=@_;
    my ($ret,@existing);

    $ret=0;
    $hosts=~s/,$//g;
    $targets=~s/,$//g;
    if ($hosts && $targets) {
        @existing=(split /,/,$hosts);
        for my $target (split /,/, $targets) {
            next unless (EDRu::inarr($target,@existing));
            $ret=1;
            last;
        }
    }

    return $ret;
}

sub find_conflict_deploy_sfha_lock {
    my ($deploy,$lines,$action_type,$action,$image,$target)=@_;
    my ($conflict,@fields,@conflicts,%seen);

    if ($target) {
        @conflicts=(split /\s+|,/, $target);
        $target=join ",", @conflicts;
    }
    @conflicts=();
    for (my $i=0; $i<=$#$lines; $i++) {
        next if ($lines->[$i] =~ /^(#|\s*$)/);
        @fields=(split /\s+/, $lines->[$i]);
        if ($action eq "change_repository") {
            # for repository change, any exisitng actions for the images are conflicts
            #push (@conflicts, $lines->[$i]);
            push (@conflicts, [@fields]);
        } elsif ($action eq "install") {
            # if the same target being used for another action
            # then we skip this
            #push (@conflicts, $lines->[$i]) if ($deploy->duplicate_actions_on_host($target,$fields[6]));
            push (@conflicts, [@fields]) if ($deploy->duplicate_actions_on_host($target,$fields[6]));
            $conflict=$i+1;
            # if the same image is used for add/delete image, or change_repository
            # then we skip this
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|load|remove)/)));
        } elsif ($action eq "download") {
            # if the action is download an image
            # then we have to make sure the image is not being downloaded|loaded|removed by another deploy_sfha
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|load|remove)/)));
        } elsif ($action eq "load") {
            # if the action is load an image
            # then we have to make sure the image is not being downloaded|loaded|installed|removed by another deploy_sfha
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|install|load|remove)/)));
        } elsif ($action eq "remove") {
            # if the action is delete an image
            # then we have to make sure the image is not being used by anybody else
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      ($image && ($fields[3] eq $image)));
            $conflict=$i+1;
        } elsif ($action =~ /^install_(bundle|template)/) {
            if ($deploy->duplicate_actions_on_host($target,$fields[6])) {
                # only one circumstance that we don't think there's conflict:
                # Same Install Bundle, Different Image
                unless (($action eq $fields[5]) && ($image ne ($fields[3]))) {
                    push (@conflicts, [@fields]);
                }
            }
            $conflict=$i+1;
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                    (($fields[3] eq $image) && ($fields[5] =~ /(download|load|remove)/)));
        } else {
            last;
        }
    }

    @conflicts = grep { ! $seen{ $_ }++ } @conflicts;
    return \@conflicts;
}

sub display_deploy_sfha_conflicts {
    my ($deploy,$conflicts)=@_;
    my ($i,$j,$contents, $titles);

    return '' if ($#$conflicts < 0);
    $titles=['hostname',[qw(pid PID)],[qw(script script)],[qw(image image)],[qw(date_time Starting_time)],[qw(action action)],[qw(targets targets)]];

    for ($i=0; $i <=$#$conflicts; $i++) {
        $contents->[$i]{$titles->[0]} = ($conflicts->[$i][0]) ? $conflicts->[$i][0] : "-";
        for ($j=1; $j <= $#$titles; $j++) {
            $contents->[$i]{$titles->[$j][0]} = ($conflicts->[$i][$j]) ? $conflicts->[$i][$j] : "-";
        }
    }

    return Msg::table($contents,$titles);
}

# get the repository/.lock status file
sub get_deploy_sfha_lockfile {
    my ($deploy,$image)=@_;
    my ($repo,$edr);

    $repo=$deploy->get_deploy_sfha_lock_dir();
    return  unless (-d $repo);
    $repo.="/$image";
    return $repo;
}

# get the repository/.lock dir
sub get_deploy_sfha_lock_dir {
    my $deploy=shift;
    my ($repo,$cpic,$edr);

    $cpic = Obj::cpic();
    $edr = Obj::edr();
    $repo=$deploy->get_image_repository_dir();
    return '' unless (-d $repo);
    $repo=~s/\/$//g;
    $repo.="/.lock";
    $edr->{localsys}->mkdir($repo) if (!-d $repo);

    return $repo;
}

sub get_image_repository_dir {
    my $deploy=shift;
    my ($repository,$cpic,$cfg);

    $cfg=Obj::cfg();
    $cpic=Obj::cpic();
    $repository||=$cfg->{opt}{repository} if (defined $cfg->{opt}{repository});
    $repository||=$deploy->{preference}{Repository}{value} if (defined $deploy->{preference}{Repository}{value});

    return $repository;
}

sub get_deploy_sfha_pids {
    my $deploy=shift;
    my ($edr,$localsys,$file,$files,@pids,$dir,$pid,$sys_pids);

    $edr = Obj::edr();
    $file=$edr->{script};
    $dir=(split /$file/,$edr->{tmpdir}, 2)[0];
    $dir.="/$file-*";
    $files="$dir/$file\.pid";
    $localsys=Obj::localsys();
    $files=$localsys->cmd("_cmd_ls $files");
    $sys_pids=$localsys->proc_pids();
    for $file (split /\n/,$files) {
        next unless (-f $file);
        $pid=$deploy->read_deploy_sfha_pid_file($file);
        $pid=~s/\s+//g;
        next unless ($pid=~/^\d+$/);
        push (@pids, $pid) if (!EDRu::inarr($pid,@pids) && EDRu::inarr($pid,@{$sys_pids}));
    }
    return \@pids;
}

sub read_deploy_sfha_pid_file {
    my ($deploy,$file)=@_;
    my ($pid,$localsys);

    $localsys=Obj::localsys();
    $pid=$localsys->readfile($file);

    return $pid;
}

sub create_deploy_sfha_pid_file {
    my ($deploy,$dir,$file)=@_;
    my ($pid,$localsys);

    $file=$dir . "/". $file . ".pid";
    $pid=$$;
    $localsys=Obj::localsys();
    $localsys->createfile($file);
    $localsys->writefile($pid,$file);

    return 0;
}

# read_deploy_sfha_locks reads the contents of all the lock files
# under <repo>/.lock/
sub read_deploy_sfha_locks {
    my $deploy=shift;
    my ($repo_dir,$locks,$files,$file_content);

    $repo_dir=$deploy->get_deploy_sfha_lock_dir();
    $files=EDR::cmd_local("_cmd_ls $repo_dir");
    for my $file (split /\n/,$files) {
        $file="$repo_dir/$file";
        if (-f $file) {
            $file_content=EDRu::readfile($file);
            $locks.=$file_content ."\n";
        }
    }
    return $locks;
}

sub write_deploy_sfha_lockfile {
    my ($deploy,$content,$file)=@_;

    return unless ($deploy->can_write_deploy_sfha_lockfile($file));
    return EDRu::writefile($content,$file);
}

sub write_deploy_sfha_locks {
    my ($deploy,$locks,$lockfile)=@_;
    my ($repo_dir,$lock,$file,%contents);

    for $lock (split /\n/, $locks) {
        next if ($lock=~/^#|^\s*$/);
        $file=(split /\s+/, $lock)[3];
        push (@{$contents{$file}}, $lock);
    }
    $repo_dir=$deploy->get_deploy_sfha_lock_dir();
    for $file (sort keys %contents) {
        $lock=join "\n",@{$contents{$file}};
        $deploy->write_deploy_sfha_lockfile($lock,"$repo_dir/$file");
    }
    $lockfile=(split /\//, $lockfile)[-1];
    if (!$contents{$lockfile}) {
        $lock="";
        $deploy->write_deploy_sfha_lockfile($lock,"$repo_dir/$lockfile");
    }

    return '';
}

# test the deploy_sfha_lockfile writable or not
sub can_write_deploy_sfha_lockfile {
    my ($deploy,$lock)=@_;
    my ($msg,$ret,$localsys);

    $ret="";
    $localsys=Obj::localsys();
    $msg="# write test from $localsys->{sys}, please ignore";
    $ret=EDR::cmd_local("echo '$msg' >> $lock");
    ($ret) ? $deploy->{repo}{read_only}=1 : $deploy->{repo}{read_only}=0;
    $msg=($ret) ? "No, we can NOT write to $lock" : "Yeah! wecan write to $lock";
    return ($ret) ? 0 : 1;
}

sub get_deploy_sfha_process_info {
    my $deploy=shift;

    $deploy->{proc_info}{pid}=$$;
    $deploy->{proc_info}{name}=$0;
    $deploy->{proc_info}{hostname}=Obj::localsys()->{sys};
    $deploy->{proc_info}{time}=EDRu::datetime();
    $deploy->{proc_info}{name}=EDRu::app_path($deploy->{proc_info}{name}) ."/". EDRu::basename($deploy->{proc_info}{name});

    return '';
}
# _END_Deploy_SFHA_Lock

1;
